@@ -327,6 +327,57 @@ void gemm(
       transa, transb, m, n, k, alpha, a, lda, b, ldb, beta, c, ldc);
 }
 
+void gemm(
+    TransposeType transa, TransposeType transb,
+    int64_t m, int64_t n, int64_t k,
+    const float alpha,
+    const at::BFloat16 *a, int64_t lda,
+    const at::BFloat16 *b, int64_t ldb,
+    const float beta,
+    float *c, int64_t ldc) {
+  internal::normalize_last_dims(transa, transb, m, n, k, &lda, &ldb, &ldc);
+#if AT_BUILD_WITH_BLAS() && defined(BLAS_HAS_SBGEMM)
+   if (use_blas_gemm(transa, transb, m, n, k, lda, ldb, ldc)) {
+      int m_ = m, n_ = n, k_ = k, lda_ = lda, ldb_ = ldb, ldc_ = ldc;
+      char transa_ = to_blas(transa), transb_ = to_blas(transb);
+      float alpha_ = alpha, beta_ = beta;
+      sbgemm_(&transa_, &transb_,
+              &m_, &n_, &k_,
+              &alpha_,
+              a, &lda_,
+              b, &ldb_,
+              &beta_,
+              c, &ldc_);
+      return;
+   }
+#endif
+#ifdef MKL_HAS_SBGEMM
+  if (use_blas_gemm(transa, transb, m, n, k, lda, ldb, ldc)) {
+    int m_ = m, n_ = n, k_ = k, lda_ = lda, ldb_ = ldb, ldc_ = ldc;
+    mkl_gemm_bf16bf16f32(transa, transb, m_, n_, k_, alpha, a, lda_, b, ldb_, beta, c, ldc_);
+    return;
+  }
+#endif
+  // for the fallback path, first compute gemm with beta = 0,
+  // and then add c in full precision.
+  int64_t c_size = n * m;
+  std::vector<at::BFloat16> bfloat_c(c_size, 0.f);
+  gemm_stub(
+      at::kCPU, at::kBFloat16,
+      transa, transb, m, n, k, alpha, a, lda, b, ldb, 0.f, bfloat_c.data(), m);
+  for (const auto j : c10::irange(n)) {
+    for (const auto i : c10::irange(m)) {
+      auto offset = j * ldc + i;
+      // beta == 0 won't propagate NaN from C
+      if (beta == 0.f) {
+        c[offset] = c10::convert<float>(bfloat_c[j * m + i]);
+      } else {
+        c[offset] = beta * c[offset] + c10::convert<float>(bfloat_c[j * m + i]);
+      }
+    }
+  }
+}
+
 void gemm(
     TransposeType transa, TransposeType transb,
     int64_t m, int64_t n, int64_t k,
@@ -421,22 +472,19 @@ void gemm_batched(
     return gemm(transa, transb, m, n, k, alpha, a[0], lda, b[0], ldb, beta, c[0], ldc);
   }
 
-  c10::guts::if_constexpr<AT_MKL_ENABLED() && is_blas_library_type<scalar_t>::value>(
-      [&](auto _) {
-        internal::normalize_last_dims(transa, transb, m, n, k, &lda, &ldb, &ldc);
-        if (use_blas_gemm(transa, transb, m, n, k, lda, ldb, ldc)) {
-          gemm_batched_mkl_impl(
-              transa, transb, batch_size, m, n, k, alpha, a, lda, b, ldb, beta, c, ldc);
-        } else {
-          gemm_batched_generic(
-              transa, transb, batch_size, m, n, k, alpha, a, lda, b, ldb, beta, c, ldc);
-        }
-      },
-      [&](auto _) {
-        gemm_batched_generic(
-            transa, transb, batch_size, m, n, k, alpha, a, lda, b, ldb, beta, c, ldc);
-      }
-    );
+  if constexpr (AT_MKL_ENABLED() && is_blas_library_type<scalar_t>::value) {
+    internal::normalize_last_dims(transa, transb, m, n, k, &lda, &ldb, &ldc);
+    if (use_blas_gemm(transa, transb, m, n, k, lda, ldb, ldc)) {
+      gemm_batched_mkl_impl(
+          transa, transb, batch_size, m, n, k, alpha, a, lda, b, ldb, beta, c, ldc);
+    } else {
+      gemm_batched_generic(
+          transa, transb, batch_size, m, n, k, alpha, a, lda, b, ldb, beta, c, ldc);
+    }
+  } else {
+    gemm_batched_generic(
+        transa, transb, batch_size, m, n, k, alpha, a, lda, b, ldb, beta, c, ldc);
+  }
 }
 
 template <typename scalar_t>
@@ -469,33 +517,31 @@ void gemm_batched_with_stride(
     return gemm(transa, transb, m, n, k, alpha, a, lda, b, ldb, beta, c, ldc);
   }
 
-  c10::guts::if_constexpr<AT_MKL_ENABLED() && is_blas_library_type<scalar_t>::value>(
-      [&](auto _) {
-        internal::normalize_last_dims(transa, transb, m, n, k, &lda, &ldb, &ldc);
-        if (use_blas_gemm(transa, transb, m, n, k, lda, ldb, ldc)) {
-          c10::SmallBuffer<const scalar_t*, 16> a_ptrs(batch_size);
-          c10::SmallBuffer<const scalar_t*, 16> b_ptrs(batch_size);
-          c10::SmallBuffer<scalar_t*, 16> c_ptrs(batch_size);
+  if constexpr (AT_MKL_ENABLED() && is_blas_library_type<scalar_t>::value) {
+    internal::normalize_last_dims(transa, transb, m, n, k, &lda, &ldb, &ldc);
+    if (use_blas_gemm(transa, transb, m, n, k, lda, ldb, ldc)) {
+      c10::SmallBuffer<const scalar_t*, 16> a_ptrs(batch_size);
+      c10::SmallBuffer<const scalar_t*, 16> b_ptrs(batch_size);
+      c10::SmallBuffer<scalar_t*, 16> c_ptrs(batch_size);
 
-          for (const auto batch : c10::irange(batch_size)) {
-            a_ptrs[batch] = a + batch_stride_a * batch;
-            b_ptrs[batch] = b + batch_stride_b * batch;
-            c_ptrs[batch] = c + batch_stride_c * batch;
-          }
-          gemm_batched_mkl_impl(
-              transa, transb, batch_size, m, n, k, alpha, a_ptrs.data(), lda,
-              b_ptrs.data(), ldb, beta, c_ptrs.data(), ldc);
-        } else {
-          gemm_batched_with_stride_generic(
-              transa, transb, batch_size, m, n, k, alpha, a, lda, batch_stride_a,
-              b, ldb, batch_stride_b, beta, c, ldc, batch_stride_c);
-        }
-      },
-      [&](auto _) {
-        gemm_batched_with_stride_generic(transa, transb, batch_size, m, n, k, alpha,
-                                         a, lda, batch_stride_a, b, ldb, batch_stride_b,
-                                         beta, c, ldc, batch_stride_c);
-      });
+      for (const auto batch : c10::irange(batch_size)) {
+        a_ptrs[batch] = a + batch_stride_a * batch;
+        b_ptrs[batch] = b + batch_stride_b * batch;
+        c_ptrs[batch] = c + batch_stride_c * batch;
+      }
+      gemm_batched_mkl_impl(
+          transa, transb, batch_size, m, n, k, alpha, a_ptrs.data(), lda,
+          b_ptrs.data(), ldb, beta, c_ptrs.data(), ldc);
+    } else {
+      gemm_batched_with_stride_generic(
+          transa, transb, batch_size, m, n, k, alpha, a, lda, batch_stride_a,
+          b, ldb, batch_stride_b, beta, c, ldc, batch_stride_c);
+    }
+  } else {
+    gemm_batched_with_stride_generic(transa, transb, batch_size, m, n, k, alpha,
+                                     a, lda, batch_stride_a, b, ldb, batch_stride_b,
+                                     beta, c, ldc, batch_stride_c);
+  }
 }
 
 #define INSTANTIATE_BATCHED_GEMM(scalar_t, DType)               \
