@@ -4,6 +4,10 @@
 #include <ATen/core/TensorBody.h>
 #include <c10/util/OptionalArrayRef.h>
 
+#ifdef AT_PER_OPERATOR_HEADERS
+#include <ATen/ops/_assert_tensor_metadata_native.h>
+#endif
+
 namespace at {
 
 class Tensor;
