@@ -105,7 +105,7 @@ static void im2col_out_cpu_template(
           output_n = output.select(0, elt);
 
           im2col<scalar_t>(
-              input_n.data_ptr<scalar_t>(),
+              input_n.const_data_ptr<scalar_t>(),
               n_input_plane,
               input_height,
               input_width,
@@ -119,7 +119,7 @@ static void im2col_out_cpu_template(
               stride_width,
               dilation_height,
               dilation_width,
-              output_n.data_ptr<scalar_t>());
+              output_n.mutable_data_ptr<scalar_t>());
         }
 
         if (!batched_input) {
