@@ -6,6 +6,7 @@
 #include <ATen/TensorOperators.h>
 #include <ATen/native/xnnpack/Engine.h>
 #include <c10/util/irange.h>
+#include <c10/core/SymInt.h>
 #include <c10/util/MaybeOwned.h>
 #include <ATen/TensorSubclassLikeUtils.h>
 
@@ -38,12 +39,44 @@
 
 namespace at { namespace native {
 
+// Parse environment variable "TORCH_LINEAR_FLATTEN_3D"
+static inline bool parseLinearFlatten3d() {
+  // Uninitialized value
+  static int value = -1;
+  if (value == -1) {
+    const char* env_str = std::getenv("TORCH_LINEAR_FLATTEN_3D");
+    if (env_str != nullptr && strcmp(env_str, "1") == 0) {
+      value = 1;
+    } else {
+      value = 0;
+    }
+  }
+  return bool(value);
+}
+
+// `_flatten_nd_linear` flattens all but the last dimension of the input tensor
+// before passing it to linear operation
+static inline Tensor _flatten_nd_linear(const Tensor& input, const Tensor& weight, const Tensor& bias) {
+    const auto input_sizes = input.sym_sizes();
+    // can't use -1 in reshape because it errors when a dimension is 0
+    c10::SymInt flattened_dim = 1;
+    for (int64_t i = 0, ndim = input_sizes.size(); i < ndim - 1; ++i) {
+      flattened_dim = flattened_dim * input_sizes[i];
+    }
+    auto inp_reshape = input.reshape_symint({flattened_dim, input_sizes.at(input_sizes.size() -1)});
+    const auto result = at::addmm(bias, inp_reshape, weight.t());
+    auto new_size = input_sizes.slice(0, input_sizes.size() - 1);
+    c10::SymDimVector sizes_vec(new_size.begin(), new_size.end());
+    sizes_vec.push_back(result.sym_size(1));
+    return result.view_symint(sizes_vec);
+}
+
+
 Tensor linear(const Tensor& input, const Tensor& weight, const c10::optional<Tensor>& bias_opt) {
   // See [Note: hacky wrapper removal for optional tensor]
   auto bias = bias_opt.has_value()
     ? c10::MaybeOwned<Tensor>::borrowed(*bias_opt)
     : c10::MaybeOwned<Tensor>::owned(c10::in_place);
-
   if (input.is_mkldnn()) {
     return at::mkldnn_linear(input, weight, *bias);
   }
@@ -52,17 +85,23 @@ Tensor linear(const Tensor& input, const Tensor& weight, const c10::optional<Ten
     return xnnpack::linear(input, weight, *bias);
   }
 #endif
-  if (input.dim() == 2 && bias->defined()) {
+  const auto input_dim = input.dim();
+  if (input_dim == 2 && bias->defined()) {
     // Fused op is marginally faster.
     return at::addmm(*bias, input, weight.t());
   }
-  if (input.dim() == 3 && bias->defined() && input.is_contiguous() &&
-      !input.is_xla()) {
+  if (bias->defined() && !input.is_xla()) {
     // Also hit the fused path for contiguous 3D input, if not using xla
     // backend. Reshaping/flattening has some performance implications on xla.
-    const auto input_sizes = input.sym_sizes();
-    const auto result = at::addmm(*bias, input.view_symint({input_sizes[0] * input_sizes[1], input_sizes[2]}), weight.t());
-    return result.view_symint({input_sizes[0], input_sizes[1], result.sym_size(1)});
+    if (input.is_contiguous() && input_dim == 3) {
+      return _flatten_nd_linear(input, weight, *bias);
+    } else if (input.is_contiguous() && input.layout() == c10::kStrided && weight.layout() == c10::kStrided && bias->dim() == 1) {
+      return _flatten_nd_linear(input, weight, *bias);
+    } else if (parseLinearFlatten3d() && input_dim == 3) {
+      // If user forces flattening via env var
+      const Tensor input_cont = input.contiguous();
+      return _flatten_nd_linear(input_cont, weight, *bias);
+    }
   }
   auto output = at::matmul(input, weight.t());
   if (bias->defined()) {
@@ -600,45 +639,45 @@ Tensor _trilinear(const Tensor& i1_, const Tensor& i2_, const Tensor& i3_,
   Tensor i1 = i1_;
   Tensor i2 = i2_;
   Tensor i3 = i3_;
-  std::vector<int64_t> output_size;
+  std::vector<c10::SymInt> output_size;
   std::vector<int64_t> sum_dims_12, sum_dims_23;
   int64_t unroll_size = -1;
   // asserts...
   for (const auto i : c10::irange(total_dim)) {
-    int64_t s = 0;
+    c10::SymInt s = 0;
     if (expand1[i]) {
       i1 = i1.unsqueeze(i);
     } else  {
-      s = i1.size(i);
+      s = i1.sym_size(i);
     }
     if (expand2[i]) {
       i2 = i2.unsqueeze(i);
     } else  {
-      s = i2.size(i);
+      s = i2.sym_size(i);
     }
     if (expand3[i]) {
       i3 = i3.unsqueeze(i);
       if (sumdim[i] && (i != unroll_dim))
         sum_dims_12.push_back(i);
     } else  {
-      s = i3.size(i);
+      s = i3.sym_size(i);
       if (sumdim[i] && (i != unroll_dim))
         sum_dims_23.push_back(i);
     }
     output_size.push_back(sumdim[i] ? 1 : s);
     if (i == unroll_dim)
-      unroll_size = s;
+      unroll_size = s.guard_int(__FILE__, __LINE__);
   }
   int64_t slicemul1 = (expand1[unroll_dim] ? 0 : 1);
   int64_t slicemul2 = (expand2[unroll_dim] ? 0 : 1);
   int64_t slicemul3 = (expand3[unroll_dim] ? 0 : 1);
 
-  auto output = at::zeros(output_size, i1.options());
+  auto output = at::zeros_symint(output_size, i1.options());
 
   // Three conditionals are necessary since this function is meant to work for both
   // forward and backward, which changes the dimensions of the inputs.
   // Note that if output has zero elems is because (at least) one of i1, i2, i3 has zero elems.
-  if (i1.numel() != 0 && i2.numel() != 0 && i3.numel() != 0) {
+  if (i1.sym_numel() != 0 && i2.sym_numel() != 0 && i3.sym_numel() != 0) {
     if (! sumdim[unroll_dim]) {
       for (const auto k : c10::irange(unroll_size)) {
         Tensor buf = at::native::sumproduct_pair(i1.narrow(unroll_dim, k * slicemul1, 1),
@@ -670,26 +709,26 @@ Tensor bilinear(const Tensor& input1, const Tensor& input2, const Tensor& weight
 
   TORCH_CHECK(input1.dim() == input2.dim(), "bilinear(): input dimensions do not match: got ", input1.dim(), " and ", input2.dim());
   for (const auto i : c10::irange(input1.dim() - 1)) {
-    TORCH_CHECK(input1.size(i) == input2.size(i),
-              "bilinear(): input batch dimensions do not match at dim ", i, ": got ", input1.size(i), " and ", input2.size(i));
+    TORCH_CHECK(input1.sym_size(i) == input2.sym_size(i),
+              "bilinear(): input batch dimensions do not match at dim ", i, ": got ", input1.sym_size(i), " and ", input2.sym_size(i));
   }
-  TORCH_CHECK(input1.size(input1.dim() - 1) == weight.size(1),
+  TORCH_CHECK(input1.sym_size(input1.dim() - 1) == weight.sym_size(1),
             "bilinear(): input1 size does not match weight size: got ",
-            input1.size(input1.dim() - 1), " but expected ", weight.size(1));
-  TORCH_CHECK(input2.size(input2.dim() - 1) == weight.size(2),
+            input1.sym_size(input1.dim() - 1), " but expected ", weight.sym_size(1));
+  TORCH_CHECK(input2.sym_size(input2.dim() - 1) == weight.sym_size(2),
             "bilinear(): input2 size does not match weight size: got ",
-            input2.size(input2.dim() - 1), " but expected ", weight.size(2));
-  TORCH_CHECK(!bias.defined() || bias.size(0) == weight.size(0),
+            input2.sym_size(input2.dim() - 1), " but expected ", weight.sym_size(2));
+  TORCH_CHECK(!bias.defined() || bias.sym_size(0) == weight.sym_size(0),
             "bilinear(): bias size does not match weight size: got ",
-            bias.size(0), " but expected ", weight.size(0));
+            bias.sym_size(0), " but expected ", weight.sym_size(0));
 
-  std::vector<int64_t> output_size;
-  auto size1 = input1.sizes();
+  std::vector<c10::SymInt> output_size;
+  auto size1 = input1.sym_sizes();
   output_size.insert(output_size.end(), size1.begin(), size1.end() - 1);
-  output_size.push_back(weight.size(0));
-  auto input1_flattened = input1.reshape({-1, input1.size(-1)});
-  auto input2_flattened = input2.reshape({-1, input2.size(-1)});
-  Tensor output = at::_trilinear(input1_flattened, weight, input2_flattened, {1,3}, {0}, {1,2}, {2,3}).reshape(output_size);
+  output_size.push_back(weight.sym_size(0));
+  auto input1_flattened = input1.reshape_symint({-1, input1.sym_size(-1)});
+  auto input2_flattened = input2.reshape_symint({-1, input2.sym_size(-1)});
+  Tensor output = at::_trilinear(input1_flattened, weight, input2_flattened, {1,3}, {0}, {1,2}, {2,3}).reshape_symint(output_size);
   if (bias.defined()) {
     output = output + bias;
   }
@@ -700,16 +739,17 @@ Tensor bilinear(const Tensor& input1, const Tensor& input2, const Tensor& weight
 // in the two dimension lists
 Tensor tensordot(const Tensor& input1, const Tensor& input2, IntArrayRef dims1, IntArrayRef dims2) {
   TORCH_CHECK(dims1.size() == dims2.size(), "both dimension lists should have same length");
-  int64_t csize = 1;  // total size of the contracted dimensions
+  TORCH_CHECK(input1.scalar_type() == input2.scalar_type(), "both inputs should have same dtype");
+  SymInt csize = 1;  // total size of the contracted dimensions
   Tensor t1 = input1;
   Tensor t2 = input2;
   for (const auto i : c10::irange(dims1.size())) {
-    int s1 = input1.size(dims1[i]);
-    int s2 = input2.size(dims2[i]);
+    SymInt s1 = input1.sym_size(dims1[i]);
+    SymInt s2 = input2.sym_size(dims2[i]);
     if (s2 == 1) { // broadcasted dimensions can be summed right away
-      t1 = t1.sum(dims1[i], true);
+      t1 = t1.sum(dims1[i], true, t1.scalar_type());
     } else if (s1 == 1) {
-      t2 = t2.sum(dims2[i], true);
+      t2 = t2.sum(dims2[i], true, t2.scalar_type());
     } else {
       TORCH_CHECK(s1 == s2, "contracted dimensions need to match, but first has size ", s1, " in dim ", dims1[i],
                " and second has size ", s2, " in dim ", dims2[i]);
@@ -719,19 +759,20 @@ Tensor tensordot(const Tensor& input1, const Tensor& input2, IntArrayRef dims1,
 
   auto cdims1 = at::dim_list_to_bitset(dims1, input1.dim());
   auto cdims2 = at::dim_list_to_bitset(dims2, input2.dim());
-  std::vector<int64_t> p1, p2, rsizes;  // p1, p2: input permutations, rsizes: sizes of the result
+  std::vector<int64_t> p1, p2;  // p1, p2: input permutations
+  std::vector<SymInt> rsizes;  // rsizes: sizes of the result
   p1.reserve(input1.dim());
   p2.reserve(input2.dim());
   rsizes.reserve(input1.dim() + input2.dim() - (int64_t) dims1.size());
-  int64_t size1 = 1; // number of non-contracted elements in input1
-  int64_t size2 = 1; // number of non-contracted elements in input2
+  SymInt size1 = 1; // number of non-contracted elements in input1
+  SymInt size2 = 1; // number of non-contracted elements in input2
 
   // fill the permutations and compute sizes
   for (const auto i : c10::irange(input1.dim())) {
     if (! cdims1[i]) {
       p1.emplace_back(i);
-      size1 *= t1.size(i);
-      rsizes.emplace_back(t1.size(i));
+      size1 *= t1.sym_size(i);
+      rsizes.emplace_back(t1.sym_size(i));
     }
   }
   for (const auto x : dims1) {
@@ -743,15 +784,15 @@ Tensor tensordot(const Tensor& input1, const Tensor& input2, IntArrayRef dims1,
   for (const auto i : c10::irange(input2.dim())) {
     if (! cdims2[i]) {
       p2.emplace_back(i);
-      size2 *= t2.size(i);
-      rsizes.emplace_back(t2.size(i));
+      size2 *= t2.sym_size(i);
+      rsizes.emplace_back(t2.sym_size(i));
     }
   }
   // permut and reshape for matrix multiplication
-  t1 = t1.permute(p1).reshape({size1, csize});
-  t2 = t2.permute(p2).reshape({csize, size2});
+  t1 = t1.permute(p1).reshape_symint({size1, csize});
+  t2 = t2.permute(p2).reshape_symint({csize, size2});
   // multiply and reshape to target size
-  return at::mm(t1, t2).reshape(rsizes);
+  return at::mm(t1, t2).reshape_symint(rsizes);
 }
 
 Tensor &tensordot_out(const Tensor& input1, const Tensor& input2, IntArrayRef dims1, IntArrayRef dims2, Tensor& result) {
