@@ -1,16 +1,38 @@
-#include <ATen/ATen.h>
-#include <ATen/AccumulateType.h>
+#define TORCH_ASSERT_ONLY_METHOD_OPERATORS
+#include <ATen/core/Tensor.h>
 #include <ATen/Dispatch.h>
 #include <ATen/Parallel.h>
+#include <ATen/TensorIndexing.h>
 #include <ATen/TensorMeta.h>
+#include <ATen/TensorOperators.h>
 #include <ATen/TensorUtils.h>
 #include <ATen/native/cpu/utils.h>
 #include <ATen/native/Resize.h>
 #include <c10/util/SmallBuffer.h>
+#include <ATen/TensorSubclassLikeUtils.h>
+
+#ifndef AT_PER_OPERATOR_HEADERS
+#include <ATen/Functions.h>
+#include <ATen/NativeFunctions.h>
+#else
+#include <ATen/ops/cross_entropy_loss_native.h>
+#include <ATen/ops/empty.h>
+#include <ATen/ops/log_softmax.h>
+#include <ATen/ops/nll_loss.h>
+#include <ATen/ops/nll_loss2d.h>
+#include <ATen/ops/nll_loss_backward_native.h>
+#include <ATen/ops/nll_loss_forward.h>
+#include <ATen/ops/nll_loss_forward_native.h>
+#include <ATen/ops/nll_loss_native.h>
+#include <ATen/ops/nll_loss_nd.h>
+#include <ATen/ops/nll_loss_nd_native.h>
+#endif
 
 #include <c10/core/TensorOptions.h>
 #include <c10/util/irange.h>
 
+#include <utility>
+
 namespace at {
 namespace meta {
 TORCH_META_FUNC(nll_loss_forward)
@@ -457,7 +479,7 @@ TORCH_IMPL_FUNC(nll_loss_backward_out_cpu)
       total_weight);
 }
 
-Tensor cross_entropy_loss_prob_target(
+static Tensor cross_entropy_loss_prob_target(
     const Tensor& self,
     const Tensor& target_,
     const Tensor& weight,
@@ -525,18 +547,18 @@ Tensor cross_entropy_loss_prob_target(
   }
 }
 
-Tensor cross_entropy_loss_label_smoothing(
+static Tensor cross_entropy_loss_label_smoothing(
     const Tensor& self,
     const Tensor& target,
     const Tensor& weight,
     int64_t reduction,
-    int64_t ignore_index,
+    c10::SymInt ignore_index,
     double label_smoothing) {
     auto class_dim = self.dim() == 1 ? 0 : 1;
     auto input = at::log_softmax(self, class_dim, self.scalar_type());
-    auto nllloss = at::nll_loss_nd(input, target, weight, reduction, ignore_index);
+    auto nllloss = at::nll_loss_nd_symint(input, target, weight, reduction, ignore_index);
 
-    auto n_classes = input.size(class_dim);
+    auto n_classes = input.sym_size(class_dim);
 
     Tensor smooth_loss;
     if (weight.defined()) {
@@ -551,18 +573,32 @@ Tensor cross_entropy_loss_label_smoothing(
       smooth_loss = -input.sum(class_dim);
     }
 
-    auto ignore_mask = target == ignore_index;
-    smooth_loss.index_put_({ignore_mask}, 0.0);
+    auto ignore_mask = target == std::move(ignore_index);
+    smooth_loss.masked_fill_(ignore_mask, 0.0);
 
     Tensor ret;
     switch (reduction) {
       case Reduction::Mean:
         if (weight.defined()) {
-          // TODO: This code can path can be removed if #61309 is resolved
-          // loss is normalized by the weights to be consistent with nll_loss_nd
-          ret = smooth_loss.sum() / weight.gather(0, target.masked_select(~ignore_mask).flatten()).sum();
+          if (isTensorSubclassLike(weight)){
+            // we will collect weights from 0 index which is always valid
+            // and mask them out if they are ignored
+            auto filtered_target = target.masked_fill(ignore_mask, 0);
+            auto tgt_weights = weight.gather(0, filtered_target.flatten());
+            auto weight_sum =
+                tgt_weights.masked_fill_(ignore_mask.flatten(), 0).sum();
+            ret = smooth_loss.sum() / weight_sum;
+          } else {
+            // TODO: This code can path can be removed if #61309 is resolved
+            // loss is normalized by the weights to be consistent with
+            // nll_loss_nd
+            ret = smooth_loss.sum() /
+                weight.gather(0, target.masked_select(~ignore_mask).flatten())
+                    .sum();
+          }
         } else {
-          ret = smooth_loss.masked_select(~ignore_mask).mean();
+          auto true_mask = ~ignore_mask;
+          ret = smooth_loss.sum()/ true_mask.sum();
         }
         break;
       case Reduction::Sum:
@@ -577,15 +613,15 @@ Tensor cross_entropy_loss_label_smoothing(
     return (1 - label_smoothing) * nllloss + ret * (label_smoothing / n_classes);
 }
 
-Tensor cross_entropy_loss(
+Tensor cross_entropy_loss_symint(
     const Tensor& self,
     const Tensor& target,
     const c10::optional<Tensor>& weight,
     int64_t reduction,
-    int64_t ignore_index,
+    c10::SymInt ignore_index,
     double label_smoothing) {
   Tensor ret;
-  if (self.sizes() == target.sizes()) {
+  if (self.sym_sizes() == target.sym_sizes()) {
     // Assume soft targets when input and target shapes are the same
     TORCH_CHECK(at::isFloatingType(target.scalar_type()),
         "Expected floating point type for target with class probabilities, got ", target.scalar_type());
@@ -601,15 +637,15 @@ Tensor cross_entropy_loss(
     // See [Note: hacky wrapper removal for optional tensor]
     c10::MaybeOwned<Tensor> weight_maybe_owned = at::borrow_from_optional_tensor(weight);
     const Tensor& weight_ = *weight_maybe_owned;
-    ret = cross_entropy_loss_label_smoothing(self, target, weight_, reduction, ignore_index, label_smoothing);
+    ret = cross_entropy_loss_label_smoothing(self, target, weight_, reduction, std::move(ignore_index), label_smoothing);
   } else {
     auto class_dim = self.dim() == 1 ? 0 : 1;
-    ret = at::nll_loss_nd(
+    ret = at::nll_loss_nd_symint(
         at::log_softmax(self, class_dim, self.scalar_type()),
         target,
         weight,
         reduction,
-        ignore_index);
+        std::move(ignore_index));
   }
   return ret;
 }
@@ -623,32 +659,41 @@ Tensor & nll_loss_out(const Tensor & self, const Tensor & target, const c10::opt
   return std::get<0>(at::nll_loss_forward_out(output, total_weight, self, target, weight, reduction, ignore_index));
 }
 
-Tensor nll_loss(const Tensor & self, const Tensor & target, const c10::optional<Tensor>& weight_opt, int64_t reduction, int64_t ignore_index) {
+Tensor nll_loss_symint(const Tensor & self, const Tensor & target, const c10::optional<Tensor>& weight_opt, int64_t reduction, c10::SymInt ignore_index) {
   // See [Note: hacky wrapper removal for optional tensor]
   c10::MaybeOwned<Tensor> weight_maybe_owned = at::borrow_from_optional_tensor(weight_opt);
   const Tensor& weight = *weight_maybe_owned;
 
-  return std::get<0>(at::nll_loss_forward(self, target, weight, reduction, ignore_index));
+  return std::get<0>(at::nll_loss_forward_symint(self, target, weight, reduction, std::move(ignore_index)));
 }
 
-Tensor nll_loss_nd(
+// Duplicate of above code for non-symbolic ints. Kept for BC purposes and to minimize breakages.
+static Tensor nll_loss(const Tensor & self, const Tensor & target, const c10::optional<Tensor>& weight_opt, int64_t reduction, int64_t ignore_index) {
+  // See [Note: hacky wrapper removal for optional tensor]
+  c10::MaybeOwned<Tensor> weight_maybe_owned = at::borrow_from_optional_tensor(weight_opt);
+  const Tensor& weight = *weight_maybe_owned;
+
+  return std::get<0>(at::nll_loss_forward_symint(self, target, weight, reduction, ignore_index));
+}
+
+Tensor nll_loss_nd_symint(
     const Tensor& self,
     const Tensor& target,
     const c10::optional<Tensor>& weight,
     int64_t reduction,
-    int64_t ignore_index) {
+    c10::SymInt ignore_index) {
   if (self.dim() < 1) {
     TORCH_CHECK_VALUE(
         false, "Expected 1 or more dimensions (got ", self.dim(), ")");
   }
 
-  if (self.dim() != 1 && self.sizes()[0] != target.sizes()[0]) {
+  if (self.dim() != 1 && self.sym_sizes()[0] != target.sym_sizes()[0]) {
     TORCH_CHECK_VALUE(
         false,
         "Expected input batch_size (",
-        self.sizes()[0],
+        self.sym_sizes()[0],
         ") to match target batch_size (",
-        target.sizes()[0],
+        target.sym_sizes()[0],
         ").");
   }
 
@@ -656,42 +701,42 @@ Tensor nll_loss_nd(
   Tensor input_ = self;
   Tensor target_ = target;
   if (input_.dim() == 1 || input_.dim() == 2) {
-    ret = at::nll_loss(input_, target_, weight, reduction, ignore_index);
+    ret = at::nll_loss_symint(input_, target_, weight, reduction, std::move(ignore_index));
   } else if (input_.dim() == 4) {
-    ret = at::nll_loss2d(input_, target_, weight, reduction, ignore_index);
+    ret = at::nll_loss2d_symint(input_, target_, weight, reduction, std::move(ignore_index));
   } else {
     // dim == 3 or dim > 4
-    auto n = input_.sizes()[0];
-    auto c = input_.sizes()[1];
-    auto out_size = input_.sizes().slice(2).vec();
+    auto n = input_.sym_sizes()[0];
+    auto c = input_.sym_sizes()[1];
+    auto out_size = input_.sym_sizes().slice(2).vec();
     out_size.insert(out_size.begin(), n);
-    if (target_.sizes().slice(1) != input_.sizes().slice(2)) {
+    if (target_.sym_sizes().slice(1) != input_.sym_sizes().slice(2)) {
       TORCH_CHECK(
           false,
           "Expected target size ",
-          IntArrayRef(out_size),
+          SymIntArrayRef(out_size),
           ", got ",
-          target_.sizes());
+          target_.sym_sizes());
     }
     input_ = input_.contiguous();
     target_ = target_.contiguous();
     // support empty batches, see #15870
     if (input_.numel() > 0) {
-      input_ = input_.view({n, c, 1, -1});
+      input_ = input_.view_symint({n, std::move(c), 1, -1});
     } else {
-      input_ = input_.view({n, c, 0, 0});
+      input_ = input_.view_symint({n, std::move(c), 0, 0});
     }
     if (target_.numel() > 0) {
-      target_ = target_.view({n, 1, -1});
+      target_ = target_.view_symint({std::move(n), 1, -1});
     } else {
-      target_ = target_.view({n, 0, 0});
+      target_ = target_.view_symint({std::move(n), 0, 0});
     }
     if (reduction != Reduction::None) {
-      ret = at::nll_loss2d(input_, target_, weight, reduction, ignore_index);
+      ret = at::nll_loss2d_symint(input_, target_, weight, reduction, std::move(ignore_index));
     } else {
       auto out =
-          at::nll_loss2d(input_, target_, weight, reduction, ignore_index);
-      ret = out.view(out_size);
+          at::nll_loss2d_symint(input_, target_, weight, reduction, std::move(ignore_index));
+      ret = out.view_symint(out_size);
     }
   }
   return ret;
