@@ -28,14 +28,20 @@
 #include <unordered_set>
 
 namespace std {
-  template<> struct hash<at::BFloat16>
-    {
-        size_t operator()(const at::BFloat16& v) const noexcept
-        {
-            return std::hash<uint16_t>()(v.x);
-        }
-    };
-}
+template <>
+struct hash<at::BFloat16> {
+  size_t operator()(const at::BFloat16& v) const noexcept {
+    return std::hash<uint16_t>()(v.x);
+  }
+};
+
+template <>
+struct hash<at::Half> {
+  size_t operator()(const at::Half& v) const noexcept {
+    return std::hash<uint16_t>()(v.x);
+  }
+};
+} // namespace std
 
 namespace at {
 namespace native{
@@ -51,7 +57,7 @@ Tensor unique_elements(const scalar_t* begin, const scalar_t* end,
 
   // Write the output tensor
   Tensor output = at::empty({static_cast<int64_t>(set.size())}, options);
-  scalar_t *output_data = output.data_ptr<scalar_t>();
+  scalar_t *output_data = output.mutable_data_ptr<scalar_t>();
   std::copy(set.begin(), set.end(), output_data);
   if (sorted) {
     std::sort(output_data, output_data + set.size());
@@ -78,7 +84,7 @@ Tensor unique_elements(const bool* begin, const bool* end,
   // Write the output tensor
   int64_t num_elem = seen[false] + seen[true];
   Tensor output = at::empty({num_elem}, options);
-  bool *output_data = output.data_ptr<bool>();
+  bool *output_data = output.mutable_data_ptr<bool>();
 
   if (seen[false]) {
     *output_data++ = false;
@@ -256,7 +262,7 @@ std::tuple<Tensor, Tensor, Tensor> _unique_dim_cpu_template(
     "There are 0 sized dimensions, and they aren't selected, so unique cannot be applied");
 
   // reshape tensor as [dim, -1]
-  Tensor input_flat = self.transpose(dim, 0);
+  Tensor input_flat = self.moveaxis(dim, 0);
   auto orig_sizes = input_flat.sizes().vec();
   input_flat = input_flat.contiguous().view({input_flat.size(0), -1});
 
@@ -305,7 +311,7 @@ std::tuple<Tensor, Tensor, Tensor> _unique_dim_cpu_template(
   auto new_sizes = std::vector<int64_t>(std::move(orig_sizes));
   new_sizes[0] = -1;
   output = output.view(new_sizes);
-  output = output.transpose(0, dim);
+  output = output.moveaxis(0, dim);
 
   return std::make_tuple(output, inverse_indices, counts);
 }
@@ -315,7 +321,7 @@ std::tuple<Tensor, Tensor, Tensor> _unique_dim_cpu_template(
 
 std::tuple<Tensor, Tensor>
 _unique_cpu(const Tensor& self, const bool sorted, const bool return_inverse) {
-  return AT_DISPATCH_ALL_TYPES_AND2(at::ScalarType::BFloat16, at::ScalarType::Bool, self.scalar_type(), "unique", [&] {
+  return AT_DISPATCH_ALL_TYPES_AND3(kBFloat16, kBool, kHalf, self.scalar_type(), "unique", [&] {
     Tensor output, inverse;
     std::tie(output, inverse, std::ignore) = unique_cpu_template<scalar_t>(self, sorted, return_inverse, false);
     return std::make_tuple(output, inverse);
@@ -324,14 +330,14 @@ _unique_cpu(const Tensor& self, const bool sorted, const bool return_inverse) {
 
 std::tuple<Tensor, Tensor, Tensor>
 _unique2_cpu(const Tensor& self, const bool sorted, const bool return_inverse, const bool return_counts) {
-  return AT_DISPATCH_ALL_TYPES_AND2(at::ScalarType::BFloat16, at::ScalarType::Bool, self.scalar_type(), "unique", [&] {
+  return AT_DISPATCH_ALL_TYPES_AND3(kBFloat16, kBool, kHalf, self.scalar_type(), "unique", [&] {
     return unique_cpu_template<scalar_t>(self, sorted, return_inverse, return_counts);
   });
 }
 
 std::tuple<Tensor, Tensor, Tensor>
 unique_dim_cpu(const Tensor& self, const int64_t dim, const bool sorted, const bool return_inverse, const bool return_counts) {
-  return AT_DISPATCH_ALL_TYPES_AND2(at::ScalarType::BFloat16, at::ScalarType::Bool, self.scalar_type(), "unique_dim", [&] {
+  return AT_DISPATCH_ALL_TYPES_AND3(kBFloat16, kBool, kHalf, self.scalar_type(), "unique_dim", [&] {
     // The current implementation using `dim` always sorts due to unhashable tensors
     return _unique_dim_cpu_template<scalar_t>(self, dim, false, return_inverse, return_counts);
   });
@@ -339,7 +345,7 @@ unique_dim_cpu(const Tensor& self, const int64_t dim, const bool sorted, const b
 
 std::tuple<Tensor, Tensor, Tensor>
 unique_dim_consecutive_cpu(const Tensor& self, const int64_t dim, const bool return_inverse, const bool return_counts) {
-  return AT_DISPATCH_ALL_TYPES_AND2(at::ScalarType::BFloat16, at::ScalarType::Bool, self.scalar_type(), "unique_dim", [&] {
+  return AT_DISPATCH_ALL_TYPES_AND3(kBFloat16, kBool, kHalf, self.scalar_type(), "unique_dim", [&] {
     return _unique_dim_cpu_template<scalar_t>(self, dim, true, return_inverse, return_counts);
   });
 }
@@ -347,7 +353,7 @@ unique_dim_consecutive_cpu(const Tensor& self, const int64_t dim, const bool ret
 std::tuple<Tensor, Tensor, Tensor>
 unique_consecutive_cpu(const Tensor& self, const bool return_inverse, const bool return_counts, c10::optional<int64_t> dim) {
   if (!dim.has_value() || (dim.value() == 0 && self.dim() == 1)) {
-    return AT_DISPATCH_ALL_TYPES_AND2(at::ScalarType::BFloat16, at::ScalarType::Bool, self.scalar_type(), "unique", [&] {
+    return AT_DISPATCH_ALL_TYPES_AND3(kBFloat16, kBool, kHalf, self.scalar_type(), "unique", [&] {
       return unique_consecutive_cpu_template<scalar_t>(self, return_inverse, return_counts);
     });
   }
