@@ -302,17 +302,14 @@ void index_copy_kernel(
   });
 }
 
-template <typename scalar_t, typename mask_t>
+template <typename scalar_t>
 void cpu_masked_fill_kernel(TensorIterator& iter, scalar_t value) {
-  auto is_mask_bool = std::is_same<mask_t, bool>::value;
   auto loop = [&](char** data, const int64_t* strides, int64_t n) {
     char* dst = data[0];
     char* mask = data[1];
     for (const auto i : c10::irange(n)) {
-      mask_t mask_value = *(mask_t*)(mask + strides[1] * i);
-      if (!is_mask_bool) {
-        TORCH_CHECK(mask_value == 0 || mask_value == 1, "Mask tensor can take 0 and 1 values only");
-      }
+      bool mask_value = *reinterpret_cast<bool*>(mask + strides[1] * i);
+
       if (mask_value) {
         *(scalar_t*)(dst + strides[0] * i) = value;
       }
@@ -326,17 +323,14 @@ void masked_fill_kernel(TensorIterator& iter, const Scalar& value) {
     iter.dtype(), "masked_fill", [&] {
       scalar_t scalar_val = value.to<scalar_t>();
       auto mask_dtype = iter.input_dtype(0);
-      if (mask_dtype == ScalarType::Bool) {
-        cpu_masked_fill_kernel<scalar_t, bool>(iter, scalar_val);
-      } else {
-        cpu_masked_fill_kernel<scalar_t, unsigned char>(iter, scalar_val);
-      }
+      TORCH_CHECK(mask_dtype == ScalarType::Bool, "masked_fill only supports boolean masks, "
+        "but got mask with dtype ", mask_dtype);
+      cpu_masked_fill_kernel<scalar_t>(iter, scalar_val);
     });
 }
 
-template <typename scalar_t, typename mask_t>
+template <typename scalar_t>
 void cpu_masked_scatter_kernel(TensorIterator& iter, const TensorBase& source) {
-  auto is_mask_bool = std::is_same<mask_t, bool>::value;
   std::ptrdiff_t source_cntr = 0;
   scalar_t* source_ptr = source.data_ptr<scalar_t>();
   auto numel = source.numel();
@@ -347,10 +341,7 @@ void cpu_masked_scatter_kernel(TensorIterator& iter, const TensorBase& source) {
     char* mask = data[1];
     const int64_t mask_stride = strides[1];
     for (const auto i : c10::irange(n)) {
-      mask_t mask_value = *(mask_t*)(mask + mask_stride * i);
-      if (!is_mask_bool) {
-        TORCH_CHECK(mask_value <= static_cast<mask_t>(1), "Mask tensor can take 0 and 1 values only");
-      }
+      auto mask_value = *reinterpret_cast<bool*>(mask + mask_stride * i);
       if (mask_value) {
         TORCH_CHECK(source_cntr < numel, "Number of elements of source < number of ones in mask");
         *(scalar_t*)(dst + dst_stride * i) = *(source_ptr);
@@ -363,6 +354,8 @@ void cpu_masked_scatter_kernel(TensorIterator& iter, const TensorBase& source) {
 }
 
 void masked_scatter_kernel(TensorIterator& iter, const TensorBase& source) {
+ TORCH_CHECK(iter.input_dtype() == ScalarType::Bool, "masked_scatter_ only supports boolean masks, "
+    "but got mask with dtype ", iter.input_dtype());
   AT_DISPATCH_ALL_TYPES_AND_COMPLEX_AND3(
       ScalarType::Bool,
       ScalarType::BFloat16,
@@ -370,18 +363,12 @@ void masked_scatter_kernel(TensorIterator& iter, const TensorBase& source) {
       iter.dtype(),
       "masked_scatter",
       [&] {
-        auto mask_dtype = iter.input_dtype(0);
-        if (mask_dtype == ScalarType::Bool) {
-          cpu_masked_scatter_kernel<scalar_t, bool>(iter, source);
-        } else {
-          cpu_masked_scatter_kernel<scalar_t, unsigned char>(iter, source);
-        }
+          cpu_masked_scatter_kernel<scalar_t>(iter, source);
       });
 }
 
 template <typename scalar_t, typename mask_t, typename func_t>
 void cpu_masked_select_serial_kernel(TensorIterator& iter, const func_t& f) {
-  auto is_mask_bool = std::is_same<mask_t, bool>::value;
   int64_t offset = 0;
   auto loop = [&](char** data, const int64_t* strides, int64_t n) {
     char* dst = data[0];
@@ -389,7 +376,7 @@ void cpu_masked_select_serial_kernel(TensorIterator& iter, const func_t& f) {
     char* mask = data[2];
     for (const auto i : c10::irange(n)) {
       mask_t mask_value = *(mask_t*)(mask + strides[2] * i);
-      if (!is_mask_bool) {
+      if constexpr (!std::is_same<mask_t, bool>::value) {
         TORCH_CHECK(mask_value == 0 || mask_value == 1, "Mask tensor can take 0 and 1 values only");
       }
       if (mask_value) {
@@ -420,7 +407,6 @@ void masked_select_serial_kernel(TensorIterator& iter, int64_t result_stride) {
 
 template <typename scalar_t, typename mask_t, typename func_t>
 void cpu_masked_select_kernel(TensorIterator& iter, const func_t& f) {
-  auto is_mask_bool = std::is_same<mask_t, bool>::value;
   auto loop = [&](char** data, const int64_t* strides, int64_t n) {
     char* dst = data[0];
     char* src = data[1];
@@ -428,7 +414,7 @@ void cpu_masked_select_kernel(TensorIterator& iter, const func_t& f) {
     char* mask_prefix_sum = data[3];
     for (const auto i : c10::irange(n)) {
       mask_t mask_value = *(mask_t*)(mask + strides[2] * i);
-      if (!is_mask_bool) {
+      if constexpr (!std::is_same<mask_t, bool>::value) {
         TORCH_CHECK(mask_value == 0 || mask_value == 1, "Mask tensor can take 0 and 1 values only");
       }
       if (mask_value) {
