@@ -9,8 +9,7 @@
 
 #include <algorithm>
 
-namespace at {
-namespace native {
+namespace at::native {
 namespace {
 
 // Load vector from a smaller type (more elements) to a larger type (fewer elements),
@@ -85,8 +84,8 @@ struct CastLoadPolicy<scalar_t, scalar_t>:
 // For inner sum, load full vec_t then sum partials down to vacc_t size
 template <typename vec_t, typename vacc_t>
 struct InnerSumCastLoadPolicy {
-  using scalar_t = typename vec_t::value_type;
-  using acc_t = typename vacc_t::value_type;
+  using scalar_t = vechold_type<vec_t>;
+  using acc_t = vechold_type<vacc_t>;
 
   static constexpr int64_t memsize() {
     return LoadPolicy<vec_t>::memsize();
@@ -125,8 +124,8 @@ struct InnerSumCastLoadPolicy<Vectorized<c10::BFloat16>, Vectorized<float>> {
 // For outer sum, load a partial vec_t of size vacc_t then cast to vacc_t
 template <typename vec_t, typename vacc_t>
 struct OuterSumCastLoadPolicy {
-  using scalar_t = typename vec_t::value_type;
-  using acc_t = typename vacc_t::value_type;
+  using scalar_t = vechold_type<vec_t>;
+  using acc_t = vechold_type<vacc_t>;
 
   static constexpr int64_t memsize() {
     return sizeof(scalar_t) * vacc_t::size();
@@ -213,8 +212,8 @@ struct NanSumCastLoadPolicy {
 
 template <typename vec_t, typename vacc_t>
 struct InnerNanSumCastLoadPolicy {
-  using scalar_t = typename vec_t::value_type;
-  using acc_t = typename vacc_t::value_type;
+  using scalar_t = vechold_type<vec_t>;
+  using acc_t = vechold_type<vacc_t>;
 
   static constexpr int64_t memsize() {
     return LoadPolicy<vec_t>::memsize();
@@ -552,18 +551,16 @@ void cascade_sum(TensorIterator &iter) {
           char* ptrs[3] = { data[0], data[0], data[1] };
           // NOLINTNEXTLINE(modernize-avoid-c-arrays,cppcoreguidelines-avoid-c-arrays)
           int64_t inner_strides[3] = { strides[0], strides[0], strides[1] };
-          c10::guts::if_constexpr<ignore_nan>(
-            [&](auto) {
+          if constexpr (ignore_nan) {
               basic_loop(ptrs, inner_strides, 0, size0, [](scalar_t a, scalar_t b) {
                 auto a_notnan = at::_isnan(a) ? scalar_t(0) : a;
                 auto b_notnan = at::_isnan(b) ? scalar_t(0) : b;
                 return a_notnan + b_notnan;
               });
-            },
-            [&](auto) {
+          } else {
               basic_loop(ptrs, inner_strides, 0, size0,
                          [](scalar_t a, scalar_t b) { return a + b; });
-            });
+          }
         });
         return;
       }
@@ -632,14 +629,10 @@ void nansum_kernel_impl(TensorIterator &iter) {
 
 }  // namespace (anonymous)
 
-REGISTER_DISPATCH(sum_stub, &sum_kernel_impl);
-
 // nansum on Float16 has poor accuracy with AVX2, and more so with AVX512.
 // So until it's fixed, it won't be dispatched with AVX512. GH issue 59415.
-#ifndef CPU_CAPABILITY_AVX512
+// Besides, these kernels are slower with AVX512 than with AVX2.
 REGISTER_DISPATCH(nansum_stub, &nansum_kernel_impl);
-#else
-REGISTER_NO_AVX512_DISPATCH(nansum_stub);
-#endif
+REGISTER_DISPATCH(sum_stub, &sum_kernel_impl);
 
-}}  // namespace at::native
+}  // namespace at::native
