@@ -327,7 +327,7 @@ void GroupNormKernelImplChannelsLastInternal(
     //
     // for each plain of HxW, scale and bias is calculated only once
     Tensor buffer = at::empty({N * G, 2 * D}, X.options().dtype(c10::CppTypeToScalarType<T_ACC>::value));
-    T_ACC* buffer_data = buffer.data_ptr<T_ACC>();
+    T_ACC* buffer_data = buffer.mutable_data_ptr<T_ACC>();
 
     at::parallel_for(0, N * G, 1, [&](int64_t begin, int64_t end) {
       int64_t n{0}, g{0};
@@ -523,43 +523,28 @@ void GroupNormKernelImpl(
   }
 }
 
-template <typename T, typename PT>
+template <typename T, typename T_ACC>
 void ComputeInternalGradients(
     int64_t N,
     int64_t C,
     int64_t HxW,
     const T* dY,
     const T* X,
-    PT* ds,
-    PT* db) {
+    T_ACC* ds,
+    T_ACC* db) {
+  using Vec = at::vec::Vectorized<T_ACC>;
   at::parallel_for(0, N * C, 1, [=](int64_t start, int64_t end) {
-    constexpr int64_t K = vec::Vectorized<T>::size();
-    const int64_t inner_size = HxW / K * K;
-    // NOLINTNEXTLINE(cppcoreguidelines-pro-type-member-init)
-    std::array<PT, K> ds_arr;
-    // NOLINTNEXTLINE(cppcoreguidelines-pro-type-member-init)
-    std::array<PT, K> db_arr;
     for (const auto i : c10::irange(start, end)) {
       const T* dY_ptr = dY + i * HxW;
       const T* X_ptr = X + i * HxW;
-      vec::Vectorized<PT> ds_vec(0);
-      vec::Vectorized<PT> db_vec(0);
-      for (int64_t j = 0; j < inner_size; j += K) {
-        const vec::Vectorized<T> dy_vec = vec::Vectorized<T>::loadu(dY_ptr + j);
-        const vec::Vectorized<T> x_vec = vec::Vectorized<T>::loadu(X_ptr + j);
-        ds_vec = ds_vec + dy_vec * x_vec;
-        db_vec = db_vec + dy_vec;
-      }
-      ds_vec.store(ds_arr.data());
-      db_vec.store(db_arr.data());
-      PT ds_val = std::accumulate(ds_arr.cbegin(), ds_arr.cend(), PT(0));
-      PT db_val = std::accumulate(db_arr.cbegin(), db_arr.cend(), PT(0));
-      for (const auto j : c10::irange(inner_size, HxW)) {
-        ds_val += dY_ptr[j] * X_ptr[j];
-        db_val += dY_ptr[j];
-      }
-      ds[i] = ds_val;
-      db[i] = db_val;
+      ds[i] = at::vec::map2_reduce_all<T>(
+          [](Vec x, Vec y) { return x * y; },
+          [](Vec x, Vec y) { return x + y; },
+          dY_ptr,
+          X_ptr,
+          HxW);
+      db[i] = at::vec::reduce_all<T>(
+          [](Vec& x, Vec& y) { return x + y; }, dY_ptr, HxW);
     }
   });
 }
@@ -579,9 +564,9 @@ void ComputeInternalGradients(
     constexpr int64_t K = bVec::size();
     const int64_t inner_size = HxW / K * K;
     // NOLINTNEXTLINE(cppcoreguidelines-pro-type-member-init)
-    std::array<float, K/2> ds_arr;
+    std::array<float, K / 2> ds_arr;
     // NOLINTNEXTLINE(cppcoreguidelines-pro-type-member-init)
-    std::array<float, K/2> db_arr;
+    std::array<float, K / 2> db_arr;
     for (const auto i : c10::irange(start, end)) {
       const BFloat16* dY_ptr = dY + i * HxW;
       const BFloat16* X_ptr = X + i * HxW;
@@ -611,18 +596,18 @@ void ComputeInternalGradients(
   });
 }
 
-template <typename PT>
+template <typename PT, typename T_ACC>
 inline void CalcDsDb(
-    const PT* ds_ptr,
-    const PT* db_ptr,
+    const T_ACC* ds_ptr,
+    const T_ACC* db_ptr,
     bool gamma_null,
     const PT* gamma_ptr,
     const int64_t d,
     const int64_t K,
     void* ds_arr,
     void* db_arr) {
-    vec::Vectorized<PT> ds_vec(0);
-    vec::Vectorized<PT> db_vec(0);
+    vec::Vectorized<T_ACC> ds_vec(0);
+    vec::Vectorized<T_ACC> db_vec(0);
     for (int64_t j = 0; j < d; j += K) {
       const vec::Vectorized<PT> gamma_vec = gamma_null
           ? vec::Vectorized<PT>(1)
@@ -634,7 +619,34 @@ inline void CalcDsDb(
     db_vec.store(db_arr);
 }
 
-template <typename T, typename PT>
+template <>
+inline void CalcDsDb(
+    const float* ds_ptr,
+    const float* db_ptr,
+    bool gamma_null,
+    const BFloat16* gamma_ptr,
+    const int64_t d,
+    const int64_t K,
+    void* ds_arr,
+    void* db_arr) {
+  using fVec = at::vec::Vectorized<float>;
+  using bVec = at::vec::Vectorized<BFloat16>;
+  fVec ds_acc(0);
+  fVec db_acc(0);
+  for (int64_t j = 0; j < d; j += K) {
+    const bVec gamma_vec = gamma_null ? bVec(1) : bVec::loadu(gamma_ptr + j);
+    fVec gamma_vec0, gamma_vec1;
+    std::tie(gamma_vec0, gamma_vec1) = convert_bfloat16_float(gamma_vec);
+    ds_acc += fVec::loadu(ds_ptr + j) * gamma_vec0;
+    ds_acc += fVec::loadu(ds_ptr + j + fVec::size()) * gamma_vec1;
+    db_acc += fVec::loadu(db_ptr + j) * gamma_vec0;
+    db_acc += fVec::loadu(db_ptr + j + fVec::size()) * gamma_vec1;
+  }
+  ds_acc.store(ds_arr);
+  db_acc.store(db_arr);
+}
+
+template <typename T, typename PT, typename T_ACC>
 void GroupNormInputBackward(
     int64_t N,
     int64_t C,
@@ -645,93 +657,217 @@ void GroupNormInputBackward(
     const PT* mean,
     const PT* rstd,
     const PT* gamma,
-    const PT* ds,
-    const PT* db,
+    const T_ACC* ds,
+    const T_ACC* db,
     T* dX) {
   const int64_t G = group;
   const int64_t D = C / G;
-  const PT s = PT(1) / static_cast<PT>(D * HxW);
+  const T_ACC s = T_ACC(1) / static_cast<T_ACC>(D * HxW);
   const bool gamma_null = (gamma == nullptr);
   at::parallel_for(0, N * G, 1, [=](int64_t start, int64_t end) {
     constexpr int64_t K = vec::Vectorized<PT>::size();
     const int64_t d = D / K * K;
     // NOLINTNEXTLINE(cppcoreguidelines-pro-type-member-init)
-    std::array<PT, K> ds_arr;
+    std::array<T_ACC, at::vec::Vectorized<T_ACC>::size()> ds_arr;
     // NOLINTNEXTLINE(cppcoreguidelines-pro-type-member-init)
-    std::array<PT, K> db_arr;
+    std::array<T_ACC, at::vec::Vectorized<T_ACC>::size()> db_arr;
     for (const auto i : c10::irange(start, end)) {
       const int64_t g = i % G;
-      const PT* ds_ptr = ds + i * D;
-      const PT* db_ptr = db + i * D;
-      const PT* gamma_ptr = gamma + g * D;
+      const T_ACC* ds_ptr = ds + i * D;
+      const T_ACC* db_ptr = db + i * D;
+      const PT* gamma_ptr = !gamma_null ? gamma + g * D : nullptr;
       CalcDsDb(ds_ptr, db_ptr, gamma_null, gamma_ptr, d, K, ds_arr.data(), db_arr.data());
-      PT ds_val = std::accumulate(ds_arr.cbegin(), ds_arr.cend(), PT(0));
-      PT db_val = std::accumulate(db_arr.cbegin(), db_arr.cend(), PT(0));
+      T_ACC ds_val = std::accumulate(ds_arr.cbegin(), ds_arr.cend(), T_ACC(0));
+      T_ACC db_val = std::accumulate(db_arr.cbegin(), db_arr.cend(), T_ACC(0));
       for (const auto j : c10::irange(d, D)) {
-        const PT gamma_v = gamma_null ? PT(1) : PT(gamma[g * D + j]);
+        const T_ACC gamma_v = gamma_null ? T_ACC(1) : T_ACC(gamma[g * D + j]);
         ds_val += ds_ptr[j] * gamma_v;
         db_val += db_ptr[j] * gamma_v;
       }
-      const PT c2 =
-          (db_val * PT(mean[i]) - ds_val) * PT(rstd[i]) * PT(rstd[i]) * PT(rstd[i]) * s;
-      const PT c3 = -c2 * PT(mean[i]) - db_val * PT(rstd[i]) * s;
+      const T_ACC c2 =
+          (db_val * T_ACC(mean[i]) - ds_val) * T_ACC(rstd[i]) * T_ACC(rstd[i]) * T_ACC(rstd[i]) * s;
+      const T_ACC c3 = -c2 * T_ACC(mean[i]) - db_val * T_ACC(rstd[i]) * s;
 
       for (const auto j : c10::irange(D)) {
         const int64_t c = g * D + j;
         const T* dY_ptr = dY + (i * D + j) * HxW;
         const T* X_ptr = X + (i * D + j) * HxW;
         T* dX_ptr = dX + (i * D + j) * HxW;
-        const PT c1 = PT(rstd[i]) * (gamma_null ? PT(1) : PT(gamma[c]));
+        const T_ACC c1 = T_ACC(rstd[i]) * (gamma_null ? T_ACC(1) : T_ACC(gamma[c]));
         for (const auto k : c10::irange(HxW)) {
-          dX_ptr[k] = c1 * PT(dY_ptr[k]) + c2 * PT(X_ptr[k]) + c3;
+          dX_ptr[k] = c1 * T_ACC(dY_ptr[k]) + c2 * T_ACC(X_ptr[k]) + c3;
         }
       }
     }
   });
 }
 
-template <typename T>
+template <typename PT, typename T_ACC>
 void GammaBackward(
     int64_t N,
     int64_t C,
     int64_t group,
-    const T* mean,
-    const T* rstd,
-    const T* ds,
-    const T* db,
-    T* dgamma) {
+    const PT* mean,
+    const PT* rstd,
+    const T_ACC* ds,
+    const T_ACC* db,
+    PT* dgamma) {
   const int64_t G = group;
   const int64_t D = C / G;
-  constexpr int64_t K = vec::Vectorized<T>::size();
-  at::parallel_for(0, D, K, [=](int64_t start, int64_t end) {
-    for (const auto i : c10::irange(G)) {
-      std::memset(dgamma + i * D + start, 0, (end - start) * sizeof(T));
-    }
-    for (int64_t i = 0; i < N * G; ++i) {
-      const T* ds_ptr = ds + i * D;
-      const T* db_ptr = db + i * D;
-      const int64_t g = i % G;
-
-      for (const auto j : c10::irange(start, end)) {
-        const int64_t c = g * D + j;
-        dgamma[c] += (ds_ptr[j] - db_ptr[j] * mean[i]) * rstd[i];
+  constexpr int64_t K = at::vec::Vectorized<PT>::size();
+  using Vec = at::vec::Vectorized<PT>;
+  const int64_t inner_size = D / K * K;
+  for (const auto g : c10::irange(G)) {
+    int64_t i = 0;
+    for (; i < inner_size; i += K) {
+      Vec acc_vec{0};
+      for (const auto n : c10::irange(N)) {
+        const PT* ds_ptr = ds + n * C + g * D + i;
+        const PT* db_ptr = db + n * C + g * D + i;
+        auto ds_vec = Vec::loadu(ds_ptr);
+        auto db_vec = Vec::loadu(db_ptr);
+        auto mean_vec = Vec(mean[n * G + g]);
+        auto rstd_vec = Vec(rstd[n * G + g]);
+        acc_vec += (ds_vec - db_vec * mean_vec) * rstd_vec;
       }
+      acc_vec.store(dgamma + g * D + i);
     }
-  });
+    if (D - i > 0) {
+      Vec acc_vec{0};
+      for (const auto n : c10::irange(N)) {
+        const PT* ds_ptr = ds + n * C + g * D + i;
+        const PT* db_ptr = db + n * C + g * D + i;
+        auto ds_vec = Vec::loadu(ds_ptr, D - i);
+        auto db_vec = Vec::loadu(db_ptr, D - i);
+        auto mean_vec = Vec(mean[n * G + g]);
+        auto rstd_vec = Vec(rstd[n * G + g]);
+        acc_vec += (ds_vec - db_vec * mean_vec) * rstd_vec;
+      }
+      acc_vec.store(dgamma + g * D + i, D - i);
+    }
+  }
 }
 
-template <typename T>
-void BetaBackward(int64_t N, int64_t C, const T* db, T* dbeta) {
-  constexpr int64_t K = vec::Vectorized<T>::size();
-  at::parallel_for(0, C, K, [=](int64_t start, int64_t end) {
-    std::memset(dbeta + start, 0, (end - start) * sizeof(T));
-    for (const auto i : c10::irange(N)) {
-      const T* db_ptr = db + i * C;
-      for (const auto j : c10::irange(start, end)) {
-        dbeta[j] += db_ptr[j];
+template <>
+void GammaBackward(
+    int64_t N,
+    int64_t C,
+    int64_t group,
+    const BFloat16* mean,
+    const BFloat16* rstd,
+    const float* ds,
+    const float* db,
+    BFloat16* dgamma) {
+  const int64_t G = group;
+  const int64_t D = C / G;
+  using Vec = at::vec::Vectorized<BFloat16>;
+  using fVec = at::vec::Vectorized<float>;
+  constexpr int64_t K = Vec::size();
+  const int64_t inner_size = D / K * K;
+  for (const auto g : c10::irange(G)) {
+    int64_t i = 0;
+    for (; i < inner_size; i += K) {
+      fVec acc0_vec{0}, acc1_vec{0};
+      for (const auto n : c10::irange(N)) {
+        const float* ds_ptr = ds + n * C + g * D + i;
+        const float* db_ptr = db + n * C + g * D + i;
+        fVec ds_vec0, ds_vec1, db_vec0, db_vec1;
+        ds_vec0 = fVec::loadu(ds_ptr);
+        ds_vec1 = fVec::loadu(ds_ptr + fVec::size());
+        db_vec0 = fVec::loadu(db_ptr);
+        db_vec1 = fVec::loadu(db_ptr + fVec::size());
+        fVec mean_vec = fVec(float(mean[n * G + g]));
+        fVec rstd_vec = fVec(float(rstd[n * G + g]));
+        acc0_vec += (ds_vec0 - db_vec0 * mean_vec) * rstd_vec;
+        acc1_vec += (ds_vec1 - db_vec1 * mean_vec) * rstd_vec;
       }
+      convert_float_bfloat16(acc0_vec, acc1_vec).store(dgamma + g * D + i);
     }
-  });
+    if (D - i > 0) {
+      fVec acc0_vec{0}, acc1_vec{0};
+      for (const auto n : c10::irange(N)) {
+        const float* ds_ptr = ds + n * C + g * D + i;
+        const float* db_ptr = db + n * C + g * D + i;
+        fVec ds_vec0, ds_vec1, db_vec0, db_vec1;
+        ds_vec0 = fVec::loadu(
+            ds_ptr, (D - i) > fVec::size() ? fVec::size() : (D - i));
+        ds_vec1 = fVec::loadu(
+            ds_ptr + fVec::size(),
+            (D - i) > fVec::size() ? (D - i - fVec::size()) : 0);
+        db_vec0 = fVec::loadu(
+            db_ptr, (D - i) > fVec::size() ? fVec::size() : (D - i));
+        db_vec1 = fVec::loadu(
+            db_ptr + fVec::size(),
+            (D - i) > fVec::size() ? (D - i - fVec::size()) : 0);
+        fVec mean_vec = fVec(float(mean[n * G + g]));
+        fVec rstd_vec = fVec(float(rstd[n * G + g]));
+        acc0_vec += (ds_vec0 - db_vec0 * mean_vec) * rstd_vec;
+        acc1_vec += (ds_vec1 - db_vec1 * mean_vec) * rstd_vec;
+      }
+      convert_float_bfloat16(acc0_vec, acc1_vec)
+          .store(dgamma + g * D + i, D - i);
+    }
+  }
+}
+
+template <typename PT, typename T_ACC>
+void BetaBackward(int64_t N, int64_t C, const T_ACC* db, PT* dbeta) {
+  using Vec = at::vec::Vectorized<PT>;
+  constexpr int64_t K = Vec::size();
+  Vec acc_vec{0}, zero{0};
+  const int64_t inner_size = C / K * K;
+  int64_t i = 0;
+  for (; i < inner_size; i += K) {
+    for (const auto n : c10::irange(N)) {
+      acc_vec += Vec::loadu(db + n * C + i);
+    }
+    acc_vec.store(dbeta + i);
+    acc_vec = Vec::set(acc_vec, zero);
+  }
+  if (C - i > 0) {
+    for (const auto n : c10::irange(N)) {
+      acc_vec += Vec::loadu(db + n * C + i, C - i);
+    }
+    acc_vec.store(dbeta + i, C - i);
+    acc_vec = Vec::set(acc_vec, zero, C - i);
+  }
+}
+
+template <>
+void BetaBackward(int64_t N, int64_t C, const float* db, BFloat16* dbeta) {
+  using Vec = at::vec::Vectorized<BFloat16>;
+  using fVec = at::vec::Vectorized<float>;
+  constexpr int64_t K = Vec::size();
+  fVec acc0_vec{0}, acc1_vec{0}, zero{0};
+  const int64_t inner_size = C / K * K;
+  int64_t i = 0;
+  for (; i < inner_size; i += K) {
+    for (const auto n : c10::irange(N)) {
+      fVec db_vec0, db_vec1;
+      db_vec0 = fVec::loadu(db + n * C + i);
+      db_vec1 = fVec::loadu(db + n * C + i + fVec::size());
+      acc0_vec += db_vec0;
+      acc1_vec += db_vec1;
+    }
+    convert_float_bfloat16(acc0_vec, acc1_vec).store(dbeta + i);
+    acc0_vec = fVec::set(acc0_vec, zero);
+    acc1_vec = fVec::set(acc1_vec, zero);
+  }
+  if (C - i > 0) {
+    for (const auto n : c10::irange(N)) {
+      fVec db_vec0, db_vec1;
+      db_vec0 = fVec::loadu(
+          db + n * C + i, (C - i) > fVec::size() ? fVec::size() : (C - i));
+      db_vec1 = fVec::loadu(
+          db + n * C + i + fVec::size(),
+          (C - i) > fVec::size() ? (C - i - fVec::size()) : 0);
+      acc0_vec += db_vec0;
+      acc1_vec += db_vec1;
+    }
+    convert_float_bfloat16(acc0_vec, acc1_vec).store(dbeta + i, C - i);
+    acc0_vec = fVec::set(acc0_vec, zero, C - i);
+    acc1_vec = fVec::set(acc1_vec, zero, C - i);
+  }
 }
 
 template <typename T, typename PT>
@@ -761,14 +897,15 @@ void GroupNormBackwardKernelImplInternal(
   T* dX_data = dX.defined() ? dX.data_ptr<T>() : nullptr;
   PT* dgamma_data = dgamma.defined() ? dgamma.data_ptr<PT>() : nullptr;
   PT* dbeta_data = dbeta.defined() ? dbeta.data_ptr<PT>() : nullptr;
-  Tensor ds = at::empty({N, C}, X.options().dtype(c10::CppTypeToScalarType<PT>::value));
-  Tensor db = at::empty({N, C}, X.options().dtype(c10::CppTypeToScalarType<PT>::value));
-  PT* ds_data = ds.data_ptr<PT>();
-  PT* db_data = db.data_ptr<PT>();
-  ComputeInternalGradients<T, PT>(N, C, HxW, dY_data, X_data, ds_data, db_data);
+  using T_ACC = at::opmath_type<T>;
+  Tensor ds = at::empty({N, C}, X.options().dtype(c10::CppTypeToScalarType<T_ACC>::value));
+  Tensor db = at::empty({N, C}, X.options().dtype(c10::CppTypeToScalarType<T_ACC>::value));
+  T_ACC* ds_data = ds.data_ptr<T_ACC>();
+  T_ACC* db_data = db.data_ptr<T_ACC>();
+  ComputeInternalGradients<T, T_ACC>(N, C, HxW, dY_data, X_data, ds_data, db_data);
 
   if (dX_data != nullptr) {
-    GroupNormInputBackward<T, PT>(
+    GroupNormInputBackward<T, PT, T_ACC>(
         N,
         C,
         HxW,
@@ -791,18 +928,18 @@ void GroupNormBackwardKernelImplInternal(
   }
 }
 
-template <typename T, typename PT>
+template <typename T, typename T_ACC>
 inline void DsDbRowwiseMomentsChannelsLast(
   const T* dY_ptr,
   const T* X_ptr,
-  PT* ds_ptr,
-  PT* db_ptr,
+  T_ACC* ds_ptr,
+  T_ACC* db_ptr,
   int64_t C) {
   using Vec = vec::Vectorized<T>;
   constexpr int64_t K = vec::Vectorized<T>::size();
   const int64_t inner_size = C / K * K;
   int64_t d = 0;
-  for (; d < inner_size; d+= K) {
+  for (; d < inner_size; d += K) {
     Vec ds_dev = Vec::loadu(ds_ptr + d);
     Vec db_vec = Vec::loadu(db_ptr + d);
     Vec x_vec = Vec::loadu(X_ptr + d);
@@ -878,15 +1015,36 @@ inline void DsDbRowwiseMomentsChannelsLast(
   }
 }
 
-template <typename T, typename PT>
-inline void ApplyInputGradientsChannelsLastColMov(
+template <typename T>
+inline std::tuple<
+    vec::Vectorized<at::opmath_type<T>>,
+    vec::Vectorized<at::opmath_type<T>>>
+load_util(const T* data_ptr, int64_t n) {
+  using Vec = vec::Vectorized<T>;
+  auto vec0 = Vec::loadu(data_ptr, n > Vec::size() ? Vec::size() : n);
+  auto vec1 = Vec::loadu(
+      data_ptr + Vec::size(), n > Vec::size() ? (n - Vec::size()) : 0);
+  return std::tuple<Vec, Vec>(vec0, vec1);
+}
+
+template <>
+inline std::tuple<at::vec::Vectorized<float>, at::vec::Vectorized<float>>
+load_util(const BFloat16* data_ptr, int64_t n) {
+  using bVec = vec::Vectorized<BFloat16>;
+  auto vec = bVec::loadu(data_ptr, n);
+  return convert_bfloat16_float(vec);
+}
+
+template <typename T, typename PT, typename T_ACC>
+inline typename std::enable_if<std::is_same<T, T_ACC>::value, void>::type
+ApplyInputGradientsChannelsLastColMov(
   const T* dY_data,
   const T* X_data,
   T* dX_data,
   const PT* rstd,
   const PT* gamma,
-  PT c2,
-  PT c3,
+  T_ACC c2,
+  T_ACC c3,
   int64_t HxW,
   int64_t C,
   int64_t D) {
@@ -894,12 +1052,13 @@ inline void ApplyInputGradientsChannelsLastColMov(
   int64_t d = 0;
   auto K = vec::Vectorized<T>::size();
   for (; d < D / K * K; d += K) {
+    auto c1 = vec::Vectorized<T>(*rstd) *
+        (gamma_null ? vec::Vectorized<T>(1)
+                    : vec::Vectorized<T>::loadu(gamma + d));
     for (const auto m : c10::irange(HxW)) {
       const T* X_ptr = X_data + m * C;
       const T* dY_ptr = dY_data + m * C;
       T* dX_ptr = dX_data + m * C;
-      auto c1 = vec::Vectorized<T>(*rstd) *
-        (gamma_null ? vec::Vectorized<T>(1) : vec::Vectorized<T>::loadu(gamma + d));
       auto dy_vec = vec::Vectorized<T>::loadu(dY_ptr + d);
       auto x_vec = vec::Vectorized<T>::loadu(X_ptr + d);
       auto dx_vec = c1 * dy_vec +
@@ -908,12 +1067,13 @@ inline void ApplyInputGradientsChannelsLastColMov(
     }
   }
   if (D - d > 0) {
+    auto c1 = vec::Vectorized<T>(*rstd) *
+        (gamma_null ? vec::Vectorized<T>(1)
+                    : vec::Vectorized<T>::loadu(gamma + d, D - d));
     for (const auto m : c10::irange(HxW)) {
       const T* X_ptr = X_data + m * C;
       const T* dY_ptr = dY_data + m * C;
       T* dX_ptr = dX_data + m * C;
-      auto c1 = vec::Vectorized<T>(*rstd) *
-      (gamma_null ? vec::Vectorized<T>(1) : vec::Vectorized<T>::loadu(gamma + d, D - d));
     auto dy_vec = vec::Vectorized<T>::loadu(dY_ptr + d, D - d);
     auto x_vec = vec::Vectorized<T>::loadu(X_ptr + d, D - d);
     auto dx_vec = c1 * dy_vec +
@@ -923,32 +1083,35 @@ inline void ApplyInputGradientsChannelsLastColMov(
   }
 }
 
-template <>
-inline void ApplyInputGradientsChannelsLastColMov(
-  const BFloat16* dY_data,
-  const BFloat16* X_data,
-  BFloat16* dX_data,
-  const float* rstd,
-  const float* gamma,
-  float c2,
-  float c3,
-  int64_t HxW,
-  int64_t C,
-  int64_t D) {
-  using bVec = vec::Vectorized<BFloat16>;
-  using fVec = vec::Vectorized<float>;
+template <typename T, typename PT, typename T_ACC>
+inline typename std::enable_if<!std::is_same<T, T_ACC>::value, void>::type
+ApplyInputGradientsChannelsLastColMov(
+    const T* dY_data,
+    const T* X_data,
+    T* dX_data,
+    const PT* rstd,
+    const PT* gamma,
+    T_ACC c2,
+    T_ACC c3,
+    int64_t HxW,
+    int64_t C,
+    int64_t D) {
+  using bVec = vec::Vectorized<T>;
+  using fVec = vec::Vectorized<T_ACC>;
   const bool gamma_null = (gamma == nullptr);
   auto K = bVec::size();
   int64_t d = 0;
   for (; d < D / K * K; d += K) {
+    fVec c1_0, c1_1;
+    std::tie(c1_0, c1_1) = gamma_null ? std::tuple<fVec, fVec>(fVec(1), fVec(1))
+                                      : load_util(gamma + d, K);
+    c1_0 = c1_0 * fVec(T_ACC(*rstd));
+    c1_1 = c1_1 * fVec(T_ACC(*rstd));
     for (const auto m : c10::irange(HxW)) {
-      const BFloat16* X_ptr = X_data + m * C;
-      const BFloat16* dY_ptr = dY_data + m * C;
-      BFloat16* dX_ptr = dX_data + m * C;
-      fVec c1_0 = fVec(*rstd) *
-        (gamma_null ? fVec(1) : fVec::loadu(gamma + d));
-      fVec c1_1 = fVec(*rstd) *
-        (gamma_null ? fVec(1) : fVec::loadu(gamma + d + fVec::size()));
+      const T* X_ptr = X_data + m * C;
+      const T* dY_ptr = dY_data + m * C;
+      T* dX_ptr = dX_data + m * C;
+
       bVec dy_vec = bVec::loadu(dY_ptr + d);
       bVec x_vec = bVec::loadu(X_ptr + d);
       fVec dy_vec0, dy_vec1, x_vec0, x_vec1;
@@ -960,14 +1123,15 @@ inline void ApplyInputGradientsChannelsLastColMov(
     }
   }
   if (D - d > 0) {
+    fVec c1_0, c1_1;
+    std::tie(c1_0, c1_1) = gamma_null ? std::tuple<fVec, fVec>(fVec(1), fVec(1))
+                                      : load_util(gamma + d, D - d);
+    c1_0 = c1_0 * fVec(T_ACC(*rstd));
+    c1_1 = c1_1 * fVec(T_ACC(*rstd));
     for (const auto m : c10::irange(HxW)) {
-      const BFloat16* X_ptr = X_data + m * C;
-      const BFloat16* dY_ptr = dY_data + m * C;
-      BFloat16* dX_ptr = dX_data + m * C;
-      fVec c1_0 = fVec(*rstd) *
-        (gamma_null ? fVec(1) : fVec::loadu(gamma + d, (D - d) > fVec::size() ? fVec::size() : (D - d)));
-      fVec c1_1 = fVec(*rstd) *
-        (gamma_null ? fVec(1) : fVec::loadu(gamma + d + fVec::size(), (D - d) > fVec::size() ? (D - d - fVec::size()) : 0));
+      const T* X_ptr = X_data + m * C;
+      const T* dY_ptr = dY_data + m * C;
+      T* dX_ptr = dX_data + m * C;
       bVec dy_vec = bVec::loadu(dY_ptr + d, D - d);
       bVec x_vec = bVec::loadu(X_ptr + d, D - d);
       fVec dy_vec0, dy_vec1, x_vec0, x_vec1;
@@ -980,15 +1144,16 @@ inline void ApplyInputGradientsChannelsLastColMov(
   }
 }
 
-template <typename T, typename PT>
-inline void ApplyInputGradientsChannelsLastRowMov(
+template <typename T, typename PT, typename T_ACC>
+inline typename std::enable_if<std::is_same<T, T_ACC>::value, void>::type
+ApplyInputGradientsChannelsLastRowMov(
   const T* dY_data,
   const T* X_data,
   T* dX_data,
   const PT* rstd,
   const PT* gamma,
-  PT c2,
-  PT c3,
+  T_ACC c2,
+  T_ACC c3,
   int64_t HxW,
   int64_t C,
   int64_t D) {
@@ -1015,28 +1180,30 @@ inline void ApplyInputGradientsChannelsLastRowMov(
   }
 }
 
-template <>
-inline void ApplyInputGradientsChannelsLastRowMov(
-  const BFloat16* dY_data,
-  const BFloat16* X_data,
-  BFloat16* dX_data,
-  const float* rstd,
-  const float* gamma,
-  float c2,
-  float c3,
-  int64_t HxW,
-  int64_t C,
-  int64_t D) {
-  using bVec = vec::Vectorized<BFloat16>;
-  using fVec = vec::Vectorized<float>;
+template <typename T, typename PT, typename T_ACC>
+inline typename std::enable_if<!std::is_same<T, T_ACC>::value, void>::type
+ApplyInputGradientsChannelsLastRowMov(
+    const T* dY_data,
+    const T* X_data,
+    T* dX_data,
+    const PT* rstd,
+    const PT* gamma,
+    T_ACC c2,
+    T_ACC c3,
+    int64_t HxW,
+    int64_t C,
+    int64_t D) {
+  using bVec = vec::Vectorized<T>;
+  using fVec = vec::Vectorized<T_ACC>;
   const bool gamma_null = (gamma == nullptr);
   auto K = bVec::size();
   int64_t d = 0;
   for (; d < D / K * K; d += K) {
-    fVec c1_0 = fVec(*rstd) *
-      (gamma_null ? fVec(1) : fVec::loadu(gamma + d));
-    fVec c1_1 = fVec(*rstd) *
-      (gamma_null ? fVec(1) : fVec::loadu(gamma + d + fVec::size()));
+    fVec c1_0, c1_1;
+    std::tie(c1_0, c1_1) = gamma_null ? std::tuple<fVec, fVec>(fVec(1), fVec(1))
+                                      : load_util(gamma + d, K);
+    c1_0 = c1_0 * fVec(T_ACC(*rstd));
+    c1_1 = c1_1 * fVec(T_ACC(*rstd));
     bVec dy_vec = bVec::loadu(dY_data + d);
     bVec x_vec = bVec::loadu(X_data + d);
     fVec dy_vec0, dy_vec1, x_vec0, x_vec1;
@@ -1047,10 +1214,11 @@ inline void ApplyInputGradientsChannelsLastRowMov(
     convert_float_bfloat16(dx_vec0, dx_vec1).store(dX_data + d);
   }
   if (D - d > 0) {
-    fVec c1_0 = fVec(*rstd) *
-      (gamma_null ? fVec(1) : fVec::loadu(gamma + d, (D - d) > fVec::size() ? fVec::size() : (D - d)));
-    fVec c1_1 = fVec(*rstd) *
-      (gamma_null ? fVec(1) : fVec::loadu(gamma + d + fVec::size(), (D - d) > fVec::size() ? (D - d - fVec::size()) : 0));
+    fVec c1_0, c1_1;
+    std::tie(c1_0, c1_1) = gamma_null ? std::tuple<fVec, fVec>(fVec(1), fVec(1))
+                                      : load_util(gamma + d, D - d);
+    c1_0 = c1_0 * fVec(T_ACC(*rstd));
+    c1_1 = c1_1 * fVec(T_ACC(*rstd));
     bVec dy_vec = bVec::loadu(dY_data + d, D - d);
     bVec x_vec = bVec::loadu(X_data + d, D - d);
     fVec dy_vec0, dy_vec1, x_vec0, x_vec1;
@@ -1062,17 +1230,20 @@ inline void ApplyInputGradientsChannelsLastRowMov(
   }
 }
 
-template <typename T, typename PT>
-std::tuple<PT, PT> CalcInternalGradientsChannelsLast(
+template <typename T, typename PT, typename T_ACC>
+inline typename std::
+    enable_if<std::is_same<T, T_ACC>::value, std::tuple<T_ACC, T_ACC>>::type
+    CalcInternalGradientsChannelsLast(
     const T* X_data,
     const T* dY_data,
     const PT* gamma_ptr,
-    PT* ds_ptr,
-    PT* db_ptr,
+    T_ACC* ds_ptr,
+    T_ACC* db_ptr,
     int64_t HxW,
     int64_t C,
     int64_t D) {
   using Vec = vec::Vectorized<T>;
+  const bool gamma_null = (gamma_ptr == nullptr);
   constexpr int64_t K = Vec::size();
   const int64_t inner_size = D / K * K;
   int64_t d = 0;
@@ -1090,9 +1261,9 @@ std::tuple<PT, PT> CalcInternalGradientsChannelsLast(
     acc0_vec.store(ds_ptr + d);
     acc1_vec.store(db_ptr + d);
     ds_gamma += vec::vec_reduce_all([](Vec& x, Vec& y) { return x + y; },
-      acc0_vec * Vec::loadu(gamma_ptr + d));
+      acc0_vec * (gamma_null ? Vec(1) : Vec::loadu(gamma_ptr + d)));
     db_gamma += vec::vec_reduce_all([](Vec& x, Vec& y) { return x + y; },
-      acc1_vec * Vec::loadu(gamma_ptr + d));
+      acc1_vec * (gamma_null ? Vec(1) : Vec::loadu(gamma_ptr + d)));
   }
   if (D - d > 0) {
     Vec acc0_vec{0}, acc1_vec{0};
@@ -1107,25 +1278,28 @@ std::tuple<PT, PT> CalcInternalGradientsChannelsLast(
     acc0_vec.store(ds_ptr + d, D - d);
     acc1_vec.store(db_ptr + d, D - d);
     ds_gamma += vec::vec_reduce_all([](Vec& x, Vec& y) { return x + y; },
-      acc0_vec * Vec::loadu(gamma_ptr + d, D - d));
+      acc0_vec * (gamma_null ? Vec(1) : Vec::loadu(gamma_ptr + d, D - d)));
     db_gamma += vec::vec_reduce_all([](Vec& x, Vec& y) { return x + y; },
-      acc1_vec * Vec::loadu(gamma_ptr + d, D - d));
+      acc1_vec * (gamma_null ? Vec(1) : Vec::loadu(gamma_ptr + d, D - d)));
   }
   return std::tuple<PT, PT>(ds_gamma, db_gamma);
 }
 
-template <>
-std::tuple<float, float> CalcInternalGradientsChannelsLast(
-  const BFloat16* X_data,
-  const BFloat16* dY_data,
-  const float* gamma_ptr,
-  float* ds_ptr,
-  float* db_ptr,
-  int64_t HxW,
-  int64_t C,
-  int64_t D) {
-  using bVec = vec::Vectorized<BFloat16>;
-  using fVec = vec::Vectorized<float>;
+template <typename T, typename PT, typename T_ACC>
+inline typename std::
+    enable_if<!std::is_same<T, T_ACC>::value, std::tuple<T_ACC, T_ACC>>::type
+    CalcInternalGradientsChannelsLast(
+        const T* X_data,
+        const T* dY_data,
+        const PT* gamma_ptr,
+        T_ACC* ds_ptr,
+        T_ACC* db_ptr,
+        int64_t HxW,
+        int64_t C,
+        int64_t D) {
+  using bVec = vec::Vectorized<T>;
+  using fVec = vec::Vectorized<T_ACC>;
+  const bool gamma_null = (gamma_ptr == nullptr);
   constexpr int64_t K = bVec::size();
   const int64_t inner_size = D / K * K;
   float ds_gamma{0}, db_gamma{0};
@@ -1133,8 +1307,8 @@ std::tuple<float, float> CalcInternalGradientsChannelsLast(
   for (; d < inner_size; d += K) {
     fVec acc0_vec0{0}, acc0_vec1{0}, acc1_vec0{0}, acc1_vec1{0};
     for (const auto m : c10::irange(HxW)) {
-      const BFloat16* X_ptr = X_data + m * C;
-      const BFloat16* dY_ptr = dY_data + m * C;
+      const T* X_ptr = X_data + m * C;
+      const T* dY_ptr = dY_data + m * C;
       bVec x_vec = bVec::loadu(X_ptr + d);
       bVec dy_vec = bVec::loadu(dY_ptr + d);
       fVec x_vec0, x_vec1, dy_vec0, dy_vec1;
@@ -1149,27 +1323,31 @@ std::tuple<float, float> CalcInternalGradientsChannelsLast(
     acc0_vec1.store(ds_ptr + d + fVec::size());
     acc1_vec0.store(db_ptr + d);
     acc1_vec1.store(db_ptr + d + fVec::size());
-    ds_gamma += vec::vec_reduce_all([](fVec& x, fVec& y) { return x + y; },
-      acc0_vec0 * fVec::loadu(gamma_ptr + d));
-    ds_gamma += vec::vec_reduce_all([](fVec& x, fVec& y) { return x + y; },
-      acc0_vec1 * fVec::loadu(gamma_ptr + d + fVec::size()));
-    db_gamma += vec::vec_reduce_all([](fVec& x, fVec& y) { return x + y; },
-      acc1_vec0 * fVec::loadu(gamma_ptr + d));
-    db_gamma += vec::vec_reduce_all([](fVec& x, fVec& y) { return x + y; },
-      acc1_vec1 * fVec::loadu(gamma_ptr + d + fVec::size()));
+    fVec gamma_vec0, gamma_vec1;
+    std::tie(gamma_vec0, gamma_vec1) = gamma_null ?
+      std::tuple<fVec, fVec>(fVec(1), fVec(1)) : load_util(gamma_ptr + d, K);
+    ds_gamma += vec::vec_reduce_all(
+        [](fVec& x, fVec& y) { return x + y; }, acc0_vec0 * gamma_vec0);
+    ds_gamma += vec::vec_reduce_all(
+        [](fVec& x, fVec& y) { return x + y; }, acc0_vec1 * gamma_vec1);
+    db_gamma += vec::vec_reduce_all(
+        [](fVec& x, fVec& y) { return x + y; }, acc1_vec0 * gamma_vec0);
+    db_gamma += vec::vec_reduce_all(
+        [](fVec& x, fVec& y) { return x + y; }, acc1_vec1 * gamma_vec1);
   }
   for (; d < D; d++) {
-    float acc0{0}, acc1{0};
+    T_ACC acc0{0}, acc1{0};
     for (const auto m : c10::irange(HxW)) {
       const BFloat16* X_ptr = X_data + m * C;
       const BFloat16* dY_ptr = dY_data + m * C;
-      acc0 += float(X_ptr[d]) * float(dY_ptr[d]);
-      acc1 += float(dY_ptr[d]);
+      acc0 += T_ACC(X_ptr[d]) * T_ACC(dY_ptr[d]);
+      acc1 += T_ACC(dY_ptr[d]);
     }
     ds_ptr[d] = acc0;
     db_ptr[d] = acc1;
-    ds_gamma += acc0 * gamma_ptr[d];
-    db_gamma += acc1 * gamma_ptr[d];
+    T_ACC gamma_val = gamma_null ? T_ACC(1) : T_ACC(gamma_ptr[d]);
+    ds_gamma += acc0 * gamma_val;
+    db_gamma += acc1 * gamma_val;
   }
 
   return std::tuple<float, float>(ds_gamma, db_gamma);
@@ -1205,11 +1383,12 @@ void GroupNormBackwardKernelImplChannelsLastInternal(
   PT* dgamma_data = dgamma.defined() ? dgamma.data_ptr<PT>() : nullptr;
   PT* dbeta_data = dbeta.defined() ? dbeta.data_ptr<PT>() : nullptr;
   const bool gamma_null = (gamma_data == nullptr);
-  Tensor ds = at::empty({N, C}, X.options().dtype(c10::CppTypeToScalarType<PT>::value));
-  Tensor db = at::empty({N, C}, X.options().dtype(c10::CppTypeToScalarType<PT>::value));
-  PT* ds_data = ds.data_ptr<PT>();
-  PT* db_data = db.data_ptr<PT>();
-  const PT s = PT(1) / static_cast<PT>(D * HxW);
+  using T_ACC = at::opmath_type<T>;
+  Tensor ds = at::empty({N, C}, X.options().dtype(c10::CppTypeToScalarType<T_ACC>::value));
+  Tensor db = at::empty({N, C}, X.options().dtype(c10::CppTypeToScalarType<T_ACC>::value));
+  T_ACC* ds_data = ds.data_ptr<T_ACC>();
+  T_ACC* db_data = db.data_ptr<T_ACC>();
+  const T_ACC s = T_ACC(1) / static_cast<T_ACC>(D * HxW);
 
   // Similar to channels last forward, channels last backward has also 2 impls.
   // impl-1: parallel on N * G. Only need one omp session for input gradients
@@ -1228,13 +1407,13 @@ void GroupNormBackwardKernelImplChannelsLastInternal(
       data_index_init(begin, n, N, g, G);
       for (const auto i : c10::irange(begin, end)) {
         // Step 1. Compute internal gradients.
-        PT* ds_ptr = ds_data + i * D;
-        PT* db_ptr = db_data + i * D;
-        PT ds_gamma, db_gamma;
+        T_ACC* ds_ptr = ds_data + i * D;
+        T_ACC* db_ptr = db_data + i * D;
+        T_ACC ds_gamma, db_gamma;
         const T* X_ptr = X_data + n * HxW * C + g * D;
         const T* dY_ptr = dY_data + n * HxW * C + g * D;
         const PT* gamma_ptr = gamma_null ? gamma_data : (gamma_data + g * D);
-        std::tie(ds_gamma, db_gamma) = CalcInternalGradientsChannelsLast<T, PT>(
+        std::tie(ds_gamma, db_gamma) = CalcInternalGradientsChannelsLast<T, PT, T_ACC>(
           X_ptr,
           dY_ptr,
           gamma_ptr,
@@ -1247,10 +1426,10 @@ void GroupNormBackwardKernelImplChannelsLastInternal(
         // Step 2. Compute dX.
         T* dX_ptr = dX_data + n * HxW * C + g * D;
         const PT* rstd_ptr = rstd_data + i;
-        const PT c2 =
-            (db_gamma * mean_data[i] - ds_gamma) * rstd_data[i] * rstd_data[i] * rstd_data[i] * s;
-        const PT c3 = -c2 * mean_data[i] - db_gamma * rstd_data[i] * s;
-        ApplyInputGradientsChannelsLastColMov<T, PT>(dY_ptr, X_ptr, dX_ptr, rstd_ptr, gamma_ptr, c2, c3, HxW, C, D);
+        const T_ACC c2 = (db_gamma * T_ACC(mean_data[i]) - ds_gamma) *
+            T_ACC(rstd_data[i]) * T_ACC(rstd_data[i]) * T_ACC(rstd_data[i]) * s;
+        const T_ACC c3 = -c2 * T_ACC(mean_data[i]) - db_gamma * T_ACC(rstd_data[i]) * s;
+        ApplyInputGradientsChannelsLastColMov<T, PT, T_ACC>(dY_ptr, X_ptr, dX_ptr, rstd_ptr, gamma_ptr, c2, c3, HxW, C, D);
         data_index_step(n, N, g, G);
       }
     });
@@ -1259,26 +1438,26 @@ void GroupNormBackwardKernelImplChannelsLastInternal(
     // impl-2: parallel on N * HxW.
     int num_threads = at::get_num_threads();
     Tensor buffer = at::empty({num_threads, N, 2 * C},
-      X.options().dtype(c10::CppTypeToScalarType<PT>::value)).zero_();
-    PT* buffer_data = buffer.data_ptr<PT>();
+      X.options().dtype(c10::CppTypeToScalarType<T_ACC>::value)).zero_();
+    T_ACC* buffer_data = buffer.data_ptr<T_ACC>();
 
     Tensor tmp_buffer = at::empty({N, 2 * G},
-      X.options().dtype(c10::CppTypeToScalarType<PT>::value));
-    PT* tmp_buffer_data = tmp_buffer.data_ptr<PT>();
+      X.options().dtype(c10::CppTypeToScalarType<T_ACC>::value));
+    T_ACC* tmp_buffer_data = tmp_buffer.data_ptr<T_ACC>();
 
     // Step 1. Each thread compute their own internal gradients to the buffer.
     at::parallel_for(0, N * HxW, 1, [&](int64_t begin, int64_t end) {
       int tid = at::get_thread_num();
-      PT* buffer_ptr = buffer_data + tid * N * 2 * C;
+      T_ACC* buffer_ptr = buffer_data + tid * N * 2 * C;
       int64_t n{0}, m{0};
       data_index_init(begin, n, N, m, HxW);
       for (const auto i : c10::irange(begin, end)) {
-        PT* ds_ptr = buffer_ptr +  n * 2 * C;
-        PT* db_ptr = ds_ptr + C;
+        T_ACC* ds_ptr = buffer_ptr +  n * 2 * C;
+        T_ACC* db_ptr = ds_ptr + C;
         const T* X_ptr = X_data + i * C;
         const T* dY_ptr = dY_data + i * C;
 
-        DsDbRowwiseMomentsChannelsLast<T, PT>(dY_ptr, X_ptr, ds_ptr, db_ptr, C);
+        DsDbRowwiseMomentsChannelsLast<T, T_ACC>(dY_ptr, X_ptr, ds_ptr, db_ptr, C);
         data_index_step(n, N, m, HxW);
       }
     });
@@ -1287,18 +1466,14 @@ void GroupNormBackwardKernelImplChannelsLastInternal(
     // get the final internal gradients to ds, db, and tmp_buffer.
     for (const auto n : c10::irange(N)) {
       for (const auto g : c10::irange(G)) {
-        PT ds_gamma{0}, db_gamma{0};
+        T_ACC ds_gamma{0}, db_gamma{0};
         for (const auto d : c10::irange(D)) {
-          PT ds_val{0}, db_val{0};
+          T_ACC ds_val{0}, db_val{0};
           for (const auto t : c10::irange(num_threads)) {
-            PT* buffer_ptr = buffer_data + t * N * 2 * C + n * 2 * C;
-            if (gamma_null) {
-              ds_gamma += buffer_ptr[g * D + d];
-              db_gamma += buffer_ptr[g * D + d + C];
-            } else {
-              ds_gamma += buffer_ptr[g * D + d] *  gamma_data[g * D + d];
-              db_gamma += buffer_ptr[g * D + d + C] * gamma_data[g * D + d];
-            }
+            T_ACC* buffer_ptr = buffer_data + t * N * 2 * C + n * 2 * C;
+            T_ACC gamma_val = gamma_null ? T_ACC(1) : T_ACC(gamma_data[g * D + d]);
+            ds_gamma += buffer_ptr[g * D + d] * gamma_val;
+            db_gamma += buffer_ptr[g * D + d + C] * gamma_val;
             ds_val += buffer_ptr[g * D + d];
             db_val += buffer_ptr[g * D + d + C];
 
@@ -1324,13 +1499,13 @@ void GroupNormBackwardKernelImplChannelsLastInternal(
             const PT* mean_ptr = mean_data + n * G + g;
             const PT* rstd_ptr = rstd_data + n * G + g;
             const PT* gamma_ptr = gamma_null ? gamma_data : (gamma_data + g * D);
-            PT ds_val = tmp_buffer_data[n * 2 * G + 2 * g];
-            PT db_val = tmp_buffer_data[n * 2 * G + 2 * g + 1];
+            T_ACC ds_val = tmp_buffer_data[n * 2 * G + 2 * g];
+            T_ACC db_val = tmp_buffer_data[n * 2 * G + 2 * g + 1];
 
-            const PT c2 =
-            (db_val * (*mean_ptr) - ds_val) * (*rstd_ptr) * (*rstd_ptr)* (*rstd_ptr) * s;
-            const PT c3 = -c2 * (*mean_ptr) - db_val * (*rstd_ptr) * s;
-            ApplyInputGradientsChannelsLastRowMov<T, PT>(dY_ptr, X_ptr, dX_ptr, rstd_ptr, gamma_ptr, c2, c3, HxW, C, D);
+            const T_ACC c2 = (db_val * T_ACC(*mean_ptr) - ds_val) *
+                T_ACC(*rstd_ptr) * T_ACC(*rstd_ptr)* T_ACC(*rstd_ptr) * s;
+            const T_ACC c3 = -c2 * T_ACC(*mean_ptr) - db_val * T_ACC(*rstd_ptr) * s;
+            ApplyInputGradientsChannelsLastRowMov<T, PT, T_ACC>(dY_ptr, X_ptr, dX_ptr, rstd_ptr, gamma_ptr, c2, c3, HxW, C, D);
           }
 
           data_index_step(n, N, m, HxW);
@@ -1342,11 +1517,11 @@ void GroupNormBackwardKernelImplChannelsLastInternal(
 
   // Finally compute dgamma and dbeta.
   if (dgamma_data != nullptr) {
-    GammaBackward<PT>(
+    GammaBackward(
         N, C, group, mean_data, rstd_data, ds_data, db_data, dgamma_data);
   }
   if (dbeta_data != nullptr) {
-    BetaBackward<PT>(N, C, db_data, dbeta_data);
+    BetaBackward(N, C, db_data, dbeta_data);
   }
 }
 
@@ -1367,7 +1542,7 @@ void GroupNormBackwardKernelImpl(
   // It will keep module parameters in acc dtype i.e. float
   // while input/output will be in BFloat16.
   // Using parameters in BFloat16 will cause high precision loss.
-  const bool mixed_type = is_mixed_type(dY, gamma);
+  const bool mixed_type = is_mixed_type(dY, mean);
   switch (X.suggest_memory_format()) {
     case at::MemoryFormat::Contiguous: {
       AT_DISPATCH_FLOATING_TYPES_AND(
