@@ -19,7 +19,8 @@ RegisterEngineAllocator cpu_alloc(
   }
 );
 
-namespace at { namespace native { namespace mkldnn {
+namespace at::native::mkldnn{
+void clear_computation_cache();
 
 void clear_computation_cache() {
   // Reset computation_cache for forward convolutions
@@ -27,6 +28,6 @@ void clear_computation_cache() {
   ideep::convolution_forward::t_store().clear();
 }
 
-}}} // namespace  at::native::mkldnn
+} // namespace  at::native::mkldnn
 
 #endif // AT_MKLDNN_ENALBED()
