@@ -7,26 +7,27 @@
 #else
 #include <ATen/ops/_nested_tensor_size_native.h>
 #include <ATen/ops/_nested_tensor_strides_native.h>
-#include <ATen/ops/_nested_tensor_offsets_native.h>
+#include <ATen/ops/_nested_tensor_storage_offsets_native.h>
 #include <ATen/ops/chunk_native.h>
+#include <ATen/ops/split_with_sizes_native.h>
 #endif
 
 namespace at {
 namespace native {
 
 /**
- * Thin wrapper around get_nested_size_tensor that is registered as a native function
+ * Thin wrapper around get_nested_sizes that is registered as a native function
  *
  * @return The nested tensors' size tensor.
  */
 at::Tensor _nested_tensor_size(const at::Tensor& self) {
-  return get_nested_size_tensor(self);
+  return get_nested_sizes(self);
 }
 
 at::Tensor _nested_tensor_strides(const at::Tensor& self){
-  return  get_nested_tensor_impl(self) -> get_nested_stride_tensor();
+  return  get_nested_tensor_impl(self) -> get_nested_strides();
 }
-std::vector<int64_t> _nested_tensor_offsets(const at::Tensor& self){
+at::Tensor _nested_tensor_storage_offsets(const at::Tensor& self){
   return get_nested_tensor_impl(self) -> get_storage_offsets();
 }
 
@@ -54,7 +55,7 @@ std::vector<int64_t> NestedTensor_get_max_size_from_size_tensor(
 
 std::vector<int64_t> NestedTensor_get_max_size(const NestedTensorImpl& nt) {
   return NestedTensor_get_max_size_from_size_tensor(
-      nt.get_nested_size_tensor());
+      nt.get_nested_sizes());
 }
 
 int64_t get_consistent_last_dim_of_nested_tensor(const NestedTensorImpl& nt) {
@@ -62,7 +63,7 @@ int64_t get_consistent_last_dim_of_nested_tensor(const NestedTensorImpl& nt) {
   TORCH_CHECK(
       last_dim != c10::nullopt,
       "Expected all tensors in nested tensor to have the same trailing dimension, instead last dimension equals: ",
-      nt.get_nested_size_tensor().select(1, -1));
+      nt.get_nested_sizes().select(1, -1));
   return *last_dim;
 }
 
@@ -84,9 +85,10 @@ std::vector<Tensor> chunk_nested_tensor(const Tensor& self, int64_t chunks, int6
   int64_t n_tensors = self.size(0);
   int64_t split_size = last_dim_size / chunks;
   std::vector<Tensor> splits(chunks);
-  const auto& sizes = self_impl->get_nested_size_tensor();
-  const auto& strides = self_impl->get_nested_stride_tensor();
-  const std::vector<int64_t>& offsets = self_impl->get_storage_offsets();
+  const auto& sizes = self_impl->get_nested_sizes();
+  const auto& strides = self_impl->get_nested_strides();
+  const auto offsets = self_impl->get_storage_offsets();
+  int64_t *offsets_ptr = offsets.data_ptr<int64_t>();
   // Account for the implicit batch dim
   --dim;
   int64_t tensor_dim = sizes.size(1);
@@ -94,16 +96,71 @@ std::vector<Tensor> chunk_nested_tensor(const Tensor& self, int64_t chunks, int6
       auto new_sizes = sizes.clone() ;
       auto new_strides = strides.clone();
       // This copys offsets so we are safe to move
-      auto new_offsets = std::vector<int64_t>(offsets);
+      auto new_offsets = offsets.clone();
       int64_t *size_ptr = new_sizes.data_ptr<int64_t>();
       // Get start val for each split
       int64_t start_val = split_idx * split_size;
       for (int64_t i : c10::irange(n_tensors)) {
         const int64_t index = i * tensor_dim + dim;
-        new_offsets[i] = offsets[i] + start_val;
+        new_offsets[i] = offsets_ptr[i] + start_val;
         size_ptr[index] = split_size;
     }
-    splits[split_idx] = create_nested_view_tensor(self, new_sizes, new_strides, std::move(new_offsets));
+    splits[split_idx] = create_nested_view_tensor(self, new_sizes, new_strides, new_offsets);
+  }
+  return splits;
+}
+
+std::vector<Tensor> split_with_sizes_nested(
+    const Tensor& self,
+    c10::IntArrayRef split_sizes,
+    int64_t dim) {
+  int64_t ndim = self.dim();
+  if (ndim == 0) {
+    TORCH_CHECK_INDEX(false, "split_with_sizes() cannot be applied to a 0-dim tensor.");
+  }
+  dim = maybe_wrap_dim(dim, ndim);
+  TORCH_CHECK(self.dim() - 1 == dim,
+           "split_with_sizes for nested tensors is currently only supported for the last dimension.");
+  auto num_splits = split_sizes.size();
+  TORCH_CHECK(num_splits > 0,
+           "split_with_sizes expects number of splits to be greater than 0, got: ", num_splits);
+  TORCH_CHECK(self.is_contiguous(), "split_with_sizes expects `self` to be contiguous.");
+
+  // Ensure entire dim is split.
+  int64_t total_size = 0;
+  for (const auto split_size : split_sizes) {
+      total_size += split_size;
+  }
+  auto self_impl = get_nested_tensor_impl(self);
+  auto self_size = get_consistent_last_dim_of_nested_tensor(*self_impl);
+  TORCH_CHECK(total_size == self_size,
+          "split_with_sizes expects split_sizes to sum exactly to ", self_size,
+          " (input tensor's size at dimension ", dim, "), but got split_sizes=", split_sizes);
+
+  int64_t n_tensors = self.size(0);
+  std::vector<Tensor> splits(num_splits);
+  const auto& sizes = self_impl->get_nested_sizes();
+  const auto& strides = self_impl->get_nested_strides();
+  const auto offsets = self_impl->get_storage_offsets();
+  int64_t *offsets_ptr = offsets.data_ptr<int64_t>();
+  // Account for the implicit batch dim
+  --dim;
+  int64_t tensor_dim = sizes.size(1);
+  int64_t start_val = 0;
+  for (const auto split_idx : c10::irange(num_splits)) {
+    auto split_size = split_sizes[split_idx];
+    auto new_sizes = sizes.clone();
+    auto new_strides = strides.clone();
+    auto new_offsets = offsets.clone();
+    int64_t *size_ptr = new_sizes.data_ptr<int64_t>();
+    // Get start val for each split
+    for (int64_t i : c10::irange(n_tensors)) {
+      const int64_t index = i * tensor_dim + dim;
+      new_offsets[i] = offsets_ptr[i] + start_val;
+      size_ptr[index] = split_size;
+    }
+    start_val += split_size;
+    splits[split_idx] = create_nested_view_tensor(self, new_sizes, new_strides, new_offsets);
   }
   return splits;
 }
