@@ -5,6 +5,8 @@
 #include <ATen/NativeFunctions.h>
 #else
 #include <ATen/ops/is_nonzero_native.h>
+#include <ATen/ops/_foobar_native.h>
+#include <ATen/ops/_test_functorch_fallback_native.h>
 #endif
 
 namespace at {
@@ -37,5 +39,10 @@ Tensor foobar(const Tensor& self, bool arg1, bool arg2, bool arg3) {
   return self;
 }
 
+// Aux function used to test functorch fallback warning
+Tensor _test_functorch_fallback(const Tensor& self, const Tensor& other) {
+  return self.clone();
+}
+
 } // namespace meta
 } // namespace at
