@@ -87,7 +87,7 @@ Tensor transformer_encoder_layer_forward(
     const Tensor& check_for_empty = src.is_nested() ? get_nested_tensor_impl(src)->get_buffer() : src;
     if (check_for_empty.numel() == 0) {
       return src.is_nested()
-        ? at::detail::make_tensor<NestedTensorImpl>(check_for_empty, get_nested_tensor_impl(src)->get_nested_size_tensor())
+        ? at::detail::make_tensor<NestedTensorImpl>(check_for_empty, get_nested_tensor_impl(src)->get_nested_sizes())
         : src.clone();
     }
   }
@@ -137,85 +137,5 @@ Tensor transformer_encoder_layer_forward(
   return x;
 }
 
-std::tuple<Tensor, Tensor, Tensor>  transformer_decoder_only_layer_forward(
-    const Tensor& src,
-    const int64_t embed_dim,
-    const int64_t num_heads,
-    const Tensor& qkv_weight,
-    const Tensor& qkv_bias,
-    const Tensor& proj_weight,
-    const Tensor& proj_bias,
-    const bool use_gelu,
-    const bool norm_first,
-    const double layer_norm_eps,
-    const Tensor& layer_norm_weight_1,
-    const Tensor& layer_norm_bias_1,
-    const Tensor& layer_norm_weight_2,
-    const Tensor& layer_norm_bias_2,
-    const Tensor& ffn_weight_1,
-    const Tensor& ffn_bias_1,
-    const Tensor& ffn_weight_2,
-    const Tensor& ffn_bias_2,
-    const c10::optional<Tensor>& mask,
-    const c10::optional<Tensor>& incr_key,
-    const c10::optional<Tensor>& incr_value) {
-  {
-    const Tensor& check_for_empty = src.is_nested() ? get_nested_tensor_impl(src)->get_buffer() : src;
-    if (check_for_empty.numel() == 0) {
-      auto src_out = src.is_nested()
-        ? at::detail::make_tensor<NestedTensorImpl>(check_for_empty, get_nested_tensor_impl(src)->get_nested_size_tensor())
-        : src.clone();
-      return std::make_tuple(src_out, incr_key.value(), incr_value.value());
-    }
-  }
-  TORCH_WARN("_transformer_decoder_only_layer_fwd is deprecated")
-
-  TORCH_CHECK(!norm_first, "norm_first is not supported yet");
-  auto mha_out = native_decoder_only_multi_head_attention(
-      src,
-      src,
-      src,
-      embed_dim,
-      num_heads,
-      qkv_weight,
-      qkv_bias,
-      proj_weight,
-      proj_bias,
-      mask,
-      incr_key,
-      incr_value,
-      false /* need_weights */);
-  auto x = std::get<0>(mha_out);
-  auto incr_key_out = std::get<2>(mha_out);
-  auto incr_value_out = std::get<3>(mha_out);
-  x.add_(src);
-  x = at::layer_norm(
-      x,
-      {embed_dim},
-      layer_norm_weight_1,
-      layer_norm_bias_1,
-      layer_norm_eps,
-      true);
-
-  auto pre_ffn_res = x;
-  x = ffn(
-      x,
-      ffn_weight_1,
-      ffn_bias_1,
-      ffn_weight_2,
-      ffn_bias_2,
-      use_gelu,
-      /* add_norm* */ false);
-  x.add_(pre_ffn_res);
-  x = at::layer_norm(
-      x,
-      {embed_dim},
-      layer_norm_weight_2,
-      layer_norm_bias_2,
-      layer_norm_eps,
-      true);
-  return std::make_tuple(std::move(x), std::move(incr_key_out), std::move(incr_value_out));
-}
-
 } // namespace native
 } // namespace at
