@@ -32,12 +32,17 @@ Tensor upsample_nearest2d(
           output_sizes[Layout::Parameter::height],
           output_sizes[Layout::Parameter::width],
       },
-      input_arg.scalar_type(),
-  };
+      input_arg.scalar_type()};
+
+  if (v_input.is_quantized()) {
+    v_output.set_is_quantized();
+    v_output.set_scale(v_input.get_scale());
+    v_output.set_zero_point(v_input.get_zero_point());
+  }
 
   const struct Block final {
     uvec3 extents;
-    uint32_t _;
+    uint32_t fill0;
     ivec2 iextents;
     vec2 scale;
   } block{
@@ -66,7 +71,8 @@ Tensor upsample_nearest2d(
 
   context->submit_compute_job(
       // shader descriptor
-      VK_KERNEL(upsample_nearest2d),
+      v_input.is_quantized() ? VK_KERNEL(quantized_upsample_nearest2d)
+                             : VK_KERNEL(upsample_nearest2d),
       // pipeline barrier
       pipeline_barrier,
       // global work group size
@@ -87,9 +93,10 @@ Tensor upsample_nearest2d(
   return convert(v_output);
 }
 
-Tensor quantized_upsample_nearest2d(
+Tensor upsample_bilinear2d(
     const Tensor& input_arg,
     const IntArrayRef output_sizes,
+    bool align_corners,
     const c10::optional<double> scales_h,
     const c10::optional<double> scales_w) {
   api::Context* const context = api::context();
@@ -100,59 +107,60 @@ Tensor quantized_upsample_nearest2d(
 
   const Tensor input = input_arg.is_vulkan() ? input_arg : input_arg.vulkan();
   const vTensor& v_input = convert(input);
-  const auto v_input_sizes = v_input.sizes();
 
   vTensor v_output{
       context,
       {
-          v_input_sizes[Layout::Activation4D::batch],
-          v_input_sizes[Layout::Activation4D::channels],
+          get_dim<Dim4D::Batch>(v_input),
+          get_dim<Dim4D::Channel>(v_input),
           output_sizes[Layout::Parameter::height],
           output_sizes[Layout::Parameter::width],
       },
-      v_input.get_scale(),
-      v_input.get_zero_point(),
       input_arg.scalar_type(),
   };
 
+  const api::utils::uvec3 output_extents = v_output.extents();
   const struct Block final {
-    uvec3 extents;
-    uint32_t _;
+    uvec3 oextents;
+    uint32_t padding;
     ivec2 iextents;
     vec2 scale;
   } block{
-      v_output.extents(),
-      0u,
+      v_output.extents(), // oextents
+      0u, // padding
       {
-          safe_downcast<int32_t>(
-              input_arg.size(Layout::Activation4D::width) - 1),
-          safe_downcast<int32_t>(
-              input_arg.size(Layout::Activation4D::height) - 1),
-      },
+          safe_downcast<int32_t>(get_dim<Dim4D::Width>(input_arg) - 1),
+          safe_downcast<int32_t>(get_dim<Dim4D::Height>(input_arg) - 1),
+      }, // iextents
       {
           compute_scales_value<float>(
               scales_w,
-              v_input_sizes[Layout::Activation4D::width],
-              output_sizes[Layout::Parameter::width]),
+              get_dim<Dim4D::Width>(input_arg),
+              get_dim<Dim4D::Width>(v_output)),
           compute_scales_value<float>(
               scales_h,
-              v_input_sizes[Layout::Activation4D::height],
-              output_sizes[Layout::Parameter::height]),
-      },
+              get_dim<Dim4D::Height>(input_arg),
+              get_dim<Dim4D::Height>(v_output)),
+      }, // scale
   };
 
   api::UniformParamsBuffer params(context, block);
   api::PipelineBarrier pipeline_barrier{};
-
+  api::ShaderInfo shader_desc;
+  if (align_corners) {
+    shader_desc = VK_KERNEL(upsample_bilinear2d_align_true);
+  } else {
+    shader_desc = VK_KERNEL(upsample_bilinear2d_align_false);
+  }
   context->submit_compute_job(
       // shader descriptor
-      VK_KERNEL(quantized_upsample_nearest2d),
+      shader_desc,
       // pipeline barrier
       pipeline_barrier,
       // global work group size
-      v_output.extents(),
+      output_extents,
       // local work group size
-      adaptive_work_group_size(v_output.extents()),
+      adaptive_work_group_size(output_extents),
       // fence handle
       VK_NULL_HANDLE,
       // shader arguments
@@ -173,6 +181,9 @@ TORCH_LIBRARY_IMPL(aten, Vulkan, m) {
   m.impl(
       TORCH_SELECTIVE_NAME("aten::upsample_nearest2d"),
       TORCH_FN(upsample_nearest2d));
+  m.impl(
+      TORCH_SELECTIVE_NAME("aten::upsample_bilinear2d"),
+      TORCH_FN(upsample_bilinear2d));
 }
 
 #endif /* USE_VULKAN_API */
