@@ -22,6 +22,10 @@ class VulkanImplRegistrar {
 };
 
 at::Tensor& vulkan_copy_(at::Tensor& self, const at::Tensor& src);
-
 } // namespace vulkan
+
+namespace native {
+  bool is_vulkan_available();
+}// namespace native
+
 } // namespace at
