@@ -1,19 +1,19 @@
-
-import operator_benchmark as op_bench
 import benchmark_caffe2 as op_bench_c2
 from benchmark_caffe2 import Caffe2BenchmarkBase  # noqa: F401
 from caffe2.python import core
 
+import operator_benchmark as op_bench
+
 """Microbenchmarks for MatMul operator"""
 
 # Configs for C2 Matmul operator
 mm_long_configs = op_bench.cross_product_configs(
     M=[8, 64, 128],
     N=range(2, 10, 3),
-    K=[2 ** x for x in range(0, 3)],
+    K=[2**x for x in range(0, 3)],
     trans_a=[True, False],
     trans_b=[True, False],
-    tags=["long"]
+    tags=["long"],
 )
 
 
@@ -32,7 +32,7 @@ class MatMulBenchmark(op_bench_c2.Caffe2BenchmarkBase):
     def init(self, M, N, K, trans_a, trans_b):
         self.input_one = self.tensor([N, M]) if trans_a else self.tensor([M, N])
         self.input_two = self.tensor([K, N]) if trans_b else self.tensor([N, K])
-        self.args = {'trans_a': trans_a, 'trans_b': trans_b}
+        self.args = {"trans_a": trans_a, "trans_b": trans_b}
         self.output = self.tensor([M, K])
         self.set_module_name("matmul")
 
