@@ -1,7 +1,9 @@
-import operator_benchmark as op_bench
-import torch
 from typing import List
 
+import torch
+
+import operator_benchmark as op_bench
+
 
 """Microbenchmarks for as_strided operator"""
 
@@ -15,7 +17,7 @@ as_strided_configs_short = op_bench.config_list(
         [512, 512, (64, 64), (2, 2), 1],
     ],
     cross_product_configs={
-        'device': ['cpu', 'cuda'],
+        "device": ["cpu", "cuda"],
     },
     tags=["short"],
 )
@@ -26,8 +28,8 @@ as_strided_configs_long = op_bench.cross_product_configs(
     size=[(16, 16), (128, 128)],
     stride=[(1, 1)],
     storage_offset=[0, 1],
-    device=['cpu', 'cuda'],
-    tags=['long']
+    device=["cpu", "cuda"],
+    tags=["long"],
 )
 
 
@@ -37,19 +39,19 @@ class As_stridedBenchmark(op_bench.TorchBenchmarkBase):
             "input_one": torch.rand(M, N, device=device),
             "size": size,
             "stride": stride,
-            "storage_offset": storage_offset
+            "storage_offset": storage_offset,
         }
-        self.set_module_name('as_strided')
+        self.set_module_name("as_strided")
 
     def forward(
         self, input_one, size: List[int], stride: List[int], storage_offset: int
     ):
-        return torch.as_strided(
-            input_one, size, stride, storage_offset)
+        return torch.as_strided(input_one, size, stride, storage_offset)
 
 
-op_bench.generate_pt_test(as_strided_configs_short + as_strided_configs_long,
-                          As_stridedBenchmark)
+op_bench.generate_pt_test(
+    as_strided_configs_short + as_strided_configs_long, As_stridedBenchmark
+)
 
 
 if __name__ == "__main__":
