@@ -1,6 +1,7 @@
-import operator_benchmark as op_bench
 import torch
 
+import operator_benchmark as op_bench
+
 """Microbenchmarks for MatMul operator"""
 
 # Configs for PT Matmul operator
@@ -12,7 +13,7 @@ mm_short_configs = op_bench.config_list(
         [256, 256, 256, False, True],
     ],
     cross_product_configs={
-        'device': ['cpu', 'cuda'],
+        "device": ["cpu", "cuda"],
     },
     tags=["short"],
 )
@@ -24,8 +25,8 @@ mm_long_configs = op_bench.cross_product_configs(
     K=[64],
     trans_a=[False, True],
     trans_b=[True, False],
-    device=['cpu', 'cuda'],
-    tags=["long"]
+    device=["cpu", "cuda"],
+    tags=["long"],
 )
 
 
