@@ -1,17 +1,18 @@
-import operator_benchmark as op_bench
 import torch
 
+import operator_benchmark as op_bench
+
 # Configs for pointwise and reduction unary ops
 qmethods_configs_short = op_bench.config_list(
-    attr_names=['M', 'N'],
+    attr_names=["M", "N"],
     attrs=[
         [32, 32],
     ],
     cross_product_configs={
-        'dtype': [torch.quint8],
-        'contig': [False, True],
+        "dtype": [torch.quint8],
+        "contig": [False, True],
     },
-    tags=['short']
+    tags=["short"],
 )
 
 qmethods_configs_long = op_bench.cross_product_configs(
@@ -19,7 +20,7 @@ qmethods_configs_long = op_bench.cross_product_configs(
     N=[256, 1024],
     dtype=[torch.qint8, torch.qint32],
     contig=[False, True],
-    tags=['long']
+    tags=["long"],
 )
 
 
@@ -28,9 +29,9 @@ class _QMethodBenchmarkBase(op_bench.TorchBenchmarkBase):
         f_input = torch.rand(M, N)
         scale = 1.0
         zero_point = 0
-        self.q_input = torch.quantize_per_tensor(f_input, scale=scale,
-                                                 zero_point=zero_point,
-                                                 dtype=dtype)
+        self.q_input = torch.quantize_per_tensor(
+            f_input, scale=scale, zero_point=zero_point, dtype=dtype
+        )
         if not contig:
             permute_dims = list(range(self.q_input.ndim))[::-1]
             self.q_input = self.q_input.permute(permute_dims)
@@ -46,8 +47,7 @@ class QMethodTensorInputCopyBenchmark(_QMethodBenchmarkBase):
 
 
 op_bench.generate_pt_test(
-    qmethods_configs_short + qmethods_configs_long,
-    QMethodTensorInputCopyBenchmark
+    qmethods_configs_short + qmethods_configs_long, QMethodTensorInputCopyBenchmark
 )
 
 if __name__ == "__main__":
