@@ -1030,7 +1030,7 @@ class CollectTensorOp final : public Operator<Context> {
         CAFFE_ENFORCE(
             // NOLINTNEXTLINE(clang-diagnostic-sign-compare)
             tensorVector->size() == numToCollect_,
-            "TensorVecotor size = ",
+            "TensorVector size = ",
             tensorVector->size(),
             " is different from numToCollect = ",
             numToCollect_);
@@ -1327,7 +1327,7 @@ Append input `B` to the end of input `A`.
 - Input `A` may have to be re-allocated in order for accommodate to the new size. Currently, an exponential growth ratio is used in order to ensure amortized constant time complexity.
 
 Github Links:
-- https://github.com/pytorch/pytorch/blob/master/caffe2/operators/dataset_ops.cc
+- https://github.com/pytorch/pytorch/blob/main/caffe2/operators/dataset_ops.cc
 
 <details>
 
