@@ -5,29 +5,31 @@
 #include <ostream>
 #include <fstream>
 #include <algorithm>
+#include <sstream>
 #include <sys/stat.h>
 #include <sys/types.h>
 
-
 #include <c10/core/Allocator.h>
 #include <c10/core/CPUAllocator.h>
 #include <c10/core/Backend.h>
 #include <c10/util/Exception.h>
+#include <c10/util/Logging.h>
+#include <c10/util/hash.h>
 
 #include "caffe2/core/common.h"
-#include "caffe2/core/logging.h"
 #include "caffe2/serialize/file_adapter.h"
 #include "caffe2/serialize/inline_container.h"
 #include "caffe2/serialize/istream_adapter.h"
 #include "caffe2/serialize/read_adapter_interface.h"
 
+#include "caffe2/serialize/versions.h"
 #include "miniz.h"
 
 namespace caffe2 {
 namespace serialize {
 constexpr c10::string_view kDebugPklSuffix(".debug_pkl");
 
-size_t istream_read_func(void *pOpaque, mz_uint64 file_ofs, void *pBuf, size_t n) {
+size_t istream_read_func(void* pOpaque, mz_uint64 file_ofs, void* pBuf, size_t n) {
   auto self = static_cast<PyTorchStreamReader*>(pOpaque);
   return self->read(file_ofs, static_cast<char*>(pBuf), n);
 }
@@ -40,8 +42,9 @@ static std::string basename(const std::string& name) {
     }
   }
 
-  if (start >= name.size())
+  if (start >= name.size()) {
     return "";
+  }
 
   size_t end = name.size();
   for(size_t i = end; i > start; --i) {
@@ -55,11 +58,13 @@ static std::string basename(const std::string& name) {
 
 static std::string parentdir(const std::string& name) {
   size_t end = name.find_last_of('/');
-  if(end == std::string::npos)
+  if(end == std::string::npos) {
     end = name.find_last_of('\\');
+  }
 
-  if(end == std::string::npos)
+  if(end == std::string::npos) {
     return "";
+  }
 
   return name.substr(0, end);
 }
@@ -116,7 +121,7 @@ void PyTorchStreamReader::init() {
 
   // figure out the archive_name (i.e. the zip folder all the other files are in)
   // all lookups to getRecord will be prefixed by this folder
-  int n = mz_zip_reader_get_num_files(ar_.get());
+  mz_uint n = mz_zip_reader_get_num_files(ar_.get());
   if (n == 0) {
     CAFFE_THROW("archive does not contain any files");
   }
@@ -132,6 +137,20 @@ void PyTorchStreamReader::init() {
   archive_name_ = buf.substr(0, pos);
   archive_name_plus_slash_ = archive_name_ + "/";
 
+  // read serialization id
+  if (hasRecord(kSerializationIdRecordName)) {
+    at::DataPtr serialization_id_ptr;
+    size_t serialization_id_size = 0;
+    std::tie(serialization_id_ptr, serialization_id_size) =
+        getRecord(kSerializationIdRecordName);
+    serialization_id_.assign(
+        static_cast<const char*>(serialization_id_ptr.get()),
+        serialization_id_size);
+  }
+  c10::LogAPIUsageMetadata(
+      "pytorch.stream.reader.metadata",
+      {{"serialization_id", serialization_id_}});
+
   // version check
   at::DataPtr version_ptr;
   // NOLINTNEXTLINE(cppcoreguidelines-init-variables)
@@ -145,13 +164,12 @@ void PyTorchStreamReader::init() {
   std::string version(static_cast<const char*>(version_ptr.get()), version_size);
   try {
     version_ = caffe2::stoull(version);
-  } catch (const std::invalid_argument &e) {
+  } catch (const std::invalid_argument& e) {
     CAFFE_THROW("Couldn't parse the version ",
                  version,
                  " as Long Long.");
   }
-  // NOLINTNEXTLINE(clang-diagnostic-sign-compare)
-  if (version_ < kMinSupportedFileFormatVersion) {
+  if (version_ < static_cast<decltype(version_)>(kMinSupportedFileFormatVersion)) {
     CAFFE_THROW(
         "Attempted to read a PyTorch file with version ",
         c10::to_string(version_),
@@ -161,8 +179,7 @@ void PyTorchStreamReader::init() {
         " with latest version of PyTorch to mitigate this issue.");
   }
 
-  // NOLINTNEXTLINE(clang-diagnostic-sign-compare)
-  if (version_ > kMaxSupportedFileFormatVersion) {
+  if (version_ > static_cast<decltype(version_)>(kMaxSupportedFileFormatVersion)) {
     CAFFE_THROW(
         "Attempted to read a PyTorch file with version ",
         version_,
@@ -187,6 +204,7 @@ void PyTorchStreamReader::valid(const char* what, const char* info) {
 constexpr int MZ_ZIP_LOCAL_DIR_HEADER_SIZE = 30;
 constexpr int MZ_ZIP_LDH_FILENAME_LEN_OFS = 26;
 constexpr int MZ_ZIP_LDH_EXTRA_LEN_OFS = 28;
+constexpr int MZ_ZIP_DATA_DESCRIPTOR_ID = 0x08074b50;
 
 namespace detail {
 size_t getPadding(
@@ -299,6 +317,75 @@ std::tuple<at::DataPtr, size_t> PyTorchStreamReader::getRecord(const std::string
   return std::make_tuple(std::move(retval), stat.m_uncomp_size);
 }
 
+// inplace memory writing
+size_t
+PyTorchStreamReader::getRecord(const std::string& name, void* dst, size_t n) {
+  std::lock_guard<std::mutex> guard(reader_lock_);
+  if ((!load_debug_symbol_) && c10::string_view(name).ends_with(kDebugPklSuffix)) {
+    return 0;
+  }
+  size_t key = getRecordID(name);
+  mz_zip_archive_file_stat stat;
+  mz_zip_reader_file_stat(ar_.get(), key, &stat);
+  TORCH_CHECK(
+      n == stat.m_uncomp_size,
+      "record size ",
+      stat.m_uncomp_size,
+      " mismatch with dst size ",
+      n);
+  valid("retrieving file meta-data for ", name.c_str());
+  mz_zip_reader_extract_to_mem(ar_.get(), key, dst, stat.m_uncomp_size, 0);
+  valid("reading file ", name.c_str());
+
+  return stat.m_uncomp_size;
+}
+
+size_t PyTorchStreamReader::getRecord(
+    const std::string& name,
+    void* dst,
+    size_t n,
+    size_t chunk_size,
+    void* buf,
+    const std::function<void(void*, const void*, size_t)>& memcpy_func) {
+  std::lock_guard<std::mutex> guard(reader_lock_);
+  if ((!load_debug_symbol_) && c10::string_view(name).ends_with(kDebugPklSuffix)) {
+    return 0;
+  }
+  size_t key = getRecordID(name);
+  mz_zip_archive_file_stat stat;
+  mz_zip_reader_file_stat(ar_.get(), key, &stat);
+  TORCH_CHECK(
+      n == stat.m_uncomp_size,
+      "record size ",
+      stat.m_uncomp_size,
+      " mismatch with dst size ",
+      n);
+  valid("retrieving file meta-data for ", name.c_str());
+
+  mz_zip_reader_extract_iter_state* iter =
+      mz_zip_reader_extract_iter_new(ar_.get(), key, 0);
+  TORCH_CHECK(
+      iter != nullptr,
+      "Failed to create zip reader iter: ",
+      mz_zip_get_error_string(mz_zip_get_last_error(ar_.get())));
+
+  for (size_t offset = 0; offset < stat.m_uncomp_size; offset += chunk_size) {
+    size_t want_size =
+        std::min(chunk_size, (size_t)stat.m_uncomp_size - offset);
+    size_t read_size =
+        mz_zip_reader_extract_iter_read(iter, buf, want_size);
+    TORCH_CHECK(
+        read_size == want_size,
+        "Failed to advance zip reader iter: ",
+        mz_zip_get_error_string(mz_zip_get_last_error(ar_.get())));
+    memcpy_func((char*)dst + offset, buf, read_size);
+  }
+  valid("reading file ", name.c_str());
+  mz_zip_reader_extract_iter_free(iter);
+
+  return stat.m_uncomp_size;
+}
+
 static int64_t read_le_16(uint8_t* buf) {
   return buf[0] + (buf[1] << 8);
 }
@@ -341,10 +428,20 @@ size_t ostream_write_func(
     self->err_seen_ = true;
   }
   self->current_pos_ += ret;
+
+  // Get the CRC32 of uncompressed data from the data descriptor, if the written
+  // data is identified as the data descriptor block.
+  if (n >= 8 && MZ_READ_LE32(pBuf) == MZ_ZIP_DATA_DESCRIPTOR_ID) {
+    const int8_t* pInt8Buf = (const int8_t*)pBuf;
+    const uint32_t uncomp_crc32 = MZ_READ_LE32(pInt8Buf + 4);
+    self->combined_uncomp_crc32_ =
+        c10::hash_combine(self->combined_uncomp_crc32_, uncomp_crc32);
+  }
+
   return ret;
 }
 
-PyTorchStreamWriter::PyTorchStreamWriter(std::string file_name)
+PyTorchStreamWriter::PyTorchStreamWriter(const std::string& file_name)
     : archive_name_(basename(file_name)) {
   setup(file_name);
 }
@@ -403,6 +500,12 @@ void PyTorchStreamWriter::writeRecord(
   AT_ASSERT(!archive_name_plus_slash_.empty());
   TORCH_INTERNAL_ASSERT(
       files_written_.count(name) == 0, "Tried to serialize file twice: ", name);
+  if (name == kSerializationIdRecordName && serialization_id_.empty()) {
+    // In case of copying records from another file, skip writing a different
+    // serialization_id than the one computed in this writer.
+    // This is to ensure serialization_id is unique per serialization output.
+    return;
+  }
   std::string full_name = archive_name_plus_slash_ + name;
   size_t padding_size =
       detail::getPadding(ar_->m_archive_size, full_name.size(), size, padding_);
@@ -454,6 +557,20 @@ void PyTorchStreamWriter::writeEndOfFile() {
     }
   }
 
+  // If no "byteorder" record in the output model, rewrites byteorder info
+  if(allRecords.find("byteorder") == allRecords.end()) {
+#if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
+    std::string byteorder = "little";
+#elif __BYTE_ORDER__ == __ORDER_BIG_ENDIAN__
+    std::string byteorder = "big";
+#else
+#error Unexpected or undefined __BYTE_ORDER__
+#endif
+    writeRecord("byteorder", byteorder.c_str(), byteorder.size());
+  }
+
+  writeSerializationId();
+
   AT_ASSERT(!finalized_);
   finalized_ = true;
 
@@ -480,6 +597,35 @@ void PyTorchStreamWriter::valid(const char* what, const char* info) {
   }
 }
 
+void PyTorchStreamWriter::writeSerializationId() {
+  // Serialization id is computed based on all files written, and is composed of
+  // 1) a combined hash of record name hashes
+  // 2) a combined crc32 of the record uncompressed data
+  // This is best effort to create a fixed-length, unique and deterministic id
+  // for the serialized files without incurring additional computation overhead.
+  if (files_written_.find(kSerializationIdRecordName) == files_written_.end()) {
+    uint64_t combined_record_name_hash = 0;
+    for (const std::string& record_name : files_written_) {
+      size_t record_name_hash = c10::hash<std::string>{}(record_name);
+      combined_record_name_hash =
+          c10::hash_combine(combined_record_name_hash, record_name_hash);
+    }
+    std::ostringstream serialization_id_oss;
+    serialization_id_oss << std::setfill('0') << std::setw(20)
+                         << combined_record_name_hash
+                         << std::setfill('0') << std::setw(20)
+                         << combined_uncomp_crc32_;
+    serialization_id_ = serialization_id_oss.str();
+    writeRecord(
+        kSerializationIdRecordName,
+        serialization_id_.c_str(),
+        serialization_id_.size());
+    c10::LogAPIUsageMetadata(
+      "pytorch.stream.writer.metadata",
+      {{"serialization_id", serialization_id_}});
+  }
+}
+
 // NOLINTNEXTLINE(bugprone-exception-escape)
 PyTorchStreamWriter::~PyTorchStreamWriter() {
   if (!finalized_) {
