@@ -2,13 +2,14 @@
 # Owner(s): ["oncall: distributed"]
 
 import torch
+import torch.distributed as dist
 import torch.nn.functional as F
-from torch.distributed.tensor.parallel import (
-    PairwiseParallel,
-    parallelize_module,
-)
+from numpy.testing import assert_array_equal
+from torch.distributed._functional_collectives import AsyncCollectiveTensor
+
 from torch.distributed._tensor import DeviceMesh, distribute_tensor, DTensor
 from torch.distributed._tensor.placement_types import _Partial, Replicate, Shard
+from torch.distributed.tensor.parallel import PairwiseParallel, parallelize_module
 
 from torch.testing._internal.common_utils import run_tests
 from torch.testing._internal.distributed._tensor.common_dtensor import (
@@ -16,6 +17,7 @@ from torch.testing._internal.distributed._tensor.common_dtensor import (
     with_comms,
 )
 
+
 class DummyMLP(torch.nn.Module):
     def __init__(self, device):
         super().__init__()
@@ -33,6 +35,7 @@ class DummyMLP(torch.nn.Module):
             self.net1.bias.fill_(1.5)
             self.net2.bias.fill_(1.2)
 
+
 class DTensorTest(DTensorTestBase):
     @with_comms
     def test_dtensor_constructor(self):
@@ -44,13 +47,23 @@ class DTensorTest(DTensorTestBase):
             local_tensor,
             device_mesh,
             shard_spec,
-            size=dist_tensor_shape,
+            shape=dist_tensor_shape,
+            dtype=local_tensor.dtype,
             requires_grad=True,
+            stride=local_tensor.stride(),
         )
         self.assertEqual(dist_tensor.size(), torch.Size((self.world_size * 3, 3)))
 
         with self.assertWarnsRegex(UserWarning, "To construct"):
-            DTensor(local_tensor, device_mesh, shard_spec, size=dist_tensor_shape)
+            DTensor(
+                local_tensor,
+                device_mesh,
+                shard_spec,
+                shape=dist_tensor_shape,
+                dtype=local_tensor.dtype,
+                requires_grad=False,
+                stride=local_tensor.stride(),
+            )
 
         local_tensor = torch.randn(3, 3, requires_grad=False)
         with self.assertWarnsRegex(UserWarning, "To construct"):
@@ -58,8 +71,10 @@ class DTensorTest(DTensorTestBase):
                 local_tensor,
                 device_mesh,
                 shard_spec,
-                size=dist_tensor_shape,
+                shape=dist_tensor_shape,
+                dtype=local_tensor.dtype,
                 requires_grad=True,
+                stride=local_tensor.stride(),
             )
 
     @with_comms
@@ -94,7 +109,9 @@ class DTensorTest(DTensorTestBase):
         model_tp.reset_parameters()
         optim = torch.optim.SGD(model_tp.parameters(), lr=0.1)
         model_regular = DummyMLP(self.device_type)
-        model_regular_tp = parallelize_module(model_regular, device_mesh, PairwiseParallel())
+        model_regular_tp = parallelize_module(
+            model_regular, device_mesh, PairwiseParallel()
+        )
         optim_regular = torch.optim.SGD(model_regular_tp.parameters(), lr=0.1)
         model_regular_tp.reset_parameters()
         torch.manual_seed(0)
@@ -120,14 +137,14 @@ class DTensorTest(DTensorTestBase):
         shard0_spec = [Shard(0)]
         local_tensor = torch.randn(4, 8)
         global_shape = torch.Size([self.world_size * 4, 8])
-        dist_tensor = DTensor(local_tensor, device_mesh, shard0_spec, size=global_shape)
+        dist_tensor = DTensor.from_local(local_tensor, device_mesh, shard0_spec)
         # won't affect stride
         self.assertEqual(dist_tensor.stride(), (8, 1))
 
         shard1_spec = [Shard(1)]
         local_tensor = torch.randn(8, 4)
         global_shape = torch.Size([8, self.world_size * 4])
-        dist_tensor = DTensor(local_tensor, device_mesh, shard1_spec, size=global_shape)
+        dist_tensor = DTensor.from_local(local_tensor, device_mesh, shard1_spec)
         # will affect stride after DT initialized
         self.assertEqual(dist_tensor.stride(), (4 * self.world_size, 1))
 
@@ -136,9 +153,7 @@ class DTensorTest(DTensorTestBase):
         local_tensor_t = local_tensor.permute(1, 2, 0)
         global_shape = torch.Size([4, self.world_size * 8, 8])
         self.assertEqual(local_tensor_t.stride(), (8, 1, 32))
-        dist_tensor = DTensor(
-            local_tensor_t, device_mesh, shard1_spec, size=global_shape
-        )
+        dist_tensor = DTensor.from_local(local_tensor_t, device_mesh, shard1_spec)
         global_stride = (8 * self.world_size, 1, 32 * self.world_size)
         self.assertEqual(dist_tensor.stride(), global_stride)
 
@@ -182,7 +197,7 @@ class DTensorTest(DTensorTestBase):
     @with_comms
     def test_to_local(self):
         device_mesh = DeviceMesh(self.device_type, list(range(self.world_size)))
-        shard_spec = [Shard(0)]
+        shard_spec = (Shard(0),)
         dist_tensor_shape = torch.Size([self.world_size * 3, 3])
         local_tensor_with_grad = torch.randn(
             3, 3, device=self.device_type, requires_grad=True
@@ -192,8 +207,10 @@ class DTensorTest(DTensorTestBase):
             local_tensor_with_grad,
             device_mesh,
             shard_spec,
-            size=dist_tensor_shape,
+            shape=dist_tensor_shape,
+            dtype=local_tensor_with_grad.dtype,
             requires_grad=True,
+            stride=local_tensor_with_grad.stride(),
         )
         self.assertEqual(sharded_tensor.size(), dist_tensor_shape)
         self.assertEqual(sharded_tensor.to_local(), local_tensor_with_grad)
@@ -214,6 +231,84 @@ class DTensorTest(DTensorTestBase):
 
         self.assertEqual(sharded_tensor.grad.to_local(), torch.ones(3, 3) * 3)
 
+        # test the case when grad stride is different from fwd input.
+        res = sharded_tensor.to_local()
+        model = torch.nn.ReLU()
+        res.register_hook(lambda grad: grad.t())
+        target = torch.randn(3, 3, device=self.device_type)
+        mae_loss = torch.nn.L1Loss()
+        output = mae_loss(model(res), target)
+        # The manual change to grad stride leads to the failure of the copy op afterwards.
+        # so that we need a try-catch here.
+        try:
+            output.backward()
+        except RuntimeError:
+            self.assertEqual(sharded_tensor.grad.stride(), [1, 3 * self.world_size])
+
+    @with_comms
+    def test_dtensor_new_empty_strided(self):
+        device_mesh = DeviceMesh(self.device_type, torch.arange(self.world_size))
+        local_tensor = torch.randn(8, 8, requires_grad=True, device=self.device_type)
+        my_dtensor = distribute_tensor(local_tensor, device_mesh, [Shard(0)])
+        new_strided_dtensor = my_dtensor.new_empty_strided(
+            (8, 8), (8, 1), requires_grad=True
+        )
+        # test the op produces new dtensor and autograd works
+        self.assertEqual(new_strided_dtensor.shape, my_dtensor.shape)
+        new_strided_dtensor.sum().backward()
+        self.assertIsNotNone(new_strided_dtensor.grad)
+        self.assertIsInstance(new_strided_dtensor.grad, DTensor)
+
+        # test backward new_empty_strided with sharding works correctly
+        my_dtensor.to_local().sum().backward()
+        local_tensor.sum().backward()
+        self.assertEqual(my_dtensor.grad, new_strided_dtensor.grad)
+        self.assertEqual(
+            my_dtensor.grad.redistribute(placements=[Replicate()]).to_local(),
+            local_tensor.grad,
+        )
+
+    @with_comms
+    def test_dtensor_async_output(self):
+        # Tests that if the output of some dtensor operations  isn't used in any compute,
+        # the output should be an AsyncCollectiveTensor (representing the fact that
+        # we haven't synced the collective yet).
+        from torch.distributed._functional_collectives_impl import _tensor_needs_wait
+
+        mesh = DeviceMesh(
+            self.device_type, torch.arange(self.world_size), _validate_mesh=False
+        )
+
+        def fn(dt):
+            dt_out_redistribute = dt.redistribute(mesh, [Replicate()])
+            # Make sure we haven't synced yet
+            # TODO: figure out why this is returning None
+            # self.assertTrue(_tensor_needs_wait(dt_out_redistribute))
+            dt_out_redistribute_view = dt_out_redistribute.view(
+                dt_out_redistribute.shape
+            )
+            local_tensor = dt_out_redistribute_view.to_local()
+            return local_tensor
+
+        x = torch.ones((4, 2), device=self.device_type)
+        dt = distribute_tensor(x, mesh, [Shard(0)])
+        out = fn(dt)
+        # Make sure we haven't synced yet
+        self.assertEqual(type(out), AsyncCollectiveTensor)
+        self.assertTrue(_tensor_needs_wait(out.elem))
+        out_view = out.view(-1)
+
+        # Assert that output is a `AsyncCollectiveTensor`
+        self.assertEqual(type(out_view), AsyncCollectiveTensor)
+        self.assertTrue(_tensor_needs_wait(out_view.elem))
+
+        # Use the daa, requiring a sync
+        ref = torch.ones((4, 2), device=self.device_type) + 1
+        ref = ref.view(-1)
+        out_data = out_view + 1
+        self.assertEqual(type(out_data), torch.Tensor)
+        self.assertEqual(out_data, ref)
+
     @with_comms
     def test_from_local_then_to_local(self):
         # this test ensure end to end from torch.Tensor -> dist tensor -> torch.Tensor works
@@ -287,12 +382,32 @@ class DTensorTest(DTensorTestBase):
         sharded_tensor = DTensor.from_local(local_tensor, device_mesh, shard_spec)
         self.assertEqual(sharded_tensor.device.type, self.device_type)
 
+    @with_comms
+    def test_dtensor_save_load(self):
+        import io
+
+        device_mesh = self.build_device_mesh()
+        shard_spec = [Shard(0)]
+        local_tensor = torch.randn(3, 3)
+        sharded_tensor = DTensor.from_local(local_tensor, device_mesh, shard_spec)
+        buffer = io.BytesIO()
+        torch.save(sharded_tensor, buffer)
+        buffer.seek(0)
+        reloaded_st = torch.load(buffer)
+        self.assertEqual(sharded_tensor, reloaded_st)
+
 
 class DTensorMeshTest(DTensorTestBase):
     @property
     def world_size(self):
         return 8
 
+    def sub_mesh_assert_equal(self, mesh, exp_in_mesh, exp_out_of_mesh, tensor):
+        if self.rank in mesh:
+            self.assertEqual(tensor, exp_in_mesh)
+        else:
+            self.assertEqual(tensor, exp_out_of_mesh)
+
     @with_comms
     def test_dtensor_device_mesh_device_conversion(self):
         # construct a cuda device mesh
@@ -325,6 +440,26 @@ class DTensorMeshTest(DTensorTestBase):
                 replica_tensor.size(), torch.Size([3 * self.world_size, 3])
             )
 
+        with DeviceMesh(self.device_type, torch.arange(self.world_size)):
+            shard_spec = [Shard(0)]
+            global_shape = torch.Size([3 * self.world_size, 3])
+            global_tensor = torch.randn(global_shape)
+            sharded_tensor = distribute_tensor(global_tensor, placements=shard_spec)
+            self.assertEqual(sharded_tensor.to_local().shape, torch.Size([3, 3]))
+
+            mesh_2d = DeviceMesh(
+                self.device_type, torch.arange(self.world_size).reshape(2, 4)
+            )
+
+            with mesh_2d:
+                shard_2d_spec = [Shard(0), Replicate()]
+                tensor_2d = distribute_tensor(global_tensor, placements=shard_2d_spec)
+
+                self.assertEqual(tensor_2d.to_local().shape, torch.Size([3 * 4, 3]))
+
+            sharded_after_2d = distribute_tensor(global_tensor, placements=shard_spec)
+            self.assertEqual(sharded_after_2d.to_local().shape, torch.Size([3, 3]))
+
     @with_comms
     def test_dtensor_2d_mesh(self):
         mesh_tensor = torch.arange(self.world_size).reshape(2, 4)
@@ -395,11 +530,159 @@ class DTensorMeshTest(DTensorTestBase):
             ),
         ]
 
+        from torch.distributed._tensor._utils import compute_local_offset
+
         # loop through all sharding specs and check local shard offsets
         logical_tensor = torch.randn(tensor_shape)
         for shard_spec, expected_shard_offsets in shard_spec_and_offsets:
             dtensor = distribute_tensor(logical_tensor, device_mesh, shard_spec)
-            self.assertEqual(expected_shard_offsets, dtensor._spec.local_offsets)
+            offset = compute_local_offset(
+                dtensor.shape, device_mesh, dtensor.placements
+            )
+            self.assertEqual(expected_shard_offsets, offset)
+
+    @with_comms
+    def test_from_local_sub_mesh(self):
+        mesh = DeviceMesh(self.device_type, [0, 2])
+        local_tensor = torch.ones(3, 4)
+
+        dtensor = DTensor.from_local(local_tensor, mesh, [Shard(0)])
+        self.assertEqual(dtensor.size(), torch.Size([6, 4]))
+
+        self.sub_mesh_assert_equal(
+            mesh.mesh,
+            torch.ones(3, 4),
+            torch.tensor([]),
+            dtensor.to_local(),
+        )
+
+        # test dtensor created in submesh, the operation should only
+        # be applied to the local shard inside the mesh, not the whole
+        # world, so only 0/2 really run the computation
+        dtensor = dtensor + 2
+
+        self.sub_mesh_assert_equal(
+            mesh.mesh,
+            torch.ones(3, 4) + 2,
+            torch.tensor([]),
+            dtensor.to_local(),
+        )
+
+    @with_comms
+    def test_default_value_sub_mesh(self):
+        mesh = DeviceMesh(self.device_type, [0, 2])
+
+        # test scalar return value
+        local_tensor1 = torch.ones(4, 3)
+        local_tensor2 = torch.ones(4, 3)
+        dtensor1 = DTensor.from_local(local_tensor1, mesh, [Shard(0)])
+        dtensor2 = DTensor.from_local(local_tensor2, mesh, [Shard(0)])
+        local_res = dtensor1.equal(dtensor2)  # equal returns local result
+        self.sub_mesh_assert_equal(
+            mesh.mesh,
+            True,
+            True,
+            local_res,
+        )
+
+        # test 0-d tensor return value
+        local_tensor = torch.ones(4, 3)
+        dtensor = DTensor.from_local(local_tensor, mesh, [Shard(0)]).sum()
+        self.sub_mesh_assert_equal(
+            mesh.mesh,
+            torch.tensor(12.0),
+            torch.tensor(0.0),
+            dtensor.to_local(),
+        )
+
+        # test List[torch.Tensor] return value
+        local_tensor = torch.ones(3, 4)
+        dtensor = DTensor.from_local(local_tensor, mesh, [Shard(0)])
+        dtensor_list = dtensor.split([2, 2], dim=1)
+        self.sub_mesh_assert_equal(
+            mesh.mesh,
+            [torch.ones(3, 2)] * 2,
+            [torch.tensor([])] * 2,
+            [dt.to_local() for dt in dtensor_list],
+        )
+
+    @with_comms
+    def test_redistribute_sub_mesh(self):
+        mesh = DeviceMesh(self.device_type, [0, 2])
+
+        # test redistribute on a submesh
+        local_tensor1 = torch.ones(4, 3)
+        sharded_dtensor = DTensor.from_local(local_tensor1, mesh, [Shard(0)])
+        replicated_dtensor = sharded_dtensor.redistribute(placements=[Replicate()])
+        self.sub_mesh_assert_equal(
+            mesh.mesh, torch.ones(8, 3), torch.tensor([]), replicated_dtensor.to_local()
+        )
+        sharded_again = replicated_dtensor.redistribute(placements=[Shard(0)])
+        self.sub_mesh_assert_equal(
+            mesh.mesh, torch.ones(4, 3), torch.tensor([]), sharded_again.to_local()
+        )
+
+
+class TestDTensorPlacementTypes(DTensorTestBase):
+    @property
+    def world_size(self):
+        return 8
+
+    def _create_tensor(self, size):
+        # Keep everything deterministic.
+        torch.manual_seed(0)
+        tensor = torch.rand(size)
+        if self.device_type == "cuda":
+            return tensor.cuda()
+        else:
+            return tensor
+
+    @with_comms
+    def test_split_tensor(self) -> None:
+        mesh = DeviceMesh(self.device_type, torch.arange(self.world_size))
+        shard_placement = Shard(0)
+
+        for size in range(8):
+            tensor = self._create_tensor(size)
+            if size == 0:
+                with self.assertRaisesRegex(
+                    Exception,
+                    "Tensor size along dim0 is 0. There is nothing to be sharded.",
+                ):
+                    _, _ = shard_placement._split_tensor(
+                        tensor,
+                        mesh.size(),
+                        with_padding=True,
+                        contiguous=True,
+                    )
+            else:
+                splitted_tensor_list, pad_sizes = shard_placement._split_tensor(
+                    tensor,
+                    mesh.size(),
+                    with_padding=True,
+                    contiguous=True,
+                )
+                expected_pad_sizes = [
+                    0 if idx < size else 1
+                    for idx, _ in enumerate(range(dist.get_world_size()))
+                ]
+                assert_array_equal(expected_pad_sizes, pad_sizes)
+
+                unpadded_list = [
+                    shard_placement._unpad_tensor(tensor, pad_sizes[i])
+                    if pad_sizes[i] > 0
+                    else tensor
+                    for i, tensor in enumerate(splitted_tensor_list)
+                ]
+                expected_is_tensor_empty = [
+                    False if idx < size else True
+                    for idx, _ in enumerate(range(dist.get_world_size()))
+                ]
+                is_tensor_empty = [
+                    False if unpadded_tensor.numel() > 0 else True
+                    for unpadded_tensor in unpadded_list
+                ]
+                assert_array_equal(expected_is_tensor_empty, is_tensor_empty)
 
 
 if __name__ == "__main__":
