@@ -3,7 +3,7 @@
 import io
 import itertools
 import sys
-from contextlib import suppress
+from contextlib import nullcontext
 from copy import deepcopy
 from functools import partial
 from typing import Any, Dict
@@ -83,7 +83,13 @@ STATE_DICT_MAPPING = {
 
 
 class Model(Module):
-    def __init__(self, wrap_fsdp, register_buffers=False, ignore_inner=False):
+    def __init__(
+        self,
+        wrap_fsdp,
+        register_buffers=False,
+        ignore_inner=False,
+        mixed_precision=False,
+    ):
         super().__init__()
         self.inner = Linear(*INNER_SHAPE)
         if register_buffers:
@@ -93,7 +99,15 @@ class Model(Module):
             )
         if wrap_fsdp:
             self.inner = FSDP(
-                self.inner, ignored_modules=([self.inner] if ignore_inner else [])
+                self.inner,
+                ignored_modules=([self.inner] if ignore_inner else []),
+                mixed_precision=MixedPrecision(
+                    param_dtype=torch.float16,
+                    reduce_dtype=torch.float16,
+                    buffer_dtype=torch.float16,
+                )
+                if mixed_precision
+                else None,
             )
         self.outer = Linear(*OUTER_SHAPE)
         if register_buffers:
@@ -134,7 +148,7 @@ class TestFSDPStateDict(FSDPTest):
                 self._state_compare(model, model_new, assert_fn)
                 if check_buffers:
                     has_buffers = any(
-                        [len(list(m.buffers())) for m in (model, model_new)]
+                        len(list(m.buffers())) for m in (model, model_new)
                     )
                     if has_buffers:
                         self._state_compare(
@@ -501,6 +515,7 @@ class TestFSDPStateDict(FSDPTest):
             use_orig_params and state_dict_type not in _UNFLATTENED_STATE_DICT_IMPLS
         ):
             return  # not supported
+        device = torch.device(self.rank)
         for model_call in [
             partial(
                 self._get_non_fsdp_root_module,
@@ -519,6 +534,14 @@ class TestFSDPStateDict(FSDPTest):
             ),
         ]:
             model = model_call()
+            if fp16:
+                model.half()
+            # Run a forward/backward to compute gradients to test the case
+            # where there are gradients populated
+            inp = torch.randn((3, 10), device=device)
+            if fp16:
+                inp = inp.half()
+            model(inp).sum().backward()
 
             ctx = self._get_state_dict_mgr(
                 model, state_dict_type, state_dict_rank0_and_offload
@@ -548,6 +571,12 @@ class TestFSDPStateDict(FSDPTest):
                 model_new = model_new.cuda()
             if fp16:
                 model_new.half()
+            # Run a forward/backward to compute gradients to test the case
+            # where there are gradients populated
+            inp = torch.randn((3, 10), device=device)
+            if fp16:
+                inp = inp.half()
+            model_new(inp).sum().backward()
 
             # zero the model to ensure parameters are different.
             _zero_model(model_new, zero_buffers=True)
@@ -770,15 +799,20 @@ class TestFSDPStateDict(FSDPTest):
     @skip_if_lt_x_gpu(2)
     @parametrize("state_dict_type", _SUPPORTED_STATE_DICT_IMPLS)
     def test_state_dict_save_load_flow(self, state_dict_type):
-        for move_to_cpu in [True, False]:
-            with self.subTest(move_to_cpu=move_to_cpu):
-                fsdp_params = self._dist_train(
-                    wrap_fsdp=True,
-                    state_dict_type=state_dict_type,
-                    move_to_cpu=move_to_cpu,
-                )
-                ddp_params = self._dist_train(wrap_fsdp=False)
-                self.assertEqual(ddp_params, fsdp_params)
+        self.run_subtests(
+            {"move_to_cpu": [True, False]},
+            self._test_state_dict_save_load_flow,
+            state_dict_type=state_dict_type,
+        )
+
+    def _test_state_dict_save_load_flow(self, state_dict_type, move_to_cpu):
+        fsdp_params = self._dist_train(
+            wrap_fsdp=True,
+            state_dict_type=state_dict_type,
+            move_to_cpu=move_to_cpu,
+        )
+        ddp_params = self._dist_train(wrap_fsdp=False)
+        self.assertEqual(ddp_params, fsdp_params)
 
     @skip_if_lt_x_gpu(2)
     @parametrize("state_dict_type", _SUPPORTED_STATE_DICT_IMPLS)
@@ -866,7 +900,6 @@ class TestFSDPStateDict(FSDPTest):
         if state_dict_rank0_and_offload:
             fsdp_state_dict = self._broadcast_state_dict(model, fsdp_state_dict)
 
-        # if self.rank == 0:
         blank_local_model.load_state_dict(fsdp_state_dict, strict=True)
         local_params = list(blank_local_model.parameters())
         for fsdp_param, local_param in zip(fsdp_params, local_params):
@@ -880,7 +913,7 @@ class TestFSDPStateDict(FSDPTest):
 
         def _create_module(wrap_fsdp=True):
             LINEAR_SKIP = "linear_skip"
-            ctx = enable_wrap(wrapper_cls=FSDP) if wrap_fsdp else suppress()
+            ctx = enable_wrap(wrapper_cls=FSDP) if wrap_fsdp else nullcontext()
             with ctx:
                 module = SkipModel(double_nest=double_nest)
                 # Full name of linear_skip param tensors in SkipModel, as would be
@@ -918,14 +951,14 @@ class TestFSDPStateDict(FSDPTest):
         # Check that it can be loaded into FSDP.
         new_fsdp, _ = _create_module()
         _zero_model(new_fsdp)
-        for (p1, p2) in zip(fsdp.parameters(), new_fsdp.parameters()):
+        for p1, p2 in zip(fsdp.parameters(), new_fsdp.parameters()):
             self.assertNotEqual(p1, p2)
         with FSDP.state_dict_type(new_fsdp, STATE_DICT_MAPPING[state_dict_type]):
             if state_dict_type != "local_state_dict":
                 # FlatParameter has not supported deepcopy yet.
                 state_dict = deepcopy(state_dict)
             new_fsdp.load_state_dict(state_dict, strict=True)
-        for (p1, p2) in zip(fsdp.parameters(), new_fsdp.parameters()):
+        for p1, p2 in zip(fsdp.parameters(), new_fsdp.parameters()):
             self.assertEqual(p1, p2)
 
         # Test that the checkpoint can be loaded into a local model.
@@ -935,7 +968,7 @@ class TestFSDPStateDict(FSDPTest):
                 param.zero_()
 
         with fsdp.summon_full_params(fsdp):
-            for (p1, p2) in zip(fsdp.parameters(), local.parameters()):
+            for p1, p2 in zip(fsdp.parameters(), local.parameters()):
                 self.assertNotEqual(p1, p2)
 
         if state_dict_type == "local_state_dict":
@@ -944,7 +977,7 @@ class TestFSDPStateDict(FSDPTest):
         with fsdp.summon_full_params(fsdp):
             if self.rank == 0:
                 local.load_state_dict(state_dict, strict=True)
-                for (p1, p2) in zip(fsdp.parameters(), local.parameters()):
+                for p1, p2 in zip(fsdp.parameters(), local.parameters()):
                     self.assertEqual(p1, p2)
 
     @skip_if_lt_x_gpu(2)
@@ -960,13 +993,17 @@ class TestFSDPStateDict(FSDPTest):
     @parametrize("state_dict_type", _UNFLATTENED_STATE_DICT_IMPLS)
     @parametrize("prefix", [True, False])
     @parametrize("ignore_inner", [True, False])
+    @parametrize("mixed_precision", [True, False])
     def test_state_dict_with_ignored_modules(
-        self, state_dict_type, prefix, ignore_inner
+        self, state_dict_type, prefix, ignore_inner, mixed_precision
     ):
         # Initialize an FSDP-wrapped model with an ignored module that includes
         # both parameters and a buffer
         model = Model(
-            wrap_fsdp=True, register_buffers=True, ignore_inner=ignore_inner
+            wrap_fsdp=True,
+            register_buffers=True,
+            ignore_inner=ignore_inner,
+            mixed_precision=mixed_precision,
         ).cuda()
         ignored_modules = [model.outer]
         ignored_tensor_to_tensor_name = {
@@ -985,7 +1022,23 @@ class TestFSDPStateDict(FSDPTest):
             model.inner.buffer: "inner.buffer",
             model.outer.buffer: "outer.buffer",
         }
-        fsdp_model = FSDP(model, ignored_modules=ignored_modules)
+        # expect fp16 model.inner.buffer with mixed_precisions
+        # expect fp32 sd.inner.buffer after restoring to original precision
+        # so skip AssertEqual
+        if mixed_precision and not ignore_inner:
+            buffer_to_buffer_name.pop(model.inner.buffer)
+
+        fsdp_model = FSDP(
+            model,
+            ignored_modules=ignored_modules,
+            mixed_precision=MixedPrecision(
+                param_dtype=torch.float16,
+                reduce_dtype=torch.float16,
+                buffer_dtype=torch.float16,
+            )
+            if mixed_precision
+            else None,
+        )
         prefix_str = "foo." if prefix else ""
         with FSDP.state_dict_type(fsdp_model, STATE_DICT_MAPPING[state_dict_type]):
             sd1 = _gather_state_dict(fsdp_model.state_dict(prefix=prefix_str))
@@ -1003,6 +1056,11 @@ class TestFSDPStateDict(FSDPTest):
                 sd1[prefixed_tensor_name].data_ptr(),
                 f"{prefixed_tensor_name}",
             )
+        # should not apply mixed_precision to ignored buffers
+        for buffer_name in buffer_to_buffer_name.values():
+            prefixed_buffer_name = f"{prefix_str}{buffer_name}"
+            self.assertTrue(prefixed_buffer_name in sd1)
+            self.assertEqual(sd1[prefixed_buffer_name].dtype, torch.float32)
         # Check that the state dict can be loaded into a non-wrapped version of
         # the model
         nonwrapped_model = Model(wrap_fsdp=False, register_buffers=True).cuda()
@@ -1084,6 +1142,67 @@ class TestFSDPStateDict(FSDPTest):
                 else:
                     self.assertEqual(v, state_dict[k])
 
+    @skip_if_lt_x_gpu(2)
+    def test_shared_module_and_shared_parameter(self):
+        class TestDummyModel(torch.nn.Module):
+            def __init__(self):
+                super().__init__()
+                torch.manual_seed(0)
+                self.net1 = nn.Sequential(nn.Linear(8, 16), nn.ReLU())
+                self.net2 = nn.Sequential(nn.Linear(16, 16), nn.ReLU())
+                self.net3 = self.net2
+                self.random_parameter = nn.Parameter(torch.Tensor(10))
+                self.shared_parameter = self.random_parameter
+
+            def forward(self, x):
+                return self.net3(self.net2(self.net1(x)))
+
+            def get_input(self):
+                return torch.rand(8, 8, device="cuda")
+
+        model = FSDP(TestDummyModel().cuda())
+        with FSDP.state_dict_type(model, StateDictType.FULL_STATE_DICT):
+            state_dict = model.state_dict()
+            self.assertEqual(
+                state_dict["random_parameter"], state_dict["shared_parameter"]
+            )
+            self.assertEqual(state_dict["net2.0.bias"], state_dict["net3.0.bias"])
+            self.assertEqual(state_dict["net2.0.weight"], state_dict["net3.0.weight"])
+
+    @skip_if_lt_x_gpu(2)
+    def test_sharded_load_multi_backend_pg(self):
+        auto_wrap_policy = ModuleWrapPolicy(
+            {TransformerEncoderLayer, TransformerDecoderLayer}
+        )
+        fsdp_kwargs = {
+            "auto_wrap_policy": auto_wrap_policy,
+            "use_orig_params": True,
+        }
+        for load_cpu in [True, False]:
+            with self.subTest(load_cpu=load_cpu):
+                pg = dist.new_group(backend="cpu:gloo,cuda:nccl")
+                fsdp_model = TransformerWithSharedParams.init(
+                    pg,
+                    FSDPInitMode.RECURSIVE,
+                    CUDAInitMode.CUDA_BEFORE,
+                    fsdp_kwargs,
+                )
+                FSDP.set_state_dict_type(fsdp_model, StateDictType.SHARDED_STATE_DICT)
+                sharded = fsdp_model.state_dict()
+                param_copy = [t.clone().detach_() for t in fsdp_model.parameters()]
+                with torch.no_grad():
+                    for p in fsdp_model.parameters():
+                        p.zero_()
+
+                if load_cpu:
+                    # Offload to CPU to simulate CPU state_dict load
+                    for k, v in sharded.items():
+                        sharded[k] = v.cpu()
+
+                fsdp_model.load_state_dict(sharded)
+                for p1, p2 in zip(param_copy, fsdp_model.parameters()):
+                    self.assertEqual(p1, p2, f"not equal: {p1.sum()} vs {p2.sum()}")
+
 
 instantiate_parametrized_tests(TestFSDPStateDict)
 
