@@ -236,7 +236,7 @@ class TestUnshardParams(TestUnshardParamsBase):
         NOTE: This method depends on FSDP internals.
         """
         model = FSDP(nn.Linear(1, 1, bias=False, device=self.device))
-        flat_param = model._handles[0].flat_param
+        flat_param = model._handle.flat_param
         self.assertEqual(1, flat_param.numel())
         # Write a known value to the *sharded* `FlatParameter`
         with torch.no_grad():
@@ -258,6 +258,11 @@ class TestUnshardParams(TestUnshardParamsBase):
         """
         Tests that unsharding parameters respects the expected reshard behavior
         between forward and backward as well as after backward.
+
+        For mixed precision, we should *not* respect the reshard behavior
+        because the ``summon_full_params()`` forces full precision, which uses
+        a different all-gather tensor than the one already in memory and will
+        not persist any modifications correctly.
         """
         self.run_subtests(
             {
@@ -288,8 +293,8 @@ class TestUnshardParams(TestUnshardParamsBase):
             ),
             **fsdp_kwargs,
         )
-        outer_flat_param = model._handles[0].flat_param
-        inner_flat_param = model.module[0]._handles[0].flat_param
+        outer_flat_param = model._handle.flat_param
+        inner_flat_param = model.module[0]._handle.flat_param
         # NOTE: This assumes uniform sharding with padding across ranks.
         expected_outer_flat_param_unsharded_numel = (
             outer_flat_param.numel() * self.world_size
@@ -321,6 +326,12 @@ class TestUnshardParams(TestUnshardParamsBase):
             offload_to_cpu=offload_to_cpu,
         ):
             pass
+        if mixed_precision is not None:
+            # After forcing full precision, we must invalidate the existing
+            # unsharded low-precision flat parameter since it will not persist
+            # changes from the `summon_full_params()` context, so we cannot
+            # respect the reshard behavior
+            expected_outer_flat_param_unsharded_numel = 0
         self.assertEqual(
             expected_outer_flat_param_unsharded_numel,
             _get_unsharded_storage_size(outer_flat_param),
@@ -372,11 +383,23 @@ class TestUnshardParams(TestUnshardParamsBase):
         # Hard code the numel values based on the model
         unsharded_inner_numel = 5 * 5
         unsharded_outer_numel = 5 * 3
+        if use_orig_params:
+            # Account for unsharded padding: since each `FlatParameter` only
+            # has one original parameter, we only need to pad for divisibility
+            # by world size and not address alignment
+            if unsharded_inner_numel % self.world_size:
+                unsharded_inner_numel += self.world_size - (
+                    unsharded_inner_numel % self.world_size
+                )
+            if unsharded_outer_numel % self.world_size:
+                unsharded_outer_numel += self.world_size - (
+                    unsharded_outer_numel % self.world_size
+                )
         # Round up the sharded numel to account for padding
         sharded_inner_numel = int(math.ceil(unsharded_inner_numel / self.world_size))
         sharded_outer_numel = int(math.ceil(unsharded_outer_numel / self.world_size))
-        inner_flat_param = model.module[0]._handles[0].flat_param
-        outer_flat_param = model._handles[0].flat_param
+        inner_flat_param = model.module[0]._handle.flat_param
+        outer_flat_param = model._handle.flat_param
         self.assertEqual(sharded_inner_numel, inner_flat_param.numel())
         self.assertEqual(sharded_outer_numel, outer_flat_param.numel())
         expected_outer_numel = (
@@ -510,7 +533,7 @@ class TestUnshardParams(TestUnshardParamsBase):
 
         def _get_error_context(is_supported: bool):
             return (
-                contextlib.suppress()
+                contextlib.nullcontext()
                 if is_supported
                 else self.assertRaises(NotImplementedError)
             )  # some configs are not implemented yet
@@ -593,8 +616,8 @@ class TestUnshardParams(TestUnshardParamsBase):
             },
         )
         for fsdp_module in FSDP.fsdp_modules(fsdp_model):
-            for handle in fsdp_module._handles:
-                assert handle.flat_param.grad is None
+            if fsdp_module._handle:
+                assert fsdp_module._handle.flat_param.grad is None
         with FSDP.summon_full_params(fsdp_model, with_grads=True):
             for param in fsdp_model.parameters():
                 self.assertTrue(param.grad is None)
