@@ -11,6 +11,7 @@ import copy
 from torch import nn
 
 from torch.distributed.pipeline.sync.skip import Namespace, skippable, stash
+from torch.testing._internal.common_utils import run_tests
 
 
 def test_namespace_difference():
@@ -45,3 +46,7 @@ def test_skippable_repr():
 ))
 """.strip()
     )
+
+
+if __name__ == "__main__":
+    run_tests()
