@@ -1,14 +1,36 @@
 # Owner(s): ["module: dynamo"]
+import re
+from textwrap import dedent
 from unittest.mock import patch
 
 import torch
 
 import torch._dynamo
 import torch._dynamo.test_case
-from torch._dynamo.testing import CompileCounter, rand_strided
+import torch.fx.traceback as fx_traceback
+import torch.utils._pytree as pytree
+from torch._dynamo.testing import CompileCounter, expectedFailureDynamic, rand_strided
+from torch._functorch.aot_autograd import _aot_export_function, create_functional_call
+from torch.profiler import profile
 from torch.testing._internal.common_utils import compare_equal_outs_and_grads
 
 
+def maybe_dupe_op(x):
+    y = x + 1
+    z = x + 2
+    if x.numel() < 5:
+        return y, y
+    else:
+        return y, z
+
+
+aten = torch.ops.aten
+lib = torch.library.Library("custom", "DEF")
+lib.define("maybe_dupe_op(Tensor a) -> (Tensor, Tensor)")
+lib.impl("maybe_dupe_op", maybe_dupe_op, "CPU")
+lib.impl("maybe_dupe_op", maybe_dupe_op, "Meta")
+
+
 class AotAutogradFallbackTests(torch._dynamo.test_case.TestCase):
     def test_LSTM(self):
         # https://github.com/pytorch/torchdynamo/issues/1147
@@ -138,7 +160,7 @@ class AotAutogradFallbackTests(torch._dynamo.test_case.TestCase):
         real = mod(rx)
 
         # Run it in export
-        graph, _ = torch._dynamo.export(mod, rx)
+        graph, _ = torch._dynamo.export(mod)(rx)
 
         # Run exported graph with AOT
         self.assertTrue(torch._dynamo.testing.same(real, graph(rx)))
@@ -169,7 +191,7 @@ class AotAutogradFallbackTests(torch._dynamo.test_case.TestCase):
         real = mod(x, y)
 
         # Run it in export
-        graph, _ = torch._dynamo.export(mod, x, y)
+        graph, _ = torch._dynamo.export(mod)(x, y)
 
         # Assert equal
         self.assertTrue(torch._dynamo.testing.same(real, graph(x, y)))
@@ -280,9 +302,9 @@ class AotAutogradFallbackTests(torch._dynamo.test_case.TestCase):
         fxy = torch._dynamo.optimize(cc, guard_fail_fn=guard_fail_fn)(F())
         compare_equal_outs_and_grads(self, F(), fxy, (x, y))
         compare_equal_outs_and_grads(self, F(), fxy, (x, z))
-        self.assertEqual(
+        self.assertExpectedInline(
             failure_reason,
-            "tensor 'y' requires_grad mismatch. expected requires_grad=1",
+            """tensor 'L['y']' requires_grad mismatch. expected requires_grad=1""",
         )
 
         # Reset failure reason
@@ -361,8 +383,8 @@ class AotAutogradFallbackTests(torch._dynamo.test_case.TestCase):
     def test_arg_dupe_via_dynamo_recompiles(self):
         class F(torch.nn.Module):
             def forward(self, x, y):
-                x = x.t_()
-                y = y.t_()
+                x = x.trunc_()
+                y = y.trunc_()
                 return (x + y,)
 
         x = torch.randn(3, 3, requires_grad=True)
@@ -399,37 +421,7 @@ class AotAutogradFallbackTests(torch._dynamo.test_case.TestCase):
         fxx(x3, x3)
         fxx(x4, y4)
         self.assertEqual(cc.frame_count, 2)
-        self.assertEqual(failure_reason, "x is y")
-
-    @patch("torch._functorch.config.debug_assert", True)
-    def test_arg_metadata_mutation_on_input_causes_recompile(self):
-        class F(torch.nn.Module):
-            def forward(self, a):
-                a.t_()
-                return (a * 2,)
-
-        a = torch.randn(3, 3, requires_grad=True).clone()
-
-        failure_reason = None
-
-        def guard_fail_fn(failure):
-            nonlocal failure_reason
-            failure_reason = failure[0]
-
-        self.assertTrue(failure_reason is None)
-
-        cc = torch._dynamo.testing.CompileCounterWithBackend("aot_eager")
-
-        f = torch._dynamo.optimize(cc, guard_fail_fn=guard_fail_fn)(F())
-        f(a)
-        f(a)
-        self.assertEqual(cc.frame_count, 2)
-        self.assertEqual(
-            failure_reason,
-            "tensor 'a' strides mismatch at index 0. expected 3, actual 1",
-        )
-
-        torch._dynamo.reset()
+        self.assertExpectedInline(failure_reason, """L['x'] is L['y']""")
 
     @patch("torch._functorch.config.debug_assert", True)
     def test_arg_dupe_via_dynamo_recompiles_many_args_param_non_tensor_arg(self):
@@ -439,10 +431,10 @@ class AotAutogradFallbackTests(torch._dynamo.test_case.TestCase):
                 self.mean = torch.nn.Parameter(torch.randn(3, 3))
 
             def forward(self, a, b, c, d, e, f):
-                a.t_()
-                b.t_()
-                c.t_()
-                d.t_()
+                a.trunc_()
+                b.trunc_()
+                c.trunc_()
+                d.trunc_()
                 return (a + b + c + d + self.mean) * e * f
 
         a = torch.randn(3, 3, requires_grad=True)
@@ -464,7 +456,7 @@ class AotAutogradFallbackTests(torch._dynamo.test_case.TestCase):
         f(a1, a1, a1, a1, 2, 2)
         f(a2, b2, b2, b2, 2, 2)
         self.assertEqual(cc.frame_count, 2)
-        self.assertEqual(failure_reason, "a is b")
+        self.assertExpectedInline(failure_reason, """L['a'] is L['b']""")
 
         torch._dynamo.reset()
 
@@ -479,7 +471,7 @@ class AotAutogradFallbackTests(torch._dynamo.test_case.TestCase):
         f(a3, b3, c3, c3, 3, 3)
         f(a4, b4, c4, d4, 3, 3)
         self.assertEqual(cc.frame_count, 2)
-        self.assertEqual(failure_reason, "c is d")
+        self.assertExpectedInline(failure_reason, """L['c'] is L['d']""")
 
     @patch("torch._functorch.config.debug_assert", True)
     def test_arg_dupe_via_dynamo_recompiles_many_with_global(self):
@@ -491,10 +483,10 @@ class AotAutogradFallbackTests(torch._dynamo.test_case.TestCase):
                 self.mean = torch.nn.Parameter(torch.randn(3, 3))
 
             def forward(self, a, b, c, d, e, f):
-                a.t_()
-                b.t_()
-                c.t_()
-                d.t_()
+                a.trunc_()
+                b.trunc_()
+                c.trunc_()
+                d.trunc_()
                 return (a + b + c + d + z + self.mean) * e * f
 
         a = torch.randn(3, 3, requires_grad=True)
@@ -517,7 +509,7 @@ class AotAutogradFallbackTests(torch._dynamo.test_case.TestCase):
         f(a1, a1, a1, a1, 2, 2)
         f(a2, b2, b2, b2, 2, 2)
         self.assertEqual(cc.frame_count, 2)
-        self.assertEqual(failure_reason, "a is b")
+        self.assertExpectedInline(failure_reason, """L['a'] is L['b']""")
 
     @patch("torch._functorch.config.debug_assert", True)
     def test_arg_dupe_via_dynamo_recompiles_many_args_param_non_tensor_arg_list(self):
@@ -527,10 +519,10 @@ class AotAutogradFallbackTests(torch._dynamo.test_case.TestCase):
                 self.mean = torch.nn.Parameter(torch.randn(3, 3))
 
             def forward(self, e, f, a, b, c, d):
-                a.t_()
-                b.t_()
-                c.t_()
-                d.t_()
+                a.trunc_()
+                b.trunc_()
+                c.trunc_()
+                d.trunc_()
                 return (a + b + c + d + self.mean) * e[0] * f[0]
 
         a = torch.randn(3, 3, requires_grad=True)
@@ -552,7 +544,7 @@ class AotAutogradFallbackTests(torch._dynamo.test_case.TestCase):
         f([3, 2, 1], [4, 5, 6], a1, a1, a1, a1)
         f([3, 2, 1], [4, 5, 6], a2, b2, b2, b2)
         self.assertEqual(cc.frame_count, 2)
-        self.assertEqual(failure_reason, "a is b")
+        self.assertExpectedInline(failure_reason, """L['a'] is L['b']""")
 
         torch._dynamo.reset()
 
@@ -576,10 +568,10 @@ class AotAutogradFallbackTests(torch._dynamo.test_case.TestCase):
                 self.mean = torch.nn.Parameter(torch.randn(3, 3))
 
             def forward(self, a, b, c, d):
-                a.t_()
-                b.t_()
-                c.t_()
-                d.t_()
+                a.trunc_()
+                b.trunc_()
+                c.trunc_()
+                d.trunc_()
                 return a + b + c + d + self.mean
 
         a = torch.randn(3, 3, requires_grad=True)
@@ -601,7 +593,7 @@ class AotAutogradFallbackTests(torch._dynamo.test_case.TestCase):
         f(a1, a1, a1, a1)
         f(a2, b2, b2, b2)
         self.assertEqual(cc.frame_count, 2)
-        self.assertEqual(failure_reason, "a is b")
+        self.assertExpectedInline(failure_reason, """L['a'] is L['b']""")
 
         torch._dynamo.reset()
 
@@ -616,16 +608,16 @@ class AotAutogradFallbackTests(torch._dynamo.test_case.TestCase):
         f(a3, b3, c3, c3)
         f(a4, b4, c4, d4)
         self.assertEqual(cc.frame_count, 2)
-        self.assertEqual(failure_reason, "c is d")
+        self.assertExpectedInline(failure_reason, """L['c'] is L['d']""")
 
     @patch("torch._functorch.config.debug_assert", True)
     def test_arg_dupe_via_dynamo_recompiles_many_args(self):
         class F(torch.nn.Module):
             def forward(self, a, b, c, d):
-                a.t_()
-                b.t_()
-                c.t_()
-                d.t_()
+                a.trunc_()
+                b.trunc_()
+                c.trunc_()
+                d.trunc_()
                 return (a + b + c + d,)
 
         a = torch.randn(3, 3, requires_grad=True)
@@ -647,7 +639,7 @@ class AotAutogradFallbackTests(torch._dynamo.test_case.TestCase):
         f(a1, a1, a1, a1)
         f(a2, b2, b2, b2)
         self.assertEqual(cc.frame_count, 2)
-        self.assertEqual(failure_reason, "a is b")
+        self.assertExpectedInline(failure_reason, """L['a'] is L['b']""")
 
         torch._dynamo.reset()
 
@@ -662,24 +654,12 @@ class AotAutogradFallbackTests(torch._dynamo.test_case.TestCase):
         f(a3, b3, c3, c3)
         f(a4, b4, c4, d4)
         self.assertEqual(cc.frame_count, 2)
-        self.assertEqual(failure_reason, "c is d")
+        self.assertExpectedInline(failure_reason, """L['c'] is L['d']""")
 
+    @expectedFailureDynamic  # https://github.com/pytorch/pytorch/issues/103539
+    @torch._dynamo.config.patch(automatic_dynamic_shapes=False)
     @patch("torch._functorch.config.debug_assert", True)
     def test_multiple_aot_autograd_calls_dupe_args(self):
-        def maybe_dupe_op(x):
-            y = x + 1
-            z = x + 2
-            if x.numel() < 5:
-                return y, y
-            else:
-                return y, z
-
-        aten = torch.ops.aten
-        lib = torch.library.Library("custom", "DEF")
-        lib.define("maybe_dupe_op(Tensor a) -> (Tensor, Tensor)")
-        lib.impl("maybe_dupe_op", maybe_dupe_op, "CPU")
-        lib.impl("maybe_dupe_op", maybe_dupe_op, "Meta")
-
         # this is just dealing with the fact that
         # aot_module_simplified expects submods to always return tuples/lists
         class WrapperModule(torch.nn.Module):
@@ -729,6 +709,208 @@ class AotAutogradFallbackTests(torch._dynamo.test_case.TestCase):
         f(torch.ones(4))
         f(torch.ones(6))
 
+    def test_nn_parameter_construction(self):
+        # https://github.com/pytorch/pytorch/issues/99569
+        def fn(x):
+            y = x.sin()
+            z = torch.nn.Parameter(torch.ones(1))
+            return y + z
+
+        x = torch.rand((4, 4))
+
+        opt_fn = torch._dynamo.optimize("aot_eager")(fn)
+        self.assertTrue(torch._dynamo.testing.same(fn(x), opt_fn(x)))
+
+    def test_aot_sequence_nr(self):
+        class Model(torch.nn.Module):
+            def __init__(self):
+                super().__init__()
+                self.conv1 = torch.nn.Conv2d(
+                    in_channels=16,
+                    out_channels=16,
+                    kernel_size=(1, 1),
+                    stride=1,
+                    padding="same",
+                    bias=True,
+                )
+                self.bn1 = torch.nn.BatchNorm2d(num_features=16)
+                self.relu1 = torch.nn.ReLU()
+                self.fc1 = torch.nn.Linear(in_features=1638400, out_features=1)
+                self.loss_fn = torch.nn.L1Loss()
+
+            def forward(self, x, target):
+                y = x
+                x = self.conv1(x)
+                x = self.bn1(x)
+                x = self.relu1(x)
+                x = x + y
+                x = torch.flatten(x)
+                x = self.fc1(x)
+                output = self.loss_fn(x, target)
+
+                return (output,)
+
+        mod = Model()
+        mod.train()
+        x = torch.rand(100, 16, 32, 32, requires_grad=True)
+        target = torch.rand(1)
+
+        # Use dynamo export to get the fx graph module
+        g_mod, _ = torch._dynamo.export(mod, x, target)
+
+        def _prepare_model_args():
+            named_parameters = dict(g_mod.named_parameters(remove_duplicate=False))
+            named_buffers = dict(g_mod.named_buffers(remove_duplicate=False))
+            params_and_buffers = {
+                **dict(named_parameters),
+                **dict(named_buffers),
+            }
+            params_and_buffers_flat, params_spec = pytree.tree_flatten(
+                params_and_buffers
+            )
+            params_len = len(params_and_buffers_flat)
+            functional_call = create_functional_call(g_mod, params_spec, params_len)
+            return params_and_buffers_flat, functional_call
+
+        full_args, fn_to_trace = _prepare_model_args()
+        param_and_buf_len = len(full_args)
+        full_args.extend([x, target])
+
+        # aot_export requires a graph mod input of fwd graph
+        # returns the full fwd/bwd graph in graph mod format
+        with torch.enable_grad(), fx_traceback.preserve_node_meta():
+            fx_g, _, _, _ = _aot_export_function(
+                fn_to_trace,
+                full_args,
+                decompositions=None,
+                num_params_buffers=param_and_buf_len,
+                no_tangents=True,
+            )
+
+        # Walk all the nodes in fx graph.
+        # Write the resulting ops to a table
+        min_seq_nr = -1
+        seq_table = "SeqNr|OrigAten|SrcFn\n"
+        for node in fx_g.graph.nodes:
+            if "call_" in node.op and "getitem" not in str(node.target):
+                seq_nr = node.meta.get("seq_nr", -1)
+                if seq_nr < 0:
+                    continue
+                if min_seq_nr < 0:
+                    min_seq_nr = seq_nr
+                mod_name = node.meta.get("source_fn", "")
+                orig_aten = node.meta.get("original_aten", "")
+                if isinstance(mod_name, tuple):
+                    mod_name = mod_name[0]
+                # Make all seq_nr relative so it starts at 0
+                seq_nr = seq_nr - min_seq_nr
+                seq_table = seq_table + f"{seq_nr}|{orig_aten}|{mod_name}\n"
+
+        self.maxDiff = None
+        self.assertExpectedInline(
+            seq_table,
+            dedent(
+                """\
+SeqNr|OrigAten|SrcFn
+0|aten.convolution.default|l__self___conv1
+0|aten.add.Tensor|l__self___bn1
+1|aten._native_batch_norm_legit_functional.default|l__self___bn1
+2|aten.relu.default|l__self___relu1
+3|aten.add.Tensor|add
+4|aten.view.default|flatten
+5|aten.view.default|l__self___fc1
+6|aten.t.default|l__self___fc1
+7|aten.addmm.default|l__self___fc1
+8|aten.view.default|l__self___fc1
+9|aten.sub.Tensor|l__self___loss_fn
+10|aten.abs.default|l__self___loss_fn
+11|aten.mean.default|l__self___loss_fn
+11|aten.ones_like.default|
+11|aten.expand.default|
+11|aten.div.Scalar|
+10|aten.sgn.default|
+10|aten.mul.Tensor|
+8|aten.view.default|
+7|aten.t.default|
+7|aten.mm.default|
+7|aten.t.default|
+7|aten.mm.default|
+7|aten.t.default|
+7|aten.sum.dim_IntList|
+7|aten.view.default|
+6|aten.t.default|
+5|aten.view.default|
+4|aten.view.default|
+2|aten.threshold_backward.default|
+1|aten.native_batch_norm_backward.default|
+0|aten.convolution_backward.default|
+11|aten.add.Tensor|
+"""
+            ),
+        )
+
+    def test_eager_sequence_nr(self):
+        class Model(torch.nn.Module):
+            def __init__(self):
+                super().__init__()
+                self.conv1 = torch.nn.Conv2d(
+                    in_channels=16,
+                    out_channels=16,
+                    kernel_size=(1, 1),
+                    stride=1,
+                    padding="same",
+                    bias=True,
+                )
+                self.bn1 = torch.nn.BatchNorm2d(num_features=16)
+                self.relu1 = torch.nn.ReLU()
+                self.fc1 = torch.nn.Linear(in_features=1638400, out_features=1)
+                self.loss_fn = torch.nn.L1Loss()
+
+            def forward(self, x, target):
+                y = x
+                x = self.conv1(x)
+                x = self.bn1(x)
+                x = self.relu1(x)
+                x = x + y
+                x = torch.flatten(x)
+                x = self.fc1(x)
+                output = self.loss_fn(x, target)
+
+                return (output,)
+
+        def grad_with_create_graph(mod, x, target):
+            y = mod(x, target)
+            # Set create_graph=True to ensure that the sequence_nr
+            # for backward ops continues to count down.
+            (gx,) = torch.autograd.grad(
+                y[0], x, create_graph=True, grad_outputs=grad_output
+            )
+            return gx
+
+        x = torch.rand(100, 16, 32, 32, requires_grad=True)
+        target = torch.rand(1)
+        mod = Model()
+        args = [mod, x, target]
+        grad_output = torch.tensor(1.0, requires_grad=True)
+        compiled_f1 = torch.compile(backend="aot_eager")(grad_with_create_graph)
+        model_instance = compiled_f1
+        with profile(
+            activities=[torch.profiler.ProfilerActivity.CPU],
+            record_shapes=True,
+        ) as kineto_prof:
+            res = model_instance(*args)
+        bwd_set = set()
+        prof_str = "SeqNr|Thread|FwdThread|Name\n"
+        for event in kineto_prof.events():
+            if event.sequence_nr >= 0:
+                prof_str = (
+                    prof_str + f"{event.sequence_nr}|{event.thread}"
+                    f"|{event.fwd_thread}|{event.name}|\n"
+                )
+                if re.search(r"Backward[01]", event.name):
+                    bwd_set.add(event.sequence_nr)
+        self.assertTrue(len(bwd_set), 13)
+
 
 if __name__ == "__main__":
     from torch._dynamo.test_case import run_tests
