@@ -12,14 +12,7 @@ from torch.fx.experimental.rewriter import RewritingTracer
 from torch.fx import GraphModule
 from torch.fx.passes.shape_prop import ShapeProp
 from torch.testing._internal.common_utils import TestCase
-
-
-try:
-    import sympy
-    HAS_SYMPY = True
-except ImportError:
-    HAS_SYMPY = False
-skipIfNoSympy = unittest.skipIf(not HAS_SYMPY, "no sympy")
+import sympy
 
 
 try:
@@ -813,7 +806,6 @@ class TypeCheckerTest(TestCase):
                 if n.op == 'output':
                     assert is_consistent(n.type, TensorType(b.size()))
 
-    @skipIfNoSympy
     @skipIfNoTorchVision
     def test_resnet50(self):
         gm_run = symbolic_trace(resnet50())
@@ -860,7 +852,6 @@ class TypeCheckerTest(TestCase):
             batch_sizes.add(n.type.__args__[0])
         assert (len(batch_sizes) == 1)
 
-    @skipIfNoSympy
     def test_type_check_batch_norm_symbolic(self):
         class BasicBlock(torch.nn.Module):
 
@@ -892,7 +883,6 @@ class TypeCheckerTest(TestCase):
         for n in graph.nodes:
             assert n.type == next(my_types)
 
-    @skipIfNoSympy
     def test_symbolic_add_with_broadcast(self):
         class M(torch.nn.Module):
             def forward(self, x: TensorType((1, 2, 3, Dyn)), y: TensorType((2, 3, 4))):
@@ -921,7 +911,6 @@ class TypeCheckerTest(TestCase):
         for n in symbolic_traced.graph.nodes:
             assert n.type == next(expected_iter)
 
-    @skipIfNoSympy
     def test_symbolic_add_with_broadcast_2(self):
         class M(torch.nn.Module):
             def forward(self, x: TensorType((1, 2)), y: TensorType((Dyn, 2))):
@@ -943,7 +932,6 @@ class TypeCheckerTest(TestCase):
         for n in symbolic_traced.graph.nodes:
             assert n.type == next(expected_iter)
 
-    @skipIfNoSympy
     def test_type_check_conv2D_types(self):
         class BasicBlock(torch.nn.Module):
             def __init__(self, inplanes, planes, stride=1):
@@ -971,7 +959,6 @@ class TypeCheckerTest(TestCase):
                 assert isinstance(n.type.__args__[2], sympy.floor)
                 assert isinstance(n.type.__args__[3], sympy.floor)
 
-    @skipIfNoSympy
     def test_type_check_symbolic_inferenceconv2D_maxpool2d_flatten(self):
 
         class BasicBlock(torch.nn.Module):
@@ -1003,12 +990,12 @@ class TypeCheckerTest(TestCase):
 
         for n in traced.graph.nodes:
             if n.target == 'conv1':
-                assert n.type == TensorType((4, 6, sympy.floor((sympy.symbols('~0') - 4)),
-                                             sympy.floor((sympy.symbols('~1') - 4))))
+                assert n.type == TensorType((4, 6, sympy.floor(sympy.symbols('~0') - 4),
+                                             sympy.floor(sympy.symbols('~1') - 4)))
 
             elif n.target == 'conv2':
-                assert n.type == TensorType((4, 16, sympy.floor((sympy.symbols('~4') - 4)),
-                                             sympy.floor((sympy.symbols('~5') - 4))))
+                assert n.type == TensorType((4, 16, sympy.floor(sympy.symbols('~4') - 4),
+                                             sympy.floor(sympy.symbols('~5') - 4)))
 
 if __name__ == '__main__':
     unittest.main()
