@@ -4,7 +4,6 @@ import io
 import os
 import pathlib
 import sys
-import unittest
 from typing import NamedTuple, Optional
 
 import torch
@@ -16,7 +15,6 @@ pytorch_test_dir = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
 sys.path.append(pytorch_test_dir)
 from torch.testing._internal.jit_utils import JitTestCase, clear_class_registry
 
-ENABLE_FLATBUFFER = os.environ.get("ENABLE_FLATBUFFER", "0") == "1"
 
 if __name__ == "__main__":
     raise RuntimeError(
@@ -433,6 +431,24 @@ class TestSaveLoad(JitTestCase):
         output = m_loaded(FooTuple(a=5))
         self.assertEqual(output, torch.tensor(3))
 
+    def test_save_namedtuple_input_only_forwardref(self):
+        """
+        Even if a NamedTuple is only used as an input argument, saving and
+        loading should work correctly.
+        """
+        global FooTuple  # see [local resolution in python]
+
+        class FooTuple(NamedTuple):
+            a: 'int'
+
+        class MyModule(torch.nn.Module):
+            def forward(self, x: FooTuple) -> torch.Tensor:
+                return torch.tensor(3)
+
+        m_loaded = self.getExportImportCopy(torch.jit.script(MyModule()))
+        output = m_loaded(FooTuple(a=5))
+        self.assertEqual(output, torch.tensor(3))
+
     def test_save_namedtuple_output_only(self):
         """
         Even if a NamedTuple is only used as an output argument, saving and
@@ -549,6 +565,50 @@ class TestSaveLoad(JitTestCase):
         self.assertTrue(m_buffers["buffer"].is_meta)
         self.assertTrue(m_loaded_buffers["buffer"].is_meta)
 
+    def test_save_load_meta_tensors_to_device(self):
+        """
+        Check that when loading a module with meta tensors to device, the meta tensors
+        stay on meta, but non-meta tensors are set to the indicated device.
+        """
+
+        class Foo(torch.nn.Module):
+            def __init__(self):
+                super().__init__()
+                self.foo = torch.nn.Linear(2, 3, device="meta")
+                self.bar = torch.nn.Linear(3, 4)
+
+            def forward(self, x):
+                x = self.foo(x)
+                x = self.bar(x)
+                return x
+
+        m = Foo()
+
+        m_loaded = self.getExportImportCopy(torch.jit.script(m), map_location="cpu")
+        # Check submodules.
+        self.assertEqual(
+            len(list(m.named_modules())), len(list(m_loaded.named_modules()))
+        )
+        self.assertEqual(
+            {name for name, _ in m.named_modules()},
+            {name for name, _ in m_loaded.named_modules()},
+        )
+        # Check parameters.
+        m_params = dict(m.named_parameters())
+        m_loaded_params = dict(m_loaded.named_parameters())
+        self.assertEqual(len(m_params), len(m_loaded_params))
+        self.assertEqual(m_params, m_loaded_params)
+        # Check params and buffers that are/are not meta tensors
+        self.assertTrue(m_params["foo.weight"].is_meta)
+        self.assertTrue(m_loaded_params["foo.weight"].is_meta)
+        self.assertTrue(m_params["foo.bias"].is_meta)
+        self.assertTrue(m_loaded_params["foo.bias"].is_meta)
+        self.assertTrue(m_params["bar.weight"].is_cpu)
+        self.assertTrue(m_loaded_params["bar.weight"].is_cpu)
+        self.assertTrue(m_params["bar.bias"].is_cpu)
+        self.assertTrue(m_loaded_params["bar.bias"].is_cpu)
+
+
     def test_save_load_with_saved_traced_inputs(self):
         """
         Check that saving and loading with traced inputs works as expected
@@ -577,7 +637,7 @@ class TestSaveLoad(JitTestCase):
         # Validate that with no input specified the traced inputs are stored
         traced_module = torch.jit.trace(module, input_tensor)
         traced_inputs = list(traced_module.graph.inputs())
-        self.assertEquals(traced_module._c._retrieve_traced_inputs()['forward'], [input_tensor])
+        self.assertEqual(traced_module._c._retrieve_traced_inputs()['forward'], [input_tensor])
         with TemporaryFileName() as fname:
             path = pathlib.Path(fname)
             traced_module.save(path)
@@ -593,7 +653,7 @@ class TestSaveLoad(JitTestCase):
         # Validate that inputs aren't saved when requested not to
         traced_module = torch.jit.trace(module, input_tensor, _store_inputs=False)
         traced_inputs = list(traced_module.graph.inputs())
-        self.assertEquals(len(traced_module._c._retrieve_traced_inputs()), 0)
+        self.assertEqual(len(traced_module._c._retrieve_traced_inputs()), 0)
 
         with TemporaryFileName() as fname:
             path = pathlib.Path(fname)
@@ -649,6 +709,36 @@ class TestSaveLoad(JitTestCase):
         traced_inputs, loaded_inputs = get_loaded_inputs(input4)
         self.assertEqual(traced_inputs[1].type(), loaded_inputs[1].type())
 
+    def test_save_load_large_string_attribute(self):
+        """
+        Check if the model with string > 4GB can be loaded.
+        """
+        import psutil
+        if psutil.virtual_memory().available < 60 * 1024 * 1024 * 1024:
+            # Profiled the test execution, and got this number to be safe to run the test
+            self.skipTest("Doesn't have enough memory to run test_save_load_large_string_attribute")
+
+        class Model(torch.nn.Module):
+            def __init__(self):
+                super().__init__()
+                self.x = "x" * (2 ** 32 + 1)
+
+            def forward(self, i) -> int:
+                return len(self.x) + i.numel()
+
+        inp = torch.ones(0)
+        ts = torch.jit.script(Model())
+        ts_output = ts(inp)
+
+        b = io.BytesIO(ts.save_to_buffer())
+        del ts
+
+        loaded_ts = torch.jit.load(b)
+        del b
+        loaded_output = loaded_ts(inp)
+        self.assertEqual(ts_output, loaded_output)
+
+
 def script_module_to_buffer(script_module):
     module_buffer = io.BytesIO(
         script_module._save_to_buffer_for_lite_interpreter(_use_flatbuffer=True)
@@ -657,9 +747,6 @@ def script_module_to_buffer(script_module):
     return module_buffer
 
 
-@unittest.skipIf(
-    not ENABLE_FLATBUFFER, "Need to enable flatbuffer to run the below tests"
-)
 class TestSaveLoadFlatbuffer(JitTestCase):
     def test_different_modules(self):
         """
@@ -1033,15 +1120,14 @@ class TestSaveLoadFlatbuffer(JitTestCase):
         torch.jit.save_jit_module_to_flatbuffer(
             first_script_module, first_saved_module)
         first_saved_module.seek(0)
-        expected = {
-            'bytecode_version': 4,
-            'operator_version': 4,
-            'function_names': {'__torch__.___torch_mangle_0.Foo.forward'},
-            'type_names': set(),
-            'opname_to_num_args': {'aten::linear': 3}}
-        self.assertEqual(
-            torch.jit._serialization.get_flatbuffer_module_info(first_saved_module),
-            expected)
+        ff_info = torch.jit._serialization.get_flatbuffer_module_info(first_saved_module)
+        self.assertEqual(ff_info['bytecode_version'], 9)
+        self.assertEqual(ff_info['operator_version'], 1)
+        self.assertEqual(ff_info['type_names'], set())
+        self.assertEqual(ff_info['opname_to_num_args'], {'aten::linear': 3})
+
+        self.assertEqual(len(ff_info['function_names']), 1)
+        self.assertTrue(next(iter(ff_info['function_names'])).endswith('forward'))
 
 
     def test_save_load_params_buffers_submodules(self):
@@ -1105,12 +1191,11 @@ class TestSaveLoadFlatbuffer(JitTestCase):
         module = Module()
         script_module = torch.jit.script(module)
 
-        script_module_io = io.BytesIO()
-        extra_files = {"abc.json": "[1,2,3]"}
-        script_module._save_for_lite_interpreter(script_module_io, _extra_files=extra_files, _use_flatbuffer=True)
-        script_module_io.seek(0)
+        extra_files = {"abc.json": b"[1,2,3]"}
+        script_module_io = script_module._save_to_buffer_for_lite_interpreter(
+            _extra_files=extra_files, _use_flatbuffer=True)
 
         re_extra_files = {}
-        torch._C._get_model_extra_files_from_buffer(script_module_io, _extra_files=re_extra_files)
+        torch._C._get_model_extra_files_from_buffer(script_module_io, re_extra_files)
 
         self.assertEqual(extra_files, re_extra_files)
