@@ -35,12 +35,13 @@ if __name__ == '__main__':
                        "\tpython test/test_jit.py TESTNAME\n\n"
                        "instead.")
 
+@skipIfTorchDynamo("Not a suitable test for TorchDynamo")
 class TestTracer(JitTestCase):
     @unittest.skipIf(not RUN_CUDA, "requires CUDA")
     def test_large_nbr_kernel_args(self):
         class Recurrence(nn.Module):
             def __init__(self, seq_len):
-                super(Recurrence, self).__init__()
+                super().__init__()
                 self.seq_len = seq_len
 
             def forward(self, input):
@@ -87,9 +88,6 @@ class TestTracer(JitTestCase):
 
     def test_trace_checking_with_global_name(self):
         class MyClass(torch.nn.Module):
-            def __init__(self):
-                super(MyClass, self).__init__()
-
             def forward(self, xs: List[Tensor]):
                 y = torch.cat(xs, dim=0)
                 return y
@@ -105,7 +103,7 @@ class TestTracer(JitTestCase):
     def test_trace_aliased_parameter(self):
         class M(nn.Module):
             def __init__(self, x):
-                super(M, self).__init__()
+                super().__init__()
                 self.x = nn.Parameter(x)
 
             def forward(self, y):
@@ -140,6 +138,16 @@ class TestTracer(JitTestCase):
         jit_f2 = torch.jit.trace(f2, x)
         assert f2(x) == jit_f2(x)  # fails
 
+    def test_trace_out_operator_with_two_output(self):
+        example_input = torch.rand(2, 8)
+        out_1, out_2 = torch.cummax(example_input, 1)
+
+        def run_cummax(example_input, out_1, out_2):
+            output_1, output_2 = torch.cummax(example_input, 1, out=(out_1, out_2))
+            return output_1, output_2
+
+        trace_model = torch.jit.trace(run_cummax, (example_input, out_1, out_2))
+
     def test_trace_namedtuple(self):
         Point = namedtuple('point', ['x', 'y'])
 
@@ -622,9 +630,6 @@ class TestTracer(JitTestCase):
     def test_input_dict_remembers_keys(self):
         """Check that the trace remembers which keys were in a dict input"""
         class TestModule(torch.nn.Module):
-            def __init__(self):
-                super(TestModule, self).__init__()
-
             def forward(self, dict_input):
                 return dict_input['x']
 
@@ -649,9 +654,6 @@ class TestTracer(JitTestCase):
     def test_input_dict_insertion_order(self):
         """Check that dictionary access doesn't care about insertion order"""
         class TestModule(torch.nn.Module):
-            def __init__(self):
-                super(TestModule, self).__init__()
-
             def forward(self, dict_input):
                 return dict_input['x'], dict_input['y']
         input_x_then_y = {}
@@ -671,9 +673,6 @@ class TestTracer(JitTestCase):
 
     def test_input_dict_recursive(self):
         class TestModule(torch.nn.Module):
-            def __init__(self):
-                super(TestModule, self).__init__()
-
             def forward(self, dict_input):
                 return dict_input['x'][1]
 
@@ -833,7 +832,7 @@ class TestTracer(JitTestCase):
     def test_shared_param(self):
         class MyModule(torch.nn.Module):
             def __init__(self):
-                super(MyModule, self).__init__()
+                super().__init__()
                 self.b = self.a = nn.Parameter(torch.randn(2, 2))
 
             def forward(self, x):
@@ -852,9 +851,6 @@ class TestTracer(JitTestCase):
             self.skipTest("Skip the test since c2 ops are not registered.")
 
         class MyModel(torch.nn.Module):
-            def __init__(self):
-                super(MyModel, self).__init__()
-
             def forward(self, scores, bbox_deltas, im_info, anchors):
                 a, b = torch.ops._caffe2.GenerateProposals(
                     (scores), (bbox_deltas), (im_info), (anchors),
@@ -955,7 +951,7 @@ class TestTracer(JitTestCase):
     def test_traced_module_cuda(self):
         class Model(nn.Module):
             def __init__(self, num_features, num_layers):
-                super(Model, self).__init__()
+                super().__init__()
                 self.num_layers = num_layers
                 layers = [[nn.Linear(num_features, num_features), nn.Sigmoid()]
                           for _ in range(num_layers)]
@@ -1001,7 +997,7 @@ class TestTracer(JitTestCase):
         traced_model.to('cpu')
         cpu_out = traced_model(x.float())
         self.assertEqual(cpu_out, cuda_out)
-        traced_model.double()
+        traced_model.to(torch.get_default_dtype())
 
         # state_dict + load_state_dict
         state = {k: v.clone() for k, v in traced_model.state_dict().items()}
@@ -1124,14 +1120,6 @@ class TestTracer(JitTestCase):
         # With `check_trace=True` it will run with `@torch.no_grad()` and break assert.
         torch.jit.trace(foo, (x, w), check_trace=False)
 
-    def test_trace_detach_onnx_erase(self):
-        class Mod(torch.nn.Module):
-            def forward(self, x, w):
-                return torch.matmul(x, w).detach()
-
-        torch.onnx.export_to_pretty_string(
-            Mod(), (torch.rand(3, 4), torch.rand(4, 5)))
-
     def test_trace_slice_full_dim(self):
         def foo(x):
             return x[0:5, 0] + 1.0
@@ -1143,7 +1131,7 @@ class TestTracer(JitTestCase):
     def test_trace_dict_input(self):
         class Bar(torch.nn.Module):
             def __init__(self):
-                super(Bar, self).__init__()
+                super().__init__()
                 self.foo = Foo()
 
             def forward(self, a, b):
@@ -1275,7 +1263,7 @@ class TestTracer(JitTestCase):
     def test_trace_save_load_copy(self):
         class Test(torch.nn.Module):
             def __init__(self):
-                super(Test, self).__init__()
+                super().__init__()
                 self.conv = torch.nn.Conv2d(3, 3, 3)
 
             def forward(self, x):
@@ -1293,7 +1281,7 @@ class TestTracer(JitTestCase):
     def test_trace_export_fns(self):
         class Foo(torch.nn.Module):
             def __init__(self):
-                super(Foo, self).__init__()
+                super().__init__()
                 self.a = 3
 
             @torch.jit.export
@@ -1324,7 +1312,7 @@ class TestTracer(JitTestCase):
     def test_trace_export_fns_recursive(self):
         class Foo(torch.nn.Module):
             def __init__(self):
-                super(Foo, self).__init__()
+                super().__init__()
                 self.a = 3
 
             @torch.jit.export
@@ -1341,7 +1329,7 @@ class TestTracer(JitTestCase):
 
         class Wrapper(torch.nn.Module):
             def __init__(self):
-                super(Wrapper, self).__init__()
+                super().__init__()
                 self.foo = Foo()
 
             def forward(self, x):
@@ -1362,9 +1350,6 @@ class TestTracer(JitTestCase):
 
         # Note that Bar's forward can only be traced, but not scripted
         class Bar(nn.Module):
-            def __init__(self):
-                super().__init__()
-
             @torch.jit.export
             def addTwo(self, x):
                 return x + 2
@@ -1377,7 +1362,7 @@ class TestTracer(JitTestCase):
         # being traced.
         class WrapperExports(torch.nn.Module):
             def __init__(self):
-                super(WrapperExports, self).__init__()
+                super().__init__()
                 self.bar = Bar()
 
             @torch.jit.export
@@ -1411,7 +1396,7 @@ class TestTracer(JitTestCase):
 
         class Wrapper(torch.nn.Module):
             def __init__(self):
-                super(Wrapper, self).__init__()
+                super().__init__()
                 self.tm = TracedModule()
 
             def forward(self, x):
@@ -1463,7 +1448,7 @@ class TestTracer(JitTestCase):
     def test_interpolate_trace(self):
         class test(nn.Module):
             def __init__(self):
-                super(test, self).__init__()
+                super().__init__()
                 self.conv = nn.Conv2d(1, 32, kernel_size=3, padding=1)
 
             def forward(self, x):
@@ -1523,7 +1508,7 @@ class TestTracer(JitTestCase):
 
         class TracedModule(torch.nn.Module):
             def __init__(self):
-                super(TracedModule, self).__init__()
+                super().__init__()
                 self.param = torch.nn.Parameter(torch.rand(4, 5))
 
             def forward(self, x):
@@ -1541,7 +1526,7 @@ class TestTracer(JitTestCase):
     def test_call_traced_module_from_traced_module(self):
         class TracedModule1(torch.nn.Module):
             def __init__(self):
-                super(TracedModule1, self).__init__()
+                super().__init__()
                 self.param = torch.nn.Parameter(torch.rand(5, 7))
 
             def forward(self, x):
@@ -1549,7 +1534,7 @@ class TestTracer(JitTestCase):
 
         class TracedModule(torch.nn.Module):
             def __init__(self):
-                super(TracedModule, self).__init__()
+                super().__init__()
                 self.param = torch.nn.Parameter(torch.rand(4, 5))
                 self.mod = torch.jit.trace(TracedModule1(), torch.rand(3, 5))
 
@@ -1705,7 +1690,7 @@ class TestTracer(JitTestCase):
     def test_trace_modulelist(self):
         class MySubmod(torch.nn.Module):
             def __init__(self):
-                super(MySubmod, self).__init__()
+                super().__init__()
                 self.relu = torch.nn.ReLU()
 
             def forward(self, x):
@@ -1713,7 +1698,7 @@ class TestTracer(JitTestCase):
 
         class MyMod(torch.nn.Module):
             def __init__(self):
-                super(MyMod, self).__init__()
+                super().__init__()
                 self.ml = torch.nn.ModuleList([
                     MySubmod(),
                     MySubmod()
@@ -1729,7 +1714,7 @@ class TestTracer(JitTestCase):
     def test_trace_fork_join_and_module(self):
         class MySubmod(torch.nn.Module):
             def __init__(self):
-                super(MySubmod, self).__init__()
+                super().__init__()
                 self.relu = torch.nn.ReLU()
 
             def forward(self, x):
@@ -1737,7 +1722,7 @@ class TestTracer(JitTestCase):
 
         class Mod(torch.nn.Module):
             def __init__(self):
-                super(Mod, self).__init__()
+                super().__init__()
                 self.ml = torch.nn.ModuleList([
                     MySubmod() for i in range(2)
                 ])
@@ -1759,7 +1744,7 @@ class TestTracer(JitTestCase):
     def test_trace_invert_module_hierarchy(self):
         class MySubmod(torch.nn.Module):
             def __init__(self):
-                super(MySubmod, self).__init__()
+                super().__init__()
                 self.relu = torch.nn.ReLU()
 
             def forward(self, x):
@@ -1771,7 +1756,7 @@ class TestTracer(JitTestCase):
 
         class Mod(torch.nn.Module):
             def __init__(self):
-                super(Mod, self).__init__()
+                super().__init__()
                 self.sm = MySubmod()
                 self.fm = MyFunctionalMod()
 
@@ -1798,9 +1783,6 @@ class TestTracer(JitTestCase):
     @skipIfTorchDynamo("Not a suitable test for TorchDynamo")
     def test_tracing_hooks(self):
         class Net(nn.Module):
-            def __init__(self):
-                super(Net, self).__init__()
-
             def forward(self, x):
                 return x + x
 
@@ -1859,9 +1841,6 @@ class TestTracer(JitTestCase):
 
     def test_tracing_backward_hook_error(self):
         class Net(nn.Module):
-            def __init__(self):
-                super(Net, self).__init__()
-
             def forward(self, x):
                 return x + x
 
@@ -1877,7 +1856,7 @@ class TestTracer(JitTestCase):
     def test_tracing_multiple_methods(self):
         class Net(nn.Module):
             def __init__(self):
-                super(Net, self).__init__()
+                super().__init__()
                 self.conv = nn.Conv2d(1, 1, 3)
 
             def forward(self, x):
@@ -1938,7 +1917,7 @@ class TestTracer(JitTestCase):
     def test_trace_with_conditional_property(self):
         class Net(nn.Module):
             def __init__(self, attr=None):
-                super(Net, self).__init__()
+                super().__init__()
                 if attr is not None:
                     self._attr = attr
                 self.attr_name = '_attr'
@@ -1972,7 +1951,7 @@ class TestTracer(JitTestCase):
     def test_trace_module_argument_names_captured(self):
         class TestModule(nn.Module):
             def __init__(self):
-                super(TestModule, self).__init__()
+                super().__init__()
                 self.conv = nn.Conv2d(1, 1, 3)
 
             def forward(self, first_arg: torch.Tensor, second_arg: torch.Tensor):
@@ -1991,7 +1970,80 @@ class TestTracer(JitTestCase):
         FileCheck().check("first_arg").check_next("second_arg") \
             .run(str(traced_module.graph))
 
+    def test_trace_checking_with_deprecated_name(self):
+        class MyClass(torch.nn.Module):
+            def __init__(self):
+                super(MyClass, self).__init__()
 
+            def forward(self, x, y, **deprecated_arguments):
+                if len(deprecated_arguments) > 0:
+                    raise RuntimeError(f"Got unexpected arguments: {deprecated_arguments}")
+                return x + y
+
+        model = MyClass()
+        m2 = torch.jit.trace(model, (torch.ones(1), torch.ones(1)))
+        m3 = torch.jit.trace(model, example_kwarg_inputs={'x': torch.ones(1), "y": torch.ones(1)}, strict=False)
+
+    def test_trace_with_tuple_tensor(self):
+        class MyClass(torch.nn.Module):
+            def __init__(self):
+                super(MyClass, self).__init__()
+
+            def forward(self, x, y):
+                return x + y[0] + y[1]
+
+        model = MyClass()
+        traced_model = torch.jit.trace(model, (torch.ones(1), (torch.ones(1), torch.ones(1))))
+        input_dict = {"x": torch.tensor([2, 3]), "y": (torch.tensor([5, 6]), torch.tensor([7, 8]))}
+        self.assertEqual(model(**input_dict), traced_model(**input_dict))
+        traced_model = torch.jit.trace(model, example_kwarg_inputs={
+                                       'x': torch.ones(1), "y": (torch.ones(1), torch.ones(1))})
+        self.assertEqual(model(**input_dict), traced_model(**input_dict))
+
+    def test_trace_no_duplicated_lifted_input_output(self):
+        class Normalize(nn.Module):
+            def __init__(self):
+                super().__init__()
+                self.norm = nn.GroupNorm(num_groups=32, num_channels=32)
+
+            def forward(self, x, y):
+                if y is None:
+                    y = x
+                else:
+                    y = self.norm(y)
+                y = y * 2
+                return y
+
+        class G(nn.Module):
+            def __init__(self):
+                super().__init__()
+                self.norm = Normalize()
+
+            def forward(self, x):
+                A = self.norm(x, None)
+                B = F.relu(A)
+                return A, B
+
+        class Net(nn.Module):
+            def __init__(self):
+                super().__init__()
+                self.g = G()
+                self.norm_1 = Normalize()
+
+            def forward(self, x):
+                hs = self.g(x)
+                A, B = hs
+                h = self.norm_1(B, A)
+                return h
+
+        net = Net()
+        net = net.eval()
+        x = torch.randn(1, 32, 16, 16)
+        traced = torch.jit.trace(net, x)
+        FileCheck().check_not("prim::TupleUnpack").run(str(traced.graph))
+
+
+@skipIfTorchDynamo("Not a suitable test for TorchDynamo")
 class TestMixTracingScripting(JitTestCase):
     def test_trace_script(self):
         @torch.jit.script
@@ -2113,7 +2165,7 @@ class TestMixTracingScripting(JitTestCase):
 
         class AnotherScriptMod(torch.jit.ScriptModule):
             def __init__(self):
-                super(AnotherScriptMod, self).__init__()
+                super().__init__()
                 self.param = torch.nn.Parameter(torch.rand(1, 2, 3))
 
             @torch.jit.script_method
@@ -2122,7 +2174,7 @@ class TestMixTracingScripting(JitTestCase):
 
         class SomeScriptMod(torch.jit.ScriptModule):
             def __init__(self):
-                super(SomeScriptMod, self).__init__()
+                super().__init__()
                 self.asm = AnotherScriptMod()
 
             @torch.jit.script_method
@@ -2135,7 +2187,7 @@ class TestMixTracingScripting(JitTestCase):
 
         class TraceMe(torch.nn.Module):
             def __init__(self):
-                super(TraceMe, self).__init__()
+                super().__init__()
                 self.ssm = SomeScriptMod()
 
             def forward(self, x):
@@ -2162,7 +2214,7 @@ class TestMixTracingScripting(JitTestCase):
     def test_trace_parameter(self):
         class Param(nn.Module):
             def __init__(self):
-                super(Param, self).__init__()
+                super().__init__()
                 self.register_parameter("bias", nn.Parameter(torch.empty(4, 4)))
 
             def forward(self, x):
@@ -2170,7 +2222,7 @@ class TestMixTracingScripting(JitTestCase):
 
         class M3(torch.jit.ScriptModule):
             def __init__(self, model):
-                super(M3, self).__init__()
+                super().__init__()
                 self.traced = torch.jit.trace(model, (torch.rand(3, 3)))
 
             @torch.jit.script_method
@@ -2179,7 +2231,7 @@ class TestMixTracingScripting(JitTestCase):
 
         class M2(nn.Module):
             def __init__(self, model):
-                super(M2, self).__init__()
+                super().__init__()
                 self.module = M3(model)
 
             def forward(self, x):
@@ -2187,7 +2239,7 @@ class TestMixTracingScripting(JitTestCase):
 
         class M1(torch.jit.ScriptModule):
             def __init__(self, model):
-                super(M1, self).__init__()
+                super().__init__()
                 self.traced = torch.jit.trace(M2(model), (torch.rand(3, 3)))
 
             @torch.jit.script_method
@@ -2207,7 +2259,7 @@ class TestMixTracingScripting(JitTestCase):
 
         class TracedModule(torch.nn.Module):
             def __init__(self):
-                super(TracedModule, self).__init__()
+                super().__init__()
                 self.param = torch.nn.Parameter(torch.rand(4, 5))
 
             def forward(self, x):
@@ -2220,7 +2272,7 @@ class TestMixTracingScripting(JitTestCase):
     def test_call_script_module_from_traced_module(self):
         class ScriptMod(torch.jit.ScriptModule):
             def __init__(self):
-                super(ScriptMod, self).__init__()
+                super().__init__()
                 self.param_foo = torch.nn.Parameter(torch.rand(5, 7))
 
             @torch.jit.script_method
@@ -2229,7 +2281,7 @@ class TestMixTracingScripting(JitTestCase):
 
         class TracedModule(torch.nn.Module):
             def __init__(self):
-                super(TracedModule, self).__init__()
+                super().__init__()
                 self.param = torch.nn.Parameter(torch.rand(4, 5))
                 self.mod = ScriptMod()
 
@@ -2255,9 +2307,6 @@ class TestMixTracingScripting(JitTestCase):
     def test_call_traced_mod_from_script_fn(self):
         with self.assertRaisesRegex(RuntimeError, "Cannot call a ScriptModule that is not a submodule of the caller"):
             class TracedModule(torch.nn.Module):
-                def __init__(self):
-                    super(TracedModule, self).__init__()
-
                 def forward(self, x):
                     return torch.mm(x, torch.zeros(4, 3))
 
@@ -2275,7 +2324,7 @@ class TestMixTracingScripting(JitTestCase):
 
         class ScriptMod(torch.jit.ScriptModule):
             def __init__(self):
-                super(ScriptMod, self).__init__()
+                super().__init__()
                 self.param = torch.nn.Parameter(torch.rand(4, 3))
 
             @torch.jit.script_method
@@ -2289,7 +2338,7 @@ class TestMixTracingScripting(JitTestCase):
     def test_call_tracing_mod_from_script_module(self):
         class TracedMod(torch.nn.Module):
             def __init__(self):
-                super(TracedMod, self).__init__()
+                super().__init__()
                 self.param = torch.nn.Parameter(torch.rand(3, 5))
 
             def forward(self, x):
@@ -2297,7 +2346,7 @@ class TestMixTracingScripting(JitTestCase):
 
         class ScriptMod(torch.jit.ScriptModule):
             def __init__(self):
-                super(ScriptMod, self).__init__()
+                super().__init__()
                 self.param = torch.nn.Parameter(torch.rand(4, 3))
                 self.tm = torch.jit.trace(TracedMod(), torch.rand(3, 3))
 
@@ -2310,15 +2359,12 @@ class TestMixTracingScripting(JitTestCase):
 
     def test_script_inline_trace_multiple_args(self):
         class M(torch.nn.Module):
-            def __init__(self):
-                super(M, self).__init__()
-
             def forward(self, input, input2):
                 return input + input2
 
         class M2(torch.jit.ScriptModule):
             def __init__(self):
-                super(M2, self).__init__()
+                super().__init__()
                 self.m = torch.jit.trace(M(), (torch.zeros(4, 3), torch.zeros(4, 3)))
 
             @torch.jit.script_method
@@ -2332,19 +2378,19 @@ class TestMixTracingScripting(JitTestCase):
     def test_trace_dict_mix_script(self):
         class testB(torch.nn.Module):
             def __init__(self):
-                super(testB, self).__init__()
+                super().__init__()
                 self.linear = torch.nn.Linear(2, 2)
 
             def forward(self, feature_map: Dict[str, List[Tensor]]) -> Tensor:
                 output = []
-                for i, j in feature_map.items():
+                for j in feature_map.values():
                     output.append(self.linear(j[0]))
 
                 return torch.stack(output)
 
         class testA(torch.nn.Module):
             def __init__(self):
-                super(testA, self).__init__()
+                super().__init__()
                 self.b = torch.jit.script(testB())
 
             def forward(self, input_map: Dict[str, List[Tensor]]) -> Tensor:
@@ -2365,9 +2411,6 @@ class TestMixTracingScripting(JitTestCase):
         The dictionary can should be able to contain other containers (like a tuple) recursively.
         """
         class ReturnsDict(torch.nn.Module):
-            def __init__(self):
-                super().__init__()
-
             def forward(
                 self, id_score_list: Dict[str, Tuple[torch.Tensor, torch.Tensor, torch.Tensor]]
             ) -> Dict[str, Tuple[torch.Tensor, torch.Tensor, torch.Tensor]]:
@@ -2381,9 +2424,6 @@ class TestMixTracingScripting(JitTestCase):
                 return result
 
         class ChecksDict(torch.nn.Module):
-            def __init__(self):
-                super().__init__()
-
             def forward(self, input: Dict[str, Tuple[torch.Tensor, torch.Tensor, torch.Tensor]]):
                 v = input["1000"]
                 return v[1] + 1
@@ -2426,9 +2466,6 @@ class TestMixTracingScripting(JitTestCase):
         should work.
         """
         class ReturnsDict(torch.nn.Module):
-            def __init__(self):
-                super().__init__()
-
             def forward(
                 self, k: torch.Tensor, v: torch.Tensor
             ) -> Dict[str, Tuple[torch.Tensor, torch.Tensor]]:
@@ -2440,9 +2477,6 @@ class TestMixTracingScripting(JitTestCase):
                 return result
 
         class ReturnsBadDict(torch.nn.Module):
-            def __init__(self):
-                super().__init__()
-
             def forward(
                 self, k: torch.Tensor, v: torch.Tensor
             ) -> Dict[str, Tuple[torch.Tensor, float]]:
@@ -2481,7 +2515,7 @@ class TestMixTracingScripting(JitTestCase):
 
         class TestModule(nn.Module):
             def __init__(self):
-                super(TestModule, self).__init__()
+                super().__init__()
                 self.conv = nn.Conv2d(1, 1, 3)
 
             def forward(self, first_arg: torch.Tensor, second_arg: torch.Tensor) -> torch.Tensor:
@@ -2549,3 +2583,21 @@ class TestMixTracingScripting(JitTestCase):
         top = TopModule()
         top_example_input = torch.ones(1)
         torch.jit.trace(top, top_example_input)
+
+    def test_jit_trace_callfunction_return_shapes(self):
+        # a torch.jit.script function gets inserted as a CallFunction node
+        @torch.jit.script
+        def inner_fn(x):
+            return torch.cat((x, x))
+
+        def outer_fn(x, y):
+            return inner_fn(x + y).relu()
+
+        x, y = [torch.rand((2, 2), dtype=torch.float) for _ in range(2)]
+        fn_t = torch.jit.trace(outer_fn, (x, y))
+
+        # expect that the CallFunction node return type has shape information on it.
+        FileCheck().check("Float").check("4, 2").check("CallFunction").run(fn_t.graph)
+        for n in fn_t.graph.nodes():
+            if n.kind() == "prim::CallFunction":
+                self.assertTrue(n.output().isCompleteTensor())
