@@ -21,9 +21,9 @@ class TestLiteScriptModule(TestCase):
         upgrader_mobile_cpp_path = pytorch_caffe2_dir / "torch" / "csrc" / "jit" / "mobile" / "upgrader_mobile.cpp"
         with tempfile.TemporaryDirectory() as tmpdirname:
             write_cpp(tmpdirname, sorted_upgrader_list)
-            with open(os.path.join(tmpdirname, 'upgrader_mobile.cpp'), 'r') as file_name:
+            with open(os.path.join(tmpdirname, 'upgrader_mobile.cpp')) as file_name:
                 actual_output = [line.strip() for line in file_name.readlines() if line]
-            with open(str(upgrader_mobile_cpp_path), 'r') as file_name:
+            with open(str(upgrader_mobile_cpp_path)) as file_name:
                 expect_output = [line.strip() for line in file_name.readlines() if line]
             actual_output_filtered = list(filter(lambda token: len(token) != 0, actual_output))
             expect_output_filtered = list(filter(lambda token: len(token) != 0, expect_output))
