@@ -6,7 +6,7 @@ import unittest.mock as mock
 
 from torch.nn import MultiheadAttention
 from torch.testing._internal.common_device_type import instantiate_device_type_tests, dtypes, \
-    onlyCUDA
+    onlyCUDAAndPRIVATEUSE1
 from torch.testing._internal.common_nn import NNTestCase
 from torch.testing._internal.common_utils import run_tests, \
     TEST_NUMPY, TEST_WITH_CROSSREF, \
@@ -118,7 +118,7 @@ class TestMultiheadAttentionNN(NNTestCase):
                                         saved_kv=False, same_embed_dim=False,
                                         average_attn_weights=average_attn_weights):
             for _ in range(100):
-                batch_sz, seq_len = [random.randint(2, 10) for r in range(2)]
+                batch_sz, seq_len = (random.randint(2, 10) for r in range(2))
                 d_head = random.randint(3, 10)
                 nheads = random.randint(2, 5) * 2
                 d_model = d_head * nheads
@@ -152,7 +152,6 @@ class TestMultiheadAttentionNN(NNTestCase):
                 attn_mask_tensor = torch.from_numpy(attn_mask).float()
                 attn_mask_tensor.masked_fill_(attn_mask_tensor == 0, float('-inf'))
                 attn_mask_tensor.masked_fill_(attn_mask_tensor > 0, float('0.0'))
-                attn_mask_tensor = attn_mask_tensor.double()
 
                 decoder_state_tensor = torch.from_numpy(decoder_state).to(torch.get_default_dtype())
                 source_hid_tensor = torch.from_numpy(K).to(torch.get_default_dtype()).transpose(0, 1)
@@ -330,7 +329,7 @@ class TestMultiheadAttentionNN(NNTestCase):
         key = torch.rand(batch_size, src_len, embed_dim)  # [N, S, D]
         value = key  # [N, S, D]
         attn_mask = torch.randint(0, 2, (batch_size, tgt_len, src_len)).float()  # [N, T, S]
-        attn_mask = attn_mask.masked_fill(attn_mask == 0, float('-inf')).masked_fill(attn_mask == 1, float(0.0))
+        attn_mask = attn_mask.masked_fill(attn_mask == 0, float('-inf')).masked_fill(attn_mask == 1, 0.0)
 
         mta_model = torch.nn.MultiheadAttention(embed_dim, num_heads)
 
@@ -588,13 +587,15 @@ class TestMultiheadAttentionNNDeviceType(NNTestCase):
     def test_multihead_self_attn_two_masks_fast_path_mock(self, device):
         """
         Multihead self-attention should take fast path when both attention mask (mask type 0)
-        and key padding mask (mask type 1) are provided at the same time on CPU and CUDA
+        and key padding mask (mask type 1) are provided at the same time on CPU and CUDA and PrivateUse1
         """
-        if device not in ['cpu', 'cuda']:
-            self.skipTest("Fastpath only runs on CPU and CUDA.")
+        device = device.rstrip(':0123456789')
+        if device not in ['cpu', 'cuda', torch._C._get_privateuse1_backend_name()]:
+            self.skipTest("Fastpath only runs on CPU and CUDA and PrivateUse1.")
+
         with torch.autocast(device_type=device, enabled=False):
-            embed_dim = 14
-            num_heads = 7
+            embed_dim = 16
+            num_heads = 8
             batch_size = 8
             src_len = 5
 
@@ -603,7 +604,9 @@ class TestMultiheadAttentionNNDeviceType(NNTestCase):
             attn_mask = torch.randint(0, 2, (src_len, src_len)).bool().to(device)
             key_padding_mask = torch.randint(0, 2, (batch_size, src_len)).bool().to(device)
 
-            with mock.patch('torch._native_multi_head_attention') as fastpath_mock:
+            with mock.patch('torch._native_multi_head_attention', new=mock.MagicMock(
+                    return_value=(torch.Tensor(), torch.Tensor()))
+            ) as fastpath_mock:
                 # Compute attention on the fast path
                 mta_model = torch.nn.MultiheadAttention(embed_dim, num_heads, batch_first=True, device=device).eval()
                 mta_model.training = False
@@ -611,14 +614,14 @@ class TestMultiheadAttentionNNDeviceType(NNTestCase):
                 # If mock was called, fastpath was taken
                 self.assertTrue(fastpath_mock.called)
 
-    @onlyCUDA
+    @onlyCUDAAndPRIVATEUSE1
     @dtypes(torch.half, torch.float, torch.double)
     def test_multihead_attention_dtype(self, device, dtype):
         embed_dim = 128
         num_heads = 8
         sl = 10
         bs = 8
-        model = nn.MultiheadAttention(embed_dim, num_heads).cuda().to(dtype)
+        model = nn.MultiheadAttention(embed_dim, num_heads).to(device).to(dtype)
         q = torch.randn(sl, bs, embed_dim, device=device, dtype=dtype)
         k = torch.randn(sl, bs, embed_dim, device=device, dtype=dtype)
         v = torch.randn(sl, bs, embed_dim, device=device, dtype=dtype)
@@ -626,7 +629,7 @@ class TestMultiheadAttentionNNDeviceType(NNTestCase):
         self.assertEqual(q.size(), out[0].size())
         self.assertEqual(dtype, out[0].dtype)
 
-    @onlyCUDA
+    @onlyCUDAAndPRIVATEUSE1
     @dtypes(torch.half, torch.float, torch.double)
     def test_multihead_attention_dtype_batch_first(self, device, dtype):
         embed_dim = 128
@@ -637,7 +640,7 @@ class TestMultiheadAttentionNNDeviceType(NNTestCase):
         # the native fast path if we call .eval() and enable inference
         # mode. Test both paths.
         for training in (True, False):
-            model = nn.MultiheadAttention(embed_dim, num_heads, batch_first=True).cuda().to(dtype)
+            model = nn.MultiheadAttention(embed_dim, num_heads, batch_first=True).to(device).to(dtype)
             if not training:
                 model = model.eval()
                 cm = torch.no_grad()
