@@ -37,7 +37,7 @@ class TestCustomAutogradFunction(pytorch_test_common.ExportTestCase):
         model = MyModule()
         onnx_test_common.run_model_test(self, model, input_args=(x,))
 
-    def test_register_custom_op(self):
+    def test_register_op(self):
         class MyClip(torch.autograd.Function):
             @staticmethod
             def forward(ctx, input, scalar):
