@@ -7,7 +7,7 @@ import onnx
 import pytorch_test_common
 import torch
 from pytorch_test_common import skipIfUnsupportedMinOpsetVersion
-from torch.onnx import _constants, symbolic_helper
+from torch.onnx import _constants, symbolic_helper, utils
 from torch.onnx._internal import jit_utils
 from torch.testing._internal import common_utils
 
@@ -23,20 +23,24 @@ def expect_tensor(scalar_type, shape=None):
     return verify
 
 
-def g_op(graph: torch.Graph, op_name: str, *args, **kwargs):
+def as_graphcontext(graph: torch.Graph) -> jit_utils.GraphContext:
     return jit_utils.GraphContext(
         graph=graph,
         block=graph.block(),
-        opset=_constants.ONNX_MAX_OPSET,
+        opset=_constants.ONNX_TORCHSCRIPT_EXPORTER_MAX_OPSET,
         original_node=None,  # type: ignore[arg-type]
         params_dict={},
         env={},
-    ).op(op_name, *args, **kwargs)
+    )
+
+
+def g_op(graph: torch.Graph, op_name: str, *args, **kwargs):
+    return as_graphcontext(graph).op(op_name, *args, **kwargs)
 
 
 class TestONNXShapeInference(pytorch_test_common.ExportTestCase):
     def setUp(self):
-        self.opset_version = _constants.ONNX_MAX_OPSET
+        self.opset_version = _constants.ONNX_TORCHSCRIPT_EXPORTER_MAX_OPSET
         symbolic_helper._set_onnx_shape_inference(True)
         symbolic_helper._set_opset_version(self.opset_version)
 
@@ -179,6 +183,16 @@ class TestONNXShapeInference(pytorch_test_common.ExportTestCase):
         slice = g_op(g, "Slice", input, start_input, end, axis, step)
         self.run_test(g, slice.node(), expect_tensor(None, shape=(None, None)))
 
+    def test_slice_with_dynamic_start_index(self):
+        g = self.create_empty_graph()
+        input = self.insert_tensor_constant(g, torch.ones(2, 3, 4, 5))
+        start_input = g.addInput()
+        start_input.setType(start_input.type().with_sizes([2]))
+        end = self.insert_tensor_constant(g, torch.tensor([3, 4]))
+        axis = self.insert_tensor_constant(g, torch.tensor([1, -1]))
+        slice = g_op(g, "Slice", input, start_input, end, axis)
+        self.run_test(g, slice.node(), expect_tensor(None, shape=(2, None, 4, None)))
+
     def test_broadcast_matmul(self):
         g = self.create_empty_graph()
         constant = self.insert_tensor_constant(g, torch.ones(5, 1, 2))
@@ -321,14 +335,45 @@ class TestONNXShapeInference(pytorch_test_common.ExportTestCase):
         gather = g_op(g, "Gather", batch_size, gather_idx, axis_i=0)
         self.run_test(g, gather.node(), expect_tensor("Long", shape=(None,)))
 
+    def test_squeeze_after_dynamic_if(self):
+        from torch.onnx.symbolic_opset11 import squeeze as squeeze11
+
+        g = self.create_empty_graph()
+
+        input = g.addInput()
+        input.setType(input.type().with_dtype(torch.float).with_sizes([1, None, 5]))
+
+        # Type is intentionally not bool to test that
+        # the added "Cast" node doesn't stop shape inference.
+        cond = g.addInput()
+        cond.setType(input.type().with_dtype(torch.int32).with_sizes([1]))
+        if_op, (if_context, else_context), new_node = jit_utils.add_op_with_blocks(
+            as_graphcontext(g), "If", cond, n_blocks=2
+        )
+        block1_output = if_context.op("Add", input, input)
+        block2_output = else_context.op("Identity", input)
+        utils._add_output_to_block(if_context.block, block1_output)
+        utils._add_output_to_block(else_context.block, block2_output)
+        if_output = torch._C._jit_pass_fixup_onnx_controlflow_node(
+            new_node, _constants.ONNX_TORCHSCRIPT_EXPORTER_MAX_OPSET
+        )[0]
+        torch._C._jit_pass_onnx_node_shape_type_inference(
+            new_node, {}, _constants.ONNX_TORCHSCRIPT_EXPORTER_MAX_OPSET
+        )
+
+        # Exporter will add "If" instead of raw "Squeeze" if it does not know
+        # that if the dimension it is squeezing has size 1.
+        squeezed = squeeze11(as_graphcontext(g), if_output, dim=0)
+        assert squeezed.node().kind() == "onnx::Squeeze"
+        self.run_test(g, squeezed.node(), expect_tensor("Float", shape=(None, 5)))
+
 
 class TestONNXCustomOpShapeInference(pytorch_test_common.ExportTestCase):
     def setUp(self):
         super().setUp()
-        self.opset_version = _constants.ONNX_MAX_OPSET
+        self.opset_version = _constants.ONNX_TORCHSCRIPT_EXPORTER_MAX_OPSET
 
     def test_setType_maintains_output_shape_for_single_custom_op(self):
-
         self.addCleanup(torch.onnx.unregister_custom_op_symbolic, "::linalg_inv", 9)
 
         class CustomInverse(torch.nn.Module):
@@ -363,7 +408,6 @@ class TestONNXCustomOpShapeInference(pytorch_test_common.ExportTestCase):
             self.assertEqual(dim.dim_value, rank)
 
     def test_no_setType_for_single_custom_op(self):
-
         self.addCleanup(torch.onnx.unregister_custom_op_symbolic, "::linalg_inv", 9)
 
         class CustomInverse(torch.nn.Module):
@@ -398,7 +442,6 @@ class TestONNXCustomOpShapeInference(pytorch_test_common.ExportTestCase):
     def test_setType_maintains_output_shape_for_single_custom_op_with_dynamic_axes(
         self,
     ):
-
         self.addCleanup(torch.onnx.unregister_custom_op_symbolic, "::linalg_inv", 9)
 
         class CustomInverse(torch.nn.Module):
@@ -438,7 +481,6 @@ class TestONNXCustomOpShapeInference(pytorch_test_common.ExportTestCase):
             self.assertEqual(dims[i].dim_value, x.size()[i])
 
     def test_setType_maintains_output_shape_for_single_custom_op_with_onnx_ops(self):
-
         self.addCleanup(torch.onnx.unregister_custom_op_symbolic, "::linalg_inv", 9)
 
         class CustomInverse(torch.nn.Module):
