@@ -104,7 +104,7 @@ class TestNumericSuiteEager(QuantizationTestCase):
                 float_model.state_dict(), q_model.state_dict()
             )
             self.assertEqual(len(weight_dict), 1)
-            for k, v in weight_dict.items():
+            for v in weight_dict.values():
                 self.assertTrue(v["float"].shape == v["quantized"].shape)
 
         model_list = [AnnotatedConvModel(qengine), AnnotatedConvBnReLUModel(qengine)]
@@ -126,7 +126,7 @@ class TestNumericSuiteEager(QuantizationTestCase):
                 float_model.state_dict(), q_model.state_dict()
             )
             self.assertEqual(len(weight_dict), 1)
-            for k, v in weight_dict.items():
+            for v in weight_dict.values():
                 self.assertTrue(v["float"].shape == v["quantized"].shape)
 
         model_list = [AnnotatedSingleLayerLinearModel(qengine)]
@@ -148,7 +148,7 @@ class TestNumericSuiteEager(QuantizationTestCase):
                 float_model.state_dict(), q_model.state_dict()
             )
             self.assertEqual(len(weight_dict), 1)
-            for k, v in weight_dict.items():
+            for v in weight_dict.values():
                 self.assertTrue(len(v["float"]) == len(v["quantized"]))
                 for i, val in enumerate(v["quantized"]):
                     self.assertTrue(v["float"][i].shape == v["quantized"][i].shape)
@@ -172,7 +172,7 @@ class TestNumericSuiteEager(QuantizationTestCase):
                 float_model.state_dict(), q_model.state_dict()
             )
             self.assertEqual(len(weight_dict), 1)
-            for k, v in weight_dict.items():
+            for v in weight_dict.values():
                 self.assertTrue(len(v["float"]) == len(v["quantized"]))
                 for i, val in enumerate(v["quantized"]):
                     self.assertTrue(v["float"][i].shape == v["quantized"][i].shape)
@@ -194,7 +194,7 @@ class TestNumericSuiteEager(QuantizationTestCase):
         def compare_and_validate_results(float_model, q_model, module_swap_list, data):
             ob_dict = compare_model_stub(float_model, q_model, module_swap_list, data)
             self.assertEqual(len(ob_dict), 1)
-            for k, v in ob_dict.items():
+            for v in ob_dict.values():
                 self.assertTrue(len(v["float"]) == len(v["quantized"]))
                 for i, val in enumerate(v["quantized"]):
                     self.assertTrue(v["float"][i].shape == v["quantized"][i].shape)
@@ -221,7 +221,7 @@ class TestNumericSuiteEager(QuantizationTestCase):
         def compare_and_validate_results(float_model, q_model, module_swap_list, data):
             ob_dict = compare_model_stub(float_model, q_model, module_swap_list, data)
             self.assertEqual(len(ob_dict), 1)
-            for k, v in ob_dict.items():
+            for v in ob_dict.values():
                 self.assertTrue(len(v["float"]) == len(v["quantized"]))
                 for i, val in enumerate(v["quantized"]):
                     self.assertTrue(v["float"][i].shape == v["quantized"][i].shape)
@@ -246,7 +246,7 @@ class TestNumericSuiteEager(QuantizationTestCase):
         def compare_and_validate_results(float_model, q_model, module_swap_list, data):
             ob_dict = compare_model_stub(float_model, q_model, module_swap_list, data)
             self.assertEqual(len(ob_dict), 1)
-            for k, v in ob_dict.items():
+            for v in ob_dict.values():
                 self.assertTrue(len(v["float"]) == len(v["quantized"]))
                 for i, val in enumerate(v["quantized"]):
                     self.assertTrue(v["float"][i].shape == v["quantized"][i].shape)
@@ -301,7 +301,7 @@ class TestNumericSuiteEager(QuantizationTestCase):
         self.assertTrue(isinstance(q_model.myadd_relu, Shadow))
         self.assertTrue(isinstance(q_model.my_scalar_add, Shadow))
         self.assertTrue(isinstance(q_model.my_scalar_mul, Shadow))
-        for k, v in ob_dict.items():
+        for v in ob_dict.values():
             self.assertTrue(len(v["float"]) == len(v["quantized"]))
             for i, val in enumerate(v["quantized"]):
                 self.assertTrue(v["float"][i].shape == v["quantized"][i].shape)
@@ -315,7 +315,7 @@ class TestNumericSuiteEager(QuantizationTestCase):
         def compare_and_validate_results(float_model, q_model, module_swap_list, data):
             ob_dict = compare_model_stub(float_model, q_model, module_swap_list, data)
             self.assertEqual(len(ob_dict), 1)
-            for k, v in ob_dict.items():
+            for v in ob_dict.values():
                 self.assertTrue(len(v["float"]) == len(v["quantized"]))
                 for i, val in enumerate(v["quantized"]):
                     self.assertTrue(v["float"][i].shape == v["quantized"][i].shape)
@@ -344,7 +344,7 @@ class TestNumericSuiteEager(QuantizationTestCase):
                 float_model, q_model, module_swap_list, input, hidden
             )
             self.assertEqual(len(ob_dict), 1)
-            for k, v in ob_dict.items():
+            for v in ob_dict.values():
                 self.assertTrue(len(v["float"]) == len(v["quantized"]))
                 for i, val in enumerate(v["quantized"]):
                     self.assertTrue(v["float"][i].shape == v["quantized"][i].shape)
@@ -375,7 +375,7 @@ class TestNumericSuiteEager(QuantizationTestCase):
             expected_act_compare_dict_keys = {"conv.stats", "quant.stats"}
 
             self.assertTrue(act_compare_dict.keys() == expected_act_compare_dict_keys)
-            for k, v in act_compare_dict.items():
+            for v in act_compare_dict.values():
                 self.assertTrue(v["float"][0].shape == v["quantized"][0].shape)
 
         model_list = [AnnotatedConvModel(qengine), AnnotatedConvBnReLUModel(qengine)]
@@ -398,7 +398,7 @@ class TestNumericSuiteEager(QuantizationTestCase):
             expected_act_compare_dict_keys = {"fc1.quant.stats", "fc1.module.stats"}
 
             self.assertTrue(act_compare_dict.keys() == expected_act_compare_dict_keys)
-            for k, v in act_compare_dict.items():
+            for v in act_compare_dict.values():
                 self.assertTrue(len(v["float"]) == len(v["quantized"]))
                 for i, val in enumerate(v["quantized"]):
                     self.assertTrue(v["float"][i].shape == v["quantized"][i].shape)
@@ -434,7 +434,7 @@ class TestNumericSuiteEager(QuantizationTestCase):
             "quant.stats",
         }
         self.assertTrue(act_compare_dict.keys() == expected_act_compare_dict_keys)
-        for k, v in act_compare_dict.items():
+        for v in act_compare_dict.values():
             self.assertTrue(len(v["float"]) == len(v["quantized"]))
             for i, val in enumerate(v["quantized"]):
                 self.assertTrue(v["float"][i].shape == v["quantized"][i].shape)
@@ -451,7 +451,7 @@ class TestNumericSuiteEager(QuantizationTestCase):
             expected_act_compare_dict_keys = {"fc1.stats"}
 
             self.assertTrue(act_compare_dict.keys() == expected_act_compare_dict_keys)
-            for k, v in act_compare_dict.items():
+            for v in act_compare_dict.values():
                 self.assertTrue(len(v["float"]) == len(v["quantized"]))
                 for i, val in enumerate(v["quantized"]):
                     self.assertTrue(v["float"][i].shape == v["quantized"][i].shape)
@@ -480,7 +480,7 @@ class TestNumericSuiteEager(QuantizationTestCase):
             expected_act_compare_dict_keys = {"lstm.stats"}
 
             self.assertTrue(act_compare_dict.keys() == expected_act_compare_dict_keys)
-            for k, v in act_compare_dict.items():
+            for v in act_compare_dict.values():
                 self.assertTrue(len(v["float"]) == len(v["quantized"]))
                 for i, val in enumerate(v["quantized"]):
                     self.assertTrue(len(v["float"][i]) == len(v["quantized"][i]))
