@@ -14,7 +14,7 @@ class LoggingTest(TestCase):
         s = TestCase.runWithPytorchAPIUsageStderr("import torch")
         self.assertRegex(s, "PYTORCH_API_USAGE.*import")
         # import the shared library directly - it triggers static init but doesn't call anything
-        s = TestCase.runWithPytorchAPIUsageStderr("from ctypes import CDLL; CDLL('{}')".format(torch._C.__file__))
+        s = TestCase.runWithPytorchAPIUsageStderr(f"from ctypes import CDLL; CDLL('{torch._C.__file__}')")
         self.assertNotRegex(s, "PYTORCH_API_USAGE")
 
 
