@@ -38,6 +38,9 @@ def build_constructor_arg_db():
         torch.nn.ConstantPad1d: ((2, 3.5), {}),
         torch.nn.ConstantPad2d: ((2, 3.5), {}),
         torch.nn.ConstantPad3d: ((2, 3.5), {}),
+        torch.nn.CircularPad1d: ((2,), {}),
+        torch.nn.CircularPad2d: ((2,), {}),
+        torch.nn.CircularPad3d: ((2,), {}),
         torch.nn.Conv1d: ((3, 3, 3), {}),
         torch.nn.Conv2d: ((3, 3, 3), {}),
         torch.nn.Conv3d: ((3, 3, 3), {}),
@@ -166,7 +169,9 @@ def build_constructor_arg_db():
         torch.nn.Upsample: ((), {}),
         torch.nn.UpsamplingBilinear2d: ((), {}),
         torch.nn.UpsamplingNearest2d: ((), {}),
+        torch.nn.ZeroPad1d: ((0,), {}),
         torch.nn.ZeroPad2d: ((0,), {}),
+        torch.nn.ZeroPad3d: ((0,), {}),
         torch.ao.nn.qat.Conv1d: ((3, 3, 3), {
             'qconfig': torch.ao.quantization.default_qconfig,
         }),
