@@ -3,20 +3,42 @@
 import tempfile
 import torch
 from copy import deepcopy
-from torch.library import Library
+from torch.library import Library, impl, fallthrough_kernel
+from torch.fx.experimental.proxy_tensor import ShapeEnv
+from torch import SymInt
+from torch._subclasses.fake_tensor import FakeTensorMode
 from torch.cuda.jiterator import _create_jit_fn
 import unittest
-from torch.testing._internal.common_utils import TestCase, run_tests, TEST_WITH_ROCM, IS_WINDOWS
+from torch.testing._internal.common_utils import *  # noqa: F403
 from torch.utils._mode_utils import no_dispatch, all_same_mode
 from torch.testing._internal.logging_tensor import LoggingTensor, LoggingTensorReentrant, LoggingTensorMode, \
     log_input, capture_logs, capture_logs_with_logging_tensor_mode
 from torch.utils._pytree import tree_map, tree_map_only
 from torch.utils._python_dispatch import TorchDispatchMode, _get_current_dispatch_mode, _get_current_dispatch_mode_stack
+from torch._custom_op.functional import register_functional_op
+import torch.utils._pytree as pytree
+from torch.fx.experimental.proxy_tensor import make_fx
 
 import logging
+import sys
+import torch._dynamo
+
+
+class TestDispatcherPythonBindings(TestCase):
+    def test_call_boxed(self) -> None:
+        sin = torch._C._dispatch_find_schema_or_throw("aten::sin", "")
+        x = torch.randn(3)
+        y = torch._C._dispatch_call_boxed(sin, x)
+        self.assertEqual(y, x.sin())
 
 
 class TestPythonRegistration(TestCase):
+    test_ns = '_test_python_registration'
+
+    def tearDown(self):
+        if hasattr(torch.ops, self.test_ns):
+            del torch.ops._test_python_registration
+
     def test_override_aten_ops_with_multiple_libraries(self) -> None:
         x = torch.tensor([1, 2])
         my_lib1 = Library("aten", "IMPL")
@@ -71,13 +93,52 @@ class TestPythonRegistration(TestCase):
             my_lib = Library("aten", "IMPL")
             my_lib.impl(torch.ops.aten.neg.default, [], "AutogradCPU")
 
+    def test_finalizer(self):
+        impls_refcnt = sys.getrefcount(torch.library._impls)
+        lib = Library(self.test_ns, "FRAGMENT")
+        lib.define("foo123(Tensor x) -> Tensor")
+
+        # 1 for `lib`, 1 for sys.getrefcount
+        self.assertEqual(sys.getrefcount(lib), 2)
+        # We gained an additional reference that gets cleared when the finalizer runs
+        self.assertEqual(sys.getrefcount(torch.library._impls), impls_refcnt + 1)
+        # 1 for `lib`
+        # 1 for the finalizer
+        # 1 for sys.getrefcount
+        self.assertEqual(sys.getrefcount(lib._op_impls), 3)
+
+        def foo123(x):
+            pass
+
+        lib.impl(f"{self.test_ns}::foo123", foo123, "CPU")
+        key = f'{self.test_ns}/foo123/CPU'
+        self.assertTrue(key in torch.library._impls)
+
+        saved_op_impls = lib._op_impls
+
+        # del will definitely work if the following passes
+        self.assertEqual(sys.getrefcount(lib), 2)
+        del lib
+
+        # 1 for saved_op_impls
+        # 1 for sys.getrefcount
+        # This function should be the last user of lib._op_impls:
+        # - lib should not have a reference anymore (it was del'ed)
+        # - lib's finalizer should not have a reference anymore
+        self.assertEqual(sys.getrefcount(saved_op_impls), 2)
+
+        self.assertTrue(key not in torch.library._impls)
+
+        # lib's finalizer should not have a reference anymore
+        self.assertEqual(sys.getrefcount(torch.library._impls), impls_refcnt)
+
     def test_override_cpu_sum(self) -> None:
         # Example 1
         run = [False]
 
         def my_sum(*args, **kwargs):
             run[0] = True
-            return args[0]
+            return args[0].clone()
 
         my_lib1 = Library("aten", "IMPL")
         my_lib1.impl('aten::sum', my_sum, "CPU")
@@ -213,7 +274,7 @@ class TestPythonRegistration(TestCase):
 
     def test_extend_library_with_dispatch_key_arg(self):
         def my_sum(*args, **kwargs):
-            return args[0]
+            return args[0].clone()
         my_lib1 = Library("aten", "IMPL", dispatch_key="CPU")
 
         # RuntimeError: Explicitly provided dispatch key (Conjugate) is
@@ -226,39 +287,85 @@ class TestPythonRegistration(TestCase):
         del my_lib1
 
     def test_create_new_library(self) -> None:
-        my_lib1 = Library("foo", "DEF")
+        my_lib1 = Library(self.test_ns, "DEF")
 
         my_lib1.define("sum(Tensor self) -> Tensor")
 
         # Example 1
         @torch.library.impl(my_lib1, "sum", "CPU")
         def my_sum(*args, **kwargs):
-            return args[0]
+            return args[0].clone()
 
         x = torch.tensor([1, 2])
-        self.assertEqual(torch.ops.foo.sum(x), x)
+        op = getattr(torch.ops, self.test_ns).sum
+        self.assertEqual(op(x), x)
 
-        my_lib2 = Library("foo", "IMPL")
+        my_lib2 = Library(self.test_ns, "IMPL")
 
         # Example 2
-        @torch.library.impl(my_lib2, torch.ops.foo.sum.default, "ZeroTensor")
+        @torch.library.impl(my_lib2, op.default, "ZeroTensor")
         def my_sum_zt(*args, **kwargs):
             if args[0]._is_zerotensor():
                 return torch._efficientzerotensor(args[0].shape)
             else:
-                return args[0]
+                return args[0].clone()
 
         y = torch._efficientzerotensor(3)
-        self.assertTrue(torch.ops.foo.sum(y)._is_zerotensor())
-        self.assertEqual(torch.ops.foo.sum(x), x)
+        self.assertTrue(op(y)._is_zerotensor())
+        self.assertEqual(op(x), x)
 
         del my_lib2
         del my_lib1
 
+    def test_create_new_library_fragment_no_existing(self):
+        my_lib = Library(self.test_ns, "FRAGMENT")
+
+        my_lib.define("sum2(Tensor self) -> Tensor")
+
+        @torch.library.impl(my_lib, "sum2", "CPU")
+        def my_sum(*args, **kwargs):
+            return args[0]
+
+        x = torch.tensor([1, 2])
+        self.assertEqual(getattr(torch.ops, self.test_ns).sum2(x), x)
+
+        del my_lib
+
+    def test_create_new_library_fragment_with_existing(self):
+        my_lib1 = Library(self.test_ns, "DEF")
+
+        # Create a fragment
+        my_lib2 = Library(self.test_ns, "FRAGMENT")
+
+        my_lib2.define("sum4(Tensor self) -> Tensor")
+
+        @torch.library.impl(my_lib2, "sum4", "CPU")
+        def my_sum4(*args, **kwargs):
+            return args[0]
+
+        x = torch.tensor([1, 2])
+        self.assertEqual(getattr(torch.ops, self.test_ns).sum4(x), x)
+
+        # Create another fragment
+        my_lib3 = Library(self.test_ns, "FRAGMENT")
+
+        my_lib3.define("sum3(Tensor self) -> Tensor")
+
+        @torch.library.impl(my_lib3, "sum3", "CPU")
+        def my_sum3(*args, **kwargs):
+            return args[0]
+
+        x = torch.tensor([1, 2])
+        self.assertEqual(getattr(torch.ops, self.test_ns).sum3(x), x)
+
+        del my_lib1
+        del my_lib2
+        del my_lib3
+
     @unittest.skipIf(IS_WINDOWS, "Skipped under Windows")
     def test_alias_analysis(self):
         def test_helper(alias_analysis=""):
-            my_lib1 = Library("foo", "DEF")
+            my_lib1 = Library(self.test_ns, "DEF")
 
             called = [0]
 
@@ -268,9 +375,9 @@ class TestPythonRegistration(TestCase):
 
             @torch.jit.script
             def _test():
-                torch.ops.foo._op()
+                torch.ops._test_python_registration._op()
 
-            assert "foo::_op" in str(_test.graph)
+            assert "_test_python_registration::_op" in str(_test.graph)
 
         with self.assertRaises(AssertionError):
             test_helper("")  # alias_analysis="FROM_SCHEMA"
@@ -281,8 +388,180 @@ class TestPythonRegistration(TestCase):
         with self.assertRaisesRegex(ValueError, "Unsupported kind"):
             my_lib1 = Library("myns", "BLA")
 
-        with self.assertRaisesRegex(ValueError, "reserved namespace"):
-            my_lib1 = Library("prim", "DEF")
+        for kind in ('DEF', 'FRAGMENT'):
+            with self.assertRaisesRegex(ValueError, "reserved namespace"):
+                my_lib1 = Library("prim", kind)
+
+    def test_returning_symint(self) -> None:
+        shape_env = ShapeEnv()
+        fake_tensor_mode = FakeTensorMode(shape_env=shape_env)
+
+        ft = fake_tensor_mode.from_tensor(torch.rand(2, 3))
+
+        s0, s1 = ft.shape
+
+        tlib = Library(self.test_ns, "DEF")
+        tlib.define("sqsum(SymInt a, SymInt b) -> SymInt")
+
+        @impl(tlib, "sqsum", "CompositeExplicitAutograd")
+        def sqsum(a: SymInt, b: SymInt):
+            return a * a + b * b
+
+        out = getattr(torch.ops, self.test_ns).sqsum.default(s0, s1)
+        out_val = shape_env.evaluate_expr(out.node.expr)
+        self.assertEqual(out_val, 13)
+
+    def test_register_functional_op_error_cases(self):
+        lib = Library(self.test_ns, "FRAGMENT")
+        with self.assertRaisesRegex(TypeError, "instance of OpOverload"):
+            register_functional_op(lib, "abs", torch.ops.aten.abs_)
+        with self.assertRaisesRegex(RuntimeError, "Expected op to be mutable"):
+            register_functional_op(lib, "abs", torch.ops.aten.abs_.default)
+        with self.assertRaisesRegex(RuntimeError, "Expected op to be mutable"):
+            register_functional_op(lib, "abs", torch.ops.aten.abs.out)
+
+        schemas = [
+            'foo(Tensor x, Tensor(a!)? y) -> ()',
+            'foo(Tensor x, Tensor(a!)[] y) -> ()',
+            'foo(Tensor x, Tensor(a!) y, Tensor(b) z) -> Tensor(b)',
+            'foo(Tensor x, Tensor(a!) y) -> (Tensor, Tensor(a))',
+        ]
+        del lib
+
+        for schema in schemas:
+            lib = Library(self.test_ns, "FRAGMENT")
+            try:
+                lib.define(schema)
+                with self.assertRaisesRegex(RuntimeError, "NYI"):
+                    register_functional_op(
+                        lib,
+                        "foo_functional",
+                        getattr(torch.ops, self.test_ns).foo.default)
+            finally:
+                del lib
+                delattr(torch.ops, self.test_ns)
+
+    def _check_is_functional_variant(self, mutable_op, functional_op, args):
+        # functional op should not mutate
+        cloned_args = pytree.tree_map_only(torch.Tensor, torch.clone, args)
+        functional_result = functional_op(*cloned_args)
+        self.assertEqual(cloned_args, args)
+
+        # check functional_result includes mutable_result
+        mutable_result = mutable_op(*cloned_args)
+        if mutable_result is None:
+            flat_mutable_result = []
+        else:
+            flat_mutable_result, _ = pytree.tree_flatten(mutable_result)
+        flat_functional_result, _ = pytree.tree_flatten(functional_result)
+        assert len(flat_functional_result) > len(flat_mutable_result)
+        self.assertEqual(flat_functional_result[:len(flat_mutable_result)], flat_mutable_result)
+
+        # check rest of functional_result is the mutated args
+        mutated_args = [maybe_mutated_arg for maybe_mutated_arg, arg in zip(cloned_args, args)
+                        if not torch.allclose(maybe_mutated_arg, arg)]
+        self.assertEqual(flat_functional_result[len(flat_mutable_result):], mutated_args)
+
+        # check that functionalization kernel was indeed registered
+        def fn(*args):
+            cloned_args = pytree.tree_map_only(torch.Tensor, torch.clone, args)
+            mutable_op(*cloned_args)
+            return cloned_args
+
+        gm = make_fx(torch.func.functionalize(fn))(*args)
+        has_functional_op = False
+        for node in gm.graph.nodes:
+            self.assertFalse(node.target is mutable_op)
+            if node.target is functional_op:
+                has_functional_op = True
+        self.assertTrue(has_functional_op)
+
+    def test_register_functional_op_no_returns(self):
+        lib = Library(self.test_ns, 'FRAGMENT')
+        lib.define('foo(Tensor x, Tensor(a!) y, Tensor z, Tensor(b!) w) -> ()')
+
+        def foo_impl(x, y, z, w):
+            y.fill_(3.14)
+            w.fill_(2.71)
+
+        lib.impl('foo', foo_impl, 'CPU')
+        register_functional_op(
+            lib,
+            'foo_functional',
+            getattr(torch.ops, self.test_ns).foo.default)
+        x = torch.randn([])
+        y = torch.randn([])
+        z = torch.randn([])
+        w = torch.randn([])
+        self._check_is_functional_variant(
+            getattr(torch.ops, self.test_ns).foo.default,
+            getattr(torch.ops, self.test_ns).foo_functional.default, (x, y, z, w))
+
+    def test_register_functional_op_one_return(self):
+        lib = Library(self.test_ns, 'FRAGMENT')
+        lib.define('foo(Tensor x, Tensor(a!) y, Tensor(c!) z, Tensor(b!) w) -> Tensor')
+
+        def foo_impl(x, y, z, w):
+            y.fill_(3.14)
+            w.fill_(2.71)
+            z.fill_(0.99)
+            return x.clone()
+
+        lib.impl('foo', foo_impl, 'CPU')
+        register_functional_op(
+            lib,
+            "foo_functional",
+            getattr(torch.ops, self.test_ns).foo.default)
+        x = torch.randn([])
+        y = torch.randn([])
+        z = torch.randn([])
+        w = torch.randn([])
+        self._check_is_functional_variant(
+            getattr(torch.ops, self.test_ns).foo.default,
+            getattr(torch.ops, self.test_ns).foo_functional.default, (x, y, z, w))
+
+    def test_register_functional_op_multiple_returns(self):
+        lib = Library(self.test_ns, 'FRAGMENT')
+        lib.define('foo(Tensor x, Tensor(a!) y, Tensor z, Tensor(b!) w) -> (Tensor, Tensor)')
+
+        def foo_impl(x, y, z, w):
+            y.fill_(3.14)
+            w.fill_(2.71)
+            return x.clone(), z.clone()
+
+        lib.impl('foo', foo_impl, 'CPU')
+        register_functional_op(
+            lib,
+            'foo_functional',
+            getattr(torch.ops, self.test_ns).foo.default)
+
+        x = torch.randn([])
+        y = torch.randn([])
+        z = torch.randn([])
+        w = torch.randn([])
+        self._check_is_functional_variant(
+            getattr(torch.ops, self.test_ns).foo.default,
+            getattr(torch.ops, self.test_ns).foo_functional.default, (x, y, z, w))
+
+    def test_register_fallthrough(self):
+        try:
+            my_lib = Library('aten', 'IMPL')
+            my_lib.impl("mm", fallthrough_kernel, "AutocastCPU")
+
+            a = torch.randn(2, 3, device='cpu', dtype=torch.float32)
+            b = torch.randn(3, 2, device='cpu', dtype=torch.float32)
+            with torch.autocast(device_type="cpu", dtype=torch.bfloat16):
+                # dtype for mm should be float32 since we registered a fallthrough
+                self.assertEqual(torch.mm(a, b).dtype, torch.float32)
+                # ops that don't have a fallthrough registered should not be affected
+                self.assertEqual(torch.matmul(a, b).dtype, torch.bfloat16)
+        finally:
+            del my_lib
+
+        with torch.autocast(device_type="cpu", dtype=torch.bfloat16):
+            # default behavior should have been restored
+            self.assertEqual(torch.mm(a, b).dtype, torch.bfloat16)
+
 
 class TestPythonDispatch(TestCase):
     def test_basic(self) -> None:
@@ -304,13 +583,13 @@ class TestPythonDispatch(TestCase):
             # TODO: figure out why broken
             # self.assertEqual(saved_x._version, x._version)
         self.assertExpectedInline('\n'.join(logs), '''\
-$0 = input('x')
-$1 = torch._ops.aten.mul.Tensor($0, $0)
-$2 = input('grad_y')
+$0: f32[1] = input('x')
+$1: f32[1] = torch._ops.aten.mul.Tensor($0, $0)
+$2: f32[1] = input('grad_y')
 True = torch._ops.aten.is_same_size.default($1, $2)
-$3 = torch._ops.aten.mul.Tensor($2, $0)
-$4 = torch._ops.aten.mul.Tensor($2, $0)
-$5 = torch._ops.aten.add.Tensor($4, $3)''')
+$3: f32[1] = torch._ops.aten.mul.Tensor($2, $0)
+$4: f32[1] = torch._ops.aten.mul.Tensor($2, $0)
+$5: f32[1] = torch._ops.aten.add.Tensor($4, $3)''')
 
     def test_out(self) -> None:
         with capture_logs() as logs:
@@ -324,9 +603,9 @@ $5 = torch._ops.aten.add.Tensor($4, $3)''')
         # TODO: arguably this shouldn't pass and we should complain
         # that out isn't a kwarg
         self.assertExpectedInline('\n'.join(logs), '''\
-$0 = input('x')
-$1 = input('y')
-$2 = torch._ops.aten.abs.out($0, out=$1)''')
+$0: f32[1] = input('x')
+$1: f32[1] = input('y')
+$2: f32[1] = torch._ops.aten.abs.out($0, out=$1)''')
 
     def test_kwarg_only(self) -> None:
         with capture_logs() as logs:
@@ -345,14 +624,14 @@ $2 = torch._ops.aten.abs.out($0, out=$1)''')
         # The expectation is that beta/alpha don't show up when they're
         # defaulted.  This is even if the user explicitly specified it.
         self.assertExpectedInline('\n'.join(logs), '''\
-$0 = input('x')
-$1 = input('y')
-$2 = input('z')
-$3 = torch._ops.aten.addmv.default($0, $1, $2)
-$4 = torch._ops.aten.addmv.default($0, $1, $2)
-$5 = torch._ops.aten.addmv.default($0, $1, $2, beta=2)
-$6 = torch._ops.aten.addmv.default($0, $1, $2, alpha=2)
-$7 = torch._ops.aten.addmv.default($0, $1, $2, beta=2, alpha=2)''')
+$0: f32[1] = input('x')
+$1: f32[1, 1] = input('y')
+$2: f32[1] = input('z')
+$3: f32[1] = torch._ops.aten.addmv.default($0, $1, $2)
+$4: f32[1] = torch._ops.aten.addmv.default($0, $1, $2)
+$5: f32[1] = torch._ops.aten.addmv.default($0, $1, $2, beta=2)
+$6: f32[1] = torch._ops.aten.addmv.default($0, $1, $2, alpha=2)
+$7: f32[1] = torch._ops.aten.addmv.default($0, $1, $2, beta=2, alpha=2)''')
 
     def test_kwarg_only_and_positional_default(self) -> None:
         with capture_logs() as logs:
@@ -366,11 +645,11 @@ $7 = torch._ops.aten.addmv.default($0, $1, $2, beta=2, alpha=2)''')
         # What we are testing here is that we omit arg2
         # if it is defaulted, even if a kwarg is set
         self.assertExpectedInline('\n'.join(logs), '''\
-$0 = input('x')
-$1 = torch._ops.aten._foobar.default($0)
-$2 = torch._ops.aten._foobar.default($0, False)
-$3 = torch._ops.aten._foobar.default($0, arg3=False)
-$4 = torch._ops.aten._foobar.default($0, False, arg3=False)''')
+$0: f32[1] = input('x')
+$1: f32[1] = torch._ops.aten._foobar.default($0)
+$2: f32[1] = torch._ops.aten._foobar.default($0, False)
+$3: f32[1] = torch._ops.aten._foobar.default($0, arg3=False)
+$4: f32[1] = torch._ops.aten._foobar.default($0, False, arg3=False)''')
 
     def test_produce_real_type(self) -> None:
         with capture_logs() as logs:
@@ -383,12 +662,12 @@ $4 = torch._ops.aten._foobar.default($0, False, arg3=False)''')
             # triggerable using tensor subclasses (need to use a mode)
 
         self.assertExpectedInline('\n'.join(logs), '''\
-$0 = input('x')
-$1 = torch._ops.aten._to_copy.default($0, dtype=torch.float64)
-$2 = torch._ops.aten.cumprod.default($0, 0, dtype=torch.float64)
-$3 = torch._ops.aten.slice.Tensor($0, 0, 0, 9223372036854775807)
-$4 = torch._ops.aten.select.int($3, 1, 1)
-$5 = torch._ops.aten.clone.default($4, memory_format=torch.contiguous_format)''')
+$0: f32[2, 2] = input('x')
+$1: f64[2, 2] = torch._ops.aten._to_copy.default($0, dtype=torch.float64)
+$2: f64[2, 2] = torch._ops.aten.cumprod.default($0, 0, dtype=torch.float64)
+$3: f32[2, 2] = torch._ops.aten.slice.Tensor($0, 0, 0, 9223372036854775807)
+$4: f32[2] = torch._ops.aten.select.int($3, 1, 1)
+$5: f32[2] = torch._ops.aten.clone.default($4, memory_format=torch.contiguous_format)''')
 
     def test_optional_tensor_list(self) -> None:
         def weird(xs):
@@ -404,9 +683,8 @@ $5 = torch._ops.aten.clone.default($4, memory_format=torch.contiguous_format)'''
             torch.ops.my_lib.weird.default([None, x])
 
         self.assertExpectedInline('\n'.join(logs), '''\
-$0 = input('x')
-$1 = torch._ops.my_lib.weird.default([None, LoggingTensor(tensor([[1., 1.],
-        [1., 1.]]))])''')
+$0: f32[2, 2] = input('x')
+$1: f32[] = torch._ops.my_lib.weird.default(['None', '$0'])''')
 
     def test_list_ret(self) -> None:
         # test all sequence types are permissible returns
@@ -458,15 +736,16 @@ $1 = torch._ops.my_lib.weird.default([None, LoggingTensor(tensor([[1., 1.],
         # this test here to make sure we don't regress even further (it
         # would be bad if calling .detach() once emits 3+ detaches).
         self.assertExpectedInline('\n'.join(logs), '''\
-$0 = input('x')
-$1 = torch._ops.aten.detach.default($0)
-$2 = torch._ops.aten.detach.default($1)''')
+$0: f32[1] = input('x')
+$1: f32[1] = torch._ops.aten.detach.default($0)
+$2: f32[1] = torch._ops.aten.detach.default($1)''')
 
     def test_storage(self) -> None:
         # For now, just make sure it doesn't crash.  Ideally, we should
         # return some virtual storage that is safe to work with
         x = LoggingTensor(torch.ones(1))
-        self.assertRaises(RuntimeError, lambda: x.storage())
+        storage = x.untyped_storage()
+        self.assertRaises(RuntimeError, lambda: storage.data_ptr())
 
     def test_make_wrapper_subclass_noalloc(self) -> None:
         # This is ludicrously big (8TB) and this should pass because wrapper
@@ -563,14 +842,14 @@ $2 = torch._ops.aten.detach.default($1)''')
             # self.assertEqual(escape[0]._version, x._version)
 
         self.assertExpectedInline('\n'.join(logs), '''\
-$0 = input('x')
-$1 = input('x.grad')
-$2 = torch._ops.aten.pow.Tensor_Scalar($0, 2)
-$3 = input('grad_output')
+$0: f32[1] = input('x')
+$1: f32[1] = input('x.grad')
+$2: f32[1] = torch._ops.aten.pow.Tensor_Scalar($0, 2)
+$3: f32[1] = input('grad_output')
 True = torch._ops.aten.is_same_size.default($2, $3)
-$4 = torch._ops.aten.mul.Tensor($3, 2)
-$5 = torch._ops.aten.mul.Tensor($4, $0)
-$6 = torch._ops.aten.add_.Tensor($1, $5)''')
+$4: f32[1] = torch._ops.aten.mul.Tensor($3, 2)
+$5: f32[1] = torch._ops.aten.mul.Tensor($4, $0)
+$6: f32[1] = torch._ops.aten.add_.Tensor($1, $5)''')
 
     def test_subclass_creation(self):
         # Make sure these statements runs without error
@@ -753,7 +1032,7 @@ $6 = torch._ops.aten.add_.Tensor($1, $5)''')
             with LoggingTensorMode():
                 torch.empty([])
         self.assertExpectedInline('\n'.join(logs), """\
-$0 = torch._ops.aten.empty.memory_format([], device=device(type='cpu'), pin_memory=False)""")
+$0: f32[] = torch._ops.aten.empty.memory_format([], device=device(type='cpu'), pin_memory=False)""")
 
     def test_torch_dispatch_mode_unrelated_tensors(self) -> None:
         x = torch.randn([])
@@ -761,8 +1040,7 @@ $0 = torch._ops.aten.empty.memory_format([], device=device(type='cpu'), pin_memo
         with capture_logs(is_mode=True) as logs:
             with LoggingTensorMode():
                 x + y
-        self.assertExpectedInline('\n'.join(logs), """\
-$2 = torch._ops.aten.add.Tensor($0, $1)""")
+        self.assertExpectedInline('\n'.join(logs), """$2: f32[] = torch._ops.aten.add.Tensor($0, $1)""")
 
     def test_nested_push_logging_tensor_mode(self):
         x = torch.randn([])
@@ -774,10 +1052,10 @@ $2 = torch._ops.aten.add.Tensor($0, $1)""")
                     x + y
 
         self.assertExpectedInline('\n'.join(logs), """\
-$0 = torch._ops.aten.empty.memory_format([], device=device(type='cpu'), pin_memory=False)
-$0 = torch._ops.aten.empty.memory_format([], device=device(type='cpu'), pin_memory=False)
-$3 = torch._ops.aten.add.Tensor($1, $2)
-$3 = torch._ops.aten.add.Tensor($1, $2)""")
+$0: f32[] = torch._ops.aten.empty.memory_format([], device=device(type='cpu'), pin_memory=False)
+$0: f32[] = torch._ops.aten.empty.memory_format([], device=device(type='cpu'), pin_memory=False)
+$3: f32[] = torch._ops.aten.add.Tensor($1, $2)
+$3: f32[] = torch._ops.aten.add.Tensor($1, $2)""")
 
     def test_capture_logs_with_torch_dispatch_mode(self):
         x = torch.randn([])
@@ -786,8 +1064,8 @@ $3 = torch._ops.aten.add.Tensor($1, $2)""")
             torch.empty([])
             x + y
         self.assertExpectedInline('\n'.join(logs), """\
-$0 = torch._ops.aten.empty.memory_format([], device=device(type='cpu'), pin_memory=False)
-$3 = torch._ops.aten.add.Tensor($1, $2)""")
+$0: f32[] = torch._ops.aten.empty.memory_format([], device=device(type='cpu'), pin_memory=False)
+$3: f32[] = torch._ops.aten.add.Tensor($1, $2)""")
 
         x = torch.randn([])
         y = torch.randn([])
@@ -798,10 +1076,10 @@ $3 = torch._ops.aten.add.Tensor($1, $2)""")
                 x + y
 
         self.assertExpectedInline('\n'.join(logs2), """\
-$0 = torch._ops.aten.empty.memory_format([], device=device(type='cpu'), pin_memory=False)
-$0 = torch._ops.aten.empty.memory_format([], device=device(type='cpu'), pin_memory=False)
-$3 = torch._ops.aten.add.Tensor($1, $2)
-$3 = torch._ops.aten.add.Tensor($1, $2)""")
+$0: f32[] = torch._ops.aten.empty.memory_format([], device=device(type='cpu'), pin_memory=False)
+$0: f32[] = torch._ops.aten.empty.memory_format([], device=device(type='cpu'), pin_memory=False)
+$3: f32[] = torch._ops.aten.add.Tensor($1, $2)
+$3: f32[] = torch._ops.aten.add.Tensor($1, $2)""")
 
         self.assertEqual(logs1, logs2)
 
@@ -938,7 +1216,7 @@ $3 = torch._ops.aten.add.Tensor($1, $2)""")
     def test_with_nested_modes(self):
         class ErrorA(RuntimeError):
             def __init__(self, msg):
-                return super().__init__(msg)
+                super().__init__(msg)
 
         class A(TorchDispatchMode):
             def __init__(self, msg):
@@ -1057,8 +1335,8 @@ $3 = torch._ops.aten.add.Tensor($1, $2)""")
                 with reenabled:
                     torch.empty([])
             self.assertExpectedInline('\n'.join(logs), """\
-$0 = torch._ops.aten.empty.memory_format([], device=device(type='cpu'), pin_memory=False)
-$0 = torch._ops.aten.empty.memory_format([], device=device(type='cpu'), pin_memory=False)""")
+$0: f32[] = torch._ops.aten.empty.memory_format([], device=device(type='cpu'), pin_memory=False)
+$0: f32[] = torch._ops.aten.empty.memory_format([], device=device(type='cpu'), pin_memory=False)""")
 
 
     def test_error_using_class_method_on_mode(self):
@@ -1116,6 +1394,48 @@ $0 = torch._ops.aten.empty.memory_format([], device=device(type='cpu'), pin_memo
         with self.assertRaises(AssertionError):
             self.assertEqual(x, None)
 
+    def test_record_stream(self) -> None:
+        class TestMode(TorchDispatchMode):
+            def __init__(self, testcase):
+                self.testcase = testcase
+
+            def __torch_dispatch__(self, func, types, args=(), kwargs=None):
+                self.testcase.assertEqual(func.name(), "aten::record_stream")
+                self.testcase.assertIsInstance(args[0], torch.Tensor)
+                self.testcase.assertIsInstance(args[1], torch.Stream)
+                self.testcase.assertEqual(args[1].stream_id, 1)
+                self.testcase.assertEqual(args[1].device_index, 2)
+                self.testcase.assertEqual(args[1].device_type, 3)
+
+        t = torch.tensor(5.)
+        s = torch.Stream(stream_id=1, device_index=2, device_type=3)
+        with TestMode(self):
+            t.record_stream(s)
+
+    def test_return_stream(self) -> None:
+        l_def = torch.library.Library("test_return_stream", "DEF")
+        l_def.define("return_stream(Tensor self) -> Stream")
+        l_impl = torch.library.Library("test_return_stream", "IMPL", "CPU")
+        l_impl.impl("return_stream", lambda _: torch.Stream(stream_id=0, device_index=1, device_type=2))
+
+        class TestMode(TorchDispatchMode):
+            def __torch_dispatch__(self, func, types, args=(), kwargs=None):
+                return torch.Stream(stream_id=1, device_index=2, device_type=3)
+
+        t = torch.tensor(5.)
+        s = torch.ops.test_return_stream.return_stream(t)
+        self.assertIsInstance(s, torch.Stream)
+        self.assertEqual(s.stream_id, 0)
+        self.assertEqual(s.device_index, 1)
+        self.assertEqual(s.device_type, 2)
+
+        with TestMode():
+            s = torch.ops.test_return_stream.return_stream(t)
+        self.assertIsInstance(s, torch.Stream)
+        self.assertEqual(s.stream_id, 1)
+        self.assertEqual(s.device_index, 2)
+        self.assertEqual(s.device_type, 3)
+
     def test_subclass_autograd_device_check(self) -> None:
         class NonWrapperSubclass(torch.Tensor):
             elem: torch.Tensor
@@ -1403,7 +1723,7 @@ $0 = torch._ops.aten.empty.memory_format([], device=device(type='cpu'), pin_memo
                         return not_contiguous_data.is_contiguous()
                     return NotImplemented
 
-            err_msg = "no implementation found for 'torch.ops.aten.is_contiguous'"
+            err_msg = "Multiple dispatch failed for 'torch.ops.aten.is_contiguous'"
             e = ExampleTensor1(torch.randn(3, 3), use_wrapper_subclass)
             with self.assertRaisesRegex(TypeError, err_msg):
                 e.is_contiguous()
@@ -1414,7 +1734,7 @@ $0 = torch._ops.aten.empty.memory_format([], device=device(type='cpu'), pin_memo
             self.assertEqual(e.is_contiguous(), True)
             e.contiguous()  # this will just return the original TensorImpl since is_contiguous = True
 
-            err_msg = "no implementation found for"
+            err_msg = "Multiple dispatch failed for"
             e = ExampleTensor3(torch.randn(3, 3), use_wrapper_subclass)
             self.assertEqual(e.is_contiguous(), False)
             with self.assertRaisesRegex(TypeError, err_msg):
@@ -1485,7 +1805,7 @@ $0 = torch._ops.aten.empty.memory_format([], device=device(type='cpu'), pin_memo
                         return torch.device('meta')
                     return NotImplemented
 
-            err_msg = "no implementation found for 'torch.ops.prim.device'"
+            err_msg = "Multiple dispatch failed for 'torch.ops.prim.device'"
             with self.assertRaisesRegex(TypeError, err_msg):
                 e = ExampleTensor1(torch.randn(3, 3), use_wrapper_subclass)
                 e.device()
@@ -1523,7 +1843,7 @@ $0 = torch._ops.aten.empty.memory_format([], device=device(type='cpu'), pin_memo
                         return data.dim()
                     return NotImplemented
 
-            err_msg = "no implementation found for 'torch.ops.aten.dim'"
+            err_msg = "Multiple dispatch failed for 'torch.ops.aten.dim'"
             e = DimNotImplementedTensor(torch.randn(3, 3), use_wrapper_subclass)
             with self.assertRaisesRegex(TypeError, err_msg):
                 e.dim()
@@ -1544,6 +1864,40 @@ $0 = torch._ops.aten.empty.memory_format([], device=device(type='cpu'), pin_memo
         # https://github.com/pytorch/pytorch/issues/79079
         self.assertFalse(torch._C._dispatch_isTensorSubclassLike(torch.empty(0)))
 
+    def test_sym_sizes_strides_slow_path(self):
+        class TestTensor(torch.Tensor):
+            @staticmethod
+            def __new__(cls, *args, **kwargs):
+                r = torch.Tensor._make_wrapper_subclass(  # type: ignore[attr-defined]
+                    cls, (0,), dispatch_sizes_strides_policy="sizes")
+                return r
+
+            @classmethod
+            def __torch_dispatch__(cls, func, types, args=(), kwargs=None):
+                if func in (
+                    torch.ops.aten.sym_size.default,
+                    torch.ops.aten.sym_stride.default
+                ):
+                    from torch._dynamo.source import ConstantSource
+                    from torch.fx.experimental.symbolic_shapes import ShapeEnv, DimDynamic
+                    shape_env = ShapeEnv()
+                    si = shape_env.create_symintnode(
+                        shape_env.create_symbol(
+                            123,
+                            source=ConstantSource("abc"),
+                            dynamic_dim=DimDynamic.DUCK,
+                            constraint_dim=None,
+                        ),
+                        hint=123
+                    )
+                    return (si,)
+
+        t = TestTensor()
+        si = t.size()[0]
+        self.assertIsInstance(si, torch.SymInt)
+        si = t.stride()[0]
+        self.assertIsInstance(si, torch.SymInt)
+
     def test_strides_slow_path(self):
         for use_wrapper_subclass in [True, False]:
             class StridesNotImplemented(torch.Tensor):
@@ -1577,7 +1931,7 @@ $0 = torch._ops.aten.empty.memory_format([], device=device(type='cpu'), pin_memo
                         return None
                     return NotImplemented
 
-            err_msg = "no implementation found for 'torch.ops.aten.sym_stride'"
+            err_msg = "Multiple dispatch failed for 'torch.ops.aten.sym_stride'"
             e = StridesNotImplemented(torch.randn(3, 3), use_wrapper_subclass)
             with self.assertRaisesRegex(TypeError, err_msg):
                 e.stride()
@@ -1629,7 +1983,7 @@ $0 = torch._ops.aten.empty.memory_format([], device=device(type='cpu'), pin_memo
                         return None
                     return NotImplemented
 
-            err_msg = "no implementation found for 'torch.ops.aten.sym_size'"
+            err_msg = "Multiple dispatch failed for 'torch.ops.aten.sym_size'"
             e = SizesNotImplemented(torch.randn(3, 3), use_wrapper_subclass)
             with self.assertRaisesRegex(TypeError, err_msg):
                 e.size()
@@ -1640,6 +1994,29 @@ $0 = torch._ops.aten.empty.memory_format([], device=device(type='cpu'), pin_memo
             e = SizesDefaultReturn(torch.randn(4, 2), use_wrapper_subclass)
             self.assertEqual(e.size(), (4, 2))
 
+    def test_data_ptr_respects_numel_slow_path(self):
+        data = torch.randn(6, 2)
+
+        class NumelDefaultReturn(torch.Tensor):
+            @staticmethod
+            def __new__(cls, data, wrapper):
+                return TestPythonDispatch.subclass_helper(cls, data, wrapper, dispatch_sizes_strides_policy="sizes")
+
+            @classmethod
+            def __torch_dispatch__(cls, func, types, args, kwargs):
+                if func.overloadpacket == torch.ops.aten.dim:
+                    return data.dim()
+                if func.overloadpacket == torch.ops.aten.sym_numel:
+                    numel_called[0] = True
+                    return None
+                return NotImplemented
+
+        for use_wrapper_subclass in (False, True):
+            numel_called = [False]
+            e = NumelDefaultReturn(torch.randn(2, 2), use_wrapper_subclass)
+            e.data_ptr()
+            self.assertTrue(numel_called[0])
+
     def test_layout_slow_path(self):
         for use_wrapper_subclass in [True, False]:
             data = torch.randn(6, 2)
@@ -1675,7 +2052,7 @@ $0 = torch._ops.aten.empty.memory_format([], device=device(type='cpu'), pin_memo
                         return data.layout
                     return NotImplemented
 
-            err_msg = "no implementation found for 'torch.ops.prim.layout'"
+            err_msg = "Multiple dispatch failed for 'torch.ops.prim.layout'"
             e = LayoutNotImplemented(torch.randn(3, 3), use_wrapper_subclass)
             with self.assertRaisesRegex(TypeError, err_msg):
                 e.layout
