@@ -18,9 +18,10 @@ from torch.testing._internal.common_device_type import \
 from torch.testing._internal.common_methods_invocations import (
     spectral_funcs, SpectralFuncType)
 from torch.testing._internal.common_cuda import SM53OrLater
+from torch._prims_common import corresponding_complex_dtype
 
-from setuptools import distutils
 from typing import Optional, List
+from packaging import version
 
 
 if TEST_NUMPY:
@@ -37,11 +38,10 @@ try:
 except ModuleNotFoundError:
     pass
 
-LooseVersion = distutils.version.LooseVersion
 REFERENCE_NORM_MODES = (
     (None, "forward", "backward", "ortho")
-    if LooseVersion(np.__version__) >= '1.20.0' and (
-        not has_scipy_fft or LooseVersion(scipy.__version__) >= '1.6.0')
+    if version.parse(np.__version__) >= version.parse('1.20.0') and (
+        not has_scipy_fft or version.parse(scipy.__version__) >= version.parse('1.6.0'))
     else (None, "ortho"))
 
 
@@ -737,11 +737,7 @@ class TestFFT(TestCase):
 
     # Legacy fft tests
     def _test_fft_ifft_rfft_irfft(self, device, dtype):
-        complex_dtype = {
-            torch.float16: torch.complex32,
-            torch.float32: torch.complex64,
-            torch.float64: torch.complex128
-        }[dtype]
+        complex_dtype = corresponding_complex_dtype(dtype)
 
         def _test_complex(sizes, signal_ndim, prepro_fn=lambda x: x):
             x = prepro_fn(torch.randn(*sizes, dtype=complex_dtype, device=device))
@@ -814,8 +810,10 @@ class TestFFT(TestCase):
                 plan_cache = torch.backends.cuda.cufft_plan_cache[device]
             original = plan_cache.max_size
             plan_cache.max_size = n
-            yield
-            plan_cache.max_size = original
+            try:
+                yield
+            finally:
+                plan_cache.max_size = original
 
         with plan_cache_max_size(devices[0], max(1, torch.backends.cuda.cufft_plan_cache.size - 10)):
             self._test_fft_ifft_rfft_irfft(devices[0], dtype)
@@ -874,6 +872,22 @@ class TestFFT(TestCase):
                             self.assertEqual(torch.backends.cuda.cufft_plan_cache.max_size, 10)  # default is cuda:0
                         self.assertEqual(torch.backends.cuda.cufft_plan_cache.max_size, 11)  # default is cuda:1
 
+    @onlyCUDA
+    @dtypes(torch.cfloat, torch.cdouble)
+    def test_cufft_context(self, device, dtype):
+        # Regression test for https://github.com/pytorch/pytorch/issues/109448
+        x = torch.randn(32, dtype=dtype, device=device, requires_grad=True)
+        dout = torch.zeros(32, dtype=dtype, device=device)
+
+        # compute iFFT(FFT(x))
+        out = torch.fft.ifft(torch.fft.fft(x))
+        out.backward(dout, retain_graph=True)
+
+        dx = torch.fft.fft(torch.fft.ifft(dout))
+
+        self.assertTrue((x.grad - dx).abs().max() == 0)
+        self.assertFalse((x.grad - x).abs().max() == 0)
+
     # passes on ROCm w/ python 2.7, fails w/ python 3.6
     @skipCPUIfNoFFT
     @onlyNativeDeviceTypes
@@ -1181,9 +1195,8 @@ class TestFFT(TestCase):
     @skipCPUIfNoFFT
     def test_stft_requires_complex(self, device):
         x = torch.rand(100)
-        y = x.stft(10, pad_mode='constant')
-        # with self.assertRaisesRegex(RuntimeError, 'stft requires the return_complex parameter'):
-        #     y = x.stft(10, pad_mode='constant')
+        with self.assertRaisesRegex(RuntimeError, 'stft requires the return_complex parameter'):
+            y = x.stft(10, pad_mode='constant')
 
     @skipCPUIfNoFFT
     def test_fft_input_modification(self, device):
@@ -1391,20 +1404,22 @@ class TestFFT(TestCase):
     @skipCPUIfNoFFT
     @dtypes(torch.double)
     def test_istft_of_sine(self, device, dtype):
+        complex_dtype = corresponding_complex_dtype(dtype)
+
         def _test(amplitude, L, n):
             # stft of amplitude*sin(2*pi/L*n*x) with the hop length and window size equaling L
             x = torch.arange(2 * L + 1, device=device, dtype=dtype)
             original = amplitude * torch.sin(2 * math.pi / L * x * n)
             # stft = torch.stft(original, L, hop_length=L, win_length=L,
             #                   window=torch.ones(L), center=False, normalized=False)
-            stft = torch.zeros((L // 2 + 1, 2, 2), device=device, dtype=dtype)
+            stft = torch.zeros((L // 2 + 1, 2), device=device, dtype=complex_dtype)
             stft_largest_val = (amplitude * L) / 2.0
             if n < stft.size(0):
-                stft[n, :, 1] = -stft_largest_val
+                stft[n].imag = torch.tensor(-stft_largest_val, dtype=dtype)
 
             if 0 <= L - n < stft.size(0):
                 # symmetric about L // 2
-                stft[L - n, :, 1] = stft_largest_val
+                stft[L - n].imag = torch.tensor(stft_largest_val, dtype=dtype)
 
             inverse = torch.istft(
                 stft, L, hop_length=L, win_length=L,
@@ -1426,11 +1441,12 @@ class TestFFT(TestCase):
     @dtypes(torch.double)
     def test_istft_linearity(self, device, dtype):
         num_trials = 100
+        complex_dtype = corresponding_complex_dtype(dtype)
 
         def _test(data_size, kwargs):
             for i in range(num_trials):
-                tensor1 = torch.randn(data_size, device=device, dtype=dtype)
-                tensor2 = torch.randn(data_size, device=device, dtype=dtype)
+                tensor1 = torch.randn(data_size, device=device, dtype=complex_dtype)
+                tensor2 = torch.randn(data_size, device=device, dtype=complex_dtype)
                 a, b = torch.rand(2, dtype=dtype, device=device)
                 # Also compare method vs. functional call signature
                 istft1 = tensor1.istft(**kwargs)
@@ -1441,7 +1457,7 @@ class TestFFT(TestCase):
         patterns = [
             # hann_window, centered, normalized, onesided
             (
-                (2, 7, 7, 2),
+                (2, 7, 7),
                 {
                     'n_fft': 12,
                     'window': torch.hann_window(12, device=device, dtype=dtype),
@@ -1452,7 +1468,7 @@ class TestFFT(TestCase):
             ),
             # hann_window, centered, not normalized, not onesided
             (
-                (2, 12, 7, 2),
+                (2, 12, 7),
                 {
                     'n_fft': 12,
                     'window': torch.hann_window(12, device=device, dtype=dtype),
@@ -1463,7 +1479,7 @@ class TestFFT(TestCase):
             ),
             # hamming_window, centered, normalized, not onesided
             (
-                (2, 12, 7, 2),
+                (2, 12, 7),
                 {
                     'n_fft': 12,
                     'window': torch.hamming_window(12, device=device, dtype=dtype),
@@ -1474,7 +1490,7 @@ class TestFFT(TestCase):
             ),
             # hamming_window, not centered, not normalized, onesided
             (
-                (2, 7, 3, 2),
+                (2, 7, 3),
                 {
                     'n_fft': 12,
                     'window': torch.hamming_window(12, device=device, dtype=dtype),
@@ -1491,13 +1507,13 @@ class TestFFT(TestCase):
     @skipCPUIfNoFFT
     def test_batch_istft(self, device):
         original = torch.tensor([
-            [[4., 0.], [4., 0.], [4., 0.], [4., 0.], [4., 0.]],
-            [[0., 0.], [0., 0.], [0., 0.], [0., 0.], [0., 0.]],
-            [[0., 0.], [0., 0.], [0., 0.], [0., 0.], [0., 0.]]
-        ], device=device)
+            [4., 4., 4., 4., 4.],
+            [0., 0., 0., 0., 0.],
+            [0., 0., 0., 0., 0.]
+        ], device=device, dtype=torch.complex64)
 
-        single = original.repeat(1, 1, 1, 1)
-        multi = original.repeat(4, 1, 1, 1)
+        single = original.repeat(1, 1, 1)
+        multi = original.repeat(4, 1, 1)
 
         i_original = torch.istft(original, n_fft=4, length=4)
         i_single = torch.istft(single, n_fft=4, length=4)
