@@ -10,7 +10,7 @@ try:
     # use faster C loader if available
     from yaml import CSafeLoader as YamlLoader
 except ImportError:
-    from yaml import SafeLoader as YamlLoader  # type: ignore[misc]
+    from yaml import SafeLoader as YamlLoader  # type: ignore[assignment, misc]
 
 NATIVE_FUNCTIONS_PATH = "aten/src/ATen/native/native_functions.yaml"
 TAGS_PATH = "aten/src/ATen/native/tags.yaml"
@@ -54,7 +54,6 @@ def generate_code(
         operator_selector = SelectiveBuilder.get_nop_selector()
 
     if subset == "libtorch" or not subset:
-
         gen_autograd(
             native_functions_path or NATIVE_FUNCTIONS_PATH,
             tags_path or TAGS_PATH,
@@ -76,7 +75,7 @@ def generate_code(
 def get_selector_from_legacy_operator_selection_list(
     selected_op_list_path: str,
 ) -> Any:
-    with open(selected_op_list_path, "r") as f:
+    with open(selected_op_list_path) as f:
         # strip out the overload part
         # It's only for legacy config - do NOT copy this code!
         selected_op_list = {
@@ -86,7 +85,7 @@ def get_selector_from_legacy_operator_selection_list(
     # Internal build doesn't use this flag any more. Only used by OSS
     # build now. Every operator should be considered a root operator
     # (hence generating unboxing code for it, which is consistent with
-    # the current behaviour), and also be considered as used for
+    # the current behavior), and also be considered as used for
     # training, since OSS doesn't support training on mobile for now.
     #
     is_root_operator = True
