@@ -2,7 +2,7 @@ import torch
 from torch.ao.quantization.experimental.quantizer import APoTQuantizer
 
 # class to store APoT quantized tensor
-class TensorAPoT():
+class TensorAPoT:
     quantizer: APoTQuantizer
     data: torch.Tensor
 
