@@ -1,5 +1,4 @@
 import copy
-import re
 import torch
 import torch.nn as nn
 from torch.ao.quantization import (
@@ -7,18 +6,27 @@ from torch.ao.quantization import (
     QuantType,
 )
 from torch.ao.quantization.backend_config import (
-    BackendConfig,
     DTypeWithConstraints,
 )
-from torch.ao.quantization.fake_quantize import FakeQuantizeBase
-from torch.ao.quantization.observer import ObserverBase
+from torch.ao.quantization.fake_quantize import (
+    FakeQuantizeBase,
+    FixedQParamsFakeQuantize,
+)
+from torch.ao.quantization.observer import (
+    FixedQParamsObserver,
+    ObserverBase,
+)
+from torch.ao.quantization.qconfig import (
+    float16_static_qconfig,
+    float16_dynamic_qconfig,
+    qconfig_equals,
+)
 from torch.ao.quantization.stubs import DeQuantStub
 from torch.ao.quantization.utils import (
     activation_is_statically_quantized,
-    is_per_tensor,
-    is_per_channel,
 )
-from torch.ao.quantization.quantize import is_activation_post_process
+from torch.ao.quantization.observer import _is_activation_post_process
+from torch.ao.quantization.qconfig_mapping import QConfigMapping
 
 from torch.fx import GraphModule, map_arg
 
@@ -27,8 +35,11 @@ from torch.fx.graph import (
     Node,
 )
 from .custom_config import PrepareCustomConfig
+# importing the lib so that the quantized_decomposed ops are registered
+from ._decomposed import quantized_decomposed_lib  # noqa: F401
 
 from typing import Callable, Optional, List, Dict, Any, Set, Tuple, Union, Type
+from dataclasses import dataclass
 from collections import namedtuple
 import operator
 import warnings
@@ -41,227 +52,56 @@ __all__ = [
     "collect_producer_nodes",
     "create_getattr_from_value",
     "create_node_from_old_node_preserve_meta",
-    "create_qparam_nodes",
     "EMPTY_ARG_DICT",
     "get_custom_module_class_keys",
     "get_linear_prepack_op_for_dtype",
     "get_new_attr_name_with_prefix",
     "get_non_observable_arg_indexes_and_types",
-    "get_per_tensor_qparams",
-    "get_qconv_op",
     "get_qconv_prepack_op",
-    "get_quantize_node_info",
     "get_skipped_module_name_and_classes",
     "graph_module_from_producer_nodes",
-    "graph_pretty_str",
-    "is_get_tensor_info_node",
     "maybe_get_next_module",
     "NodeInfo",
-    "node_return_type_is_int",
     "node_arg_is_bias",
     "node_arg_is_weight",
     "NON_OBSERVABLE_ARG_DICT",
     "NON_QUANTIZABLE_WEIGHT_OPS",
-    "quantize_node",
     "return_arg_list",
+    "ObservedGraphModuleAttrs",
 ]
 
 NON_QUANTIZABLE_WEIGHT_OPS = {torch.nn.functional.layer_norm, torch.nn.functional.group_norm, torch.nn.functional.instance_norm}
 
-def node_arg_is_weight(node: Node, arg: Any, backend_config: BackendConfig) -> bool:
+@dataclass
+class ObservedGraphModuleAttrs:
+    node_name_to_qconfig: Dict[str, QConfigAny]
+    node_name_to_scope: Dict[str, Tuple[str, type]]
+    prepare_custom_config: PrepareCustomConfig
+    equalization_node_name_to_qconfig: Dict[str, Any]
+    qconfig_mapping: QConfigMapping
+    is_qat: bool
+    observed_node_names: Set[str]
+    is_observed_standalone_module: bool = False
+    standalone_module_input_quantized_idxs: Optional[List[int]] = None
+    standalone_module_output_quantized_idxs: Optional[List[int]] = None
+
+def node_arg_is_weight(node: Node, arg: Any) -> bool:
     """Returns if node arg is weight"""
-    if isinstance(node, Node) and node.op == "call_function" and node.target in backend_config.configs:
-        weight_index = backend_config.configs[node.target]._input_type_to_index.get("weight")
-        if weight_index is not None and weight_index < len(node.args) and node.args[weight_index] is arg:
-            return True
-        return node.kwargs.get("weight") is arg
-    return False
+    weight_index = None
+    if "target_dtype_info" in node.meta:
+        weight_index = node.meta["target_dtype_info"].get("weight_index", None)
+    if weight_index is not None and weight_index < len(node.args) and node.args[weight_index] is arg:
+        return True
+    return node.kwargs.get("weight") is arg
 
-def node_arg_is_bias(node: Node, arg: Any, backend_config: BackendConfig) -> bool:
+def node_arg_is_bias(node: Node, arg: Any) -> bool:
     """Returns if node arg is bias"""
-    if isinstance(node, Node) and node.op == "call_function" and node.target in backend_config.configs:
-        bias_index = backend_config.configs[node.target]._input_type_to_index.get("bias")
-        if bias_index is not None and bias_index < len(node.args) and node.args[bias_index] is arg:
-            return True
-        return node.kwargs.get("bias") is arg
-    return False
-
-def graph_pretty_str(g, shorten=True) -> str:
-    """Returns a printable representation of the ops in the graph of g.
-    If shorten is True, tries to abbreviate fields.
-    """
-    built_in_func_re = re.compile('<built-in function (.*)>')
-    built_in_meth_re = re.compile('<built-in method (.*) of type.*>')
-    op_dict = {
-        'placeholder': 'plchdr',
-        'get_attr': 'gt_prm',
-        'call_function': 'cl_fun',
-        'call_module': 'cl_mod',
-        'call_method': 'cl_meth',
-    }
-
-    max_lens = {}
-    col_names = ("name", "op", "target", "args", "kwargs")
-    for s in col_names:
-        max_lens[s] = len(s)
-
-    results = []
-    for n in g.nodes:
-
-        # activation_post_process_0 -> obs_0
-        name = str(n.name)
-        if shorten:
-            name = name.replace("activation_post_process", "obs")
-
-        op = str(n.op)
-        # placeholder -> plchdr, and so on
-        if shorten and op in op_dict:
-            op = op_dict[op]
-
-        target = str(n.target)
-        # <built-in function foo> -> <bi_fun foo>, and so on
-        if shorten:
-            built_in_func = built_in_func_re.search(target)
-            if built_in_func:
-                target = f"<bi_fun {built_in_func.group(1)}>"
-            built_in_meth = built_in_meth_re.search(target)
-            if built_in_meth:
-                target = f"<bi_meth {built_in_meth.group(1)}>"
-            target = target.replace("activation_post_process", "obs")
-
-        args = str(n.args)
-        if shorten:
-            args = args.replace("activation_post_process", "obs")
-
-        kwargs = str(n.kwargs)
-
-        # calculate maximum length of each column, so we can tabulate properly
-        for k, v in zip(col_names, (name, op, target, args, kwargs)):
-            max_lens[k] = max(max_lens[k], len(v))
-        results.append([name, op, target, args, kwargs])
-
-    res_str = ""
-    format_str = "{:<{name}} {:<{op}} {:<{target}} {:<{args}} {:<{kwargs}}\n"
-    res_str += format_str.format(*col_names, **max_lens)
-    for result in results:
-        res_str += format_str.format(*result, **max_lens)
-
-    # print an exra note on abbreviations which change attribute names,
-    # since users will have to un-abbreviate for further debugging
-    if shorten:
-        res_str += "*obs_{n} = activation_post_process_{n}\n"
-    return res_str
-
-def get_per_tensor_qparams(activation_post_process):
-    assert is_per_tensor(activation_post_process.qscheme), 'Only per tensor quantization is supported'
-    scale, zero_point = activation_post_process.calculate_qparams()
-    scale = float(scale)
-    zero_point = int(zero_point)
-    dtype = activation_post_process.dtype
-    return scale, zero_point, dtype
-
-def get_quantize_node_info(activation_post_process: Callable) -> Optional[Tuple[str, Union[Callable, str], Dict[str, Any]]]:
-    ''' Given an activation_post_process module,
-    return node_type(e.g. call_function), quantize op(e.g. quantize_per_tensor) and a dictionary
-    of extracted qparams from the module
-    '''
-    dtype = activation_post_process.dtype  # type: ignore[attr-defined]
-    compute_dtype = None
-    if hasattr(activation_post_process, "compute_dtype"):
-        compute_dtype = activation_post_process.compute_dtype  # type: ignore[attr-defined]
-    quantize_op : Optional[Union[Callable, str]] = None
-    if dtype in [torch.quint8, torch.qint8] and \
-            not hasattr(activation_post_process, 'compute_dtype'):
-        node_type = "call_function"
-        scale, zero_point = activation_post_process.calculate_qparams()  # type: ignore[attr-defined]
-        if is_per_channel(activation_post_process.qscheme):  # type: ignore[attr-defined]
-            ch_axis = int(activation_post_process.ch_axis)  # type: ignore[attr-defined]
-            qparams = {"_scale_": scale, "_zero_point_": zero_point, "_axis_": ch_axis, "_dtype_": dtype}
-            quantize_op = torch.quantize_per_channel
-        else:
-            scale = float(scale)
-            zero_point = int(zero_point)
-            qparams = {"_scale_": scale, "_zero_point_": zero_point, "_dtype_": dtype}
-            quantize_op = torch.quantize_per_tensor
-    elif compute_dtype in [torch.quint8, torch.qint8, torch.float16]:
-        # TODO(future PR): switch compute_dtype to is_dynamic
-        # dynamic quantization
-        node_type = "call_function"
-        quantize_op = torch.quantize_per_tensor_dynamic
-        # TODO: get reduce range from observer
-        # reduce_range = activation_post_process.reduce_range
-        reduce_range = torch.backends.quantized.engine in ("fbgemm", "x86")
-        qparams = {"_dtype_": compute_dtype, "_reduce_range_": reduce_range}
-    elif dtype == torch.float16:
-        node_type = "call_method"
-        quantize_op = "to"
-        qparams = {"_dtype_": dtype}
-    else:
-        warnings.warn(f"Unsupported activation_post_process in get_quantize_node_info: {activation_post_process}")
-        return None
-    return node_type, quantize_op, qparams
-
-def quantize_node(
-        in_node: Node,
-        obs_module: torch.nn.Module,
-        obs_node: Node,
-        modules: Dict[str, torch.nn.Module],
-        quantized_graph: Graph,
-        node_name_to_scope: Dict[str, Tuple[str, type]],
-        is_input: bool,
-        output_prefix: str = "_output") -> Node:
-    ''' Add quantization nodes (eg. quantize_per_tensor/per_channel) for given node to graph
-    with the qparams calculated from activation_post_process (obs_module).
-    The observer node (obs_node) is used to find the FQN of the user of act_post_process.
-    e.g. Given input `node` in `node = self.conv(x)`, insert node:
-    `quantized_node = torch.quantize_per_tensor(x, self._scale_0, self._zer_point_0, self._dtype_0)`
-    where self._scale_0, self._zero_point_0 and self._dtype_0 are
-    calculated from `obs_module`
-    '''
-    # Find the first use of the observer node, we use this to get the scope of the module.
-    if is_input:
-        # if the quantize function is at the input of op, then we find the first user of the observer_node
-        # to get the path. If a linear call_function is in the user list, we return the first instance
-        # of linear node to get the FQN.
-        users = list(obs_node.users)
-        first_linear_use_or_first_use = users[0] if users else None
-        linear_node = None
-        for n in users:
-            if n.op == "call_function" and n.target == torch.nn.functional.linear:
-                linear_node = n
-                break
-        if linear_node:
-            first_linear_use_or_first_use = linear_node
-        prefix = "_input"
-    else:
-        # if the quantize function is at the output of the op, we use the observer input node to get the path
-        first_linear_use_or_first_use = in_node
-        prefix = output_prefix
-
-    if first_linear_use_or_first_use and first_linear_use_or_first_use.name in node_name_to_scope:
-        module_path, _ = node_name_to_scope[first_linear_use_or_first_use.name]
-    else:
-        # TODO: it's not used, so actually we can skip quantization
-        # but this requires changing return type of quantize_node
-        # we can fix it later if needed
-        module_path = ""
-    root_module = modules['']
-    graph = quantized_graph
-    maybe_quantize_node_info = get_quantize_node_info(obs_module)
-    assert maybe_quantize_node_info is not None, \
-        f"Expecting quantize node info not to be None, observer: {obs_module}"
-    node_type, quantize_op, qparams = maybe_quantize_node_info
-    inputs = [in_node]
-
-    for key, value in qparams.items():
-        if key in ['_scale_', '_zero_point_']:
-            # For scale and zero_point values we register them as buffers in the root module.
-            qparam_node = create_getattr_from_value(root_module, graph, module_path + prefix + key, value)
-            inputs.append(qparam_node)
-        else:
-            # for qparams that are not scale/zero_point (like axis, dtype) we store them as literals in the graph.
-            inputs.append(value)
-    return graph.create_node(node_type, quantize_op, tuple(inputs), {})
+    bias_index = None
+    if "target_dtype_info" in node.meta:
+        bias_index = node.meta["target_dtype_info"].get("bias_index", None)
+    if bias_index is not None and bias_index < len(node.args) and node.args[bias_index] is arg:
+        return True
+    return node.kwargs.get("bias") is arg
 
 def get_custom_module_class_keys(custom_module_mapping: Dict[QuantType, Dict[Type, Type]]) -> List[Any]:
     r""" Get all the unique custom module keys in the custom config dict
@@ -303,30 +143,15 @@ def get_qconv_prepack_op(conv_op: Callable) -> Callable:
     prepack_ops = {
         torch.nn.functional.conv1d: torch.ops.quantized.conv1d_prepack,
         torch.nn.functional.conv2d: torch.ops.quantized.conv2d_prepack,
-        torch.nn.functional.conv3d: torch.ops.quantized.conv3d_prepack
+        torch.nn.functional.conv3d: torch.ops.quantized.conv3d_prepack,
+        torch.nn.functional.conv_transpose1d: torch.ops.quantized.conv_transpose1d_prepack,
+        torch.nn.functional.conv_transpose2d: torch.ops.quantized.conv_transpose2d_prepack,
+        torch.nn.functional.conv_transpose3d: torch.ops.quantized.conv_transpose3d_prepack,
     }
     prepack_op = prepack_ops.get(conv_op, None)
-    assert prepack_op, "Didn't find prepack op for {}".format(conv_op)
+    assert prepack_op, f"Didn't find prepack op for {conv_op}"
     return prepack_op
 
-def get_qconv_op(conv_op: Callable, has_relu: bool) -> Callable:
-    qconv_op = {
-        # has relu
-        True: {
-            torch.nn.functional.conv1d: torch.ops.quantized.conv1d_relu,
-            torch.nn.functional.conv2d: torch.ops.quantized.conv2d_relu,
-            torch.nn.functional.conv3d: torch.ops.quantized.conv3d_relu
-        },
-        False: {
-            torch.nn.functional.conv1d: torch.ops.quantized.conv1d,
-            torch.nn.functional.conv2d: torch.ops.quantized.conv2d,
-            torch.nn.functional.conv3d: torch.ops.quantized.conv3d
-        }
-    }
-    qconv = qconv_op[has_relu].get(conv_op)
-    assert qconv, "Can't find corresponding quantized conv op for {} {}".format(conv_op, has_relu)
-    return qconv
-
 # Returns a function that can get a new attribute name for module with given
 # prefix, for example,
 # >> get_new_observer_name = get_new_attr_name_with_prefix('_observer')
@@ -384,7 +209,7 @@ def graph_module_from_producer_nodes(
       A graph module constructed from the producer nodes
     '''
     assert len(producer_nodes) > 0, 'list of producer nodes can not be empty'
-    # since we traced back from node to getattrr
+    # since we traced back from node to getattr
     producer_nodes.reverse()
     graph = Graph()
     env: Dict[Any, Any] = {}
@@ -406,7 +231,7 @@ def assert_and_get_unique_device(module: torch.nn.Module) -> Any:
         {p.device for p in module.buffers()}
     assert len(devices) <= 1, (
         "prepare only works with cpu or single-device CUDA modules, "
-        "but got devices {}".format(devices)
+        f"but got devices {devices}"
     )
     device = next(iter(devices)) if len(devices) > 0 else None
     return device
@@ -426,25 +251,6 @@ def create_getattr_from_value(module: torch.nn.Module, graph: Graph, prefix: str
     attr_node = graph.create_node("get_attr", attr_name)
     return attr_node
 
-def create_qparam_nodes(
-        node_name: str,
-        scale: Any,
-        zero_point: Any,
-        modules: Dict[str, torch.nn.Module],
-        quantized_graph: Graph,
-        node_name_to_scope: Dict[str, Tuple[str, type]]
-) -> Tuple[Node, Node]:
-    """
-    Create getattr nodes in the quantized graph for scale and zero point values.
-    The nodes are registered with the root_module of the model.
-    """
-    root_module = modules['']
-    module_path, _ = node_name_to_scope[node_name]
-    scale_node = create_getattr_from_value(root_module, quantized_graph, (module_path + "_scale_"), scale)
-    zero_point_node = create_getattr_from_value(root_module, quantized_graph, (module_path + "_zero_point_"), zero_point)
-    return (scale_node, zero_point_node)
-
-
 def all_node_args_have_no_tensors(node: Node, modules: Dict[str, torch.nn.Module], cache: Dict[Node, bool]) -> bool:
     """
     If we know for sure that all of this node's args have no
@@ -462,7 +268,7 @@ def all_node_args_have_no_tensors(node: Node, modules: Dict[str, torch.nn.Module
         result = False
     elif node.op == 'call_module':
         assert isinstance(node.target, str)
-        if is_activation_post_process(modules[node.target]):
+        if _is_activation_post_process(modules[node.target]):
             result = all_node_args_have_no_tensors(node.args[0], modules, cache)  # type: ignore[arg-type]
     elif node.op == 'call_module':
         result = False
@@ -589,22 +395,6 @@ def get_non_observable_arg_indexes_and_types(node: Node) -> Dict[Union[type, tor
 
     return NON_OBSERVABLE_ARG_DICT.get(info, EMPTY_ARG_DICT)
 
-def node_return_type_is_int(node: Node) -> bool:
-    """
-    Returns true if this node results in an integer, even if some of the args
-    are Tensors.
-    """
-    return node.op == 'call_method' and node.target == 'size'
-
-
-def is_get_tensor_info_node(node: Node) -> bool:
-    """ Returns True if this node is a node that takes a Tensor as input and output some
-    meta information about the Tensor, e.g. shape, size etc.
-    """
-    result: bool = \
-        node.op == "call_function" and node.target == getattr and node.args[1] == "shape"  # type: ignore[assignment]
-    return result
-
 def maybe_get_next_module(
     node: Node,
     modules: Dict[str, nn.Module],
@@ -620,7 +410,7 @@ def maybe_get_next_module(
         target_functional_type: Functional type that we want to check
     """
 
-    for user, _ in node.users.items():
+    for user in node.users.keys():
         if user.op == 'call_module' and target_module_type is not None and \
            isinstance(modules[str(user.target)], target_module_type):
             return user
@@ -667,13 +457,32 @@ def _is_custom_module_lstm(
     """
     mod = _get_module(node, named_modules)
     if qconfig is not None and qhandler is not None:
-        assert isinstance(qhandler, torch.ao.quantization.fx.quantization_patterns.QuantizeHandler)  # type: ignore[attr-defined]
+        assert isinstance(qhandler, torch.ao.quantization.fx.quantize_handler.QuantizeHandler)  # type: ignore[attr-defined]
         return isinstance(mod, torch.nn.LSTM) and \
             activation_is_statically_quantized(qconfig) and \
             qhandler.is_custom_module()
     else:
         return isinstance(mod, torch.ao.nn.quantizable.LSTM)
 
+def _is_custom_module_mha(
+        node: Node,
+        named_modules: Dict[str, torch.nn.Module],
+        qconfig: QConfigAny = None,
+        # QuantizeHandler, but we cannot include the type here due to circular imports
+        qhandler: Optional[Any] = None,
+) -> bool:
+    """
+    Return whether this refers to the custom module MultiheadAttention flow.
+    """
+    mod = _get_module(node, named_modules)
+    if qconfig is not None and qhandler is not None:
+        assert isinstance(qhandler, torch.ao.quantization.fx.quantize_handler.QuantizeHandler)  # type: ignore[attr-defined]
+        return isinstance(mod, torch.nn.MultiheadAttention) and \
+            activation_is_statically_quantized(qconfig) and \
+            qhandler.is_custom_module()
+    else:
+        return isinstance(mod, torch.ao.nn.quantizable.MultiheadAttention)
+
 def _get_module(node: Node, named_modules: Dict[str, torch.nn.Module]) -> Optional[torch.nn.Module]:
     """
     If `node` refers to a call_module node, return the module, else None.
@@ -711,7 +520,7 @@ def _insert_dequant_stubs_for_custom_module_lstm_output(
     """
     Insert DeQuantStubs after each internal output node of custom module LSTM.
 
-    Custom module LSTM outputs are nested tuples of the sturcture (output, (hidden0, hidden1)),
+    Custom module LSTM outputs are nested tuples of the structure (output, (hidden0, hidden1)),
     Since we cannot dequantize a tuple as a whole, we must first break down the tuple into its
     components through `getitem`. This function transforms the graph as follows:
 
@@ -977,10 +786,13 @@ def _qconfig_satisfies_dtype_config_constraints(
 
         1. QConfig specified a quantization range that falls within the backend's, if any
         2. QConfig specified a min scale value that is >= the backend's, if any
+        3. QConfig specified a FixedQParamsObserver or FixedQParamsFakeQuantize that has
+           scale and zero point that match the backend's, if any
 
     If `is_activation` is True, we check `qconfig.activation`, else we check `qconfig.weight`.
     If `qconfig` or `dtype_with_constraints.dtype` is None, or the dtypes do not match, return True.
     """
+    # TODO: log warnings only when the user enabled a debug flag
     def _activation_post_process_satisfies_dtype_config_constraints(
             activation_post_process: Union[ObserverBase, FakeQuantizeBase],
             dtype_with_constraints: DTypeWithConstraints,
@@ -994,27 +806,58 @@ def _qconfig_satisfies_dtype_config_constraints(
         backend_quant_min = dtype_with_constraints.quant_min_lower_bound
         backend_quant_max = dtype_with_constraints.quant_max_upper_bound
         backend_scale_min = dtype_with_constraints.scale_min_lower_bound
+        backend_scale_exact_match = dtype_with_constraints.scale_exact_match
+        backend_zero_point_exact_match = dtype_with_constraints.zero_point_exact_match
         # check quantization ranges
         if backend_quant_min is not None and backend_quant_max is not None:
             if app_quant_min is None or app_quant_max is None:
-                warnings.warn("QConfig %s must specify 'quant_min' and 'quant_max', ignoring %s" %
-                              (debug_string, qconfig))
+                warnings.warn(f"QConfig {debug_string} must specify 'quant_min' and 'quant_max', ignoring {qconfig}")
                 return False
             elif app_quant_min < backend_quant_min or app_quant_max > backend_quant_max:
-                warnings.warn(("QConfig %s quantization range must fall within the backend's:\n"
-                              "QConfig range = (%s, %s), BackendConfig range = (%s, %s), ignoring %s") %
-                              (debug_string, app_quant_min, app_quant_max,
-                              backend_quant_min, backend_quant_max, qconfig))
+                warnings.warn(
+                    f"QConfig {debug_string} quantization range must fall within the backend's:\n"
+                    f"QConfig range = ({app_quant_min}, {app_quant_max}), "
+                    f"BackendConfig range = ({backend_quant_min}, {backend_quant_max}), "
+                    f"ignoring {qconfig}"
+                )
                 return False
         # check scale min
         if backend_scale_min is not None:
             if app_scale_min is None:
-                warnings.warn("QConfig %s must specify 'eps', ignoring %s" % (debug_string, qconfig))
+                warnings.warn(f"QConfig {debug_string} must specify 'eps', ignoring {qconfig}")
                 return False
-            elif app_scale_min < backend_scale_min:
-                warnings.warn(("QConfig %s eps (%s) must be greater than or equal to "
-                              "the backend's min scale value (%s), ignoring %s") %
-                              (debug_string, app_scale_min, backend_scale_min, qconfig))
+            if app_scale_min < backend_scale_min:
+                warnings.warn(
+                    f"QConfig {debug_string} eps ({app_scale_min}) must be greater than or equal to "
+                    f"the backend's min scale value ({backend_scale_min}), ignoring {qconfig}"
+                )
+                return False
+        # check fixed scale and zero point
+        if backend_scale_exact_match is not None and backend_zero_point_exact_match is not None:
+            # For tests only, accept the following qconfigs for now
+            # TODO: handle fp16 qconfigs properly
+            for accepted_qconfig in [float16_static_qconfig, float16_dynamic_qconfig]:
+                if qconfig_equals(qconfig, accepted_qconfig):
+                    return True
+            suggestion_str = (
+                "Please use torch.ao.quantization.get_default_qconfig_mapping or "
+                "torch.ao.quantization.get_default_qat_qconfig_mapping. Example:\n"
+                "    qconfig_mapping = get_default_qconfig_mapping(\"fbgemm\")\n"
+                "    model = prepare_fx(model, qconfig_mapping, example_inputs)"
+            )
+            if not isinstance(activation_post_process, FixedQParamsObserver) and \
+                    not isinstance(activation_post_process, FixedQParamsFakeQuantize):
+                warnings.warn(
+                    f"QConfig must specify a FixedQParamsObserver or a FixedQParamsFakeQuantize "
+                    f"for fixed qparams ops, ignoring {qconfig}.\n{suggestion_str}"
+                )
+                return False
+            if observer.scale != backend_scale_exact_match or observer.zero_point != backend_zero_point_exact_match:
+                warnings.warn(
+                    f"QConfig fixed scale ({observer.scale}) and zero point ({observer.zero_point}) "
+                    f"do not match the backend's ({backend_scale_exact_match} and {backend_zero_point_exact_match}), "
+                    f"ignoring {qconfig}.\n{suggestion_str}"
+                )
                 return False
         return True
 
@@ -1026,7 +869,7 @@ def _qconfig_satisfies_dtype_config_constraints(
     satisfies_constraints = True
     if activation_post_process_ctr is not None:
         activation_post_process = activation_post_process_ctr()
-        assert is_activation_post_process(activation_post_process)
+        assert _is_activation_post_process(activation_post_process)
         # If dtypes don't match, don't check the activation_post_process and return True early
         if activation_post_process.dtype != dtype_with_constraints.dtype:
             return True
