@@ -5,7 +5,6 @@
 #include <torch/csrc/DynamicTypes.h>
 #include <torch/csrc/Exceptions.h>
 #include <torch/csrc/Layout.h>
-#include <torch/csrc/PythonTypes.h>
 #include <torch/csrc/Storage.h>
 #include <torch/csrc/autograd/generated/VariableType.h>
 #include <torch/csrc/utils/cuda_enabled.h>
@@ -34,17 +33,21 @@ std::array<THPLayout*, static_cast<int>(at::Layout::NumOptions)>
 at::DeprecatedTypeProperties* get_type_properties(
     at::DeviceType device_type,
     at::ScalarType scalarType) {
-  at::Backend backend;
+  at::Backend backend = at::Backend::Undefined;
   if (device_type == at::kCPU) {
     backend = at::Backend::CPU;
   } else if (device_type == at::kCUDA) {
     backend = at::Backend::CUDA;
   } else if (device_type == at::kXPU) {
     backend = at::Backend::XPU;
+  } else if (device_type == at::kHPU) {
+    backend = at::Backend::HPU;
   } else if (device_type == at::kMPS) {
     backend = at::Backend::MPS;
   } else if (device_type == at::DeviceType::Meta) {
     backend = at::Backend::Undefined;
+  } else if (device_type == at::DeviceType::PrivateUse1) {
+    backend = at::Backend::PrivateUse1;
   } else {
     TORCH_CHECK(false, "Invalid device for storage: ", device_type);
   }
@@ -77,22 +80,20 @@ THPLayout* getTHPLayout(at::Layout layout) {
 }
 
 PyObject* createPyObject(const at::Storage& storage) {
-  if (storage.device_type() != at::DeviceType::Meta &&
-      storage.data() == nullptr && storage.sym_nbytes() != 0 &&
-      // Grabbing storage() from FunctionalTensorWrapper is allowed.
-      // This is useful for checking aliasing info from python
-      dynamic_cast<at::functionalization::FunctionalStorageImpl*>(
-          storage.unsafeGetStorageImpl()) == nullptr) {
-    TORCH_CHECK_NOT_IMPLEMENTED(
-        false,
-        "python bindings to nullptr storage (e.g., from torch.Tensor._make_wrapper_subclass) are currently unsafe and thus disabled.  See https://github.com/pytorch/pytorch/issues/61669 for more details");
-  }
+  // Note [Invalid Python Storages]
+  // When a user creates a python tensor wrapper subclass, the subclass
+  // is a tensor object that has a nullptr storage.
+  // We still allow users to call `my_subclass.untyped_storage()`, and get back
+  // a valid storage object (this can be useful for detecting aliasing
+  // information about storages from python). However, any accesses to the
+  // data_ptr is not allowed, through methods like
+  // x.untyped_storage().data_ptr()
   PyTypeObject* type = reinterpret_cast<PyTypeObject*>(THPStorageClass);
   auto obj = THPObjectPtr(type->tp_alloc(type, 0));
   if (!obj)
     throw python_error();
-  ((THPVoidStorage*)obj.get())->cdata =
-      at::Storage(/* copy */ storage).unsafeReleaseStorageImpl();
+  ((THPStorage*)obj.get())->cdata =
+      c10::MaybeOwned<at::Storage>::owned(at::Storage(/* copy */ storage));
   return obj.release();
 }
 
@@ -127,7 +128,7 @@ at::Storage createStorageGetType(
     at::ScalarType& scalar_type,
     bool& is_typed_storage) {
   is_typed_storage = PyObject_TypeCheck(obj, getTypedStorageTypeObject());
-  PyObject* untyped_storage_obj;
+  PyObject* untyped_storage_obj = nullptr;
 
   if (is_typed_storage) {
     // NOTE: `PyObject_GetAttrString` increments the refcounts to `dtype` and
@@ -154,18 +155,15 @@ at::Storage createStorageGetType(
     throw TypeError("not a storage '%s'", Py_TYPE(obj)->tp_name);
   }
 
-  c10::StorageImpl* impl = static_cast<c10::StorageImpl*>(
-      ((THPVoidStorage*)untyped_storage_obj)->cdata);
-  c10::DeviceType device_type = impl->device().type();
-
+  const auto& storage = THPStorage_Unpack(untyped_storage_obj);
+  c10::DeviceType device_type = storage.device().type();
   auto type_properties = get_type_properties(device_type, at::kByte);
-
   return type_properties->unsafeStorageFromTH(
-      ((THPVoidStorage*)untyped_storage_obj)->cdata, true);
+      storage.unsafeGetStorageImpl(), true);
 }
 
 at::Storage createStorage(PyObject* obj) {
-  at::ScalarType scalar_type;
+  at::ScalarType scalar_type = at::ScalarType::Undefined;
   bool is_typed_storage = false;
   return createStorageGetType(obj, scalar_type, is_typed_storage);
 }
