@@ -21,6 +21,7 @@
 
 #include <ATen/ATen.h>
 #include <ATen/MapAllocator.h>
+#include <ATen/StorageUtils.h>
 #include <torch/csrc/utils/pycfunction_helpers.h>
 #include <torch/csrc/utils/python_arg_parser.h>
 #include <torch/csrc/utils/python_numbers.h>
@@ -38,17 +39,25 @@
 #define LSEEK lseek
 #endif
 
-static PyObject* THPStorage_nbytes(PyObject* _self, PyObject* noargs) {
+static PyObject* THPStorage_nbytes(PyObject* self, PyObject* noargs) {
   HANDLE_TH_ERRORS
-  auto self = (THPStorage*)_self;
-  return THPUtils_packUInt64(self->cdata->nbytes());
+  return py::cast(THPStorage_Unpack(self).sym_nbytes()).release().ptr();
   END_HANDLE_TH_ERRORS
 }
 
-static PyObject* THPStorage_dataPtr(PyObject* _self, PyObject* noargs) {
+static PyObject* THPStorage_dataPtr(PyObject* self, PyObject* noargs) {
   HANDLE_TH_ERRORS
-  auto self = (THPStorage*)_self;
-  return PyLong_FromVoidPtr(self->cdata->data<uint8_t>());
+  // PyLong_FromVoidPtr should not need to mutate the pointer in order
+  // to extract a new long object from it.
+
+  auto self_ = THPStorage_Unpack(self);
+  // See Note [Invalid Python Storages]
+  auto invalid = self_.data() == nullptr &&
+      self_.device_type() != c10::DeviceType::Meta && self_.sym_nbytes() != 0;
+  TORCH_CHECK(
+      !invalid,
+      "Attempted to access the data pointer on an invalid python storage.")
+  return PyLong_FromVoidPtr(self_.mutable_data());
   END_HANDLE_TH_ERRORS
 }
 
@@ -69,9 +78,15 @@ static PyObject* THPStorage_copy_(
   at::Storage src = r.storage(0);
   bool non_blocking = r.toBoolOptional(1).value_or(false);
 
+  // See Note [Invalid Python Storages]
+  auto invalid = src.data() == nullptr &&
+      src.device_type() != c10::DeviceType::Meta && src.sym_nbytes() != 0;
+  TORCH_CHECK(
+      !invalid, "Attempted to call copy_() on an invalid python storage.")
+
   TORCH_CHECK(self_.nbytes() == src.nbytes(), "size does not match");
 
-  storage_copy(self_, src, non_blocking);
+  at::storage_copy(self_, src, non_blocking);
 
   Py_INCREF(self);
   return self;
@@ -79,51 +94,43 @@ static PyObject* THPStorage_copy_(
   END_HANDLE_TH_ERRORS
 }
 
-static PyObject* THPStorage_isPinned(PyObject* _self, PyObject* noargs) {
-  HANDLE_TH_ERRORS
-#if defined(USE_CUDA)
-  auto self = (THPStorage*)_self;
-  return PyBool_FromLong(
-      at::globalContext().isPinnedPtr(self->cdata->data<uint8_t>()));
-#else
-  Py_RETURN_FALSE;
-#endif
-  END_HANDLE_TH_ERRORS
-}
-
 static PyObject* THPStorage_elementSize(PyObject* _self, PyObject* noargs) {
   HANDLE_TH_ERRORS
   return THPUtils_packInt64(sizeof(uint8_t));
   END_HANDLE_TH_ERRORS
 }
 
-static PyObject* THPStorage_new(PyObject* _self, PyObject* noargs) {
+static PyObject* THPStorage_new(PyObject* self, PyObject* noargs) {
   HANDLE_TH_ERRORS
-  auto self = (THPStorage*)_self;
-  c10::Allocator* allocator = self->cdata->allocator();
+  c10::Allocator* allocator = THPStorage_Unpack(self).allocator();
   auto new_storage = c10::make_intrusive<at::StorageImpl>(
       c10::StorageImpl::use_byte_size_t(),
       0,
       allocator,
       /*resizable=*/true);
 
-  // NOLINTNEXTLINE(cppcoreguidelines-init-variables)
   return THPStorage_New(std::move(new_storage));
   END_HANDLE_TH_ERRORS
 }
 
-static PyObject* THPStorage_resize_(PyObject* _self, PyObject* number_arg) {
+static PyObject* THPStorage_resize_(PyObject* self, PyObject* number_arg) {
   HANDLE_TH_ERRORS
-  auto self = (THPStorage*)_self;
+  const auto& storage = THPStorage_Unpack(self);
+  // See Note [Invalid Python Storages]
+  auto invalid = storage.data() == nullptr &&
+      storage.device_type() != c10::DeviceType::Meta &&
+      storage.sym_nbytes() != 0;
+  TORCH_CHECK(
+      !invalid, "Attempted to call resize_() on an invalid python storage.")
   THPUtils_assert(
       THPUtils_checkLong(number_arg),
       "resize_ expects an int, "
       "but got %s",
       THPUtils_typename(number_arg));
   int64_t newsize = THPUtils_unpackLong(number_arg);
-  c10::DeviceType device_type = self->cdata->device_type();
+  c10::DeviceType device_type = storage.device_type();
   if (device_type == at::kCPU) {
-    at::native::resize_bytes_cpu(self->cdata, newsize);
+    at::native::resize_bytes_cpu(storage.unsafeGetStorageImpl(), newsize);
 #ifdef USE_CUDA
   } else if (device_type == at::kCUDA) {
     ptrdiff_t size_bytes_i = newsize;
@@ -133,8 +140,38 @@ static PyObject* THPStorage_resize_(PyObject* _self, PyObject* number_arg) {
         size_bytes_i,
         ") cannot be represented as a size_t");
     const auto size_bytes = static_cast<size_t>(size_bytes_i);
-    at::native::resize_bytes_cuda(self->cdata, size_bytes);
+    at::native::resize_bytes_cuda(storage.unsafeGetStorageImpl(), size_bytes);
 #endif
+  } else if (device_type == at::kMeta) {
+    at::native::resize_bytes_meta(storage.unsafeGetStorageImpl(), newsize);
+  } else if (device_type == at::kXPU || device_type == at::kPrivateUse1) {
+    ptrdiff_t size_bytes_i = newsize;
+    TORCH_CHECK(
+        !c10::overflows<int64_t>(size_bytes_i),
+        "Requested storage size (",
+        size_bytes_i,
+        ") cannot be represented as a int64_t");
+    const auto size_bytes = static_cast<int64_t>(size_bytes_i);
+    void* original_data_ptr = storage.data_ptr().get();
+
+    auto src_option =
+        c10::TensorOptions().device(storage.device()).dtype(at::kByte);
+    auto src_tensor = at::empty({0}, {}, src_option).set_(storage);
+    src_tensor.resize_({size_bytes});
+
+    // When using resize_ to replace resize_bytes_xxx, in some cases
+    // the original data_ptr is still returned, which is an inconsistent
+    // behavior when compared to resize_bytes_xxx. For these cases,
+    // an additional memory copy and update for storage are required.
+    if (original_data_ptr == src_tensor.storage().data_ptr().get()) {
+      auto new_tensor = at::empty(src_tensor.sizes(), src_tensor.options());
+      new_tensor.copy_(src_tensor);
+      storage.set_data_ptr_noswap(
+          std::move(new_tensor.storage().mutable_data_ptr()));
+      storage.unsafeGetStorageImpl()->set_allocator(
+          new_tensor.storage().unsafeGetStorageImpl()->allocator());
+      storage.set_nbytes(new_tensor.storage().nbytes());
+    }
   } else {
     TORCH_CHECK(
         false,
@@ -142,23 +179,27 @@ static PyObject* THPStorage_resize_(PyObject* _self, PyObject* number_arg) {
         device_type);
   }
   Py_INCREF(self);
-  return (PyObject*)self;
+  return self;
   END_HANDLE_TH_ERRORS
 }
 
-static PyObject* THPStorage_fill_(PyObject* _self, PyObject* number_arg) {
+static PyObject* THPStorage_fill_(PyObject* self, PyObject* number_arg) {
   HANDLE_TH_ERRORS
-  auto self = (THPStorage*)_self;
+  const auto& storage = THPStorage_Unpack(self);
+  // See Note [Invalid Python Storages]
+  auto invalid = storage.data() == nullptr &&
+      storage.device_type() != c10::DeviceType::Meta &&
+      storage.sym_nbytes() != 0;
+  TORCH_CHECK(
+      !invalid, "Attempted to call fill_() on an invalid python storage.")
   THPUtils_assert(
       THPByteUtils_checkReal(number_arg),
       "fill_ expects int, "
       "but got %s",
       THPUtils_typename(number_arg));
-  storage_fill(
-      at::unsafeStorageFromTH(self->cdata, /*retain=*/true),
-      THPByteUtils_unpackReal(number_arg));
+  storage_fill(storage, THPByteUtils_unpackReal(number_arg));
   Py_INCREF(self);
-  return (PyObject*)self;
+  return self;
   END_HANDLE_TH_ERRORS
 }
 
@@ -173,18 +214,17 @@ static PyObject* THPStorage_fromBuffer(
   PyObject* dtype_obj = nullptr;
   c10::ScalarType scalar_type = at::kByte;
   Py_buffer buffer = {};
-  // NOLINTNEXTLINE(cppcoreguidelines-avoid-c-arrays,modernize-avoid-c-arrays,clang-diagnostic-writable-strings)
-  static char* kwlist[] = {
+  // NOLINTNEXTLINE(cppcoreguidelines-avoid-c-arrays,modernize-avoid-c-arrays)
+  constexpr const char* kwlist[] = {
       "buffer", "byte_order", "count", "offset", "dtype", nullptr};
-  // NOLINTNEXTLINE(cppcoreguidelines-init-variables)
-  const char* argtypes;
-  argtypes = "O|snnO";
+  constexpr const char* argtypes = "O|snnO";
 
   if (!PyArg_ParseTupleAndKeywords(
           args,
           keywds,
           argtypes,
-          kwlist,
+          // NOLINTNEXTLINE(cppcoreguidelines-pro-type-const-cast)
+          const_cast<char**>(kwlist),
           &obj,
           &byte_order_str,
           &count,
@@ -199,21 +239,26 @@ static PyObject* THPStorage_fromBuffer(
   auto dtype = reinterpret_cast<THPDtype*>(dtype_obj);
   scalar_type = dtype->scalar_type;
 
+  const bool is_endian_independent = (scalar_type == at::kByte) ||
+      (scalar_type == at::kChar) || (scalar_type == at::kFloat8_e5m2) ||
+      (scalar_type == at::kFloat8_e4m3fn);
+
   TORCH_CHECK(
-      (scalar_type == at::kByte) || (scalar_type == at::kChar) ||
-          (byte_order_str != nullptr),
+      is_endian_independent || (byte_order_str != nullptr),
       "function missing required argument 'byte_order' (pos 2)");
   size_t element_size = c10::elementSize(scalar_type);
 
-  // NOLINTNEXTLINE(cppcoreguidelines-init-variables)
-  torch::utils::THPByteOrder byte_order;
-  if (scalar_type != at::kByte && scalar_type != at::kChar) {
+  bool do_byte_swap = false;
+  if (!is_endian_independent) {
     if (strcmp(byte_order_str, "native") == 0) {
-      byte_order = torch::utils::THP_nativeByteOrder();
+      do_byte_swap = false;
     } else if (strcmp(byte_order_str, "big") == 0) {
-      byte_order = torch::utils::THP_BIG_ENDIAN;
+      do_byte_swap =
+          (torch::utils::THP_LITTLE_ENDIAN ==
+           torch::utils::THP_nativeByteOrder());
     } else if (strcmp(byte_order_str, "little") == 0) {
-      byte_order = torch::utils::THP_LITTLE_ENDIAN;
+      do_byte_swap =
+          (torch::utils::THP_BIG_ENDIAN == torch::utils::THP_nativeByteOrder());
     } else {
       PyErr_Format(
           PyExc_ValueError,
@@ -237,8 +282,7 @@ static PyObject* THPStorage_fromBuffer(
     return nullptr;
   }
 
-  // NOLINTNEXTLINE(cppcoreguidelines-init-variables)
-  size_t size_bytes;
+  size_t size_bytes = 0;
   if (count < 0) {
     if ((buffer.len - offset) % element_size != 0) {
       PyErr_SetString(
@@ -251,7 +295,7 @@ static PyObject* THPStorage_fromBuffer(
       return nullptr;
     }
     size_bytes = buffer.len - offset;
-    count = size_bytes / element_size;
+    count = static_cast<Py_ssize_t>(size_bytes / element_size);
   } else {
     size_bytes = count * element_size;
   }
@@ -275,41 +319,71 @@ static PyObject* THPStorage_fromBuffer(
       c10::GetDefaultCPUAllocator(),
       /*resizable=*/true);
 
-  if (scalar_type == at::kByte || scalar_type == at::kChar) {
-    memcpy(storage->data(), src + offset, count);
+  if (is_endian_independent) {
+    memcpy(storage->mutable_data(), src + offset, count);
   } else if (scalar_type == at::kBool) {
     // Because of ASAN checks, that are failing whenever
     // we are trying to get a value which is not 0 or 1, we have to manually
     // convert original values to boolean ones.
     torch::utils::THP_decodeBoolBuffer(
-        storage->data<bool>(), src + offset, byte_order, count);
+        static_cast<bool*>(storage->mutable_data()),
+        src + offset,
+        do_byte_swap,
+        count);
   } else if (scalar_type == at::kShort) {
     torch::utils::THP_decodeInt16Buffer(
-        storage->data<int16_t>(), src + offset, byte_order, count);
+        static_cast<int16_t*>(storage->mutable_data()),
+        src + offset,
+        do_byte_swap,
+        count);
   } else if (scalar_type == at::kInt) {
     torch::utils::THP_decodeInt32Buffer(
-        storage->data<int32_t>(), src + offset, byte_order, count);
+        static_cast<int32_t*>(storage->mutable_data()),
+        src + offset,
+        do_byte_swap,
+        count);
   } else if (scalar_type == at::kLong) {
     torch::utils::THP_decodeInt64Buffer(
-        storage->data<int64_t>(), src + offset, byte_order, count);
+        static_cast<int64_t*>(storage->mutable_data()),
+        src + offset,
+        do_byte_swap,
+        count);
   } else if (scalar_type == at::kHalf) {
     torch::utils::THP_decodeHalfBuffer(
-        storage->data<c10::Half>(), src + offset, byte_order, count);
+        static_cast<c10::Half*>(storage->mutable_data()),
+        src + offset,
+        do_byte_swap,
+        count);
   } else if (scalar_type == at::kBFloat16) {
     torch::utils::THP_decodeBFloat16Buffer(
-        storage->data<c10::BFloat16>(), src + offset, byte_order, count);
+        static_cast<c10::BFloat16*>(storage->mutable_data()),
+        src + offset,
+        do_byte_swap,
+        count);
   } else if (scalar_type == at::kFloat) {
     torch::utils::THP_decodeFloatBuffer(
-        storage->data<float>(), src + offset, byte_order, count);
+        static_cast<float*>(storage->mutable_data()),
+        src + offset,
+        do_byte_swap,
+        count);
   } else if (scalar_type == at::kDouble) {
     torch::utils::THP_decodeDoubleBuffer(
-        storage->data<double>(), src + offset, byte_order, count);
+        static_cast<double*>(storage->mutable_data()),
+        src + offset,
+        do_byte_swap,
+        count);
   } else if (scalar_type == at::kComplexFloat) {
     torch::utils::THP_decodeComplexFloatBuffer(
-        storage->data<c10::complex<float>>(), src + offset, byte_order, count);
+        static_cast<c10::complex<float>*>(storage->mutable_data()),
+        src + offset,
+        do_byte_swap,
+        count);
   } else if (scalar_type == at::kComplexDouble) {
     torch::utils::THP_decodeComplexDoubleBuffer(
-        storage->data<c10::complex<double>>(), src + offset, byte_order, count);
+        static_cast<c10::complex<double>*>(storage->mutable_data()),
+        src + offset,
+        do_byte_swap,
+        count);
   } else {
     TORCH_CHECK(false, "Unknown type: ", scalar_type);
   }
@@ -324,14 +398,20 @@ static PyObject* THPStorage_fromFile(
     PyObject* args,
     PyObject* keywds) {
   HANDLE_TH_ERRORS
-  // NOLINTNEXTLINE(cppcoreguidelines-init-variables)
-  const char* filename;
+  const char* filename = nullptr;
   Py_ssize_t nbytes = 0;
   int shared = 0;
-  // NOLINTNEXTLINE(cppcoreguidelines-avoid-c-arrays,modernize-avoid-c-arrays,clang-diagnostic-writable-strings)
-  static char* kwlist[] = {"filename", "shared", "nbytes", nullptr};
+  // NOLINTNEXTLINE(cppcoreguidelines-avoid-c-arrays,modernize-avoid-c-arrays)
+  constexpr const char* kwlist[] = {"filename", "shared", "nbytes", nullptr};
   if (!PyArg_ParseTupleAndKeywords(
-          args, keywds, "s|in", kwlist, &filename, &shared, &nbytes)) {
+          args,
+          keywds,
+          "s|in",
+          // NOLINTNEXTLINE(cppcoreguidelines-pro-type-const-cast)
+          const_cast<char**>(kwlist),
+          &filename,
+          &shared,
+          &nbytes)) {
     return nullptr;
   }
   if (shared)
@@ -353,9 +433,15 @@ static PyObject* THPStorage_fromFile(
   END_HANDLE_TH_ERRORS
 }
 
-PyObject* THPStorage_writeFile(PyObject* _self, PyObject* args) {
+PyObject* THPStorage_writeFile(PyObject* self, PyObject* args) {
   HANDLE_TH_ERRORS
-  auto self = (THPStorage*)_self;
+  const auto& storage = THPStorage_Unpack(self);
+  // See Note [Invalid Python Storages]
+  auto invalid = storage.data() == nullptr &&
+      storage.device_type() != c10::DeviceType::Meta &&
+      storage.sym_nbytes() != 0;
+  TORCH_CHECK(
+      !invalid, "Attempted to call _write_file() on an invalid python storage.")
   PyObject* file = PyTuple_GetItem(args, 0);
   bool is_real_file = PyTuple_GetItem(args, 1) == Py_True;
   bool save_size = PyTuple_GetItem(args, 2) == Py_True;
@@ -367,7 +453,7 @@ PyObject* THPStorage_writeFile(PyObject* _self, PyObject* args) {
 
   if (!is_real_file) {
     THPStorage_writeFileRaw<PyObject*>(
-        self->cdata, file, save_size, element_size);
+        storage.unsafeGetStorageImpl(), file, save_size, element_size);
     Py_RETURN_NONE;
   }
 
@@ -376,7 +462,8 @@ PyObject* THPStorage_writeFile(PyObject* _self, PyObject* args) {
       fd != -1,
       "_write_file couldn't retrieve a file descriptor "
       "from given object");
-  THPStorage_writeFileRaw(self->cdata, fd, save_size, element_size);
+  THPStorage_writeFileRaw(
+      storage.unsafeGetStorageImpl(), fd, save_size, element_size);
   Py_RETURN_NONE;
   END_HANDLE_TH_ERRORS
 }
@@ -403,9 +490,9 @@ PyObject* THPStorage_newWithFile(PyObject* _unused, PyObject* args) {
   END_HANDLE_TH_ERRORS
 }
 
-static PyObject* THPStorage_setFromFile(PyObject* _self, PyObject* args) {
+static PyObject* THPStorage_setFromFile(PyObject* self, PyObject* args) {
   HANDLE_TH_ERRORS
-  auto self = (THPStorage*)_self;
+  const auto& storage = THPStorage_Unpack(self);
   PyObject* file = PyTuple_GET_ITEM(args, 0);
   PyObject* offset = PyTuple_GET_ITEM(args, 1);
   bool is_real_file = PyTuple_GET_ITEM(args, 2) == Py_True;
@@ -424,11 +511,11 @@ static PyObject* THPStorage_setFromFile(PyObject* _self, PyObject* args) {
         offset == Py_None,
         "_set_from_file: offset is NYI for filelike objects");
 
-    auto self_storage =
-        c10::intrusive_ptr<c10::StorageImpl>::reclaim_copy(self->cdata);
-    auto storage = THPStorage_readFileRaw<PyObject*>(
-        file, std::move(self_storage), element_size);
-    if (!storage.defined()) {
+    auto self_storage_impl = c10::intrusive_ptr<c10::StorageImpl>::reclaim_copy(
+        storage.unsafeGetStorageImpl());
+    auto storage_impl = THPStorage_readFileRaw<PyObject*>(
+        file, std::move(self_storage_impl), element_size);
+    if (!storage_impl.defined()) {
       return nullptr;
     }
     Py_INCREF(self);
@@ -445,10 +532,11 @@ static PyObject* THPStorage_setFromFile(PyObject* _self, PyObject* args) {
       fd != -1,
       "_set_from_file couldn't retrieve a file "
       "descriptor from given object");
-  auto self_storage =
-      c10::intrusive_ptr<c10::StorageImpl>::reclaim_copy(self->cdata);
-  auto storage = THPStorage_readFileRaw<int>(fd, self_storage, element_size);
-  if (!storage.defined())
+  auto self_storage_impl = c10::intrusive_ptr<c10::StorageImpl>::reclaim_copy(
+      storage.unsafeGetStorageImpl());
+  auto storage_impl =
+      THPStorage_readFileRaw<int>(fd, self_storage_impl, element_size);
+  if (!storage_impl.defined())
     return nullptr;
   Py_INCREF(self);
 
@@ -464,7 +552,7 @@ static PyObject* THPStorage_setFromFile(PyObject* _self, PyObject* args) {
   }
   Py_DECREF(seek_return);
 
-  return (PyObject*)self;
+  return self;
   END_HANDLE_TH_ERRORS
 }
 
@@ -477,18 +565,58 @@ PyObject* THPStorage__setCdata(PyObject* _self, PyObject* new_cdata) {
       "_set_cdata - expected an int or long, but got %s",
       THPUtils_typename(new_cdata));
   c10::StorageImpl* ptr = (c10::StorageImpl*)PyLong_AsVoidPtr(new_cdata);
-  if (ptr) {
-    c10::raw::intrusive_ptr::incref(ptr);
-  }
-  if (self->cdata) {
-    c10::raw::intrusive_ptr::decref(self->cdata);
-  }
-  self->cdata = ptr;
+  self->cdata.~MaybeOwned<c10::Storage>();
+  self->cdata = c10::MaybeOwned<c10::Storage>::owned(
+      c10::Storage(c10::intrusive_ptr<c10::StorageImpl>::reclaim_copy(ptr)));
   Py_INCREF(self);
   return (PyObject*)self;
   END_HANDLE_TH_ERRORS
 }
 
+PyObject* THPStorage_byteswap(PyObject* self, PyObject* args) {
+  HANDLE_TH_ERRORS
+  THPUtils_assert(PyTuple_GET_SIZE(args) == 1, "tuple of 1 item expected");
+  PyObject* _elem_size = PyTuple_GET_ITEM(args, 0);
+  THPUtils_assert(
+      THPUtils_checkLong(_elem_size), "_byteswap(): arg must be an 'int'");
+  auto elem_size = THPUtils_unpackLong(_elem_size);
+  THPUtils_assert(
+      elem_size == 1 || elem_size == 2 || elem_size == 4 || elem_size == 8,
+      "elem_size must be 1, 2, 4, or 8");
+
+  const auto& storage = THPStorage_Unpack(self);
+  const auto nbytes = static_cast<uint64_t>(storage.nbytes());
+  const uint64_t count = nbytes / elem_size;
+
+  if (elem_size == 1) {
+    Py_RETURN_NONE;
+  }
+  THPUtils_assert(
+      nbytes % elem_size == 0,
+      "the length of data is not a multiple of %ld",
+      elem_size);
+
+  if (elem_size == 2) {
+    auto buffer = static_cast<uint16_t*>(storage.mutable_data());
+    for (uint64_t i = 0; i < count; i++, buffer++) {
+      *buffer = thp_bswap16(*buffer);
+    }
+  } else if (elem_size == 4) {
+    auto buffer = static_cast<uint32_t*>(storage.mutable_data());
+    for (uint64_t i = 0; i < count; i++, buffer++) {
+      *buffer = thp_bswap32(*buffer);
+    }
+  } else if (elem_size == 8) {
+    auto buffer = static_cast<uint64_t*>(storage.mutable_data());
+    for (uint64_t i = 0; i < count; i++, buffer++) {
+      *buffer = thp_bswap64(*buffer);
+    }
+  }
+
+  Py_RETURN_NONE;
+  END_HANDLE_TH_ERRORS
+}
+
 // NOLINTNEXTLINE(cppcoreguidelines-avoid-c-arrays,modernize-avoid-c-arrays,cppcoreguidelines-avoid-non-const-global-variables)
 static PyMethodDef THPStorage_methods[] = {
     {"copy_",
@@ -501,7 +629,6 @@ static PyMethodDef THPStorage_methods[] = {
     {"resize_", THPStorage_resize_, METH_O, nullptr},
     {"nbytes", THPStorage_nbytes, METH_NOARGS, nullptr},
     {"data_ptr", THPStorage_dataPtr, METH_NOARGS, nullptr},
-    {"is_pinned", THPStorage_isPinned, METH_NOARGS, nullptr},
     {"_write_file", THPStorage_writeFile, METH_VARARGS, nullptr},
     {"_new_with_file",
      THPStorage_newWithFile,
@@ -517,6 +644,7 @@ static PyMethodDef THPStorage_methods[] = {
      METH_VARARGS | METH_KEYWORDS | METH_STATIC,
      nullptr},
     {"_set_cdata", THPStorage__setCdata, METH_O, nullptr},
+    {"_byteswap", THPStorage_byteswap, METH_VARARGS, nullptr},
     {nullptr}};
 
 PyMethodDef* THPStorage_getMethods() {
