@@ -6,6 +6,7 @@
 #include <torch/csrc/autograd/functions/basic_ops.h>
 #include <torch/csrc/autograd/grad_mode.h>
 #include <torch/csrc/autograd/variable.h>
+#include <torch/csrc/dynamo/compiled_autograd.h>
 #include <torch/csrc/utils/memory.h>
 
 #include <ATen/DeviceGuard.h>
@@ -75,6 +76,28 @@ inline bool should_run_in_cpu_ready_queue(c10::DeviceType device) {
     return false;
   }
 }
+
+std::atomic<Engine::compiled_autograd_fn> the_compiled_autograd = nullptr;
+#define COMPILED_AUTOGRAD_POISON \
+  reinterpret_cast<Engine::compiled_autograd_fn>(1)
+std::atomic<int32_t> num_threads_in_backwards;
+struct CompiledAutogradThreadingDebugCheck {
+  CompiledAutogradThreadingDebugCheck() : incremented(true) {
+    num_threads_in_backwards++;
+  }
+  ~CompiledAutogradThreadingDebugCheck() {
+    release();
+  }
+  void release() {
+    if (std::exchange(incremented, false)) {
+      num_threads_in_backwards--;
+    }
+  }
+
+ private:
+  bool incremented;
+};
+
 } // namespace
 
 // Threads spawned by the engine are assigned a 'worker_device' specifying
@@ -347,14 +370,7 @@ void Engine::thread_init(
   // We don't have any good reason to prefer one or the other, so we've
   // arbitrarily picked to colocate devices.  Maybe the other approach is
   // better.
-
-#if defined(USE_CUDA)
-  if (at::detail::getCUDAHooks().hasPrimaryContext(device)) {
-    set_device(device);
-  }
-#else
-  set_device(device);
-#endif
+  worker_device = device;
 
   // initialize each device thread's thread local ready queue with the ready
   // queue that is created before the thread initialization
@@ -411,7 +427,7 @@ std::vector<Node*> get_current_graph_task_execution_order() {
   }
 
   // We could potentially check if there is only a single device here
-  // but explicitly require this context doens't seem bad either
+  // but explicitly require this context doesn't seem bad either
   TORCH_CHECK(
       !c10::AutogradState::get_tls_state().get_multithreading_enabled(),
       "get_current_graph_task_execution_order expects the current backward to be "
@@ -421,7 +437,8 @@ std::vector<Node*> get_current_graph_task_execution_order() {
 
   const bool check_exec_info = !task->exec_info_.empty();
   std::vector<Node*> out{};
-  std::unordered_set<Node*> seen{};
+  // Do a copy since we mutate it later
+  std::unordered_map<Node*, int> dependencies = task->dependencies_;
 
   auto compare_seq_nr = [](Node* n1, Node* n2) {
     return n1->sequence_nr() < n2->sequence_nr();
@@ -434,17 +451,14 @@ std::vector<Node*> get_current_graph_task_execution_order() {
   }
 
   // Implementation notes:
-  // - Don't need to count dependencies because we have sequence_nr
+  // - We need count dependencies even though we have sequence_nr, because
+  //   in the accumulate_grad case we cannot assume the outputs to have higher
+  //   sequence_nr than the inputs
   // - Don't need to check topological_nr because we have exec_info
   while (!heap.empty()) {
     Node* fn = heap.top();
     heap.pop();
 
-    const bool was_inserted = seen.insert(fn).second;
-    if (!was_inserted) {
-      continue;
-    }
-
     out.push_back(fn);
     for (const auto& edge : fn->next_edges()) {
       Node* next_ptr = edge.function.get();
@@ -457,7 +471,12 @@ std::vector<Node*> get_current_graph_task_execution_order() {
           continue;
         }
       }
-      heap.push(next_ptr);
+      auto it = dependencies.find(edge.function.get());
+      TORCH_INTERNAL_ASSERT(it != dependencies.end());
+      if (--it->second == 0) {
+        dependencies.erase(it);
+        heap.push(next_ptr);
+      }
     }
   }
   return out;
@@ -526,6 +545,8 @@ auto Engine::thread_main(const std::shared_ptr<GraphTask>& graph_task) -> void {
         continue;
       }
 
+      set_device(worker_device);
+
       if (task.fn_ && !local_graph_task->has_error_.load()) {
         // Set the ThreadLocalState before calling the function.
         // NB: The ThreadLocalStateGuard doesn't set the grad_mode because
@@ -779,7 +800,7 @@ void set_device(int device) {
   // as in some settings we compile with cuda, but
   // have lazy stubs for CUDA functionality (so actually
   // attempting to setup a guard(CPU_DEVICE) will cause an
-  // error, because it will still query cudaGetDevice).
+  // error, because it will still query GetDevice).
   //
   // Don't use DeviceGuard here because its destructor may be called before the
   // device is reset. This is fine because the device is thread local.
@@ -803,7 +824,7 @@ void validate_outputs(
     std::stringstream ss;
     ss << "invalid number of gradients - expected ";
     ss << edges.size() << ", but got " << grads.size();
-    AT_ERROR(format_error(ss.str()));
+    TORCH_CHECK(false, format_error(ss.str()));
   }
   for (const auto i : c10::irange(grads.size())) {
     const auto& edge = edges[i];
@@ -816,7 +837,7 @@ void validate_outputs(
       // FIXME: TestJit.test_ge_optimized fails this assertion.
       // std::stringstream ss;
       // ss << "undefined gradient at index " << i;
-      // AT_ERROR(format_error(ss.str()));
+      // TORCH_CHECK(false, format_error(ss.str()));
       continue;
     }
 
@@ -825,7 +846,7 @@ void validate_outputs(
         grad = metadata.reduce_grad(grad);
       } else {
         const auto message = metadata.incompatible_shape_error_message(i, grad);
-        AT_ERROR(format_error(message.str()));
+        TORCH_CHECK(false, format_error(message.str()));
       }
     }
 
@@ -844,12 +865,12 @@ void validate_outputs(
       std::stringstream ss;
       ss << "invalid gradient at index " << i << " - expected dtype ";
       ss << metadata.dtype() << " but got " << grad.dtype();
-      AT_ERROR(format_error(ss.str()));
+      TORCH_CHECK(false, format_error(ss.str()));
     }
     if (grad.layout() != metadata.layout()) {
       // TODO: Currently we only support (*, Sparse) combination for
       // (tensor.layout(), tensor.grad.layout()) In future, there will be an
-      // oppportunity to support more combinations of layouts if they are
+      // opportunity to support more combinations of layouts if they are
       // composable (example., operations like addition etc., are well defined
       // between tensors of different layouts.), as well as all parts of
       // autograd like AccumulateGrad correctly handle this. We allow grad to be
@@ -861,7 +882,7 @@ void validate_outputs(
         std::stringstream ss;
         ss << "invalid gradient at index " << i << " - expected layout ";
         ss << metadata.layout() << " but got " << grad.layout();
-        AT_ERROR(format_error(ss.str()));
+        TORCH_CHECK(false, format_error(ss.str()));
       }
     }
 
@@ -876,7 +897,7 @@ void validate_outputs(
           std::stringstream ss;
           ss << "invalid gradient at index " << i << " - expected device ";
           ss << metadata.device() << " but got " << grad.device();
-          AT_ERROR(format_error(ss.str()));
+          TORCH_CHECK(false, format_error(ss.str()));
         }
       }
     }
@@ -930,7 +951,6 @@ static variable_list call_function(
   });
 
   if (has_post_hooks) {
-    // NOLINTNEXTLINE(bugprone-use-after-move)
     return call_post_hooks(fn, std::move(outputs), inputs);
   }
   return outputs;
@@ -1156,6 +1176,11 @@ auto Engine::execute(
         "your parameters to None after use to break the cycle and avoid the leak.");
   }
 
+  // Allows us to assert no other threads are in backwards
+  CompiledAutogradThreadingDebugCheck _thread_check;
+  auto compiled_autograd = the_compiled_autograd.load();
+  TORCH_INTERNAL_ASSERT(compiled_autograd != COMPILED_AUTOGRAD_POISON);
+
   // accumulate_grad is true if and only if the frontend call was to
   // grad(), not backward(). grad() returns the sum of the gradients
   // w.r.t. the inputs and thus needs the inputs to be present.
@@ -1184,7 +1209,7 @@ auto Engine::execute(
       /* graph_roots */ std::move(temp_roots));
 
   // If we receive a single root, skip creating extra root node
-  bool skip_dummy_node = root_edges.size() == 1;
+  bool skip_dummy_node = root_edges.size() == 1 && compiled_autograd == nullptr;
   auto graph_root = skip_dummy_node
       ? root_edges.at(0).function
       : std::make_shared<GraphRoot>(root_edges, inputs);
@@ -1198,6 +1223,19 @@ auto Engine::execute(
         *graph_root, outputs, accumulate_grad, min_topo_nr);
   }
 
+  if (compiled_autograd != nullptr) {
+    // see [Note: Compiled Autograd]
+    TORCH_CHECK(!keep_graph, "compiled_autograd does not support keep_graph");
+    TORCH_CHECK(
+        !create_graph, "compiled_autograd does not support create_graph");
+    _thread_check.release();
+    TORCH_CHECK(
+        !AnomalyMode::is_enabled(),
+        "compiled_autograd does not support AnomalyMode")
+    return (*compiled_autograd)(
+        graph_root, *graph_task, accumulate_grad, outputs);
+  }
+
   // Queue the root
   if (skip_dummy_node) {
     InputBuffer input_buffer(root_edges.at(0).function->num_inputs());
@@ -1330,6 +1368,17 @@ Engine& Engine::get_default_engine() {
   return engine_stub.load()();
 }
 
+void Engine::set_compiled_autograd(Engine::compiled_autograd_fn fn) {
+  if (the_compiled_autograd.load() == fn) {
+    return;
+  }
+  auto prior = the_compiled_autograd.exchange(COMPILED_AUTOGRAD_POISON);
+  TORCH_CHECK(
+      num_threads_in_backwards.load() == 0 && prior != COMPILED_AUTOGRAD_POISON,
+      "compiled_autograd.enable() requires no threads in backwards()");
+  the_compiled_autograd.store(fn);
+}
+
 void Engine::queue_callback(std::function<void()> callback) {
   TORCH_CHECK(
       current_graph_task,
@@ -1501,7 +1550,7 @@ void GraphTask::init_to_execute(
   // recursion, but the actual code does this iteratively. Refer to the
   // numbering to see how the actual code corresponds. A difference to note is
   // that in the iterative version, when you are working with the current Node,
-  // you are reponsible to update your parent's is_needed after all your
+  // you are responsible to update your parent's is_needed after all your
   // children have been updated.
   //
   // is_needed = {fn: True for fn in outputs}             # (0)
