@@ -2,6 +2,7 @@
 
 #include <ATen/PythonTorchFunctionTLS.h>
 #include <ATen/SavedTensorHooks.h>
+#include <ATen/SequenceNumber.h>
 #include <ATen/autocast_mode.h>
 #include <ATen/core/PythonFallbackKernel.h>
 #include <ATen/record_function.h>
@@ -10,7 +11,9 @@
 #include <c10/core/ScalarType.h>
 #include <c10/core/impl/PythonDispatcherTLS.h>
 #include <torch/csrc/Exceptions.h>
+#include <torch/csrc/autograd/VariableTypeUtils.h>
 #include <torch/csrc/autograd/autograd.h>
+#include <torch/csrc/autograd/autograd_not_implemented_fallback.h>
 #include <torch/csrc/autograd/function.h>
 #include <torch/csrc/autograd/grad_mode.h>
 #include <torch/csrc/autograd/profiler.h>
@@ -25,15 +28,20 @@
 #include <torch/csrc/jit/python/pybind_utils.h>
 #include <torch/csrc/profiler/collection.h>
 #include <torch/csrc/profiler/kineto_shim.h>
+#include <torch/csrc/utils.h>
 #include <torch/csrc/utils/disable_torch_function.h>
 #include <torch/csrc/utils/pybind.h>
 #include <torch/csrc/utils/pycfunction_helpers.h>
+#include <torch/csrc/utils/python_raii.h>
 #include <torch/csrc/utils/python_torch_function_mode.h>
 
 #include <set>
 #include <unordered_set>
 #include <utility>
 
+using torch::impl::py_context_manager;
+using torch::impl::py_context_manager_DEPRECATED;
+
 namespace {
 
 struct DisableFuncTorch {
@@ -44,28 +52,6 @@ struct DisableFuncTorch {
   c10::impl::ExcludeDispatchKeyGuard back_guard_;
 };
 
-struct MultithreadingEnabled {
-  MultithreadingEnabled(bool enabled)
-      : old_(c10::AutogradState::get_tls_state().get_multithreading_enabled()) {
-    c10::AutogradState::get_tls_state().set_multithreading_enabled(enabled);
-  }
-  ~MultithreadingEnabled() {
-    c10::AutogradState::get_tls_state().set_multithreading_enabled(old_);
-  }
-  bool old_;
-};
-
-struct ViewReplayEnabled {
-  ViewReplayEnabled(bool enabled)
-      : old_(c10::AutogradState::get_tls_state().get_view_replay_enabled()) {
-    c10::AutogradState::get_tls_state().set_view_replay_enabled(enabled);
-  }
-  ~ViewReplayEnabled() {
-    c10::AutogradState::get_tls_state().set_view_replay_enabled(old_);
-  }
-  bool old_;
-};
-
 struct DisableAutocast {
   c10::impl::ExcludeDispatchKeyGuard guard_{c10::autocast_dispatch_keyset};
 };
@@ -92,6 +78,11 @@ struct EnablePythonDispatcher {
   c10::impl::PyInterpreter* old_;
 };
 
+struct EnablePreDispatch {
+  EnablePreDispatch() : guard_(c10::DispatchKey::PreDispatch) {}
+  c10::impl::IncludeDispatchKeyGuard guard_;
+};
+
 } // namespace
 
 PyObject* THPAutograd_initExtension(PyObject* _unused, PyObject* unused) {
@@ -173,7 +164,16 @@ PyObject* THPAutograd_initExtension(PyObject* _unused, PyObject* unused) {
       .value("HPU", c10::DeviceType::HPU)
       .value("VE", c10::DeviceType::VE)
       .value("Lazy", c10::DeviceType::Lazy)
-      .value("IPU", c10::DeviceType::IPU);
+      .value("IPU", c10::DeviceType::IPU)
+      .value("PrivateUse1", c10::DeviceType::PrivateUse1);
+
+  using torch::autograd::CreationMeta;
+  py::enum_<CreationMeta>(m, "CreationMeta")
+      .value("DEFAULT", CreationMeta::DEFAULT)
+      .value("IN_CUSTOM_FUNCTION", CreationMeta::IN_CUSTOM_FUNCTION)
+      .value("MULTI_OUTPUT_NODE", CreationMeta::MULTI_OUTPUT_NODE)
+      .value("NO_GRAD_MODE", CreationMeta::NO_GRAD_MODE)
+      .value("INFERENCE_MODE", CreationMeta::INFERENCE_MODE);
 
   py::class_<KinetoEvent>(m, "_KinetoEvent")
       // name of the event
@@ -204,6 +204,19 @@ PyObject* THPAutograd_initExtension(PyObject* _unused, PyObject* unused) {
       // shapes of input tensors
       .def("shapes", [](const KinetoEvent& e) { return e.shapes().vec(); })
       .def("dtypes", [](const KinetoEvent& e) { return e.dtypes().vec(); })
+      .def(
+          "concrete_inputs",
+          [](const KinetoEvent& e) {
+            std::vector<py::object> as_pyobj;
+            std::transform(
+                e.concreteInputs().begin(),
+                e.concreteInputs().end(),
+                std::back_inserter(as_pyobj),
+                [](const c10::IValue& val) {
+                  return torch::jit::toPyObject(val);
+                });
+            return as_pyobj;
+          })
       // stack traces of the PyTorch CPU events
       .def("stack", [](const KinetoEvent& e) { return e.stack().vec(); })
       // type of the RecordFunction that generated a PyTorch CPU event
@@ -226,9 +239,11 @@ PyObject* THPAutograd_initExtension(PyObject* _unused, PyObject* unused) {
       // Whether this is async event or not
       .def("is_async", [](const KinetoEvent& e) { return e.isAsync(); })
       .def("cuda_elapsed_us", &KinetoEvent::cudaElapsedUs)
+      .def("privateuse1_elapsed_us", &KinetoEvent::privateuse1ElapsedUs)
       .def("nbytes", [](const KinetoEvent& e) { return e.nBytes(); });
 
   m.def("_soft_assert_raises", &setSoftAssertRaises);
+  m.def("_get_sequence_nr", &at::sequence_number::peek);
 
   py::class_<ProfilerResult>(m, "_ProfilerResult")
       .def("trace_start_us", &ProfilerResult::trace_start_us)
@@ -297,6 +312,13 @@ PyObject* THPAutograd_initExtension(PyObject* _unused, PyObject* unused) {
     if (at::getNumGPUs() > 0) {
       activities.insert(ActivityType::CUDA);
     }
+#elif defined(USE_KINETO)
+    if (at::hasXPU()) {
+      activities.insert(ActivityType::XPU);
+    }
+    if (at::hasMTIA()) {
+      activities.insert(ActivityType::MTIA);
+    }
 #endif
     return activities;
   });
@@ -347,37 +369,76 @@ PyObject* THPAutograd_initExtension(PyObject* _unused, PyObject* unused) {
     torch::autograd::PyDefaultSavedVariableHooks::pop_hooks();
   });
 
+  m.def("_get_creation_meta", [](const at::Tensor& t) {
+    auto* meta = torch::autograd::impl::get_view_autograd_meta(t);
+    TORCH_CHECK(meta != nullptr);
+    return meta->get_creation_meta();
+  });
+
+  m.def(
+      "_set_creation_meta",
+      [](const at::Tensor& t, CreationMeta new_creation_meta) {
+        auto* meta = torch::autograd::impl::get_view_autograd_meta(t);
+        TORCH_CHECK(meta != nullptr);
+        meta->set_creation_meta(new_creation_meta);
+      });
+
   _C_m.def(
       "_register_py_class_for_device",
       [](const std::string& device, py::object python_type_class) {
         auto cls = python_type_class.ptr();
         registerPythonTensorClass(device, cls);
       });
+  _C_m.def("_set_autograd_fallback_mode", [](const std::string& mode) {
+    if (mode == "nothing") {
+      torch::autograd::setAutogradFallbackMode(
+          torch::autograd::AutogradFallbackMode::Nothing);
+      return;
+    }
+    if (mode == "warn") {
+      torch::autograd::setAutogradFallbackMode(
+          torch::autograd::AutogradFallbackMode::Warn);
+      return;
+    }
+    if (mode == "error") {
+      torch::autograd::setAutogradFallbackMode(
+          torch::autograd::AutogradFallbackMode::Error);
+      return;
+    }
+    TORCH_INTERNAL_ASSERT(false, "Unsupported AutogradFallbackMode: ", mode);
+  });
+  _C_m.def("_get_autograd_fallback_mode", []() {
+    auto mode = torch::autograd::getAutogradFallbackMode();
+    switch (mode) {
+      case torch::autograd::AutogradFallbackMode::Nothing:
+        return "nothing";
+      case torch::autograd::AutogradFallbackMode::Warn:
+        return "warn";
+      case torch::autograd::AutogradFallbackMode::Error:
+        return "error";
+      default:
+        TORCH_INTERNAL_ASSERT(false, "Unsupported AutogradFallbackMode");
+    }
+  });
 
   _C_m.def("_activate_cuda_trace", []() { activateCUDATrace(); });
 
-  py::class_<c10::InferenceMode>(_C_m, "_InferenceMode").def(py::init<bool>());
+  py_context_manager_DEPRECATED<c10::InferenceMode, bool>(
+      _C_m, "_InferenceMode");
+  py_context_manager<at::impl::RestorePythonTLSSnapshot>(
+      _C_m, "_RestorePythonTLSSnapshot");
 
-  py::class_<at::impl::RestorePythonTLSSnapshot>(
-      _C_m, "_RestorePythonTLSSnapshot")
-      .def(py::init<>());
-
-  py::class_<torch::DisableTorchDispatch>(_C_m, "_DisableTorchDispatch")
-      .def(py::init<>());
-  py::class_<EnableTorchFunction>(_C_m, "_EnableTorchFunction")
-      .def(py::init<>());
-  py::class_<EnablePythonDispatcher>(_C_m, "_EnablePythonDispatcher")
-      .def(py::init<>());
-  py::class_<c10::impl::DisablePythonDispatcher>(
-      _C_m, "_DisablePythonDispatcher")
-      .def(py::init<>());
-  py::class_<DisableFuncTorch>(_C_m, "_DisableFuncTorch").def(py::init<>());
-  py::class_<MultithreadingEnabled>(_C_m, "_MultithreadingEnabled")
-      .def(py::init<bool>());
-  py::class_<DisableAutocast>(std::move(_C_m), "_DisableAutocast")
-      .def(py::init<>());
-  py::class_<ViewReplayEnabled>(_C_m, "_ViewReplayEnabled")
-      .def(py::init<bool>());
+  py_context_manager_DEPRECATED<torch::DisableTorchDispatch>(
+      _C_m, "_DisableTorchDispatch");
+  py_context_manager_DEPRECATED<EnableTorchFunction>(
+      _C_m, "_EnableTorchFunction");
+  py_context_manager_DEPRECATED<EnablePythonDispatcher>(
+      _C_m, "_EnablePythonDispatcher");
+  py_context_manager<c10::impl::DisablePythonDispatcher>(
+      _C_m, "_DisablePythonDispatcher");
+  py_context_manager<EnablePreDispatch>(_C_m, "_EnablePreDispatch");
+  py_context_manager_DEPRECATED<DisableFuncTorch>(_C_m, "_DisableFuncTorch");
+  py_context_manager<DisableAutocast>(_C_m, "_DisableAutocast");
   py::class_<torch::autograd::SavedVariable>(std::move(m), "SavedTensor")
       .def(py::init([]() -> torch::autograd::SavedVariable {
         TORCH_CHECK(
@@ -426,7 +487,8 @@ static PyObject* is_autocast_enabled(PyObject* _unused, PyObject* arg) {
 static PyObject* is_any_autocast_enabled(PyObject* _unused, PyObject* arg) {
   HANDLE_TH_ERRORS
   if (at::autocast::is_enabled() || at::autocast::is_cpu_enabled() ||
-      at::autocast::is_xpu_enabled()) {
+      at::autocast::is_xpu_enabled() || at::autocast::is_ipu_enabled() ||
+      at::autocast::is_xla_enabled() || at::autocast::is_hpu_enabled()) {
     Py_RETURN_TRUE;
   } else {
     Py_RETURN_FALSE;
@@ -454,6 +516,46 @@ static PyObject* is_autocast_cpu_enabled(PyObject* _unused, PyObject* arg) {
   END_HANDLE_TH_ERRORS
 }
 
+static PyObject* set_autocast_ipu_enabled(PyObject* _unused, PyObject* arg) {
+  HANDLE_TH_ERRORS
+  if (!PyBool_Check(arg)) {
+    throw TypeError("enabled must be a bool (got %s)", Py_TYPE(arg)->tp_name);
+  }
+  at::autocast::set_ipu_enabled(arg == Py_True);
+  Py_RETURN_NONE;
+  END_HANDLE_TH_ERRORS
+}
+
+static PyObject* is_autocast_ipu_enabled(PyObject* _unused, PyObject* arg) {
+  HANDLE_TH_ERRORS
+  if (at::autocast::is_ipu_enabled()) {
+    Py_RETURN_TRUE;
+  } else {
+    Py_RETURN_FALSE;
+  }
+  END_HANDLE_TH_ERRORS
+}
+
+static PyObject* set_autocast_xla_enabled(PyObject* _unused, PyObject* arg) {
+  HANDLE_TH_ERRORS
+  if (!PyBool_Check(arg)) {
+    throw TypeError("enabled must be a bool (got %s)", Py_TYPE(arg)->tp_name);
+  }
+  at::autocast::set_xla_enabled(arg == Py_True);
+  Py_RETURN_NONE;
+  END_HANDLE_TH_ERRORS
+}
+
+static PyObject* is_autocast_xla_enabled(PyObject* _unused, PyObject* arg) {
+  HANDLE_TH_ERRORS
+  if (at::autocast::is_xla_enabled()) {
+    Py_RETURN_TRUE;
+  } else {
+    Py_RETURN_FALSE;
+  }
+  END_HANDLE_TH_ERRORS
+}
+
 static PyObject* set_autocast_gpu_dtype(PyObject* _unused, PyObject* arg) {
   HANDLE_TH_ERRORS
   if (!THPDtype_Check(arg)) {
@@ -478,6 +580,30 @@ static PyObject* set_autocast_cpu_dtype(PyObject* _unused, PyObject* arg) {
   END_HANDLE_TH_ERRORS
 }
 
+static PyObject* set_autocast_ipu_dtype(PyObject* _unused, PyObject* arg) {
+  HANDLE_TH_ERRORS
+  if (!THPDtype_Check(arg)) {
+    throw TypeError(
+        "dtype must be a torch.dtype (got %s)", Py_TYPE(arg)->tp_name);
+  }
+  at::ScalarType targetType = reinterpret_cast<THPDtype*>(arg)->scalar_type;
+  at::autocast::set_autocast_ipu_dtype(targetType);
+  Py_RETURN_NONE;
+  END_HANDLE_TH_ERRORS
+}
+
+static PyObject* set_autocast_xla_dtype(PyObject* _unused, PyObject* arg) {
+  HANDLE_TH_ERRORS
+  if (!THPDtype_Check(arg)) {
+    throw TypeError(
+        "dtype must be a torch.dtype (got %s)", Py_TYPE(arg)->tp_name);
+  }
+  at::ScalarType targetType = reinterpret_cast<THPDtype*>(arg)->scalar_type;
+  at::autocast::set_autocast_xla_dtype(targetType);
+  Py_RETURN_NONE;
+  END_HANDLE_TH_ERRORS
+}
+
 static PyObject* get_autocast_gpu_dtype(PyObject* _unused, PyObject* arg) {
   HANDLE_TH_ERRORS
   at::ScalarType current_dtype = at::autocast::get_autocast_gpu_dtype();
@@ -496,6 +622,24 @@ static PyObject* get_autocast_cpu_dtype(PyObject* _unused, PyObject* arg) {
   END_HANDLE_TH_ERRORS
 }
 
+static PyObject* get_autocast_ipu_dtype(PyObject* _unused, PyObject* arg) {
+  HANDLE_TH_ERRORS
+  at::ScalarType current_dtype = at::autocast::get_autocast_ipu_dtype();
+  auto dtype = (PyObject*)torch::getTHPDtype(current_dtype);
+  Py_INCREF(dtype);
+  return dtype;
+  END_HANDLE_TH_ERRORS
+}
+
+static PyObject* get_autocast_xla_dtype(PyObject* _unused, PyObject* arg) {
+  HANDLE_TH_ERRORS
+  at::ScalarType current_dtype = at::autocast::get_autocast_xla_dtype();
+  auto dtype = (PyObject*)torch::getTHPDtype(current_dtype);
+  Py_INCREF(dtype);
+  return dtype;
+  END_HANDLE_TH_ERRORS
+}
+
 static PyObject* clear_autocast_cache(PyObject* _unused, PyObject* arg) {
   HANDLE_TH_ERRORS
   at::autocast::clear_cache();
@@ -535,12 +679,24 @@ static PyObject* set_autocast_cache_enabled(PyObject* _unused, PyObject* arg) {
   END_HANDLE_TH_ERRORS
 }
 
-static PyObject* set_grad_enabled(PyObject* _unused, PyObject* arg) {
+static PyObject* set_grad_enabled(
+    PyObject* _unused,
+    PyObject* args,
+    PyObject* kwargs) {
   HANDLE_TH_ERRORS
-  if (!PyBool_Check(arg)) {
-    throw TypeError("enabled must be a bool (got %s)", Py_TYPE(arg)->tp_name);
+  static PythonArgParser parser({
+      "set_grad_enabled(bool enabled)",
+  });
+  ParsedArgs<1> parsed_args;
+  auto r = parser.parse(args, kwargs, parsed_args);
+
+  if (at::impl::torch_function_mode_enabled()) {
+    auto torch_C_module = THPObjectPtr(PyImport_ImportModule("torch._C"));
+    return handle_torch_function(
+        r, args, kwargs, torch_C_module, "torch._C", "_set_grad_enabled");
   }
-  GradMode::set_enabled(arg == Py_True);
+  auto grad_enabled = r.toBool(0);
+  GradMode::set_enabled(grad_enabled);
   Py_RETURN_NONE;
   END_HANDLE_TH_ERRORS
 }
@@ -575,6 +731,82 @@ static PyObject* is_fwd_grad_enabled(PyObject* _unused, PyObject* arg) {
   END_HANDLE_TH_ERRORS
 }
 
+static PyObject* set_multithreading_enabled(
+    PyObject* self,
+    PyObject* args,
+    PyObject* kwargs) {
+  HANDLE_TH_ERRORS
+  static PythonArgParser parser({
+      "set_multithreading_enabled(bool enabled)",
+  });
+  ParsedArgs<1> parsed_args;
+  auto r = parser.parse(args, kwargs, parsed_args);
+
+  if (at::impl::torch_function_mode_enabled()) {
+    auto torch_C_module = THPObjectPtr(PyImport_ImportModule("torch._C"));
+    return handle_torch_function(
+        r,
+        args,
+        kwargs,
+        torch_C_module,
+        "torch._C",
+        "_set_multithreading_enabled");
+  }
+  auto multithreading_enabled = r.toBool(0);
+  c10::AutogradState::get_tls_state().set_multithreading_enabled(
+      multithreading_enabled);
+  Py_RETURN_NONE;
+  END_HANDLE_TH_ERRORS
+}
+
+static PyObject* is_multithreading_enabled(PyObject* self, PyObject* args) {
+  HANDLE_TH_ERRORS
+  if (c10::AutogradState::get_tls_state().get_multithreading_enabled()) {
+    Py_RETURN_TRUE;
+  } else {
+    Py_RETURN_FALSE;
+  }
+  END_HANDLE_TH_ERRORS
+}
+
+static PyObject* set_view_replay_enabled(
+    PyObject* self,
+    PyObject* args,
+    PyObject* kwargs) {
+  HANDLE_TH_ERRORS
+  static PythonArgParser parser({
+      "set_view_replay_enabled(bool enabled)",
+  });
+  ParsedArgs<1> parsed_args;
+  auto r = parser.parse(args, kwargs, parsed_args);
+
+  if (at::impl::torch_function_mode_enabled()) {
+    auto torch_C_module = THPObjectPtr(PyImport_ImportModule("torch._C"));
+    return handle_torch_function(
+        r,
+        args,
+        kwargs,
+        torch_C_module,
+        "torch._C",
+        "_set_view_replay_enabled");
+  }
+  auto view_replay_enabled = r.toBool(0);
+  c10::AutogradState::get_tls_state().set_view_replay_enabled(
+      view_replay_enabled);
+  Py_RETURN_NONE;
+  END_HANDLE_TH_ERRORS
+}
+
+static PyObject* is_view_replay_enabled(PyObject* self, PyObject* args) {
+  HANDLE_TH_ERRORS
+  if (c10::AutogradState::get_tls_state().get_view_replay_enabled()) {
+    Py_RETURN_TRUE;
+  } else {
+    Py_RETURN_FALSE;
+  }
+  END_HANDLE_TH_ERRORS
+}
+
 static PyObject* is_inference_mode_enabled(PyObject* _unused, PyObject* arg) {
   HANDLE_TH_ERRORS
   if (c10::InferenceMode::is_enabled()) {
@@ -762,9 +994,21 @@ static PyObject* len_torch_dispatch_stack(
   END_HANDLE_TH_ERRORS
 }
 
+PyObject* THPModule_increment_version(PyObject* _unused, PyObject* tensor) {
+  HANDLE_TH_ERRORS
+  THPUtils_assert(
+      THPVariable_Check(tensor), "increment_version expect a Tensor as input");
+  torch::autograd::increment_version((THPVariable_Unpack(tensor)));
+  Py_RETURN_NONE;
+  END_HANDLE_TH_ERRORS
+}
+
 // autograd methods on torch._C
 static PyMethodDef methods[] = { // NOLINT
-    {"_set_grad_enabled", set_grad_enabled, METH_O, nullptr},
+    {"_set_grad_enabled",
+     castPyCFunctionWithKeywords(set_grad_enabled),
+     METH_VARARGS | METH_KEYWORDS,
+     nullptr},
     {"is_grad_enabled", is_grad_enabled, METH_NOARGS, nullptr},
     {"_set_fwd_grad_enabled", set_fwd_grad_enabled, METH_O, nullptr},
     {"_is_fwd_grad_enabled", is_fwd_grad_enabled, METH_NOARGS, nullptr},
@@ -782,6 +1026,14 @@ static PyMethodDef methods[] = { // NOLINT
     {"get_autocast_cpu_dtype", get_autocast_cpu_dtype, METH_NOARGS, nullptr},
     {"set_autocast_gpu_dtype", set_autocast_gpu_dtype, METH_O, nullptr},
     {"get_autocast_gpu_dtype", get_autocast_gpu_dtype, METH_NOARGS, nullptr},
+    {"set_autocast_xla_enabled", set_autocast_xla_enabled, METH_O, nullptr},
+    {"is_autocast_xla_enabled", is_autocast_xla_enabled, METH_NOARGS, nullptr},
+    {"set_autocast_xla_dtype", set_autocast_xla_dtype, METH_O, nullptr},
+    {"get_autocast_xla_dtype", get_autocast_xla_dtype, METH_NOARGS, nullptr},
+    {"set_autocast_ipu_enabled", set_autocast_ipu_enabled, METH_O, nullptr},
+    {"is_autocast_ipu_enabled", is_autocast_ipu_enabled, METH_NOARGS, nullptr},
+    {"set_autocast_ipu_dtype", set_autocast_ipu_dtype, METH_O, nullptr},
+    {"get_autocast_ipu_dtype", get_autocast_ipu_dtype, METH_NOARGS, nullptr},
     {"autocast_increment_nesting",
      autocast_increment_nesting,
      METH_NOARGS,
@@ -795,6 +1047,7 @@ static PyMethodDef methods[] = { // NOLINT
      METH_NOARGS,
      nullptr},
     {"set_autocast_cache_enabled", set_autocast_cache_enabled, METH_O, nullptr},
+    {"_increment_version", THPModule_increment_version, METH_O, nullptr},
     {"set_anomaly_enabled",
      castPyCFunctionWithKeywords(set_anomaly_mode_enabled),
      METH_VARARGS | METH_KEYWORDS,
@@ -804,6 +1057,19 @@ static PyMethodDef methods[] = { // NOLINT
      is_anomaly_check_nan_enabled,
      METH_NOARGS,
      nullptr},
+    {"_is_multithreading_enabled",
+     is_multithreading_enabled,
+     METH_NOARGS,
+     nullptr},
+    {"_set_multithreading_enabled",
+     castPyCFunctionWithKeywords(set_multithreading_enabled),
+     METH_VARARGS | METH_KEYWORDS,
+     nullptr},
+    {"_is_view_replay_enabled", is_view_replay_enabled, METH_NOARGS, nullptr},
+    {"_set_view_replay_enabled",
+     castPyCFunctionWithKeywords(set_view_replay_enabled),
+     METH_VARARGS | METH_KEYWORDS,
+     nullptr},
     {"_enter_dual_level", python_enter_dual_level, METH_NOARGS, nullptr},
     {"_exit_dual_level",
      castPyCFunctionWithKeywords(python_exit_dual_level),
