@@ -26,7 +26,7 @@ static PyObject* THPVariable_pynew(
   const char* name = nullptr;
 
   // NOLINTNEXTLINE(cppcoreguidelines-avoid-c-arrays,modernize-avoid-c-arrays)
-  constexpr char* accepted_args[] = {
+  constexpr const char* accepted_args[] = {
       "data", "requires_grad", "volatile", "_grad_fn", "name", nullptr};
   if (!PyArg_ParseTupleAndKeywords(
           args,
