@@ -4,6 +4,7 @@
 #ifdef USE_C10D_GLOO
 
 #include <torch/csrc/distributed/c10d/GlooDeviceFactory.hpp>
+#include <torch/csrc/distributed/c10d/PrefixStore.hpp>
 #include <chrono>
 #include <exception>
 #include <ratio>
@@ -33,8 +34,8 @@
 #include <gloo/reduce.h>
 #include <gloo/scatter.h>
 
-#include <ATen/SparseTensorUtils.h>
 #include <ATen/ThreadLocalState.h>
+#include <ATen/native/SparseTensorUtils.h>
 
 #include <c10/util/StringUtil.h>
 #include <c10/util/intrusive_ptr.h>
@@ -59,6 +60,7 @@
       func<int8_t>(__VA_ARGS__);                 \
       break;                                     \
     case ::at::ScalarType::Byte:                 \
+    case ::at::ScalarType::Bool:                 \
       func<uint8_t>(__VA_ARGS__);                \
       break;                                     \
     case ::at::ScalarType::Int:                  \
@@ -88,6 +90,7 @@
       func<int8_t>(args);                        \
       break;                                     \
     case ::at::ScalarType::Byte:                 \
+    case ::at::ScalarType::Bool:                 \
       func<uint8_t>(args);                       \
       break;                                     \
     case ::at::ScalarType::Int:                  \
@@ -105,8 +108,6 @@ namespace c10d {
 
 namespace {
 
-constexpr int kBytes = 8;
-
 using steady_clock_time_point =
     std::chrono::time_point<std::chrono::steady_clock>;
 
@@ -153,7 +154,7 @@ void checkRemainingTime(
       " ms.");
   if (remainingTime.count() < 0) {
     std::string rankInfo;
-    if (processedRanks.size() > 0) {
+    if (!processedRanks.empty()) {
       rankInfo = c10::str(
           "Successfully processed ranks: ", c10::Join(", ", processedRanks));
     } else {
@@ -445,8 +446,8 @@ std::vector<at::Tensor> ProcessGroupGloo::AsyncWork::result() {
   TORCH_CHECK(
       outputTensors_.size() <= 1,
       "work result does not support list of lists, use .getFuture() and value()");
-  return outputTensors_.size() == 0 ? std::vector<at::Tensor>()
-                                    : outputTensors_.at(0);
+  return outputTensors_.empty() ? std::vector<at::Tensor>()
+                                : outputTensors_.at(0);
 }
 
 c10::intrusive_ptr<c10::ivalue::Future> ProcessGroupGloo::AsyncWork::
@@ -468,7 +469,7 @@ c10::intrusive_ptr<c10::ivalue::Future> createFutureAsOutput(
 void returnFutureWithOutput(
     c10::intrusive_ptr<c10::ivalue::Future>& future,
     const std::vector<std::vector<at::Tensor>>& outputTensors) {
-  if (outputTensors.size() == 0) {
+  if (outputTensors.empty()) {
     future->markCompleted(c10::IValue(std::vector<at::Tensor>()));
     return;
   }
@@ -518,7 +519,7 @@ ProcessGroupGloo::AsyncWork::AsyncWork(
     // correct timestamps for work that is asynchronously executed.
     : Work(-1, OpType::UNKNOWN, nullptr, inputTensors),
       outputTensors_(std::move(outputTensors)),
-      future_(createFutureAsOutput(outputTensors)) {
+      future_(createFutureAsOutput(outputTensors_)) {
   if (profilingTitle != nullptr) {
     recordAsyncWorkProfilingInfo(profilingTitle, inputTensors);
   }
@@ -608,7 +609,7 @@ void ProcessGroupGloo::RecvWork::abort() {
 }
 
 ProcessGroupGloo::Options::Options(std::chrono::milliseconds timeout)
-    : ProcessGroup::Options(GLOO_BACKEND_NAME, timeout), threads(2) {}
+    : Backend::Options(GLOO_BACKEND_NAME, timeout), threads(2) {}
 
 namespace {
 
@@ -625,16 +626,16 @@ void socketInitialize() {
 // gracefully fall back to an alternative if it doesn't.
 bool doesHostnameResolveToUsableAddress(const std::string& hostname) {
   socketInitialize();
-  struct addrinfo hints;
+  struct addrinfo hints {};
   memset(&hints, 0, sizeof(hints));
   hints.ai_family = AF_UNSPEC;
   hints.ai_socktype = SOCK_STREAM;
-  struct addrinfo* result;
+  struct addrinfo* result = nullptr;
   auto rv = getaddrinfo(hostname.c_str(), nullptr, &hints, &result);
   if (rv < 0) {
     return false;
   }
-  struct addrinfo* rp;
+  struct addrinfo* rp = nullptr;
   for (rp = result; rp != nullptr; rp = rp->ai_next) {
     auto fd = socket(rp->ai_family, rp->ai_socktype, rp->ai_protocol);
     if (fd == -1) {
@@ -731,7 +732,7 @@ ProcessGroupGloo::ProcessGroupGloo(
     int rank,
     int size,
     c10::intrusive_ptr<Options> options)
-    : ProcessGroup(rank, size),
+    : Backend(rank, size),
       store_(new GlooStore(store)),
       options_(options),
       stop_(false),
@@ -758,7 +759,14 @@ ProcessGroupGloo::ProcessGroupGloo(
     auto context = std::make_shared<::gloo::rendezvous::Context>(rank_, size_);
     auto store = ::gloo::rendezvous::PrefixStore(std::to_string(i), *store_);
     context->setTimeout(options->timeout);
-    context->connectFullMesh(store, options->devices[i]);
+    try {
+      context->connectFullMesh(store, options->devices[i]);
+    } catch (const std::runtime_error& e) {
+      auto err = e.what();
+      // TORCH_CHECK to print the cpp stacktrace.
+      auto msg = c10::str("Gloo connectFullMesh failed with ", err);
+      logAndThrow(msg, msg);
+    }
     contexts_.push_back(std::move(context));
   }
 
@@ -1092,7 +1100,7 @@ class AsyncSparseAllreduceWork : public ProcessGroupGloo::AsyncWork {
     // Construct from an existing metadata tensor to facilitate structured
     // access to metadata from peers, after gathering it.
     explicit SparseTensorMetadata(at::Tensor metadata)
-        : metadata_(metadata), data_(metadata_.data_ptr<int64_t>()) {
+        : metadata_(metadata), data_(metadata_.mutable_data_ptr<int64_t>()) {
       AT_ASSERT(metadata.scalar_type() == at::kLong);
       AT_ASSERT(metadata.dim() == 1);
       AT_ASSERT(metadata.size(0) == dim);
@@ -1149,7 +1157,6 @@ class AsyncSparseAllreduceWork : public ProcessGroupGloo::AsyncWork {
   // Every process then sums the resulting sparse tensors locally.
   // The nnz for sparse tensors may be different across processes, so first
   // we run allgather on the nnz, and then allgather with max(nnz).
-  // We could use an allgatherv for this, if it were available.
   at::Tensor allreduce(std::vector<at::Tensor>& tensors) {
     // TODO: This is a massive hack!  There is some confusion about
     // Variable/Tensor inside the body of this function.  Turning off
@@ -1230,7 +1237,7 @@ class AsyncSparseAllreduceWork : public ProcessGroupGloo::AsyncWork {
 
     // Allgather metadata
     gloo::AllgatherOptions opts(context);
-    opts.setOutput(buffer.data_ptr<int64_t>(), buffer.numel());
+    opts.setOutput(buffer.mutable_data_ptr<int64_t>(), buffer.numel());
     opts.setTag(tag);
     gloo::allgather(opts);
 
@@ -1257,8 +1264,9 @@ class AsyncSparseAllreduceWork : public ProcessGroupGloo::AsyncWork {
 
     // Allgatherv indices.
     gloo::AllgathervOptions opts(context);
-    opts.setInput(input.data_ptr<int64_t>(), input.numel());
-    opts.setOutput(output.data_ptr<int64_t>(), counts);
+    opts.setInput(
+        const_cast<int64_t*>(input.const_data_ptr<int64_t>()), input.numel());
+    opts.setOutput(output.mutable_data_ptr<int64_t>(), counts);
     opts.setTag(tag);
     gloo::allgatherv(opts);
 
@@ -1847,7 +1855,7 @@ c10::intrusive_ptr<Work> ProcessGroupGloo::allgather(
     TORCH_CHECK(false, "ProcessGroupGloo::allgather: " + msg);
   };
 
-  if (inputs.size() == 0) {
+  if (inputs.empty()) {
     invalidArgument("requires non-empty input tensor list");
   }
 
@@ -1985,7 +1993,7 @@ c10::intrusive_ptr<Work> ProcessGroupGloo::allgather_coalesced(
     invalidArgument("requires non-empty input tensor list");
   }
 
-  if (output_lists.size() != getSize()) {
+  if (output_lists.size() != static_cast<size_t>(getSize())) {
     invalidArgument("output lists should be equal to world size");
   }
 
@@ -2199,7 +2207,7 @@ c10::intrusive_ptr<Work> ProcessGroupGloo::gather(
     const auto& sizes = inputs[0].sizes();
     assertTypeAndSizesMatch(invalidArgument, outputs[0], options, sizes);
   } else {
-    if (outputs.size() != 0) {
+    if (!outputs.empty()) {
       invalidArgument("requires empty output on non-root");
     }
   }
@@ -2245,9 +2253,8 @@ class AsyncScatterWork : public ProcessGroupGloo::AsyncWork {
       : ProcessGroupGloo::AsyncWork(
             {outputs},
             "gloo:scatter",
-            inputs.size() > 0
-                ? c10::optional<std::vector<at::Tensor>>(inputs[0])
-                : c10::nullopt),
+            !inputs.empty() ? c10::optional<std::vector<at::Tensor>>(inputs[0])
+                            : c10::nullopt),
         context(context),
         outputs(outputs),
         inputs(inputs),
@@ -2383,7 +2390,7 @@ c10::intrusive_ptr<Work> ProcessGroupGloo::scatter(
     const auto& sizes = outputs[0].sizes();
     assertTypeAndSizesMatch(invalidArgument, inputs[0], options, sizes);
   } else {
-    if (inputs.size() != 0) {
+    if (!inputs.empty()) {
       invalidArgument("requires empty input on non-root");
     }
   }
@@ -2454,7 +2461,7 @@ class AsyncAlltoallWork : public ProcessGroupGloo::AsyncWork {
 
   void alltoall(at::Tensor& outputTensor, at::Tensor& inputTensor) {
     const auto scalarType = outputTensor.scalar_type();
-    if (outputCounts.size() == 0 && inputCounts.size() == 0) {
+    if (outputCounts.empty() && inputCounts.empty()) {
       // Gloo alltoall
       gloo::AlltoallOptions opts(context);
       opts.setTag(tag);
@@ -2591,7 +2598,7 @@ c10::intrusive_ptr<Work> ProcessGroupGloo::alltoall_base(
   return work;
 }
 
-at::Tensor& checkSingleTensor(std::vector<at::Tensor>& tensors) {
+static at::Tensor& checkSingleTensor(std::vector<at::Tensor>& tensors) {
   if (tensors.size() != 1) {
     TORCH_CHECK(false, "ProcessGroupGloo::send takes a single tensor");
   }
@@ -2605,7 +2612,7 @@ at::Tensor& checkSingleTensor(std::vector<at::Tensor>& tensors) {
   return tensor;
 }
 
-uint32_t checkTag(int32_t tag) {
+static uint32_t checkTag(int32_t tag) {
   TORCH_CHECK(tag >= 0, "Tag must be nonnegative");
   return (uint32_t)tag;
 }
@@ -2616,12 +2623,12 @@ c10::intrusive_ptr<Work> ProcessGroupGloo::send(
     int tag) {
   auto& tensor = checkSingleTensor(tensors);
   auto utag = checkTag(tag);
-  auto ptr = tensor.data_ptr();
+  auto ptr = tensor.const_data_ptr();
   auto size = tensor.numel() * tensor.element_size();
 
   // Construct unbound buffer.
   auto context = getContext(tag);
-  auto buf = context->createUnboundBuffer(ptr, size);
+  auto buf = context->createUnboundBuffer(const_cast<void*>(ptr), size);
   buf->send(dstRank, utag);
 
   // The work captures the tensor to prevent it being deallocated and
@@ -2635,7 +2642,7 @@ c10::intrusive_ptr<Work> ProcessGroupGloo::recv(
     int tag) {
   auto& tensor = checkSingleTensor(tensors);
   auto utag = checkTag(tag);
-  auto ptr = tensor.data_ptr();
+  auto ptr = tensor.mutable_data_ptr();
   auto size = tensor.numel() * tensor.element_size();
 
   // Construct unbound buffer.
@@ -2653,7 +2660,7 @@ c10::intrusive_ptr<Work> ProcessGroupGloo::recvAnysource(
     int tag) {
   auto& tensor = checkSingleTensor(tensors);
   auto utag = checkTag(tag);
-  auto ptr = tensor.data_ptr();
+  auto ptr = tensor.mutable_data_ptr();
   auto size = tensor.numel() * tensor.element_size();
 
   // Construct unbound buffer.
@@ -2814,7 +2821,8 @@ void ProcessGroupGloo::monitoredBarrier(
     // some ranks have not responded.
     // Ensure all ranks from 1, ... WORLD_SIZE -1 have been successfully
     // processed.
-    auto rankFailure = (processedRanks.size() != size_ - 1);
+    auto rankFailure =
+        (processedRanks.size() != static_cast<size_t>(size_ - 1));
     if (waitAllRanks && rankFailure) {
       std::vector<int> failedRanks;
       for (const auto i : c10::irange(1, size_)) {
@@ -2849,20 +2857,7 @@ void ProcessGroupGloo::monitoredBarrier(
 }
 
 void ProcessGroupGloo::setSequenceNumberForGroup() {
-  if (rank_ == 0) {
-    // Create and broadcast sequence number
-    auto seq = 1 + rand();
-    sequenceNum_ = c10d::SequenceNum(seq);
-    std::vector<char> values = c10d::toVec<char>(seq, kBytes);
-    store_->set(kSeqNumStoreKey, values);
-  } else {
-    // Read rank 0's sequence number from store.
-    sequenceNum_ = c10d::SequenceNum();
-    store_->wait({kSeqNumStoreKey}, options_->timeout);
-    std::vector<char> values = store_->get(kSeqNumStoreKey);
-    uint64_t num = c10d::fromVec<char>(values);
-    sequenceNum_->set(num);
-  }
+  sequenceNum_ = c10d::SequenceNum(0);
 }
 
 uint64_t ProcessGroupGloo::getSequenceNumberForGroup() {
