@@ -31,11 +31,15 @@ struct CollectiveFingerPrint {
   std::vector<int8_t> tensor_device_types_;
   // input tensor sizes
   std::vector<std::vector<int64_t>> tensor_sizes_;
+  int sequence_number_;
 
-  explicit CollectiveFingerPrint(
+  CollectiveFingerPrint(
       OpType op_type,
-      const std::vector<at::Tensor>& input_tensors)
-      : op_type_(op_type), num_tensors_(input_tensors.size()) {
+      const std::vector<at::Tensor>& input_tensors,
+      int sequence_number)
+      : op_type_(op_type),
+        num_tensors_(input_tensors.size()),
+        sequence_number_(sequence_number) {
     tensor_dtypes_.reserve(num_tensors_);
     tensor_device_types_.reserve(num_tensors_);
     tensor_sizes_.reserve(num_tensors_);
@@ -49,13 +53,17 @@ struct CollectiveFingerPrint {
   // Constructor for the data received from deserialized fingerprint
   CollectiveFingerPrint(
       OpType op_type,
+      size_t num_tensors,
       std::vector<int8_t> tensor_dtypes,
       std::vector<int8_t> tensor_device_types,
-      std::vector<std::vector<int64_t>> tensor_sizes)
+      std::vector<std::vector<int64_t>> tensor_sizes,
+      int sequence_number)
       : op_type_(op_type),
+        num_tensors_(num_tensors),
         tensor_dtypes_(std::move(tensor_dtypes)),
         tensor_device_types_(std::move(tensor_device_types)),
-        tensor_sizes_(std::move(tensor_sizes)) {}
+        tensor_sizes_(std::move(tensor_sizes)),
+        sequence_number_(sequence_number) {}
 
   // Logs collective information in case of a failure.
   friend std::ostream& operator<<(
@@ -87,13 +95,16 @@ struct CollectiveFingerPrint {
     auto device_types = std::vector<int8_t>();
     auto sizes = std::vector<std::vector<int64_t>>();
     int index = 0;
+    int seq = 0;
     // 1. OpType
     optype = OpType(serialized_tensor[index].item<int>());
     index++;
-
+    int num_tensors = 0;
     if (index < serialized_tensor.size(0)) {
+      seq = serialized_tensor[index].item<int64_t>();
+      index++;
       // 2. Num tensors
-      int num_tensors = serialized_tensor[index].item<int>();
+      num_tensors = serialized_tensor[index].item<int>();
       index++;
       dtypes.reserve(num_tensors);
       device_types.reserve(num_tensors);
@@ -124,7 +135,8 @@ struct CollectiveFingerPrint {
         sizes.push_back(shapeVec);
       }
     }
-    return CollectiveFingerPrint(optype, dtypes, device_types, sizes);
+    return CollectiveFingerPrint(
+        optype, num_tensors, dtypes, device_types, sizes, seq);
   }
 
  private:
@@ -154,7 +166,7 @@ struct CollectiveFingerPrint {
     for (const auto i : c10::irange(output_tensors.size())) {
       const std::vector<at::Tensor> gathered_tensors = output_tensors[i];
       const at::Tensor reference_tensor = tensors_to_verify[i];
-      for (int rank = 0; rank < gathered_tensors.size(); rank++) {
+      for (const auto rank : c10::irange(gathered_tensors.size())) {
         const auto& rank_tensor = gathered_tensors[rank];
         if (!rank_tensor.equal(reference_tensor)) {
           CollectiveFingerPrint rank_fingerprint =
@@ -164,12 +176,118 @@ struct CollectiveFingerPrint {
              << backend->getRank() << " is running collective: " << *this
              << ", but Rank " << rank
              << " is running collective: " << rank_fingerprint << ".";
+          auto diff_result = compute_collective_diff(rank_fingerprint);
+          if (std::get<0>(diff_result)) {
+            ss << std::get<1>(diff_result);
+          }
+
           TORCH_CHECK(false, ss.str());
         }
       }
     }
   }
 
+  static std::vector<std::string> get_size_strs(
+      const CollectiveFingerPrint& collective_fingerprint) {
+    std::vector<std::string> size_strs;
+    if (!collective_fingerprint.tensor_sizes_.empty()) {
+      for (const auto& single_tensor_shape_num :
+           collective_fingerprint.tensor_sizes_[0]) {
+        size_strs.emplace_back(std::to_string(single_tensor_shape_num));
+      }
+    }
+    return size_strs;
+  }
+
+  static std::vector<std::string> get_dtype_strs(
+      const CollectiveFingerPrint& collective_fingerprint) {
+    std::vector<std::string> dtype_strs;
+    dtype_strs.reserve(collective_fingerprint.tensor_dtypes_.size());
+    for (const auto& tensor_dtype : collective_fingerprint.tensor_dtypes_) {
+      dtype_strs.emplace_back(
+          c10::toString(static_cast<at::ScalarType>(tensor_dtype)));
+    }
+    return dtype_strs;
+  }
+
+  static std::vector<std::string> get_device_type_strs(
+      const CollectiveFingerPrint& collective_fingerprint) {
+    std::vector<std::string> device_type_strs;
+    device_type_strs.reserve(
+        collective_fingerprint.tensor_device_types_.size());
+    for (const auto& tensor_device_type :
+         collective_fingerprint.tensor_device_types_) {
+      device_type_strs.emplace_back(
+          c10::toString(static_cast<at::DeviceType>(tensor_device_type)));
+    }
+    return device_type_strs;
+  }
+
+  std::pair<bool, std::string> compute_collective_diff(
+      CollectiveFingerPrint& other) {
+    // Computes the difference between two collectives (seq num, tensor shapes,
+    // collective type, etc) for easier understanding of how mismatched
+    // collectives across ranks differ.
+    bool found_diff = false;
+    std::stringstream ss;
+    ss << "Collectives differ in the following aspects: ";
+    // Check seq_num
+    if (other.sequence_number_ != sequence_number_) {
+      found_diff = true;
+      ss << c10::str(
+          "\t Sequence number: ",
+          sequence_number_,
+          "vs ",
+          other.sequence_number_);
+    }
+    // Check op type
+    auto other_op = opTypeToString(other.op_type_);
+    auto this_op = opTypeToString(op_type_);
+    if (other_op.compare(this_op) != 0) {
+      found_diff = true;
+      ss << c10::str("  Op type: ", this_op, "vs ", other_op);
+    }
+
+    auto check = [&ss, &found_diff](
+                     const char* arg,
+                     std::vector<std::string> other,
+                     std::vector<std::string> curr) {
+      if (other.size() != curr.size()) {
+        found_diff = true;
+        ss << c10::str("  Tensor ", arg, ": ", curr, "vs ", other);
+        return;
+      }
+      for (size_t i = 0; i < other.size(); ++i) {
+        if (other[i].compare(curr[i]) != 0) {
+          found_diff = true;
+          ss << c10::str("  Tensor ", arg, ": ", curr, "vs ", other);
+          return;
+        }
+      }
+    };
+
+    // check tensor sizes
+    auto other_sizes = get_size_strs(other);
+    auto this_sizes = get_size_strs(*this);
+    check("Tensor shapes", other_sizes, this_sizes);
+
+    // check tensor dtypes
+    auto other_dtypes = get_dtype_strs(other);
+    auto this_dtypes = get_dtype_strs(*this);
+    check("Tensor dtypes", other_dtypes, this_dtypes);
+
+    // check tensor devices
+    auto other_devices = get_device_type_strs(other);
+    auto this_devices = get_device_type_strs(*this);
+
+    check("Tensor devices", other_devices, this_devices);
+    if (!found_diff) {
+      return std::make_pair(false, ss.str());
+    } else {
+      return std::make_pair(true, ss.str());
+    }
+  }
+
   // Serializes the information (op type, input shapes, data types, device
   // types) about the collective fingerprint into a tensor
   at::Tensor serialize_fingerprint() {
@@ -177,6 +295,8 @@ struct CollectiveFingerPrint {
     // std::vector<int64_t> data;
     // 1. OpType
     data->push_back(static_cast<int64_t>(op_type_));
+    // sequence number
+    data->push_back(sequence_number_);
     // 2. Num tensors
     data->push_back(static_cast<int64_t>(num_tensors_));
     // 3. Tensor dtypes
@@ -216,34 +336,22 @@ std::ostream& operator<<(
     std::ostream& output,
     const CollectiveFingerPrint& collective_fingerprint) {
   std::string collectiveInfo;
+  auto op_type_str = opTypeToString(collective_fingerprint.op_type_);
   if (collective_fingerprint.num_tensors_ != 0) {
     // Convert dtype and device type info to string.
-    std::vector<std::string> dtype_strs;
-    std::vector<std::string> device_type_strs;
-    std::vector<std::string> size_strs;
-    dtype_strs.reserve(collective_fingerprint.tensor_dtypes_.size());
-    for (const auto& tensor_dtype : collective_fingerprint.tensor_dtypes_) {
-      dtype_strs.emplace_back(
-          c10::toString(static_cast<at::ScalarType>(tensor_dtype)));
-    }
-    device_type_strs.reserve(
-        collective_fingerprint.tensor_device_types_.size());
-    for (const auto& tensor_device_type :
-         collective_fingerprint.tensor_device_types_) {
-      device_type_strs.emplace_back(
-          c10::toString(static_cast<at::DeviceType>(tensor_device_type)));
-    }
-    if (!collective_fingerprint.tensor_sizes_.empty()) {
-      for (const auto& single_tensor_shape_num :
-           collective_fingerprint.tensor_sizes_[0]) {
-        size_strs.emplace_back(std::to_string(single_tensor_shape_num));
-      }
-    }
+    std::vector<std::string> dtype_strs =
+        CollectiveFingerPrint::get_dtype_strs(collective_fingerprint);
+    std::vector<std::string> device_type_strs =
+        CollectiveFingerPrint::get_device_type_strs(collective_fingerprint);
+    std::vector<std::string> size_strs =
+        CollectiveFingerPrint::get_size_strs(collective_fingerprint);
 
     collectiveInfo = c10::str(
         "CollectiveFingerPrint(",
-        "OpType=",
-        opTypeToString(collective_fingerprint.op_type_),
+        "SequenceNumber=",
+        collective_fingerprint.sequence_number_,
+        ", OpType=",
+        op_type_str,
         ", TensorShape=[",
         c10::Join(", ", size_strs),
         "], TensorDtypes=",
@@ -254,8 +362,10 @@ std::ostream& operator<<(
   } else {
     collectiveInfo = c10::str(
         "CollectiveFingerPrint(",
+        "SequenceNumber=",
+        collective_fingerprint.sequence_number_,
         "OpType=",
-        opTypeToString(collective_fingerprint.op_type_),
+        op_type_str,
         ")");
   }
   return output << collectiveInfo;
@@ -442,12 +552,15 @@ c10::intrusive_ptr<Backend> ProcessGroupWrapper::getWrappedPg() const {
 
 void ProcessGroupWrapper::runCollectiveChecks(
     OpType op_type,
-    const std::vector<at::Tensor>& tensors) const {
+    const std::vector<at::Tensor>& tensors) {
   // first perform a monitored barrier to ensure all ranks can synchronize.
   c10d::BarrierOptions options;
   // TODO: we should use wrapped backend_'s timeout here, but C++ ProcessGroup
   // API does not expose timeout.
-  auto finger_print = CollectiveFingerPrint(op_type, tensors);
+  auto seq = getSequenceNumberForGroup();
+  auto finger_print = CollectiveFingerPrint(op_type, tensors, seq);
+  LOG(INFO) << "[Rank " << getRank() << "] "
+            << "Running collective: " << finger_print;
   try {
     glooBackend_->monitoredBarrier(options, /* waitAllRanks */ true);
   } catch (const std::runtime_error& e) {
