@@ -38,7 +38,7 @@ Work::Work(
   }
 }
 
-OpType Work::retrieveOpType() {
+OpType Work::retrieveOpType() const {
   return opType_;
 }
 
@@ -105,7 +105,7 @@ c10::intrusive_ptr<c10::ivalue::Future> Work::getFuture() {
 void Work::finish(std::exception_ptr exception) {
   std::unique_lock<std::mutex> lock(mutex_);
   completed_ = true;
-  exception_ = exception;
+  exception_ = std::move(exception);
   if (recordFunctionEndCallback_) {
     recordFunctionEndCallback_();
     recordFunctionEndCallback_ = nullptr;
@@ -117,7 +117,7 @@ void Work::finish(std::exception_ptr exception) {
 void Work::finishAndThrow(std::exception_ptr exception) {
   std::unique_lock<std::mutex> lock(mutex_);
   completed_ = true;
-  exception_ = exception;
+  exception_ = std::move(exception);
   if (recordFunctionEndCallback_) {
     recordFunctionEndCallback_();
     recordFunctionEndCallback_ = nullptr;
@@ -127,6 +127,10 @@ void Work::finishAndThrow(std::exception_ptr exception) {
   }
 }
 
+float Work::getDuration() const {
+  TORCH_CHECK(false, "Only ProcessGrouppNCCL::WorkNCCL supports getDuration.");
+}
+
 class FutureWrappingWork : public Work {
  public:
   FutureWrappingWork(c10::intrusive_ptr<c10::ivalue::Future> fut)
@@ -176,7 +180,7 @@ class FutureWrappingWork : public Work {
 };
 
 c10::intrusive_ptr<Work> Work::create_from_future(
-    c10::intrusive_ptr<c10::ivalue::Future> future) {
+    const c10::intrusive_ptr<c10::ivalue::Future>& future) {
   return c10::make_intrusive<FutureWrappingWork>(future);
 }
 
