@@ -18,6 +18,8 @@
 #include <ATen/functorch/TensorWrapper.h>
 #include <c10/core/AutogradState.h>
 
+#include <iostream>
+
 // This file contains functorch's Python bindings.
 
 namespace torch {
@@ -266,10 +268,12 @@ int64_t _jvp_decrement_nesting() {
 }
 
 int64_t _vmap_increment_nesting(
-    int64_t batch_size,
+    c10::SymInt batch_size,
     const std::string& randomness) {
   return initAndPushDynamicLayer(
-      TransformType::Vmap, batch_size, get_randomness_enum(randomness));
+      TransformType::Vmap,
+      std::move(batch_size),
+      get_randomness_enum(randomness));
 }
 
 int64_t _vmap_decrement_nesting() {
