@@ -85,7 +85,7 @@ class FlatbufferLoader final {
   void registerIValueParser(
       mobile::serialization::IValueUnion ivalue_type,
       IValueParser parser);
-  mobile::Module parseModule(mobile::serialization::Module* module);
+  mobile::Module parseModule(mobile::serialization::Module* module, char* end);
 
   void extractJitSourceAndConstants(
       ExtraFilesMap* jit_sources,
@@ -281,7 +281,8 @@ void FlatbufferLoader::parseAndPopulate(
 }
 
 mobile::Module FlatbufferLoader::parseModule(
-    mobile::serialization::Module* module) {
+    mobile::serialization::Module* module,
+    char* end) {
   module_ = module;
   all_ivalues_.clear();
   all_types_.clear();
@@ -290,6 +291,11 @@ mobile::Module FlatbufferLoader::parseModule(
   module_parsed_ = false;
 
   const auto* ivalues = module->ivalues();
+  TORCH_CHECK(
+      ivalues && module->object_types(),
+      "Parsing flatbuffer module: Corrupted ivalues/object_types field");
+  TORCH_CHECK(
+      reinterpret_cast<const char*>(ivalues) < end, "Corrupted ivalues field");
   all_ivalues_.resize(ivalues->size());
   all_types_.resize(module->object_types()->size());
   storages_.resize(module->storage_data_size());
@@ -302,6 +308,8 @@ mobile::Module FlatbufferLoader::parseModule(
 
   for (uint32_t i = 0; i < mobile_ivalue_size_; i++) {
     const auto* ival = ivalues->Get(i);
+    TORCH_CHECK(
+        reinterpret_cast<const char*>(ival) < end, "Corrupted ivalue item")
     parseAndPopulate(i, ival);
   }
   IValue& module_ivalue = getIValue(module->state_obj());
@@ -733,7 +741,7 @@ void FlatbufferLoader::extractJitSourceAndConstants(
     }
   }
   const auto* jit_constants = module_->jit_constants();
-  for (auto i = 0; i < jit_constants->size(); ++i) {
+  for (const auto i : c10::irange(jit_constants->size())) {
     constants->emplace_back(getIValue(jit_constants->Get(i)));
   }
   parseExtraFilesFromVector(module_->jit_sources(), jit_sources);
@@ -743,7 +751,7 @@ void FlatbufferLoader::extractJitSourceAndConstants(
 
 mobile::Module parse_and_initialize_mobile_module(
     void* data,
-    size_t,
+    size_t size,
     c10::optional<at::Device>,
     ExtraFilesMap* extra_files,
     bool should_copy_tensor_memory) {
@@ -752,13 +760,20 @@ mobile::Module parse_and_initialize_mobile_module(
   // TODO(T128189662): If not copying, enforce that data is aligned to
   // kFlatbufferDataAlignmentBytes, and add unit tests.
 
+  // Validate Flatbuffer module before parsing.
+  flatbuffers::Verifier verifier(reinterpret_cast<uint8_t*>(data), size);
+  TORCH_CHECK(
+      mobile::serialization::VerifyModuleBuffer(verifier),
+      "Malformed Flatbuffer module");
+
   FlatbufferLoader loader;
   loader.setShouldCopyTensorMemory(should_copy_tensor_memory);
 
   // Flatbuffer doesn't seem to have a way to provide the buffer size when
   // interacting with the buffer.
   auto* flatbuffer_module = mobile::serialization::GetMutableModule(data);
-  mobile::Module m = loader.parseModule(flatbuffer_module);
+  auto* end = static_cast<char*>(data) + size;
+  mobile::Module m = loader.parseModule(flatbuffer_module, end);
   if (extra_files != nullptr) {
     parseExtraFiles(flatbuffer_module, *extra_files);
   }
@@ -782,7 +797,7 @@ mobile::Module parse_and_initialize_mobile_module(
 
 mobile::Module parse_and_initialize_mobile_module_for_jit(
     void* data,
-    size_t,
+    size_t size,
     ExtraFilesMap& jit_sources,
     std::vector<IValue>& jit_constants,
     c10::optional<at::Device>,
@@ -792,9 +807,16 @@ mobile::Module parse_and_initialize_mobile_module_for_jit(
   // TODO(T128189662): Enforce that data is aligned to
   // kFlatbufferDataAlignmentBytes, and add unit tests.
 
+  // Validate Flatbuffer module before parsing.
+  flatbuffers::Verifier verifier(reinterpret_cast<uint8_t*>(data), size);
+  TORCH_CHECK(
+      mobile::serialization::VerifyModuleBuffer(verifier),
+      "Malformed Flatbuffer module");
+
   FlatbufferLoader loader;
   auto* flatbuffer_module = mobile::serialization::GetMutableModule(data);
-  mobile::Module m = loader.parseModule(flatbuffer_module);
+  auto* end = static_cast<char*>(data) + size;
+  mobile::Module m = loader.parseModule(flatbuffer_module, end);
   if (extra_files != nullptr) {
     parseExtraFiles(flatbuffer_module, *extra_files);
   }
@@ -886,6 +908,13 @@ mobile::Module parse_flatbuffer_no_object(
     c10::optional<at::Device> device) {
   (void)device;
   (void)size;
+
+  // Validate Flatbuffer module before parsing.
+  flatbuffers::Verifier verifier(reinterpret_cast<uint8_t*>(data.get()), size);
+  TORCH_CHECK(
+      mobile::serialization::VerifyModuleBuffer(verifier),
+      "Malformed Flatbuffer module");
+
   auto* flatbuffer_module = mobile::serialization::GetMutableModule(data.get());
   FlatbufferLoader loader;
   // replace parserObject with to handle only class with field case
@@ -905,7 +934,8 @@ mobile::Module parse_flatbuffer_no_object(
         return static_cast<c10::IValue>(obj);
       });
 
-  mobile::Module m = loader.parseModule(flatbuffer_module);
+  auto* end = data.get() + size;
+  mobile::Module m = loader.parseModule(flatbuffer_module, end);
   m.set_delete_memory(std::move(data));
   return m;
 }
