@@ -5,6 +5,7 @@
 #include <ATen/core/ivalue.h>
 #include <ATen/core/qualified_name.h>
 #include <c10/util/Exception.h>
+#include <c10/util/Optional.h>
 #include <c10/util/ScopeExit.h>
 #include <c10/util/irange.h>
 #include <caffe2/serialize/in_memory_adapter.h>
@@ -13,6 +14,7 @@
 #include <caffe2/serialize/versions.h>
 #include <torch/csrc/jit/api/compilation_unit.h>
 #include <torch/csrc/jit/mobile/file_format.h>
+#include <torch/csrc/jit/mobile/flatbuffer_loader.h>
 #include <torch/csrc/jit/mobile/interpreter.h>
 #include <torch/csrc/jit/mobile/observer.h>
 #include <torch/csrc/jit/mobile/type_parser.h>
@@ -88,19 +90,6 @@ using caffe2::serialize::MemoryReadAdapter;
 using caffe2::serialize::PyTorchStreamReader;
 using caffe2::serialize::ReadAdapterInterface;
 
-mobile::Module (*load_flatbuffer_bytes)(
-    std::shared_ptr<char>,
-    size_t size,
-    c10::optional<at::Device>,
-    ExtraFilesMap*) = nullptr;
-
-mobile::Module (*load_flatbuffer_bytes_no_object)(
-    std::shared_ptr<char>,
-    size_t size,
-    c10::optional<at::Device>) = nullptr;
-
-uint64_t (*get_flatbuffer_bytecode_version)(char* flatbuffer_content) = nullptr;
-
 OpCode parseOpCode(const char* str);
 
 TypePtr resolveTypeNameMobile(
@@ -128,7 +117,7 @@ TypePtr resolveTypeNameMobile(
 
 c10::StrongTypePtr typeResolverMobile(
     const c10::QualifiedName& qn,
-    std::shared_ptr<CompilationUnit> compilation_unit) {
+    const std::shared_ptr<CompilationUnit>& compilation_unit) {
   return c10::StrongTypePtr(
       compilation_unit, resolveTypeNameMobile(qn, compilation_unit));
 }
@@ -321,7 +310,7 @@ void BytecodeDeserializer::parseMethods(
     c10::ivalue::TupleElements&& vals,
     c10::optional<c10::ivalue::TupleElements>&& debug_handles,
     mobile::CompilationUnit& mcu) {
-  TORCH_CHECK(vals.size() > 0, "Bytecode has no elements. ");
+  TORCH_CHECK(!vals.empty(), "Bytecode has no elements. ");
   // Initialized with the version number when kProducedBytecodeVersion was
   // introduced. The old models (some of them already in production) without
   // version number are seen as version 3 (deprecated).
@@ -437,9 +426,7 @@ void BytecodeDeserializer::deserialize_only_extra(
   for (const auto& kv : extra_files) {
     const std::string& key = "extra/" + kv.first;
     if (reader_->hasRecord(key)) {
-      at::DataPtr meta_ptr;
-      size_t meta_size = 0;
-      std::tie(meta_ptr, meta_size) = reader_->getRecord(key);
+      auto [meta_ptr, meta_size] = reader_->getRecord(key);
       extra_files[kv.first] =
           std::string(static_cast<char*>(meta_ptr.get()), meta_size);
     }
@@ -497,7 +484,7 @@ c10::IValue BytecodeDeserializer::readArchive(
     return typeResolverMobile(qn, compilation_unit_);
   };
 
-  auto obj_loader = [&](at::StrongTypePtr type, IValue input) {
+  auto obj_loader = [&](const at::StrongTypePtr& type, const IValue& input) {
     return objLoaderMobile(type, input, *mcu);
   };
 
@@ -518,132 +505,6 @@ c10::IValue BytecodeDeserializer::readArchive(
   return ivalues;
 }
 
-} // namespace
-
-// Forward declare so that _load_for_mobile() overloads can
-// call this method directly.
-mobile::Module _load_for_mobile_impl(
-    std::unique_ptr<ReadAdapterInterface> rai,
-    c10::optional<c10::Device> device,
-    ExtraFilesMap& extra_files,
-    uint64_t module_load_options);
-
-mobile::Module _load_mobile_from_bytes(
-    std::shared_ptr<char> data,
-    size_t size,
-    c10::optional<c10::Device> device,
-    ExtraFilesMap& extra_files,
-    uint64_t module_load_options);
-
-mobile::Module _load_for_mobile(
-    std::istream& in,
-    c10::optional<at::Device> device) {
-  ExtraFilesMap extra_files;
-  return _load_for_mobile(in, device, extra_files);
-}
-
-mobile::Module _load_for_mobile(
-    const std::string& filename,
-    c10::optional<at::Device> device) {
-  ExtraFilesMap extra_files;
-  return _load_for_mobile(filename, device, extra_files);
-}
-
-mobile::Module _load_for_mobile(
-    std::unique_ptr<ReadAdapterInterface> rai,
-    c10::optional<c10::Device> device) {
-  ExtraFilesMap extra_files;
-  return _load_for_mobile(std::move(rai), device, extra_files);
-}
-
-mobile::Module _load_for_mobile(
-    std::istream& in,
-    c10::optional<at::Device> device,
-    ExtraFilesMap& extra_files) {
-  if (getFileFormat(in) == FileFormat::FlatbufferFileFormat) {
-    std::shared_ptr<char> data;
-    size_t size = 0;
-    std::tie(data, size) = get_stream_content(in);
-    return _load_mobile_from_bytes(
-        data, size, device, extra_files, kDefaultMobileLoadOptions);
-  }
-  std::unique_ptr<IStreamAdapter> rai = std::make_unique<IStreamAdapter>(&in);
-  auto module = _load_for_mobile_impl(
-      std::move(rai), device, extra_files, kDefaultMobileLoadOptions);
-  return module;
-}
-
-mobile::Module _load_for_mobile(
-    const std::string& filename,
-    c10::optional<at::Device> device,
-    ExtraFilesMap& extra_files) {
-  return _load_for_mobile(
-      filename, device, extra_files, kDefaultMobileLoadOptions);
-}
-
-mobile::Module _load_for_mobile(
-    const std::string& filename,
-    c10::optional<at::Device> device,
-    ExtraFilesMap& extra_files,
-    uint64_t module_load_options) {
-  auto format = getFileFormat(filename);
-
-  if (format == FileFormat::FlatbufferFileFormat) {
-    std::shared_ptr<char> data;
-    size_t size = 0;
-    std::tie(data, size) = get_file_content(filename.c_str());
-    return _load_mobile_from_bytes(
-        data, size, device, extra_files, module_load_options);
-  }
-
-  std::unique_ptr<FileAdapter> rai = std::make_unique<FileAdapter>(filename);
-  return _load_for_mobile_impl(
-      std::move(rai), device, extra_files, module_load_options);
-}
-
-TORCH_API mobile::Module _load_for_mobile(
-    std::unique_ptr<ReadAdapterInterface> rai,
-    c10::optional<c10::Device> device,
-    ExtraFilesMap& extra_files,
-    uint64_t module_load_options) {
-  // TODO optimize file read for non-flatbuffer models
-  std::shared_ptr<char> data;
-  size_t size = 0;
-  std::tie(data, size) = get_rai_content(rai.get());
-  return _load_mobile_from_bytes(
-      data, size, device, extra_files, module_load_options);
-}
-
-mobile::Module _load_mobile_from_bytes(
-    std::shared_ptr<char> data,
-    size_t size,
-    c10::optional<c10::Device> device,
-    ExtraFilesMap& extra_files,
-    uint64_t module_load_options) {
-  TORCH_CHECK(size >= kFileFormatHeaderSize, "Format error");
-  auto format = getFileFormat(data.get());
-  switch (format) {
-    case FileFormat::ZipFileFormat: {
-      std::unique_ptr<ReadAdapterInterface> rai =
-          std::make_unique<MemoryReadAdapter>(data.get(), size);
-      return _load_for_mobile_impl(
-          std::move(rai), device, extra_files, module_load_options);
-    }
-    case FileFormat::FlatbufferFileFormat: {
-      if (load_flatbuffer_bytes != nullptr) {
-        return load_flatbuffer_bytes(data, size, device, &extra_files);
-      } else {
-        TORCH_CHECK(
-            false,
-            "Flatbuffer input file but the build hasn't enabled flatbuffer");
-      }
-    }
-    default: {
-      TORCH_CHECK(false, "Format error");
-    }
-  }
-}
-
 mobile::Module _load_for_mobile_impl(
     std::unique_ptr<ReadAdapterInterface> rai,
     c10::optional<c10::Device> device,
@@ -665,6 +526,21 @@ mobile::Module _load_for_mobile_impl(
 
   const size_t model_size = rai != nullptr ? rai->size() : 0;
   auto reader = torch::make_unique<PyTorchStreamReader>(std::move(rai));
+  if (module_load_options &
+      MobileModuleLoadOptions::PARSE_ALL_EXTRA_FILE_MAPS) {
+    // ExtraFilesMap is serialized with a "extra/", hence it is necessary to
+    // account for when we de-serialize de-serialized filemap key values contain
+    // prefix and we need to remove prior to construct the map. "extra/" string
+    // has a length of 6 characters, hence we need only sub-string 6th position
+    // of a string. Please refer to following link for a detail:
+    // https://www.internalfb.com/code/fbsource/[9996fcb7a6fb]/fbcode/caffe2/torch/csrc/jit/mobile/import.cpp?lines=427-434
+    std::vector<std::string> all_files = reader->getAllRecords();
+    for (auto& file_name : all_files) {
+      if (file_name.find("extra/") == 0) {
+        extra_files[file_name.substr(6)] = "";
+      }
+    }
+  }
   BytecodeDeserializer deserializer(std::move(reader), module_load_options);
 
   std::string error_message;
@@ -701,6 +577,107 @@ mobile::Module _load_for_mobile_impl(
   }
 }
 
+mobile::Module _load_mobile_from_bytes(
+    const std::shared_ptr<char>& data,
+    size_t size,
+    c10::optional<c10::Device> device,
+    ExtraFilesMap& extra_files,
+    uint64_t module_load_options) {
+  TORCH_CHECK(size >= kFileFormatHeaderSize, "Format error");
+  auto format = getFileFormat(data.get());
+  switch (format) {
+    case FileFormat::ZipFileFormat: {
+      std::unique_ptr<ReadAdapterInterface> rai =
+          std::make_unique<MemoryReadAdapter>(data.get(), size);
+      return _load_for_mobile_impl(
+          std::move(rai), device, extra_files, module_load_options);
+    }
+    case FileFormat::FlatbufferFileFormat: {
+      return parse_and_initialize_mobile_module(
+          data, size, device, &extra_files);
+    }
+    default: {
+      TORCH_CHECK(false, "Format error");
+    }
+  }
+}
+
+} // namespace
+
+mobile::Module _load_for_mobile(
+    std::istream& in,
+    c10::optional<at::Device> device) {
+  ExtraFilesMap extra_files;
+  return _load_for_mobile(in, device, extra_files);
+}
+
+mobile::Module _load_for_mobile(
+    const std::string& filename,
+    c10::optional<at::Device> device) {
+  ExtraFilesMap extra_files;
+  return _load_for_mobile(filename, device, extra_files);
+}
+
+mobile::Module _load_for_mobile(
+    std::unique_ptr<ReadAdapterInterface> rai,
+    c10::optional<c10::Device> device) {
+  ExtraFilesMap extra_files;
+  return _load_for_mobile(std::move(rai), device, extra_files);
+}
+
+mobile::Module _load_for_mobile(
+    std::istream& in,
+    c10::optional<at::Device> device,
+    ExtraFilesMap& extra_files,
+    uint64_t module_load_options) {
+  if (getFileFormat(in) == FileFormat::FlatbufferFileFormat) {
+    auto [data, size] = get_stream_content(in);
+    return _load_mobile_from_bytes(
+        data, size, device, extra_files, module_load_options);
+  }
+  std::unique_ptr<IStreamAdapter> rai = std::make_unique<IStreamAdapter>(&in);
+  auto module = _load_for_mobile_impl(
+      std::move(rai), device, extra_files, module_load_options);
+  return module;
+}
+
+mobile::Module _load_for_mobile(
+    const std::string& filename,
+    c10::optional<at::Device> device,
+    ExtraFilesMap& extra_files) {
+  return _load_for_mobile(
+      filename, device, extra_files, kDefaultMobileLoadOptions);
+}
+
+mobile::Module _load_for_mobile(
+    const std::string& filename,
+    c10::optional<at::Device> device,
+    ExtraFilesMap& extra_files,
+    uint64_t module_load_options) {
+  auto format = getFileFormat(filename);
+
+  if (format == FileFormat::FlatbufferFileFormat) {
+    auto [data, size] = get_file_content(filename.c_str());
+    return _load_mobile_from_bytes(
+        data, size, device, extra_files, module_load_options);
+  }
+
+  std::unique_ptr<FileAdapter> rai = std::make_unique<FileAdapter>(filename);
+  return _load_for_mobile_impl(
+      std::move(rai), device, extra_files, module_load_options);
+}
+
+TORCH_API mobile::Module _load_for_mobile(
+    std::unique_ptr<ReadAdapterInterface> rai,
+    c10::optional<c10::Device> device,
+    ExtraFilesMap& extra_files,
+    uint64_t module_load_options) {
+  // TODO optimize file read for non-flatbuffer models
+  auto [data, size] = get_rai_content(rai.get());
+  return _load_mobile_from_bytes(
+      data, size, device, extra_files, module_load_options);
+}
+
 void _load_extra_only_for_mobile(
     const std::string& filename,
     c10::optional<at::Device> device,
@@ -726,16 +703,7 @@ void _load_extra_only_for_mobile(
       // TODO: the current flatbuffers implementation will always load the
       // whole module including the extra files. Ideally it should be
       // possible to just get the extra files given data
-      std::shared_ptr<char> data;
-      size_t size = 0;
-      std::tie(data, size) = get_file_content(filename.c_str());
-      if (load_flatbuffer_bytes != nullptr) {
-        load_flatbuffer_bytes(data, size, device, &extra_files);
-      } else {
-        TORCH_CHECK(
-            false,
-            "Flatbuffer input file but the build hasn't enabled flatbuffer");
-      }
+      load_mobile_module_from_file(filename, c10::nullopt, &extra_files);
       break;
     }
     default: {
