@@ -7,6 +7,8 @@
 #include <c10/util/irange.h>
 #include <torch/csrc/jit/jit_log.h>
 
+#include <utility>
+
 namespace torch {
 namespace jit {
 namespace SubgraphUtils {
@@ -178,7 +180,7 @@ void collectNodesToUnfuse(Node* start, std::set<Node*, topo_cmp_node>& s) {
 std::vector<std::set<Value*, topo_cmp_value>> buildAliasedSets(
     std::shared_ptr<Graph> subgraph) {
   auto outputs = subgraph->outputs();
-  AliasDb alias_db(subgraph);
+  AliasDb alias_db(std::move(subgraph));
   TORCH_INTERNAL_ASSERT(outputs.size() > 1);
   std::vector<std::set<Value*, topo_cmp_value>> res;
   for (auto o : outputs) {
@@ -225,7 +227,7 @@ void unmergeSubgraph(Node* subgraphNode) {
   subgraphNode->destroy();
 }
 
-void collectNestedUses(
+static void collectNestedUses(
     std::unordered_set<Value*>& closed_over_values,
     std::unordered_set<Value*>& new_values,
     std::unordered_map<Value*, Value*>& externalValuesMap,
@@ -261,7 +263,7 @@ void collectNestedUses(
       collectNestedUses(
           closed_over_values, new_values, externalValuesMap, node);
     }
-  } else if (input_node->blocks().size() != 0) {
+  } else if (!input_node->blocks().empty()) {
     TORCH_INTERNAL_ASSERT(false, input_node, " kind not handled yet");
   }
   for (Value* output : input_node->outputs()) {
@@ -269,7 +271,7 @@ void collectNestedUses(
   }
 }
 
-std::unordered_set<Value*> closedOverValues(
+static std::unordered_set<Value*> closedOverValues(
     Node* toMerge,
     std::unordered_map<Value*, Value*>& externalValuesMap) {
   std::unordered_set<Value*> closed_over_values;
@@ -460,7 +462,7 @@ bool unmergeAliasedOutputs(Node* subgraphNode) {
 
   auto subgraph = subgraphNode->g(attr::Subgraph);
   GRAPH_DUMP("unfuseAliasedOutputs Subgraph ", subgraph);
-  auto sets = buildAliasedSets(subgraph);
+  auto sets = buildAliasedSets(std::move(subgraph));
   GRAPH_DEBUG("buildAliasedSets sets.size() = ", sets.size());
 
   std::set<Node*, topo_cmp_node> nodes;
@@ -600,7 +602,7 @@ void unmergeNode(Node* n, Node* subgraphNode) {
   n->destroy();
 }
 
-std::string truncateStrWithHash(const std::string& s, size_t maxlen) {
+static std::string truncateStrWithHash(const std::string& s, size_t maxlen) {
   if (s.size() <= maxlen) {
     return s;
   }
@@ -608,7 +610,7 @@ std::string truncateStrWithHash(const std::string& s, size_t maxlen) {
   // If hash-string plus '_' can fit into maxlen, then truncate the original
   // string correspondingly so that the final string with the hash included fits
   // into maxlen. If that's not possible, at least truncate the original string
-  // to maxlen (and appen the hash to it).
+  // to maxlen (and append the hash to it).
   size_t trunc_len =
       (maxlen > hash_str.size() + 1) ? (maxlen - hash_str.size() - 1) : maxlen;
   std::stringstream truncated;
