@@ -6,9 +6,7 @@
 
 #include <c10/util/irange.h>
 
-namespace torch {
-namespace jit {
-namespace tensorexpr {
+namespace torch::jit::tensorexpr {
 
 RegisterCodeGen<SimpleIREvaluator> ir_eval_codegen_reg("simple_ir_eval");
 
@@ -431,13 +429,13 @@ class SimpleIREvaluatorImpl : public IRVisitor {
   TORCH_API void visit(BlockPtr v) override {
     BlockPtr last = scope_;
     scope_ = v;
-    for (StmtPtr s : v->stmts()) {
+    for (const StmtPtr& s : v->stmts()) {
       s->accept(this);
     }
 
     auto it = var_by_scope_.find(v);
     if (it != var_by_scope_.end()) {
-      for (ExprPtr v : it->second) {
+      for (const ExprPtr& v : it->second) {
         eval_context_.erase(v);
       }
       var_by_scope_.erase(it);
@@ -690,8 +688,8 @@ class SimpleIREvaluatorImpl : public IRVisitor {
           throw malformed_input(
               "Number of dimensions did not match number of strides", buf);
         }
-        size_t buf_size = 1;
-        if (dims.size() > 0) {
+        int64_t buf_size = 1;
+        if (!dims.empty()) {
           ExprHandle buf_size_expr = ExprHandle(immLike(dims[0], 1));
           ExprHandle negative_one = ExprHandle(immLike(dims[0], -1));
           for (const auto& i : c10::irange(dims.size())) {
@@ -847,7 +845,7 @@ class SimpleIREvaluatorImpl : public IRVisitor {
     std::vector<int8_t> buf_dtypes;
     std::vector<int64_t> extra_args;
 
-    for (BufPtr b : bufs) {
+    for (const BufPtr& b : bufs) {
       auto iter = buffer_mapping_.find(b);
       if (iter == buffer_mapping_.end()) {
         throw malformed_input("could not find buf", v);
@@ -856,7 +854,7 @@ class SimpleIREvaluatorImpl : public IRVisitor {
       buf_ptrs.push_back(iter->second);
       buf_ranks.push_back(b->dims().size());
       buf_dtypes.push_back((int8_t)b->dtype().scalar_type());
-      for (ExprPtr dim_expr : b->dims()) {
+      for (const ExprPtr& dim_expr : b->dims()) {
         dim_expr->accept(this);
         buf_dims.push_back(value().intValue());
       }
@@ -865,7 +863,7 @@ class SimpleIREvaluatorImpl : public IRVisitor {
         buf_strides.push_back(value().intValue());
       }
     }
-    for (ExprPtr a : v->args()) {
+    for (const ExprPtr& a : v->args()) {
       a->accept(this);
       // NOLINTNEXTLINE(cppcoreguidelines-init-variables)
       int64_t val;
@@ -986,7 +984,7 @@ class SimpleIREvaluatorImpl : public IRVisitor {
       values[i] = this->value();
     }
     std::vector<TInput> v1;
-    if (values.size() >= 1ULL) {
+    if (!values.empty()) {
       v1 = values[0].as_vec<TInput>();
     }
     std::vector<TInput> v2;
@@ -1311,6 +1309,4 @@ c10::optional<int64_t> evalInt(ExprPtr e) {
   }
 }
 
-} // namespace tensorexpr
-} // namespace jit
-} // namespace torch
+} // namespace torch::jit::tensorexpr
