@@ -48,338 +48,7 @@ bool TensorCompare(const at::Tensor& t1, const at::Tensor& t2) {
              contiguous_t1.numel() * contiguous_t1.itemsize()) == 0;
 }
 
-// Locking:
-// We perform two kinds of operations of tensors, synchronous and asynchronous.
-// The ApplyPendingGraph() are synchronous, as we need the device data result
-// immediately. Before the synchronous operations can start, they need to wait
-// that the pending asynchronous operations have completed.
-// Synchronous operations do not hold device locks, since they are strictly
-// sequential, dictated by the PyTorch execution order.
-// The SyncTensorsGraph() is asynchronous, and returns immediately after having
-// scheduled the asynchronous operation. While executing, the asynchronous
-// operations will hold locks on all the participating devices (in most common
-// cases there will be only one device).
-// Since asynchronous operations capture device locks, only one asynchronous
-// operation can execute at the same time, on a given device. Tensor operations
-// which send data to device do not need to hold any device locks while doing
-// so. Only operations which _use_ device data (computations, and transfer from
-// server) need to wait for asynchronous operations to complete (barrier).
-
-class DeviceLocker {
- public:
-  explicit DeviceLocker(BackendDevice device) : device_(std::move(device)) {}
-
-  const BackendDevice& device() const {
-    return device_;
-  }
-
-  void Lock() {
-    std::unique_lock<std::mutex> lock(mutex_);
-    cv_.wait(lock, [this] { return !locked_; });
-    CheckResetException();
-    locked_ = true;
-  }
-
-  void Unlock(std::exception_ptr exptr) {
-    std::lock_guard<std::mutex> lock(mutex_);
-    locked_ = false;
-    exptr_ = std::move(exptr);
-    cv_.notify_all();
-  }
-
-  void Barrier() {
-    std::unique_lock<std::mutex> lock(mutex_);
-    cv_.wait(lock, [this] { return !locked_; });
-    cv_.notify_all();
-    CheckResetException();
-  }
-
- private:
-  void CheckResetException() {
-    std::exception_ptr exptr = std::move(exptr_);
-    exptr_ = nullptr;
-    if (exptr != nullptr) {
-      std::rethrow_exception(exptr);
-    }
-  }
-
-  BackendDevice device_;
-  std::mutex mutex_;
-  std::condition_variable cv_;
-  bool locked_ = false;
-  std::exception_ptr exptr_;
-};
-
-class DeviceLockerArena {
- public:
-  static DeviceLockerArena* Get() {
-    static DeviceLockerArena* arena = new DeviceLockerArena();
-    return arena;
-  }
-
-  std::shared_ptr<DeviceLocker> GetLocker(const BackendDevice& device) {
-    std::lock_guard<std::mutex> lock(mutex_);
-    auto it = lockers_.find(device);
-    if (it == lockers_.end()) {
-      it = lockers_.emplace(device, std::make_shared<DeviceLocker>(device))
-               .first;
-    }
-    return it->second;
-  }
-
-  void DeviceBarrier(const BackendDevice& device) {
-    auto locker = DeviceLockerArena::Get()->GetLocker(device);
-    locker->Barrier();
-  }
-
-  // Use a set to impose an order on the device locking sequence (ABBA
-  // prevention).
-  std::vector<ExceptionCleanup> LockDevices(
-      const std::set<BackendDevice>& devices) {
-    std::vector<ExceptionCleanup> unlocker;
-    unlocker.reserve(devices.size());
-    for (auto& device : devices) {
-      unlocker.emplace_back(LockDevice(device));
-    }
-    return unlocker;
-  }
-
- private:
-  ExceptionCleanup LockDevice(const BackendDevice& device) {
-    auto locker = DeviceLockerArena::Get()->GetLocker(device);
-    locker->Lock();
-    return ExceptionCleanup(
-        [locker = std::move(locker)](ExceptionCleanup::StatusType status) {
-          locker->Unlock(std::move(status));
-        });
-  }
-
-  std::mutex mutex_;
-  std::map<BackendDevice, std::shared_ptr<DeviceLocker>> lockers_;
-};
-
-class DataCacheArena {
- public:
-  static DataCacheArena* Get() {
-    static DataCacheArena* arena =
-        new DataCacheArena(FLAGS_torch_lazy_device_data_cache_size);
-    return arena;
-  }
-
-  explicit DataCacheArena(size_t max_cache_size)
-      : max_cache_size_(max_cache_size) {}
-
-  BackendDataPtr GetDeviceData(
-      const at::Tensor& tensor,
-      const BackendDevice& device) {
-    DataCacheArena::DataCache* cache = Get()->GetDataCache(device);
-    ;
-    BackendDataPtr device_data = cache->Get(tensor);
-    if (device_data == nullptr) {
-      at::Tensor tensor_copy = CopyTensor(tensor);
-      device_data = TensorToDataHandle(tensor_copy, device);
-      cache->Add(std::move(tensor_copy), device_data);
-      TORCH_LAZY_COUNTER("DeviceDataCacheMiss", 1);
-    }
-    return device_data;
-  }
-
-  BackendDataPtr GetDeviceData(
-      const at::Scalar& value,
-      at::ScalarType scalar_type,
-      const BackendDevice& device) {
-    // Workaround since at::scalar_tensor doesn't support bfloat16 yet.
-    at::Tensor t = at::scalar_tensor(
-        value,
-        at::TensorOptions(
-            scalar_type == at::ScalarType::BFloat16 ? at::ScalarType::Float
-                                                    : scalar_type));
-    if (scalar_type == at::ScalarType::BFloat16) {
-      t = t.to(scalar_type);
-    }
-    return GetDeviceData(t, device);
-  }
-
- private:
-  struct TensorHasher {
-    size_t operator()(const at::Tensor& tensor) const {
-      return HashReduce(
-          HashCombine(GetEnumValue(tensor.scalar_type()), TensorHash(tensor)));
-    }
-  };
-  struct TensorComparer {
-    bool operator()(const at::Tensor& tensor1, const at::Tensor& tensor2)
-        const {
-      return TensorCompare(tensor1, tensor2);
-    }
-  };
-
-  using DataCache =
-      Cache<at::Tensor, BackendData, TensorHasher, TensorComparer>;
-
-  DataCache* GetDataCache(const BackendDevice& device) {
-    std::lock_guard<std::mutex> lock(mutex_);
-    auto it = device_caches_.find(device);
-    if (it == device_caches_.end()) {
-      std::unique_ptr<DataCache> cache(new DataCache(max_cache_size_));
-      it = device_caches_.emplace(device, std::move(cache)).first;
-    }
-    return it->second.get();
-  }
-
-  size_t max_cache_size_ = 0;
-  std::mutex mutex_;
-  std::map<BackendDevice, std::unique_ptr<DataCache>> device_caches_;
-};
-
-// The DeviceContextArena holds per device live information and statistics,
-// among which the lazy tensors which are currently alive in the system. This is
-// used to create computation "barriers" in order to flush pending operations
-// and ensure the same computations are created during the training loops.
-class DeviceContextArena {
-  struct DeviceContext {
-    std::mutex lock;
-    std::map<int64_t, std::weak_ptr<LazyTensor::Data>> tensors_data;
-    uint64_t seed = 101;
-    uint64_t running_seed = 101;
-    Value seed_ir_value;
-  };
-
- public:
-  static DeviceContextArena* Get() {
-    static DeviceContextArena* arena = new DeviceContextArena();
-    return arena;
-  }
-
-  void RegisterTensor(std::shared_ptr<LazyTensor::Data> data) {
-    DeviceContext* devctx = GetDeviceContext(data->device);
-    std::lock_guard<std::mutex> lock(devctx->lock);
-    devctx->tensors_data.emplace(data->unique_id, data);
-    TORCH_LAZY_COUNTER("CreateLtcTensor", 1);
-  }
-
-  void UnregisterTensor(LazyTensor::Data* data) {
-    DeviceContext* devctx = GetDeviceContext(data->device);
-    std::lock_guard<std::mutex> lock(devctx->lock);
-    devctx->tensors_data.erase(data->unique_id);
-    TORCH_LAZY_COUNTER("DestroyLtcTensor", 1);
-  }
-
-  std::vector<LazyTensorPtr> GetLiveTensors(const BackendDevice* device) {
-    std::vector<LazyTensorPtr> tensors;
-    auto fn = [&](DeviceContext* devctx) {
-      std::lock_guard<std::mutex> lock(devctx->lock);
-      for (auto& uid_wptr : devctx->tensors_data) {
-        std::shared_ptr<LazyTensor::Data> data = uid_wptr.second.lock();
-        if (data != nullptr) {
-          tensors.push_back(LazyTensor::Create(std::move(data)));
-        }
-      }
-    };
-    ForAllDeviceContexts(fn, device);
-    return tensors;
-  }
-
-  Value GetRngSeed(const BackendDevice& device) {
-    static const at::ScalarType kSeedType = at::ScalarType::Long;
-    static const uint64_t kSeedMul = 214013;
-    static const uint64_t kSeedAdd = 2531011;
-    DeviceContext* devctx = GetDeviceContext(device);
-    std::lock_guard<std::mutex> lock(devctx->lock);
-    if (!devctx->seed_ir_value) {
-      devctx->seed_ir_value =
-          IrValueFromScalar(MakeIntScalar(devctx->seed), kSeedType, device);
-    }
-    // Keep the running seed as scalar as well, so we can return it directly
-    // without executing graphs.
-    devctx->running_seed = kSeedAdd + kSeedMul * devctx->running_seed;
-    // Compose new seeds from the root seed, to avoid creating too many
-    // computation parameters which might overflow the device capacity.
-    Value k = MakeScalar(MakeIntScalar(kSeedMul), kSeedType);
-    Value b = MakeScalar(MakeIntScalar(kSeedAdd), kSeedType);
-    devctx->seed_ir_value = b + k * devctx->seed_ir_value;
-    return devctx->seed_ir_value;
-  }
-
-  uint64_t GetRunningSeed(const BackendDevice& device) {
-    DeviceContext* devctx = GetDeviceContext(device);
-    std::lock_guard<std::mutex> lock(devctx->lock);
-    return devctx->running_seed;
-  }
-
-  void SetRngSeed(const BackendDevice& device, uint64_t seed) {
-    DeviceContext* devctx = GetDeviceContext(device);
-    std::lock_guard<std::mutex> lock(devctx->lock);
-    devctx->seed = seed;
-    devctx->running_seed = devctx->seed;
-    devctx->seed_ir_value = Value();
-  }
-
-  void MarkStep(const BackendDevice& device) {
-    DeviceContext* devctx = GetDeviceContext(device);
-    std::lock_guard<std::mutex> lock(devctx->lock);
-    devctx->seed = 1012031 + devctx->seed * 7012063;
-    devctx->running_seed = devctx->seed;
-    devctx->seed_ir_value = Value();
-  }
-
-  std::vector<BackendDevice> GetActiveDevices() {
-    std::vector<BackendDevice> active_devices;
-    std::lock_guard<std::mutex> lock(lock_);
-    active_devices.reserve(device_contexts_.size());
-    for (auto& device_contexts : device_contexts_) {
-      active_devices.push_back(device_contexts.first);
-    }
-    return active_devices;
-  }
-
- private:
-  std::vector<DeviceContext*> GetAllDeviceContexts() {
-    std::vector<DeviceContext*> all_device_contexts;
-    std::lock_guard<std::mutex> lock(lock_);
-    all_device_contexts.reserve(device_contexts_.size());
-    for (auto& device_contexts : device_contexts_) {
-      all_device_contexts.push_back(device_contexts.second);
-    }
-    return all_device_contexts;
-  }
-
-  void ForAllDeviceContexts(
-      const std::function<void(DeviceContext*)>& fn,
-      const BackendDevice* device) {
-    if (device == nullptr) {
-      for (auto devctx : GetAllDeviceContexts()) {
-        fn(devctx);
-      }
-    } else {
-      fn(GetDeviceContext(*device));
-    }
-  }
-
-  DeviceContext* GetDeviceContext(const BackendDevice& device) {
-    std::lock_guard<std::mutex> lock(lock_);
-    auto it = device_contexts_.find(device);
-    if (it == device_contexts_.end()) {
-      it = device_contexts_.emplace(device, new DeviceContext()).first;
-    }
-    return it->second;
-  }
-
-  Value IrValueFromScalar(
-      const at::Scalar& value,
-      at::ScalarType scalar_type,
-      const BackendDevice& device) {
-    at::Tensor tensor =
-        at::scalar_tensor(value, at::TensorOptions(scalar_type));
-    BackendDataPtr device_data = TensorToDataHandle(tensor, device);
-    return MakeDeviceData(std::move(device_data));
-  }
-
-  std::mutex lock_;
-  std::map<BackendDevice, DeviceContext*> device_contexts_;
-};
-
-// Return true if no tensor in the list has an underlying IR (leaf or
+// Return true if any tensor in the list has an underlying IR (leaf or
 // operation).
 bool TensorsHaveIR(const std::vector<LazyTensorPtr>& tensors) {
   for (const auto& tensor : tensors) {
@@ -390,19 +59,309 @@ bool TensorsHaveIR(const std::vector<LazyTensorPtr>& tensors) {
   return false;
 }
 
+std::atomic<LazyGraphExecutor*> lazy_graph_executor_registry;
 } // namespace
 
+auto LazyGraphExecutor::DeviceContextArena::Get()
+    -> LazyGraphExecutor::DeviceContextArena* {
+  static DeviceContextArena* arena = new DeviceContextArena();
+  return arena;
+}
+
+void LazyGraphExecutor::DeviceContextArena::RegisterTensor(
+    std::shared_ptr<LazyTensor::Data> data) {
+  DeviceContext* devctx = GetDeviceContext(data->device);
+  std::lock_guard<std::mutex> lock(devctx->lock);
+  devctx->tensors_data.emplace(data->unique_id, data);
+}
+
+void LazyGraphExecutor::DeviceContextArena::UnregisterTensor(
+    LazyTensor::Data* data) {
+  DeviceContext* devctx = GetDeviceContext(data->device);
+  std::lock_guard<std::mutex> lock(devctx->lock);
+  devctx->tensors_data.erase(data->unique_id);
+}
+
+std::vector<LazyTensorPtr> LazyGraphExecutor::DeviceContextArena::
+    GetLiveTensors(const BackendDevice* device) {
+  std::vector<LazyTensorPtr> tensors;
+  auto fn = [&](DeviceContext* devctx) {
+    std::lock_guard<std::mutex> lock(devctx->lock);
+    for (auto& uid_wptr : devctx->tensors_data) {
+      std::shared_ptr<LazyTensor::Data> data = uid_wptr.second.lock();
+      if (data != nullptr) {
+        tensors.push_back(LazyTensor::Create(std::move(data)));
+      }
+    }
+  };
+  ForAllDeviceContexts(fn, device);
+  return tensors;
+}
+
+Value LazyGraphExecutor::DeviceContextArena::GetRngSeed(
+    const BackendDevice& device) {
+  static const at::ScalarType kSeedType = at::ScalarType::Long;
+  static const uint64_t kSeedMul = 214013;
+  static const uint64_t kSeedAdd = 2531011;
+  DeviceContext* devctx = GetDeviceContext(device);
+  std::lock_guard<std::mutex> lock(devctx->lock);
+  if (!devctx->seed_ir_value) {
+    devctx->seed_ir_value =
+        IrValueFromScalar(MakeIntScalar(devctx->seed), kSeedType, device);
+  }
+  // Keep the running seed as scalar as well, so we can return it directly
+  // without executing graphs.
+  devctx->running_seed = kSeedAdd + kSeedMul * devctx->running_seed;
+  // Compose new seeds from the root seed, to avoid creating too many
+  // computation parameters which might overflow the device capacity.
+  Value k = MakeScalar(MakeIntScalar(kSeedMul), kSeedType);
+  Value b = MakeScalar(MakeIntScalar(kSeedAdd), kSeedType);
+  devctx->seed_ir_value = b + k * devctx->seed_ir_value;
+  return devctx->seed_ir_value;
+}
+
+uint64_t LazyGraphExecutor::DeviceContextArena::GetRunningSeed(
+    const BackendDevice& device) {
+  DeviceContext* devctx = GetDeviceContext(device);
+  std::lock_guard<std::mutex> lock(devctx->lock);
+  return devctx->running_seed;
+}
+
+void LazyGraphExecutor::DeviceContextArena::SetRngSeed(
+    const BackendDevice& device,
+    uint64_t seed) {
+  DeviceContext* devctx = GetDeviceContext(device);
+  std::lock_guard<std::mutex> lock(devctx->lock);
+  devctx->seed = seed;
+  devctx->running_seed = devctx->seed;
+  devctx->seed_ir_value = Value();
+}
+
+void LazyGraphExecutor::DeviceContextArena::MarkStep(
+    const BackendDevice& device) {
+  DeviceContext* devctx = GetDeviceContext(device);
+  std::lock_guard<std::mutex> lock(devctx->lock);
+  devctx->seed = 1012031 + devctx->seed * 7012063;
+  devctx->running_seed = devctx->seed;
+  devctx->seed_ir_value = Value();
+}
+
+std::vector<BackendDevice> LazyGraphExecutor::DeviceContextArena::
+    GetActiveDevices() {
+  std::vector<BackendDevice> active_devices;
+  std::lock_guard<std::mutex> lock(lock_);
+  active_devices.reserve(device_contexts_.size());
+  for (auto& device_contexts : device_contexts_) {
+    active_devices.push_back(device_contexts.first);
+  }
+  return active_devices;
+}
+
+auto LazyGraphExecutor::DeviceContextArena::GetAllDeviceContexts()
+    -> std::vector<DeviceContext*> {
+  std::vector<DeviceContext*> all_device_contexts;
+  std::lock_guard<std::mutex> lock(lock_);
+  all_device_contexts.reserve(device_contexts_.size());
+  for (auto& device_contexts : device_contexts_) {
+    all_device_contexts.push_back(device_contexts.second);
+  }
+  return all_device_contexts;
+}
+
+void LazyGraphExecutor::DeviceContextArena::ForAllDeviceContexts(
+    const std::function<void(DeviceContext*)>& fn,
+    const BackendDevice* device) {
+  if (device == nullptr) {
+    for (auto devctx : GetAllDeviceContexts()) {
+      fn(devctx);
+    }
+  } else {
+    fn(GetDeviceContext(*device));
+  }
+}
+
+auto LazyGraphExecutor::DeviceContextArena::GetDeviceContext(
+    const BackendDevice& device) -> DeviceContext* {
+  std::lock_guard<std::mutex> lock(lock_);
+  auto it = device_contexts_.find(device);
+  if (it == device_contexts_.end()) {
+    it = device_contexts_.emplace(device, new DeviceContext()).first;
+  }
+  return it->second;
+}
+
+Value LazyGraphExecutor::DeviceContextArena::IrValueFromScalar(
+    const at::Scalar& value,
+    at::ScalarType scalar_type,
+    const BackendDevice& device) {
+  at::Tensor tensor = at::scalar_tensor(value, at::TensorOptions(scalar_type));
+  BackendDataPtr device_data = TensorToDataHandle(tensor, device);
+  return MakeDeviceData(std::move(device_data));
+}
+
+void LazyGraphExecutor::DeviceLocker::Lock() {
+  std::unique_lock<std::mutex> lock(mutex_);
+  cv_.wait(lock, [this] { return !locked_; });
+  CheckResetException();
+  locked_ = true;
+}
+
+void LazyGraphExecutor::DeviceLocker::Unlock(std::exception_ptr exptr) {
+  std::lock_guard<std::mutex> lock(mutex_);
+  locked_ = false;
+  exptr_ = std::move(exptr);
+  cv_.notify_all();
+}
+
+void LazyGraphExecutor::DeviceLocker::Barrier() {
+  std::unique_lock<std::mutex> lock(mutex_);
+  cv_.wait(lock, [this] { return !locked_; });
+  cv_.notify_all();
+  CheckResetException();
+}
+
+void LazyGraphExecutor::DeviceLocker::CheckResetException() {
+  std::exception_ptr exptr = std::move(exptr_);
+  exptr_ = nullptr;
+  if (exptr != nullptr) {
+    std::rethrow_exception(exptr);
+  }
+}
+
+auto LazyGraphExecutor::DeviceLockerArena::Get() -> DeviceLockerArena* {
+  static DeviceLockerArena* arena = new DeviceLockerArena();
+  return arena;
+}
+
+auto LazyGraphExecutor::DeviceLockerArena::GetLocker(
+    const BackendDevice& device) -> std::shared_ptr<DeviceLocker> {
+  std::lock_guard<std::mutex> lock(mutex_);
+  auto it = lockers_.find(device);
+  if (it == lockers_.end()) {
+    it = lockers_.emplace(device, std::make_shared<DeviceLocker>(device)).first;
+  }
+  return it->second;
+}
+
+void LazyGraphExecutor::DeviceLockerArena::DeviceBarrier(
+    const BackendDevice& device) {
+  auto locker = DeviceLockerArena::Get()->GetLocker(device);
+  locker->Barrier();
+}
+
+std::vector<ExceptionCleanup> LazyGraphExecutor::DeviceLockerArena::LockDevices(
+    const std::set<BackendDevice>& devices) {
+  std::vector<ExceptionCleanup> unlocker;
+  unlocker.reserve(devices.size());
+  for (auto& device : devices) {
+    unlocker.emplace_back(LockDevice(device));
+  }
+  return unlocker;
+}
+
+ExceptionCleanup LazyGraphExecutor::DeviceLockerArena::LockDevice(
+    const BackendDevice& device) {
+  VLOG(4) << "Waiting on device barrier for device " << device << " ...";
+  std::shared_ptr<DeviceLocker> locker;
+  {
+    TORCH_LAZY_TIMED("DeviceLockWait");
+    locker = DeviceLockerArena::Get()->GetLocker(device);
+    locker->Lock();
+  }
+  VLOG(4) << "Waiting on device barrier for device " << device << " done!";
+  return torch::lazy::ExceptionCleanup(
+      [locker = std::move(locker)](
+          torch::lazy::ExceptionCleanup::StatusType status) {
+        locker->Unlock(std::move(status));
+      });
+}
+
+auto LazyGraphExecutor::DataCacheArena::Get() -> DataCacheArena* {
+  static DataCacheArena* arena =
+      new DataCacheArena(FLAGS_torch_lazy_device_data_cache_size);
+  return arena;
+}
+
+LazyGraphExecutor::DataCacheArena::DataCacheArena(size_t max_cache_size)
+    : max_cache_size_(max_cache_size) {}
+
+BackendDataPtr LazyGraphExecutor::DataCacheArena::GetDeviceData(
+    const at::Tensor& tensor,
+    const BackendDevice& device) {
+  DataCacheArena::DataCache* cache = Get()->GetDataCache(device);
+  ;
+  BackendDataPtr device_data = cache->Get(tensor);
+  if (device_data == nullptr) {
+    at::Tensor tensor_copy = CopyTensor(tensor);
+    device_data = TensorToDataHandle(tensor_copy, device);
+    cache->Add(std::move(tensor_copy), device_data);
+    TORCH_LAZY_COUNTER("DeviceDataCacheMiss", 1);
+  }
+  return device_data;
+}
+
+BackendDataPtr LazyGraphExecutor::DataCacheArena::GetDeviceData(
+    const at::Scalar& value,
+    at::ScalarType scalar_type,
+    const BackendDevice& device) {
+  // Workaround since at::scalar_tensor doesn't support bfloat16 yet.
+  at::Tensor t = at::scalar_tensor(
+      value,
+      at::TensorOptions(
+          scalar_type == at::ScalarType::BFloat16 ? at::ScalarType::Float
+                                                  : scalar_type));
+  if (scalar_type == at::ScalarType::BFloat16) {
+    t = t.to(scalar_type);
+  }
+  return GetDeviceData(t, device);
+}
+
+size_t LazyGraphExecutor::DataCacheArena::TensorHasher::operator()(
+    const at::Tensor& tensor) const {
+  return HashReduce(
+      HashCombine(GetEnumValue(tensor.scalar_type()), TensorHash(tensor)));
+}
+
+bool LazyGraphExecutor::DataCacheArena::TensorComparer::operator()(
+    const at::Tensor& tensor1,
+    const at::Tensor& tensor2) const {
+  return TensorCompare(tensor1, tensor2);
+}
+
+auto LazyGraphExecutor::DataCacheArena::GetDataCache(
+    const BackendDevice& device) -> DataCache* {
+  std::lock_guard<std::mutex> lock(mutex_);
+  if (FLAGS_torch_lazy_enable_device_data_cache) {
+    auto it = device_caches_.find(device);
+    if (it == device_caches_.end()) {
+      std::unique_ptr<DataCache> cache(new DataCache(max_cache_size_));
+      it = device_caches_.emplace(device, std::move(cache)).first;
+    }
+    return it->second.get();
+  } else {
+    // If cache is disabled then always return a zero size cache
+    static std::unique_ptr<DataCache> s_empty_cache(new DataCache(0));
+    return s_empty_cache.get();
+  }
+}
+
+void LazyGraphExecutor::Register(LazyGraphExecutor* executor) {
+  lazy_graph_executor_registry.store(executor);
+}
 LazyGraphExecutor* LazyGraphExecutor::Get() {
-  static LazyGraphExecutor* executor = new LazyGraphExecutor();
+  auto* executor = lazy_graph_executor_registry.load();
+  TORCH_CHECK(executor, "Lazy graph executor not registered.");
   return executor;
 }
 
 void LazyGraphExecutor::RegisterTensor(std::shared_ptr<LazyTensor::Data> data) {
   DeviceContextArena::Get()->RegisterTensor(data);
+  TORCH_LAZY_COUNTER("CreateLtcTensor", 1);
 }
 
 void LazyGraphExecutor::UnregisterTensor(LazyTensor::Data* data) {
   DeviceContextArena::Get()->UnregisterTensor(data);
+  TORCH_LAZY_COUNTER("DestroyLtcTensor", 1);
 }
 
 Value LazyGraphExecutor::GetRngSeed(const BackendDevice& device) {
@@ -468,7 +427,7 @@ void LazyGraphExecutor::MarkStep(const BackendDevice& device) {
   TORCH_LAZY_COUNTER("MarkStep", 1);
   DeviceContextArena::Get()->MarkStep(device);
   ScopePusher::ResetScopes();
-  g_tls_data.Reset();
+  ResetTrimCounter();
   // Move TrieCache's current pointer back to its root
   TrieCache::Get()->ResetCurrent();
 }
@@ -498,7 +457,11 @@ std::vector<at::Tensor> LazyGraphExecutor::GetTensors(
   return GetTensorsFused(tensors);
 }
 
-size_t LazyGraphExecutor::IncTrimCounter() {
+void LazyGraphExecutor::ResetTrimCounter() const {
+  g_tls_data.Reset();
+}
+
+size_t LazyGraphExecutor::IncTrimCounter() const {
   return ++g_tls_data.trim_counter;
 }
 
@@ -531,8 +494,7 @@ Value LazyGraphExecutor::GetIrValueForScalarFromCodegen(
   if (IsSpecialScalar(value)) {
     return MakeScalar(value, value.type());
   }
-  BackendDataPtr data =
-      getBackend()->MakeComputationDataFromScalar(value, device);
+  auto data = GetDeviceData(value, value.type(), device);
   data->SetInfo(
       std::make_shared<DeviceDataInfo>(/*tensor_id=*/-1, /*read_only=*/true));
   return MakeDeviceData(std::move(data));
@@ -604,6 +566,10 @@ void LazyGraphExecutor::Async::Wait() {
   }
 }
 
+bool LazyGraphExecutor::ShouldSyncTensor(const LazyTensorPtr& tensor) const {
+  return tensor->GetIrValue()->op() != ltc_not_supported;
+}
+
 LazyGraphExecutor::SyncTensorCollection LazyGraphExecutor::CollectSyncTensors(
     const std::vector<LazyTensorPtr>& tensors,
     const SyncTensorsConfig& config) {
@@ -635,7 +601,7 @@ LazyGraphExecutor::SyncTensorCollection LazyGraphExecutor::CollectSyncTensors(
         tensors[i]->CurrentDataHandle() == nullptr) {
       Value ir_value = tensors[i]->CurrentIrValue();
       if (ir_value) {
-        if (getBackend()->ShouldSyncTensor(tensors[i])) {
+        if (ShouldSyncTensor(tensors[i])) {
           // Add only tensors which need to be synced.
           coll.hash = HashCombine(coll.hash, ir_value.hash());
           coll.indices.push_back(i);
@@ -679,13 +645,37 @@ std::vector<Value> LazyGraphExecutor::CollectRoots(
   return roots;
 }
 
-std::vector<BackendDataPtr> LazyGraphExecutor::FetchTensorData(
+void LazyGraphExecutor::ExtractIRAndPrepareTensorData(
     std::vector<LazyTensorPtr>* tensors,
     const SyncTensorsConfig& config,
-    c10::ArrayRef<size_t> indices) {
+    c10::ArrayRef<size_t> indices,
+    std::vector<Value>& ir_values,
+    std::vector<BackendDataPtr>& tensor_data_vec) {
+  ir_values.reserve(indices.size());
+  tensor_data_vec.reserve(indices.size());
+  for (auto index : indices) {
+    LazyTensorPtr& tensor = (*tensors)[index];
+    Value ir_value = tensor->CurrentIrValue();
+    ir_values.push_back(ir_value);
+    const BackendDevice& tensor_device = tensor->GetDevice();
+    BackendDataPtr handle = getBackend()->CreateDataPlaceholder(
+        tensor_device, std::move(tensor->shape()));
+    tensor_data_vec.push_back(handle);
+    if (tensor->CurrentDataHandle() == nullptr && config.sync_ltc_data) {
+      tensor->AssignIrValue(Value());
+    }
+  }
+}
+
+std::vector<torch::lazy::BackendDataPtr> LazyGraphExecutor::SetTensorData(
+    std::vector<LazyTensorPtr>* tensors,
+    const SyncTensorsConfig& config,
+    c10::ArrayRef<size_t> indices,
+    const std::vector<BackendDataPtr>& tensor_data_vec) {
   std::vector<BackendDataPtr> tensors_data;
   tensors_data.reserve(indices.size());
-  for (auto index : indices) {
+  for (const auto i : c10::irange(indices.size())) {
+    auto index = indices[i];
     LazyTensorPtr& tensor = (*tensors)[index];
     // If the config.force_ltc_data flag is true, the purpose of this tensor
     // sync operation is to truncate the IR graph and materialize device data in
@@ -698,11 +688,12 @@ std::vector<BackendDataPtr> LazyGraphExecutor::FetchTensorData(
     // completes.
     BackendDataPtr handle = tensor->CurrentDataHandle();
     if (handle == nullptr && config.force_ltc_data) {
-      const BackendDevice& tensor_device = tensor->GetDevice();
-      handle = getBackend()->CreateDataPlaceholder(
-          tensor_device, std::move(tensor->shape()));
-
-      tensor->SetDataHandle(handle, config.sync_ltc_data);
+      handle = tensor_data_vec[i];
+      // Note: We are not using SetHandleData method here since that method
+      // resets the ir_value. We have already done the resetting as part
+      // of ExtractIRAndPrepareTensorData to overlap with previous execution.
+      tensor->data()->handle = handle;
+      tensor->data()->tensor_data = c10::nullopt;
     }
     tensors_data.emplace_back(std::move(handle));
   }
@@ -710,12 +701,11 @@ std::vector<BackendDataPtr> LazyGraphExecutor::FetchTensorData(
 }
 
 LazyGraphExecutor::PostOrderData LazyGraphExecutor::RunPostOrder(
-    const std::vector<LazyTensorPtr>& tensors,
+    const std::vector<Value>& ir_values,
     SyncTensorCollection* coll) {
-  std::vector<Node*> roots;
-  roots.reserve(coll->indices.size());
-  for (auto index : coll->indices) {
-    Value ir_value = tensors.at(index)->CurrentIrValue();
+  std::vector<const Node*> roots;
+  roots.reserve(ir_values.size());
+  for (const auto& ir_value : ir_values) {
     roots.push_back(ir_value.node.get());
   }
   PostOrderData po_data;
@@ -746,7 +736,8 @@ LazyGraphExecutor::PostOrderData LazyGraphExecutor::RunPostOrder(
 std::shared_ptr<LazyGraphExecutor::Async> LazyGraphExecutor::TryRunCachedSync(
     std::vector<LazyTensorPtr>* tensors,
     SyncTensorCollection* coll,
-    PostOrderData* po_data) {
+    PostOrderData* po_data,
+    const std::vector<BackendDataPtr>& tensor_data_vec) {
   ComputationCache::TypePtr cached_computation =
       LookupCachedCompile(coll->hash);
   if (cached_computation == nullptr) {
@@ -763,50 +754,24 @@ std::shared_ptr<LazyGraphExecutor::Async> LazyGraphExecutor::TryRunCachedSync(
       tensors,
       coll,
       std::move(po_data->parameters_data),
-      std::move(cached_computation));
+      std::move(cached_computation),
+      tensor_data_vec);
 }
 
 LazyGraphExecutor::CompilationResult LazyGraphExecutor::Compile(
     const std::vector<LazyTensorPtr>& tensors,
     c10::ArrayRef<std::string> devices,
     const SyncTensorCollection& coll,
-    PostOrderData* po_data) {
+    PostOrderData* po_data,
+    const std::vector<Value>& ir_values) {
   auto lowering_ctx = LoweringContext::Create(
       "SyncTensorsGraph",
       coll.device,
       po_data->post_order,
       std::move(po_data->emission_map));
-  for (auto index : coll.indices) {
-    Value ir_value = tensors[index]->CurrentIrValue();
+  for (const auto& ir_value : ir_values) {
     lowering_ctx->AddResult(ir_value);
   }
-  if (FLAGS_torch_lazy_param_aliasing && coll.config.sync_ltc_data) {
-    // We can only alias at the step barrier, when force_ltc_data is true.
-    // Consider the case:
-    //   1. Tensor A(DEVICE_DATA)
-    //   2. Tensor B = A + 0.9
-    //   3. A += 0.4
-    // If we activate aliasing for A's graph, and we do:
-    //   print(A)
-    //   print(A)
-    // The first print will update DEVICE_DATA' with DEVICE_DATA+0.4, and the
-    // second print will again update DEVICE_DATA" with DEVICE_DATA'+0.4, which
-    // will lead to incorrect results.
-    // We cannot normally turn A's state into DEVICE_DATA, as if any of the
-    // sources is a view, this will not lead to correct results (as A's value
-    // taken at different times need to reflect view source changes):
-    //   1. Tensor A = some_graph_with_view_source(V)
-    //   2. print(A)
-    //   3. V += 1
-    //   4. print(A)
-    // The second print should reflect the new value due to V's changes.
-    // Also in the first example, unless we are doing a step barrier and hence
-    // include all live tensors, if the B value is not part of the graph, it
-    // will later fetch the new value of A, which is incorrect.
-    // But, when we issue a step barrier (force_ltc_data == true) we have to
-    // turn everything into DEVICE_DATA, so we can activate aliasing.
-    BuildInputOutputAliases(tensors, coll.indices, lowering_ctx.get());
-  }
 
   ComputationPtr computation = lowering_ctx->Build();
   // If force_ltc_data is true it means that we did a proper sync and are
@@ -824,7 +789,8 @@ LazyGraphExecutor::CompilationResult LazyGraphExecutor::Compile(
     // TODO(whc) should computation be allowed null here? (because it is in one
     // case)
     TORCH_CHECK(
-        computation->parameters_size() == po_data->parameters_data.size());
+        computation->parameters_size() ==
+        static_cast<int>(po_data->parameters_data.size()));
   }
 
   return {
@@ -857,40 +823,6 @@ LazyGraphExecutor::ComputationCache::TypePtr LazyGraphExecutor::
 typedef SSIZE_T ssize_t;
 #endif
 
-void LazyGraphExecutor::BuildInputOutputAliases(
-    const std::vector<LazyTensorPtr>& tensors,
-    c10::ArrayRef<size_t> indices,
-    LoweringContext* lowering_ctx) {
-  std::unordered_map<int64_t, size_t> output_tensor_id_map;
-  for (const auto i : c10::irange(indices.size())) {
-    size_t tensor_index = indices[i];
-    int64_t tensor_id = tensors[tensor_index]->GetUniqueId();
-    output_tensor_id_map[tensor_id] = i;
-  }
-  const std::vector<BackendDataPtr>& parameters_data =
-      lowering_ctx->GetParametersData();
-  std::vector<ssize_t> alias_map(indices.size(), -1);
-  for (const auto i : c10::irange(parameters_data.size())) {
-    DeviceDataInfo* data_info =
-        dynamic_cast<DeviceDataInfo*>(parameters_data[i]->info());
-    if (data_info != nullptr && !data_info->read_only) {
-      auto it = output_tensor_id_map.find(data_info->tensor_id);
-      if (it != output_tensor_id_map.end()) {
-        size_t output_index = it->second;
-        if (lowering_ctx->CheckResultShape(parameters_data[i], output_index) &&
-            alias_map[output_index] < 0) {
-          lowering_ctx->SetUpAlias({static_cast<int64_t>(output_index)}, i, {});
-          alias_map[output_index] = i;
-
-          VLOG(6) << "Aliased parameter " << i << " with output "
-                  << output_index << ": " << Shape(parameters_data[i]->shape());
-        }
-      }
-    }
-  }
-  TORCH_LAZY_VALUE_METRIC("InputOutputAliasCount", alias_map.size());
-}
-
 std::shared_ptr<LazyGraphExecutor::Async> LazyGraphExecutor::
     SyncTensorsGraphInternal(
         std::vector<LazyTensorPtr>* tensors,
@@ -903,17 +835,23 @@ std::shared_ptr<LazyGraphExecutor::Async> LazyGraphExecutor::
     TensorCollectionBarrier(&coll);
     return nullptr;
   }
-  PostOrderData po_data = RunPostOrder(*tensors, &coll);
   DebugUtil::SaveTensorsGraphInfo(
       "ScheduleSyncTensorsGraph", *tensors, &coll.indices);
+  std::vector<Value> ir_values;
+  std::vector<BackendDataPtr> tensor_data_vec;
+  ExtractIRAndPrepareTensorData(
+      tensors, coll.config, coll.indices, ir_values, tensor_data_vec);
+  PostOrderData po_data = RunPostOrder(ir_values, &coll);
   coll.hash = HashCombine(coll.hash, Hash(po_data.parameter_sequence));
   VLOG(4) << "Parameter sequence graph hash " << HashToString(coll.hash);
-  std::shared_ptr<Async> async = TryRunCachedSync(tensors, &coll, &po_data);
+  std::shared_ptr<Async> async =
+      TryRunCachedSync(tensors, &coll, &po_data, tensor_data_vec);
   if (async != nullptr) {
     return async;
   }
 
-  CompilationResult compile_result = Compile(*tensors, devices, coll, &po_data);
+  CompilationResult compile_result =
+      Compile(*tensors, devices, coll, &po_data, ir_values);
   if (GRAPH_DUMP_ENABLED) {
     auto* comp = compile_result.computation.get();
     LOG(ERROR) << "Add a cached computation with hash " << coll.hash
@@ -932,7 +870,8 @@ std::shared_ptr<LazyGraphExecutor::Async> LazyGraphExecutor::
       tensors,
       &coll,
       std::move(compile_result.parameters_data),
-      std::move(cached_computation));
+      std::move(cached_computation),
+      tensor_data_vec);
 }
 
 std::shared_ptr<LazyGraphExecutor::Async> LazyGraphExecutor::
@@ -948,12 +887,7 @@ std::shared_ptr<LazyGraphExecutor::Async> LazyGraphExecutor::
       std::move(tensors_data),
       std::move(cached_computation));
 
-  auto syncfn = [this, async, hash = coll->hash]() {
-    // For profiling lazy trace overhead
-    if (noop_execution_mode_) {
-      return;
-    }
-
+  auto syncfn = [async, hash = coll->hash]() {
     try {
       VLOG(3) << "Executing IR graph hash " << HashToString(hash)
               << " on device " << async->device << " ...";
@@ -988,10 +922,8 @@ std::shared_ptr<LazyGraphExecutor::Async> LazyGraphExecutor::
       // even in case the caller does not wait, and that is accomplished by
       // setting the unlockers status. In that case the exception will be
       // surfaced when the user tries to acquire the device locks the next time.
-      // std::exception_ptr exptr = std::current_exception();
       for (auto& unlocker : async->unlocker) {
-        std::exception_ptr exptr = std::current_exception();
-        unlocker.SetStatus(std::move(exptr));
+        unlocker.SetStatus(std::current_exception());
       }
       throw;
     }
@@ -1010,8 +942,10 @@ std::shared_ptr<LazyGraphExecutor::Async> LazyGraphExecutor::
         std::vector<LazyTensorPtr>* tensors,
         SyncTensorCollection* coll,
         std::vector<BackendDataPtr> parameters_data,
-        ComputationCache::TypePtr cached_computation) {
-  auto tensors_data = FetchTensorData(tensors, coll->config, coll->indices);
+        ComputationCache::TypePtr cached_computation,
+        const std::vector<BackendDataPtr>& tensor_data_vec) {
+  auto tensors_data =
+      SetTensorData(tensors, coll->config, coll->indices, tensor_data_vec);
   return ScheduleSyncTensorsGraph(
       coll,
       std::move(parameters_data),
@@ -1107,13 +1041,12 @@ std::vector<BackendDataPtr> LazyGraphExecutor::GatherTensorsData(
 }
 
 void LazyGraphExecutor::TensorCollectionBarrier(SyncTensorCollection* coll) {
-  static const std::string invalid_device(
-      "Unknown0"); /* Temp solution to idetify unassigned devices */
-  if (coll->device.toString().compare(invalid_device) == 0 ||
-      coll->unlocker.size() > 0) {
-    return;
-  }
   if (coll) {
+    static const std::string invalid_device(
+        "Unknown0"); /* Temp solution to idetify unassigned devices */
+    if (coll->device.toString() == invalid_device || !coll->unlocker.empty()) {
+      return;
+    }
     VLOG(4) << "Waiting on device barrier for device " << coll->device
             << " ...";
     {
@@ -1131,7 +1064,12 @@ hash_t LazyGraphExecutor::GetGraphHash(
   config.sync_ltc_data = false;
 
   auto coll = CollectSyncTensors(tensors, config);
-  auto po_data = RunPostOrder(tensors, &coll);
+  std::vector<Value> ir_values;
+  for (auto index : coll.indices) {
+    Value ir_value = tensors[index]->CurrentIrValue();
+    ir_values.push_back(ir_value);
+  }
+  auto po_data = RunPostOrder(ir_values, &coll);
   coll.hash = HashCombine(coll.hash, Hash(po_data.parameter_sequence));
   return coll.hash;
 }
