@@ -19,6 +19,11 @@ using namespace at;
 namespace torch {
 namespace utils {
 
+static const char* parse_privateuseone_backend() {
+  static std::string backend_name = "torch." + get_privateuse1_backend();
+  return backend_name.c_str();
+}
+
 static const char* backend_to_string(const at::Backend& backend) {
   switch (backend) {
     case at::Backend::CPU:
@@ -41,8 +46,10 @@ static const char* backend_to_string(const at::Backend& backend) {
       return "torch.hpu";
     case at::Backend::MPS:
       return "torch.mps";
+    case at::Backend::MTIA:
+      return "torch.mtia";
     case at::Backend::PrivateUse1:
-      return "torch.privateuseone";
+      return parse_privateuseone_backend();
     case at::Backend::Lazy:
       return "torch.lazy";
     case at::Backend::XLA:
@@ -54,7 +61,7 @@ static const char* backend_to_string(const at::Backend& backend) {
   }
 }
 
-std::string options_to_string(const at::TensorOptions options) {
+std::string options_to_string(const at::TensorOptions& options) {
   std::ostringstream ss;
   ss << backend_to_string(options.backend()) << "."
      << toString(at::typeMetaToScalarType(options.dtype())) << "Tensor";
@@ -70,9 +77,12 @@ std::string type_to_string(const at::DeprecatedTypeProperties& type) {
 
 at::TensorOptions options_from_string(const std::string& str) {
   static std::string cuda_prefix("torch.cuda.");
+  static std::string xpu_prefix("torch.xpu.");
   static c10::once_flag cpu_once;
   static c10::once_flag cuda_once;
+  static c10::once_flag xpu_once;
   static std::unordered_map<std::string, at::DeprecatedTypeProperties*> cpu_map;
+  static std::unordered_map<std::string, at::DeprecatedTypeProperties*> xpu_map;
   static std::unordered_map<std::string, at::DeprecatedTypeProperties*>
       cuda_map;
 
@@ -95,6 +105,16 @@ at::TensorOptions options_from_string(const std::string& str) {
       }
     });
     map = &cuda_map;
+  } else if (
+      std::mismatch(xpu_prefix.begin(), xpu_prefix.end(), str.begin()).first ==
+      xpu_prefix.end()) {
+    // torch.xpu. is prefix of str
+    c10::call_once(xpu_once, []() {
+      for (auto type : autograd::VariableType::allXPUTypes()) {
+        xpu_map.emplace(type_to_string(*type), type);
+      }
+    });
+    map = &xpu_map;
   } else {
     c10::call_once(cpu_once, []() {
       for (auto type : autograd::VariableType::allCPUTypes()) {
