@@ -1,24 +1,32 @@
-import inspect
-import torch
-import types
 import collections
-import textwrap
 import functools
-import warnings
+import inspect
 import sys
+import textwrap
+import types
+import warnings
 from typing import Dict, List, Set, Type
 
+import torch
+
 import torch._jit_internal as _jit_internal
 from torch._sources import fake_range
-from torch.jit.frontend import get_default_args, get_jit_class_def, get_jit_def, get_class_properties
 from torch.jit._builtins import _find_builtin
 from torch.jit._check import AttributeTypeIsSupportedChecker
-from torch.jit._state import _python_cu, _add_script_class, _get_script_class
+from torch.jit._state import _add_script_class, _get_script_class, _python_cu
+from torch.jit.frontend import (
+    get_class_properties,
+    get_default_args,
+    get_jit_class_def,
+    get_jit_def,
+)
 from torch.nn import Module
 
 
-ScriptMethodStub = collections.namedtuple('ScriptMethodStub', ('resolution_callback', 'def_', 'original_method'))
-PropertyStub = collections.namedtuple('PropertyStub', ('resolution_callback', 'def_'))
+ScriptMethodStub = collections.namedtuple(
+    "ScriptMethodStub", ("resolution_callback", "def_", "original_method")
+)
+PropertyStub = collections.namedtuple("PropertyStub", ("resolution_callback", "def_"))
 
 
 # TODO: there should be a more principled way of doing this.
@@ -28,9 +36,14 @@ ignored_attributes = [
     "_buffers",
     "_non_persistent_buffers_set",
     "_backward_hooks",
+    "_backward_pre_hooks",
     "_forward_hooks",
+    "_forward_hooks_with_kwargs",
     "_forward_pre_hooks",
+    "_forward_pre_hooks_with_kwargs",
+    "_forward_hooks_always_called",
     "_state_dict_hooks",
+    "_state_dict_pre_hooks",
     "_load_state_dict_pre_hooks",
     "_load_state_dict_post_hooks",
     "_modules",
@@ -38,22 +51,27 @@ ignored_attributes = [
     "dump_patches",
 ]
 
+
 def _compile_and_register_class(obj, rcb, qualified_name):
     script_class = _get_script_class(obj)
 
     if not script_class:
         ast = get_jit_class_def(obj, obj.__name__)
         defaults = torch.jit.frontend.get_default_args_for_class(obj)
-        script_class = torch._C._jit_script_class_compile(qualified_name, ast, defaults, rcb)
+        script_class = torch._C._jit_script_class_compile(
+            qualified_name, ast, defaults, rcb
+        )
         _add_script_class(obj, script_class)
 
     return script_class
 
+
 def make_stub(func, name):
     rcb = _jit_internal.createResolutionCallbackFromClosure(func)
     ast = get_jit_def(func, name, self_name="RecursiveScriptModule")
     return ScriptMethodStub(rcb, ast, func)
 
+
 def make_stub_from_method(nn_module, method_name):
     func = getattr(nn_module, method_name)
     if isinstance(func, ScriptMethodStub):
@@ -81,11 +99,14 @@ def make_stubs_from_exported_methods(mod):
 
     return stubs
 
+
 def jit_ignored_properties(module):
-    user_annotated_ignored_attributes = getattr(module, "__jit_ignored_attributes__", list())
+    user_annotated_ignored_attributes = getattr(
+        module, "__jit_ignored_attributes__", list()
+    )
 
     def get_properties_names(module):
-        return set(k for k, v in vars(module).items() if isinstance(v, property))
+        return {k for k, v in vars(module).items() if isinstance(v, property)}
 
     properties = get_properties_names(type(module))
     user_annoted_ignored_properties = set()
@@ -95,30 +116,70 @@ def jit_ignored_properties(module):
             user_annoted_ignored_properties.add(ignored_attr)
     return user_annoted_ignored_properties
 
+
 # base types that can be constants
 # in addition, tuples and lists of these base types are also considered constants
 # If you edit this list, then you also need to edit the handlers in
 # ConstantValue in jit/script/init.cpp
-_constant_types = (bool, float, int, str, type(None), torch.device, torch.layout, torch.dtype)
+_constant_types = (
+    bool,
+    float,
+    int,
+    str,
+    type(None),
+    torch.device,
+    torch.layout,
+    torch.dtype,
+)
+
 
 def _get_valid_constant(attr, v, owner_type):
     if isinstance(v, _constant_types):
         return v
-    elif isinstance(v, tuple) or isinstance(v, list):
+    elif isinstance(v, (tuple, list)):
         return tuple(_get_valid_constant(attr, x, owner_type) for x in v)
     constants = ", ".join(torch.typename(typ) for typ in _constant_types)
-    raise TypeError(textwrap.dedent("""
-        '{}' object in attribute '{}.{}' is not a valid constant.
+    raise TypeError(
+        textwrap.dedent(
+            f"""
+        '{torch.typename(type(v))}' object in attribute '{owner_type}.{attr}' is not a valid constant.
         Valid constants are:
         1. a nn.ModuleList
-        2. a value of type {{{}}}
+        2. a value of type {{{constants}}}
         3. a list or tuple of (2)
-        """.format(torch.typename(type(v)), owner_type, attr, constants)))
+        """
+        )
+    )
 
 
 class SourceContext(torch._C._jit_tree_views.SourceRangeFactory):
     def __init__(self, source, filename, file_lineno, leading_whitespace_len):
-        super(SourceContext, self).__init__(source, filename, file_lineno, leading_whitespace_len)
+        super().__init__(source, filename, file_lineno, leading_whitespace_len)
+
+
+def get_annotations(obj):
+    if sys.version_info < (3, 10):
+        return getattr(obj, "__annotations__", {})
+    # In Python-3.10+ it is recommended to use inspect.get_annotations
+    # See https://docs.python.org/3.10/howto/annotations.html
+    # But also, in 3.10 annotations from base class are not inherited
+    # by unannotated derived one, so they must be manually extracted
+    annotations = inspect.get_annotations(obj)
+    if annotations:
+        return annotations
+
+    def get_cls_annotations(cls):
+        cls_annotations = inspect.get_annotations(cls)
+        if cls_annotations:
+            return cls_annotations
+        for base in cls.__bases__:
+            cls_annotations = get_cls_annotations(base)
+            if cls_annotations:
+                return cls_annotations
+        return {}
+
+    cls = obj if isinstance(obj, type) else type(obj)
+    return get_cls_annotations(cls)
 
 
 def infer_concrete_type_builder(nn_module, share_types=True):
@@ -137,27 +198,14 @@ def infer_concrete_type_builder(nn_module, share_types=True):
     if isinstance(nn_module, (torch.nn.ParameterDict)):
         concrete_type_builder.set_parameter_dict()
 
-    def get_annotations(obj):
-        if sys.version_info < (3, 10):
-            return getattr(obj, '__annotations__', {})
-        # In Python-3.10+ it is recommended to use inspect.get_annotations
-        # See https://docs.python.org/3.10/howto/annotations.html
-        # But also, in 3.10 annotations from base class are not inherited
-        # by unannotated derived one, so they must be manually extracted
-        annotations = inspect.get_annotations(obj)
-        if len(annotations) > 0:
-            return annotations
-        cls = obj if isinstance(obj, type) else type(obj)
-        if len(cls.__bases__) == 0:
-            return {}
-        return inspect.get_annotations(cls.__bases__[0])
-
     class_annotations = get_annotations(nn_module)
     if isinstance(nn_module, (torch.ao.quantization.QuantWrapper)):
         class_annotations = {}
 
     # Get user-annotated ignored attributes.
-    user_annotated_ignored_attributes = getattr(nn_module, "__jit_ignored_attributes__", list())
+    user_annotated_ignored_attributes = getattr(
+        nn_module, "__jit_ignored_attributes__", list()
+    )
     concrete_type_builder.add_ignored_attributes(user_annotated_ignored_attributes)
     ignored_properties = jit_ignored_properties(nn_module)
 
@@ -170,8 +218,14 @@ def infer_concrete_type_builder(nn_module, share_types=True):
         # is also true!
         inferred = False
         try:
-            if name in class_annotations and class_annotations[name] != torch.nn.Module.__annotations__["forward"]:
-                ann_to_type = torch.jit.annotations.ann_to_type(class_annotations[name], fake_range())
+            if (
+                name in class_annotations
+                and class_annotations[name]
+                != torch.nn.Module.__annotations__["forward"]
+            ):
+                ann_to_type = torch.jit.annotations.ann_to_type(
+                    class_annotations[name], fake_range()
+                )
                 attr_type = torch._C.InferredType(ann_to_type)
             elif isinstance(item, torch.jit.Attribute):
                 ann_to_type = torch.jit.annotations.ann_to_type(item.type, fake_range())
@@ -180,9 +234,7 @@ def infer_concrete_type_builder(nn_module, share_types=True):
                 attr_type = torch._C._jit_try_infer_type(item)
                 inferred = True
         except RuntimeError as re:
-            raise RuntimeError(
-                "Error inferring type for {name}: {item}: {re}".format(name=name, item=item, re=re)
-            )
+            raise RuntimeError(f"Error inferring type for {name}: {item}: {re}") from re
 
         return attr_type, inferred
 
@@ -224,7 +276,9 @@ def infer_concrete_type_builder(nn_module, share_types=True):
         if attr_type.success():
             assert attr_type.type().is_interface_type()
             # if the type can be inferred, it should be a module interface type
-            sub_concrete_type = torch._C.ConcreteModuleType.from_jit_type(attr_type.type())
+            sub_concrete_type = torch._C.ConcreteModuleType.from_jit_type(
+                attr_type.type()
+            )
         else:
             # otherwise we get the concrete module type for item and add it to concrete_type
             sub_concrete_type = get_module_concrete_type(item, share_types)
@@ -251,26 +305,38 @@ def infer_concrete_type_builder(nn_module, share_types=True):
             elif name in nn_module._parameters:
                 hint = "parameter"
             else:
-                raise AssertionError("added_names must be submodule, parameter, or buffer")
+                raise AssertionError(
+                    "added_names must be submodule, parameter, or buffer"
+                )
 
-            warnings.warn("'{}' was found in ScriptModule constants, "
-                          " but it is a non-constant {}. Consider removing it.".format(name, hint))
+            warnings.warn(
+                f"'{name}' was found in ScriptModule constants, "
+                f" but it is a non-constant {hint}. Consider removing it."
+            )
             continue
         if not hasattr(nn_module, name):
             # TODO: We should really error in this case, but its bc-breaking so
             # we need to warn for at least one release
-            warnings.warn("'{}' was found in ScriptModule constants, "
-                          "but was not actually set in __init__. "
-                          "Consider removing it.".format(name))
+            warnings.warn(
+                f"'{name}' was found in ScriptModule constants, "
+                "but was not actually set in __init__. "
+                "Consider removing it."
+            )
             continue
         value = getattr(nn_module, name)
-        concrete_type_builder.add_constant(name, _get_valid_constant(name, value, type(nn_module).__name__))
+        concrete_type_builder.add_constant(
+            name, _get_valid_constant(name, value, type(nn_module).__name__)
+        )
         added_names.add(name)
 
     # populate overloads
     overloads = getattr(nn_module, "__overloads__", {})
     # update with any annotated overloads
-    overloads.update(get_overload_name_mapping(get_overload_annotations(nn_module, ignored_properties)))
+    overloads.update(
+        get_overload_name_mapping(
+            get_overload_annotations(nn_module, ignored_properties)
+        )
+    )
     for name, overloaded_names in overloads.items():
         concrete_type_builder.add_overload(name, overloaded_names)
 
@@ -295,16 +361,17 @@ def infer_concrete_type_builder(nn_module, share_types=True):
             try:
                 scripted_fn = torch.jit.script(value)
                 concrete_type_builder.add_function_attribute(
-                    name,
-                    torch._C._jit_try_infer_type(scripted_fn).type(),
-                    value)
+                    name, torch._C._jit_try_infer_type(scripted_fn).type(), value
+                )
             except Exception as e:
                 # If we fail to script the function, it isn't a hard error.
                 # Instead, we will add it to the list of attributes we failed
                 # to convert, with the compilation error.
-                hint = ("(This function exists as an attribute on the Python module, "
-                        "but we failed to compile it to a TorchScript function. "
-                        "\nThe error stack is reproduced here:\n{}").format(e)
+                hint = (
+                    "(This function exists as an attribute on the Python module, "
+                    "but we failed to compile it to a TorchScript function. "
+                    f"\nThe error stack is reproduced here:\n{e}"
+                )
                 concrete_type_builder.add_failed_attribute(name, hint)
                 pass
 
@@ -320,9 +387,8 @@ def infer_concrete_type_builder(nn_module, share_types=True):
         # Handle Script function attributes
         if isinstance(value, torch.jit.ScriptFunction):
             concrete_type_builder.add_function_attribute(
-                name,
-                torch._C._jit_try_infer_type(value).type(),
-                value)
+                name, torch._C._jit_try_infer_type(value).type(), value
+            )
             continue
 
         # If we got here, this is a regular "data" attribute, add it to the concrete type
@@ -332,11 +398,17 @@ def infer_concrete_type_builder(nn_module, share_types=True):
         else:
             # TODO: could add more detail here. For example, what the user should do
             # when the pytype is `list` or `NoneType`
-            inferred_msg = "Its type was inferred; try adding a type annotation for the attribute." if inferred else ""
+            inferred_msg = (
+                "Its type was inferred; try adding a type annotation for the attribute."
+                if inferred
+                else ""
+            )
             additional_info = f"{attr_type.reason()}. {inferred_msg}"
-            hint = "(This attribute exists on the Python module, " \
-                f"but we failed to convert Python type: '{torch.typename(type(value))}' " \
+            hint = (
+                "(This attribute exists on the Python module, "
+                f"but we failed to convert Python type: '{torch.typename(type(value))}' "
                 f"to a TorchScript type. {additional_info})"
+            )
             concrete_type_builder.add_failed_attribute(name, hint)
 
     # add hooks to concrete type
@@ -347,7 +419,8 @@ def infer_concrete_type_builder(nn_module, share_types=True):
 
     return concrete_type_builder
 
-class ConcreteTypeStore(object):
+
+class ConcreteTypeStore:
     type_store: Dict[Type[Module], List[torch._C.ConcreteModuleType]]
     methods_compiled: Set[torch._C.ConcreteModuleType]
 
@@ -379,10 +452,13 @@ class ConcreteTypeStore(object):
         self.type_store[nn_module_type].append(concrete_type)
         return concrete_type
 
+
 concrete_type_store = ConcreteTypeStore()
 
 
-def create_methods_and_properties_from_stubs(concrete_type, method_stubs, property_stubs):
+def create_methods_and_properties_from_stubs(
+    concrete_type, method_stubs, property_stubs
+):
     method_defs = [m.def_ for m in method_stubs]
     method_rcbs = [m.resolution_callback for m in method_stubs]
     method_defaults = [get_default_args(m.original_method) for m in method_stubs]
@@ -390,7 +466,10 @@ def create_methods_and_properties_from_stubs(concrete_type, method_stubs, proper
     property_defs = [p.def_ for p in property_stubs]
     property_rcbs = [p.resolution_callback for p in property_stubs]
 
-    concrete_type._create_methods_and_properties(property_defs, property_rcbs, method_defs, method_rcbs, method_defaults)
+    concrete_type._create_methods_and_properties(
+        property_defs, property_rcbs, method_defs, method_rcbs, method_defaults
+    )
+
 
 def create_hooks_from_stubs(concrete_type, hook_stubs, pre_hook_stubs):
     hook_defs = [h.def_ for h in hook_stubs]
@@ -401,6 +480,7 @@ def create_hooks_from_stubs(concrete_type, hook_stubs, pre_hook_stubs):
 
     concrete_type._create_hooks(hook_defs, hook_rcbs, pre_hook_defs, pre_hook_rcbs)
 
+
 def get_module_concrete_type(nn_module, share_types=True):
     """
     Gets a concrete type for nn_modules. If share_types is True, the concrete
@@ -415,8 +495,9 @@ def get_module_concrete_type(nn_module, share_types=True):
         A concrete type for nn_module.
     """
     assert isinstance(nn_module, Module)
-    if isinstance(nn_module, torch.jit.ScriptModule) and \
-            hasattr(nn_module, "_concrete_type"):
+    if isinstance(nn_module, torch.jit.ScriptModule) and hasattr(
+        nn_module, "_concrete_type"
+    ):
         return nn_module._concrete_type
 
     if share_types:
@@ -431,6 +512,7 @@ def get_module_concrete_type(nn_module, share_types=True):
 
     return concrete_type
 
+
 def create_script_class(obj):
     """
     Create and return a RecursiveScriptClass instance from a Python object.
@@ -452,6 +534,7 @@ def create_script_class(obj):
     # Wrap the torch._C.ScriptObject in a RecursiveScriptClass instance.
     return wrap_cpp_class(cpp_object)
 
+
 def create_script_module(nn_module, stubs_fn, share_types=True, is_tracing=False):
     """
     Creates a new ScriptModule from an nn.Module
@@ -475,6 +558,7 @@ def create_script_module(nn_module, stubs_fn, share_types=True, is_tracing=False
         AttributeTypeIsSupportedChecker().check(nn_module)
     return create_script_module_impl(nn_module, concrete_type, stubs_fn)
 
+
 def create_script_module_impl(nn_module, concrete_type, stubs_fn):
     """
     Convert an nn.Module to a RecursiveScriptModule.
@@ -489,22 +573,30 @@ def create_script_module_impl(nn_module, concrete_type, stubs_fn):
     property_stubs = get_property_stubs(nn_module)
     hook_stubs, pre_hook_stubs = get_hook_stubs(nn_module)
 
-    user_annotated_ignored_attributes = getattr(nn_module, "__jit_ignored_attributes__", list())
+    user_annotated_ignored_attributes = getattr(
+        nn_module, "__jit_ignored_attributes__", list()
+    )
     ignored_properties = jit_ignored_properties(nn_module)
 
     def init_fn(script_module):
         # Initialize the ScriptModule:
         # 1. Copy the attributes/parameters/buffers from the original `nn_module` to the new ScriptModule.
-        for name, (attr_type, is_param) in concrete_type.get_attributes().items():
+        for name in concrete_type.get_attributes().keys():
             orig_value = getattr(nn_module, name)
-            orig_value = orig_value.value if isinstance(orig_value, torch.jit.Attribute) else orig_value
+            orig_value = (
+                orig_value.value
+                if isinstance(orig_value, torch.jit.Attribute)
+                else orig_value
+            )
             cpp_module.setattr(name, orig_value)
 
         # 2. Copy the submodules from the original `nn_module` to the new ScriptModule,
         #    recursively scripting them.
         for name, sub_concrete_type in concrete_type.get_modules():
             orig_value = getattr(nn_module, name)
-            assert isinstance(orig_value, Module), "Expected Module but got {}".format(type(orig_value))
+            assert isinstance(
+                orig_value, Module
+            ), f"Expected Module but got {type(orig_value)}"
             module_type = sub_concrete_type.jit_type
             if isinstance(module_type, torch._C.InterfaceType):
                 # use the interface inference rule to compile the module
@@ -513,7 +605,9 @@ def create_script_module_impl(nn_module, concrete_type, stubs_fn):
                 scripted = orig_value
             else:
                 # always reuse the provided stubs_fn to infer the methods to compile
-                scripted = create_script_module_impl(orig_value, sub_concrete_type, stubs_fn)
+                scripted = create_script_module_impl(
+                    orig_value, sub_concrete_type, stubs_fn
+                )
 
             cpp_module.setattr(name, scripted)
             script_module._modules[name] = scripted
@@ -539,7 +633,9 @@ def create_script_module_impl(nn_module, concrete_type, stubs_fn):
 
     # Compile methods if necessary
     if concrete_type not in concrete_type_store.methods_compiled:
-        create_methods_and_properties_from_stubs(concrete_type, method_stubs, property_stubs)
+        create_methods_and_properties_from_stubs(
+            concrete_type, method_stubs, property_stubs
+        )
         # Create hooks after methods to ensure no name collisions between hooks and methods.
         # If done before, hooks can overshadow methods that aren't exported.
         create_hooks_from_stubs(concrete_type, hook_stubs, pre_hook_stubs)
@@ -553,20 +649,26 @@ def create_script_module_impl(nn_module, concrete_type, stubs_fn):
     for idx, fn in enumerate(script_module._c._get_forward_hooks()):
         script_module._forward_hooks[idx] = fn
 
-
     # Special handling so methods like __len__ work in script methods on classes derived from containers
-    if isinstance(nn_module, (torch.nn.ModuleList, torch.nn.Sequential, torch.nn.ModuleDict)) and \
-            '__len__' not in cpp_module._method_names():
-        script_module.define("def __len__(self):\n   return {}\n".format(len(nn_module)))
-    if isinstance(nn_module, torch.nn.ModuleDict) and \
-            '__contains__' not in cpp_module._method_names():
+    if (
+        isinstance(
+            nn_module, (torch.nn.ModuleList, torch.nn.Sequential, torch.nn.ModuleDict)
+        )
+        and "__len__" not in cpp_module._method_names()
+    ):
+        script_module.define(f"def __len__(self):\n   return {len(nn_module)}\n")
+    if (
+        isinstance(nn_module, torch.nn.ModuleDict)
+        and "__contains__" not in cpp_module._method_names()
+    ):
         if len(nn_module.keys()):
             keys = repr(list(nn_module.keys()))
-            script_module.define("def __contains__(self, key: str):\n   return key in {}\n".format(keys))
+            script_module.define(
+                f"def __contains__(self, key: str):\n   return key in {keys}\n"
+            )
         else:
             script_module.define("def __contains__(self, key: str):\n   return False\n")
 
-
     # Make the compiled methods available to the Python ScriptModule class.
     for method_stub in method_stubs:
         if method_stub.original_method is None:
@@ -584,14 +686,15 @@ def create_script_module_impl(nn_module, concrete_type, stubs_fn):
         # Wrap the original to propagate docstrings and such.
         # TODO: we don't currently do this functions that are recursively
         # compiled, we should.
-        wrapped_script_method = functools.wraps(method_stub.original_method)(script_method)
+        wrapped_script_method = functools.wraps(method_stub.original_method)(
+            script_method
+        )
 
         # Add the methods to the script_module directly. This ensures they will
         # be found first when `name` is looked up (as opposed to the stubs or
         # nn.Module.forward)
         script_module.__dict__[name] = wrapped_script_method
 
-
     # Make module properties available on the Python ScriptModule class.
     for property_stub in property_stubs:
         property_name = property_stub.def_.name().name
@@ -607,7 +710,10 @@ def create_script_module_impl(nn_module, concrete_type, stubs_fn):
         if name in ignored_properties:
             continue
         item = getattr(nn_module, name, None)
-        if _jit_internal.get_torchscript_modifier(item) is _jit_internal.FunctionModifiers.COPY_TO_SCRIPT_WRAPPER:
+        if (
+            _jit_internal.get_torchscript_modifier(item)
+            is _jit_internal.FunctionModifiers.COPY_TO_SCRIPT_WRAPPER
+        ):
             add_python_attr_to_scripted_model(script_module, nn_module, name)
 
     return script_module
@@ -625,10 +731,12 @@ def script_model_defines_attr(script_model, attr):
         return False
     return script_attr != default_attr
 
+
 def add_python_attr_to_scripted_model(script_model, orig, attr):
     if hasattr(orig, attr) and script_model_defines_attr(script_model, attr):
         setattr(script_model, attr, getattr(orig, attr))
 
+
 def get_overload_annotations(mod, jit_ignored_properties):
     # original function => [(mangled overload name, overload function)]
     overloads = {}
@@ -642,19 +750,25 @@ def get_overload_annotations(mod, jit_ignored_properties):
 
         # builtin functions like repr() in python 2 do not have __module__ defined
         if hasattr(item, "__module__") and item.__module__ is not None:
-            method_overloads = _jit_internal._get_overloaded_methods(item, mod.__class__)
+            method_overloads = _jit_internal._get_overloaded_methods(
+                item, mod.__class__
+            )
             if method_overloads is None:
                 continue
 
             if item.__func__ in method_overloads:
-                raise RuntimeError(_jit_internal.get_overload_no_implementation_error_message(
-                    'method', item.__func__))
+                raise RuntimeError(
+                    _jit_internal.get_overload_no_implementation_error_message(
+                        "method", item.__func__
+                    )
+                )
 
             names = [name + "__" + str(i) for i in range(len(method_overloads))]
             overloads[item] = list(zip(names, method_overloads))
 
     return overloads
 
+
 def get_overload_name_mapping(overload_info):
     # Same format as __overloads__
     # original function => [overload names]
@@ -668,40 +782,61 @@ def get_overload_name_mapping(overload_info):
             overload_name_mappings[original_name].append(overload_name)
     return overload_name_mappings
 
+
 def _check_no_signature(func):
-    signature = torch.jit.annotations.get_signature(func, None, fake_range(), inspect.ismethod(func))
+    signature = torch.jit.annotations.get_signature(
+        func, None, fake_range(), inspect.ismethod(func)
+    )
     if signature is None:
         qual_name = _jit_internal._qualified_name(func)
-        raise RuntimeError("Must explicitly add type annotations to overloaded functions: {}".format(qual_name))
+        raise RuntimeError(
+            f"Must explicitly add type annotations to overloaded functions: {qual_name}"
+        )
+
 
 def make_stubs_for_overloads(overload_info):
     overload_stubs = []
     for orig_fn, overloads in overload_info.items():
-        orig_ast = get_jit_def(orig_fn, orig_fn.__name__, self_name="RecursiveScriptModule")
+        orig_ast = get_jit_def(
+            orig_fn, orig_fn.__name__, self_name="RecursiveScriptModule"
+        )
         for overload_name, overload_fn in overloads:
             _check_no_signature(overload_fn)
-            over_ast = get_jit_def(overload_fn, overload_fn.__name__, self_name="RecursiveScriptModule")
-            new_ast = torch._C._replace_overloaded_method_decl(over_ast.decl(), orig_ast, overload_name)
+            over_ast = get_jit_def(
+                overload_fn, overload_fn.__name__, self_name="RecursiveScriptModule"
+            )
+            new_ast = torch._C._replace_overloaded_method_decl(
+                over_ast.decl(), orig_ast, overload_name
+            )
             _rcb = _jit_internal.createResolutionCallbackFromClosure(orig_fn)
             overload_stubs.append(ScriptMethodStub(_rcb, new_ast, overload_fn))
     return overload_stubs
 
+
 def check_module_initialized(mod):
     assert isinstance(mod, torch.nn.Module)
-    if not hasattr(mod, '_parameters'):
-        raise RuntimeError("'{}' has not been initialized, did you forget to call 'super()'?"
-                           .format(torch.typename(type(mod))))
+    if not hasattr(mod, "_parameters"):
+        raise RuntimeError(
+            f"'{torch.typename(type(mod))}' has not been initialized, did you forget to call 'super()'?"
+        )
 
     # This is to avoid importing torch.distributed.nn
-    if not hasattr(mod, 'remote_parameters'):
+    if not hasattr(mod, "remote_parameters"):
         for name, param in mod._parameters.items():
             if param is not None and torch.nn.parameter.is_lazy(param):
-                raise RuntimeError("'{}' has uninitialized parameters {}. Did you forget to run a forward pass?"
-                                   .format(torch.typename(type(mod)), name))
+                raise RuntimeError(
+                    "'{}' has uninitialized parameters {}. Did you forget to run a forward pass?".format(
+                        torch.typename(type(mod)), name
+                    )
+                )
         for name, buf in mod._buffers.items():
             if buf is not None and torch.nn.parameter.is_lazy(buf):
-                raise RuntimeError("'{}' has uninitialized buffers {}. Did you forget to run a forward pass?"
-                                   .format(torch.typename(type(mod)), name))
+                raise RuntimeError(
+                    "'{}' has uninitialized buffers {}. Did you forget to run a forward pass?".format(
+                        torch.typename(type(mod)), name
+                    )
+                )
+
 
 def infer_methods_to_compile(nn_module):
     """
@@ -709,22 +844,29 @@ def infer_methods_to_compile(nn_module):
     points for compilation (TODO add a link when the rules are published).
     """
     check_module_initialized(nn_module)
-    user_annotated_ignored_attributes = getattr(nn_module, "__jit_ignored_attributes__", list())
+    user_annotated_ignored_attributes = getattr(
+        nn_module, "__jit_ignored_attributes__", list()
+    )
     ignored_properties = jit_ignored_properties(nn_module)
 
     methods: List[str] = []
-    if hasattr(nn_module, 'forward') and not _jit_internal.is_ignored_fn(nn_module.forward):
+    if hasattr(nn_module, "forward") and not _jit_internal.is_ignored_fn(
+        nn_module.forward
+    ):
         forward_func = getattr(nn_module.forward, "__func__", None)
         module_forward = getattr(torch.nn.Module, "forward", None)
         if forward_func != module_forward:
-            methods = ['forward']
+            methods = ["forward"]
 
     exported = []
     for name in dir(nn_module):
         if name in ignored_properties:
             continue
         item = getattr(nn_module, name, None)
-        if _jit_internal.get_torchscript_modifier(item) is _jit_internal.FunctionModifiers.EXPORT:
+        if (
+            _jit_internal.get_torchscript_modifier(item)
+            is _jit_internal.FunctionModifiers.EXPORT
+        ):
             exported.append(name)
 
     methods = methods + exported
@@ -807,7 +949,9 @@ def get_property_stubs(nn_module):
         item = getattr(module_ty, name, None)
         if isinstance(item, property):
             if not item.fget:
-                raise RuntimeError(f'Property {name} of {nn_module.__name__} must have a getter')
+                raise RuntimeError(
+                    f"Property {name} of {nn_module.__name__} must have a getter"
+                )
 
             rcbs[name] = _jit_internal.createResolutionCallbackFromClosure(item.fget)
 
@@ -841,6 +985,7 @@ def interface_script(mod_interface, nn_module):
 
     return create_script_module(nn_module, infer_interface_methods_to_compile)
 
+
 def try_compile_fn(fn, loc):
     if _jit_internal.is_ignored_fn(fn):
         # Don't do anything for @ignore'd functions
@@ -852,9 +997,11 @@ def try_compile_fn(fn, loc):
         return None
 
     if not inspect.isfunction(fn) and not inspect.ismethod(fn):
-        raise RuntimeError("`{}` is not a function. Recursive scripting only supports "
-                           "Python functions or methods currently.\n"
-                           "Consider manually annotating `{}` with @torch.jit.script.".format(fn, fn))
+        raise RuntimeError(
+            f"`{fn}` is not a function. Recursive scripting only supports "
+            "Python functions or methods currently.\n"
+            f"Consider manually annotating `{fn}` with @torch.jit.script."
+        )
 
     # We don't have the actual scope where the function was defined, but we can
     # extract the necessary info from the closed over variables on the function
@@ -862,20 +1009,25 @@ def try_compile_fn(fn, loc):
     rcb = _jit_internal.createResolutionCallbackFromClosure(fn)
     return torch.jit.script(fn, _rcb=rcb)
 
+
 def wrap_cpp_class(cpp_class):
     """
     Wrap this torch._C.Object in a Python RecursiveScriptClass.
     """
     return torch.jit.RecursiveScriptClass(cpp_class)
 
+
 def wrap_cpp_module(cpp_module):
     """
     Wrap this torch._C.ScriptModule in a Python ScriptModule, recursively for all submodules
     """
+
     def init_fn(script_module):
         for name, cpp_module in torch._C.ModuleDict(script_module._c).items():
             setattr(script_module, name, wrap_cpp_module(cpp_module))
-        script_module._concrete_type = torch._C.ConcreteModuleType.from_jit_type(script_module._c._type())
+        script_module._concrete_type = torch._C.ConcreteModuleType.from_jit_type(
+            script_module._c._type()
+        )
 
         for idx, fn in enumerate(script_module._c._get_forward_pre_hooks()):
             script_module._forward_pre_hooks[idx] = fn
@@ -884,6 +1036,7 @@ def wrap_cpp_module(cpp_module):
 
     return torch.jit.RecursiveScriptModule._construct(cpp_module, init_fn)
 
+
 def compile_unbound_method(concrete_type, fn):
     if _jit_internal.is_ignored_fn(fn):
         return None
@@ -894,6 +1047,7 @@ def compile_unbound_method(concrete_type, fn):
         create_methods_and_properties_from_stubs(concrete_type, (stub,), ())
     return stub
 
+
 def lazy_bind(concrete_type, unbound_method):
     """
     Returns a function that lazily binds `unbound_method` to a provided
@@ -901,6 +1055,7 @@ def lazy_bind(concrete_type, unbound_method):
     shenanigans that will poison type sharing are impossible at compile
     time.
     """
+
     def lazy_binding_method(cpp_module, *args):
         def init_fn(script_module):
             orig_class = concrete_type.py_class
